//****************
module AL_LOGIC_RAMFIFO (/*autoarg*/
                   // Inputs
                   rst, di, clkr, re, clkw, we,
                   // Outputs
                   do, empty_flag, full_flag, rdusedw, wrusedw
                   );
   
   parameter DATA_WIDTH = 8;
   parameter ADDR_WIDTH = 8;
   parameter SHOWAHEAD = 0;
   parameter IMPLEMENT = "AUTO";  // 9K | 32K | AUTO (all capitalized)
   input                     rst;
   input   [DATA_WIDTH-1:0]  di;
   input                     clkr;
   input                     re;
   input                     clkw;
   input                     we;
   output  [DATA_WIDTH-1:0]  do;
   output                    empty_flag;
   output                    full_flag;
   output  [ADDR_WIDTH:0]    rdusedw;
   output  [ADDR_WIDTH:0]    wrusedw;
endmodule
//****************
//****************
module gray2bin (
                  // Inputs
                  din,
                  // Outputs
                  dout
                  );
                  
   parameter WIDTH = 8;
   input   [WIDTH-1:0]  din;
   output  [WIDTH-1:0]  dout;
endmodule
//****************
//****************
module gray_count (
                   // Inputs
                   clk, enable, reset,
                   // Outputs
                   cnt
                   );
                  
   parameter WIDTH = 8;
   parameter INIT  = 0;
   input                clk;
   input                enable;
   input                reset;
   output  [WIDTH-1:0]  cnt;
endmodule
//****************
//****************
module showahead_buffer (/*autoarg*/
                         // Inputs
                         rstn_i, clk_i, ena_i, rd_i, data_i,
                         // Outputs
                         data_o, addr_inc_o, empty_o
                         );
   
   parameter DATA_WIDTH = 8;
   input                     rstn_i;
   input                     clk_i;
   input                     ena_i;
   input                     rd_i;
   input   [DATA_WIDTH-1:0]  data_i;
   output  [DATA_WIDTH-1:0]  data_o;
   output                    addr_inc_o;
   output                    empty_o;
endmodule
//****************
//****************
module  AL_LOGIC_BUF(o,i);     // user instantiated logic BUF
output o;
input i;
endmodule
//****************
//****************
module  AL_LOGIC_GSRN(gsrn);   // user controllable GSRN
input gsrn;
parameter USR_GSRN_EN   = "ENABLE";  // DISABLE/ENABLE set by wakeup register
endmodule
//****************
//****************
module  AL_LOGIC_BUFG(o,i);  // driver for global clk resource
output o;
input i;
endmodule
//****************
//****************
module  AL_LOGIC_BUFIO(clki,rst,coe,clko,clkdiv1,clkdivx);  // driver for IO clk resource
input clki;
input rst;
input coe;
output clko;        // for pad sinks
output clkdiv1;     // for other sinks
output clkdivx;
parameter GSR                = "DISABLE";    //  DISABLE/ENABLE 
parameter DIV                 = 2;                                 //  2/4
parameter STOPCLK = "DISABLE";                   // ENABLE,DISABLE
endmodule
//****************
//****************
module  AL_LOGIC_BUFGMUX(o,i0,i1,s); // global clk switch mux
output o;
input i0;
input i1;
input s;
parameter INIT_OUT = "0";
parameter PRESELECT_I0 = "TRUE";
parameter PRESELECT_I1 = "FALSE";
endmodule
//****************
//****************
module AL_LOGIC_CCLK(cclk);  // config clk
output cclk;
endmodule
//****************
//****************
module  AL_LOGIC_IDELAY(o,i);
output o;
input i;
parameter INDEL = 0;  // 0-31
endmodule
//****************
//****************
module AL_LOGIC_IDDR (q1, q2, clk, d, rst);
    output q1;
    output q2;
    input clk;
    input d;
    input rst;
    parameter SRMODE = "SYNC"; //SYNC, ASYNC
endmodule
//****************
//****************
module AL_LOGIC_ODDR (q, clk, d1, d2, rst);
    output q;
    input clk;
    input d1;
    input d2;
    input rst;
    parameter SRMODE = "SYNC"; //SYNC, ASYNC
endmodule
//****************
//****************
module AL_LOGIC_PROGRAMN(programn);  // user programn in AL3A10LG144
input programn;
endmodule
//****************
//****************
module AL_LOGIC_FIFO (
    rst,
    di, clkw, we, csw, 
    do, clkr, re, csr, ore,
    empty_flag, aempty_flag,
    full_flag, afull_flag
);
// 1kx9 by default
parameter DATA_WIDTH_W = 9;                 // 1 ... 9*48
parameter DATA_WIDTH_R = DATA_WIDTH_W;      // 1 ... 9*48
parameter DATA_DEPTH_W = 1024;                                        // 512, 1024, 2048, 4096, 8192
parameter DATA_DEPTH_R = DATA_WIDTH_W * DATA_DEPTH_W / DATA_WIDTH_R;  // 512, 1024, 2048, 4096, 8192
input   rst;
input   [DATA_WIDTH_W-1:0] di;
output  [DATA_WIDTH_R-1:0] do;
input   clkw, we;
input   clkr, re, ore; //DEFAULT := '0'
input [2:0] csw, csr;
output  empty_flag, aempty_flag; //DEFAULT := '0'
output  full_flag, afull_flag; //DEFAULT := '0'
parameter  MODE = "FIFO8K";
parameter  REGMODE_W   = "NOREG"; //NOREG, OUTREG
parameter  REGMODE_R   = "NOREG"; //NOREG, OUTREG   
parameter  E  = 0;
parameter  AE = 6;
parameter  AF = DATA_DEPTH_W-6;
parameter  F  = DATA_DEPTH_W;
parameter  GSR = "DISABLE";  //DISABLE, ENABLE
parameter  RESETMODE = "ASYNC"; //SYNC, ASYNC
parameter  ASYNC_RESET_RELEASE = "SYNC";  //SYNC, ASYNC
parameter ENDIAN = "LITTLE";    // LITTLE, BIG
endmodule
//****************
//****************
module AL_LOGIC_DRAM (di, waddr, we, wclk, do, raddr);
parameter DATA_WIDTH_W = 9;
parameter ADDR_WIDTH_W = 10;
parameter DATA_DEPTH_W = 2 ** ADDR_WIDTH_W;
parameter DATA_WIDTH_R = 9;
parameter ADDR_WIDTH_R = 10;
parameter DATA_DEPTH_R = 2 ** ADDR_WIDTH_R;
input [DATA_WIDTH_W-1:0]di;
input [ADDR_WIDTH_W-1:0]waddr;
input wclk;
input we;
output [DATA_WIDTH_R-1:0]do;
input [ADDR_WIDTH_R-1:0]raddr;
parameter INIT_FILE = "NONE";
endmodule
//****************
//****************
module AL_LOGIC_DRAM16X4 ( di, waddr, we, wclk, raddr, do);
parameter INIT_D0=16'h0000;
parameter INIT_D1=16'h0000;
parameter INIT_D2=16'h0000;
parameter INIT_D3=16'h0000;
  input [3:0]di;
  input [3:0]waddr;
  input wclk;
  input we;
  input [3:0]raddr;
  output [3:0]do;
endmodule
//****************
//****************
module AL_LOGIC_MULT (p, a, b, cea, ceb, cepd, clk, rstan, rstbn, rstpdn); 
    parameter INPUT_WIDTH_A = 18;
    parameter INPUT_WIDTH_B = 18;
    parameter OUTPUT_WIDTH = 36;
    output [OUTPUT_WIDTH-1:0] p;
    input [INPUT_WIDTH_A-1:0] a;
    input [INPUT_WIDTH_B-1:0] b;
    input cea; //DEFAULT := '0'
    input ceb; //DEFAULT := '0'
    input cepd; //DEFAULT := '0'
    input clk; //DEFAULT := '0'
    input rstan; //DEFAULT := '0'
    input rstbn; //DEFAULT := '0'
    input rstpdn; //DEFAULT := '0'
    parameter INPUTFORMAT = "SIGNED"; // SIGNED, UNSIGNED
    parameter INPUTREGA = "ENABLE";   // ENABLE, DISABLE
    parameter INPUTREGB = "ENABLE";   // ENABLE, DISABLE
    parameter OUTPUTREG = "ENABLE";   // ENABLE, DISABLE
    parameter SRMODE    = "ASYNC";    // ASYNC, SYNC
    parameter IMPLEMENT = "AUTO";    // AUTO, DSP, GATE
endmodule
//****************
//****************
module AL_LOGIC_SEQ_DIV (clk, rst, start, numer, denom, quotient, remain, done);

	parameter NUMER_WIDTH = 16;
  parameter DENOM_WIDTH = 16;
  input                     clk;
  input                     rst;  
  input                     start;  
  input  [NUMER_WIDTH-1:0]  numer;  
  input  [DENOM_WIDTH-1:0]  denom;  
  output [NUMER_WIDTH-1:0]  quotient;  
  output [DENOM_WIDTH-1:0]  remain;  
  output                    done;
endmodule
//****************
//****************
module  AL_LOGIC_MBOOT(rebootn, dynamic_addr); 	// config multiboot
input       rebootn;
input [7:0] dynamic_addr;		// high-8bit address
  parameter ADDR_SOURCE_SEL = "STATIC";	// STATIC/DYNAMIC 
  parameter STATIC_ADDR = 8'b0;		// high-8bit address
  parameter MBOOT_AUTO_SEL = "DISABLE";	// DISABLE/ENABLE
endmodule
//****************
//****************
module AL_LOGIC_DNA(dout, clk, din, shift_en);		// config dna 
parameter DNA = 64'h0000000000000000;
output dout;
input  clk;
input  din;
input  shift_en;
endmodule
//****************
//****************
module AL_PHY_BRAM (
  dia, csa,
  addra, cea, ocea, clka, wea, rsta,
  dib, csb,
  addrb, ceb, oceb, clkb, web, rstb,
  doa,
  dob);

output  [8:0] doa;
output  [8:0] dob;
input   [8:0] dia;
input   [8:0] dib;
input   [2:0] csa;
input   [2:0] csb;
input   cea, ocea, clka, wea, rsta;
input   ceb, oceb, clkb, web, rstb;
input   [12:0] addra;
input   [12:0] addrb;
parameter MODE = "DP8K";               // DP8K,SP8K,PDPW8K,FIFO8K : for DATA_WIDTH = 18, MODE must be "PDPW8K" or "FIFO8K" 
parameter DATA_WIDTH_A = "9";          // 1, 2, 4, 9, 18
parameter DATA_WIDTH_B = "9";          // 1, 2, 4, 9, 18
parameter READBACK = "OFF";
parameter REGMODE_A = "NOREG";         // "NOREG", "OUTREG"
parameter REGMODE_B = "NOREG";         // "NOREG", "OUTREG"
parameter WRITEMODE_A = "NORMAL";      // "NORMAL", "READBEFOREWRITE", "WRITETHROUGH"
parameter WRITEMODE_B = "NORMAL";      // "NORMAL", "READBEFOREWRITE", "WRITETHROUGH"
parameter GSR = "ENABLE";              // "DISABLE", "ENABLE"
parameter RESETMODE = "SYNC";          // "SYNC", "ASYNC"
parameter ASYNC_RESET_RELEASE = "SYNC";          // "SYNC", "ASYNC"
parameter CEAMUX = "SIG"; //0, 1, INV, SIG
parameter CEBMUX = "SIG"; //0, 1, INV, SIG
parameter OCEAMUX = "SIG"; //0, 1, INV, SIG
parameter OCEBMUX = "SIG"; //0, 1, INV, SIG
parameter RSTAMUX = "SIG"; //0, 1, INV, SIG
parameter RSTBMUX = "SIG"; //0, 1, INV, SIG
parameter CLKAMUX = "SIG"; //0, 1, INV, SIG
parameter CLKBMUX = "SIG"; //0, 1, INV, SIG
parameter WEAMUX = "SIG"; //0, 1, INV, SIG
parameter WEBMUX = "SIG"; //0, 1, INV, SIG
parameter CSA0 = "SIG" ; //1,0,INV,SIG
parameter CSA1 = "SIG" ;
parameter CSA2 = "SIG" ;
parameter CSB0 = "SIG" ;
parameter CSB1 = "SIG" ;
parameter CSB2 = "SIG" ;
    parameter INIT_FILE = "NONE";
    parameter INITP_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INITP_01 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INITP_02 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INITP_03 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_01 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_02 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_03 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_04 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_05 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_06 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_07 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_08 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_09 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_0A = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_0B = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_0C = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_0D = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_0E = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_0F = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_10 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_11 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_12 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_13 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_14 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_15 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_16 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_17 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_18 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_19 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_1A = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_1B = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_1C = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_1D = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_1E = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_1F = 256'h0000000000000000000000000000000000000000000000000000000000000000;
endmodule
//****************
//****************
module AL_PHY_PAD (ce, clk, eninr_dyn, ipad, ocomp, otrue, rst, ts, di, icomp, itrue, opad, bpad);
    parameter DEDCLK     = "DISABLE";   //DISABLE, ENABLE
    parameter GSR        = "ENABLE";    //ENABLE, DISABLE
    parameter SRMODE     = "SYNC";      //SYNC, ASYNC
    parameter TSMUX      = "1";         //1, 0, TS, INV
    parameter INCLKMUX   = "0";         //0, 1, CLK, INV
    parameter INCEMUX    = "CE";        //CE, INV, 1
    parameter INRSTMUX   = "0";         //0, RST, INV
    parameter IDDRMODE   = "OFF";       //OFF, ON
    parameter INDELMUX   = "NODEL";     //NODEL, FIXDEL
    parameter INDEL      = 0;           // 0-31
    parameter OUTCLKMUX  = "0";         //0, 1, CLK, INV
    parameter OUTCEMUX   = "CE";        //CE, INV, 1
    parameter OUTRSTMUX  = "0";         //0, RST, INV
    parameter ODDRMODE   = "OFF";       //OFF, ON
    parameter IN_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter IN_REGSET  = "RESET";     //RESET, SET
    parameter DO_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter DO_REGSET  = "RESET";     //RESET, SET
    parameter TO_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter TO_REGSET  = "RESET";     //RESET, SET
    parameter MODE       = "IN";        //IN, OUT, BI
    parameter DRIVE      = "NONE";      //NONE, 8, 12, 16, 20 24
    parameter IOTYPE     = "LVCMOS25";  //LVCMOS25, LVCMOS12, LVCMOS18, LVCMOS33, LVDS25 ...
    output opad;
    output di;
    output itrue;
    output icomp;
    input ce, clk, eninr_dyn, ipad, ocomp, otrue, rst, ts;
    inout bpad;
endmodule
//****************
//****************
module AL_PHY_BRAM32K ( 
  doa, dia, addra, bytea, cena, wena, clka, rsta, ocea,
  dob, dib, addrb, byteb, cenb, wenb, clkb, rstb, oceb
  );

output  [15:0] doa;
output  [15:0] dob;
input   [15:0] dia;
input   [15:0] dib;
input   [10:0] addra;
input   [10:0] addrb;
input   bytea; // port a byte mode, lowest address
input   byteb; // port b byte mode, lowest address
input   cena,wena; // 0 is valid
input   clka,rsta;
input   cenb,wenb; // 0 is valid
input   clkb,rstb;
input   ocea;  // port a output register ce input when REGMODE_A=="OUTREG"
input   oceb;  // port B output register ce input when REGMODE_B=="OUTREG"
parameter MODE = "DP16K";               // DP16K, SP16K
parameter DATA_WIDTH_A = "16";          // 8, 16
parameter DATA_WIDTH_B = "16";          // 8, 16
parameter REGMODE_A = "NOREG";         // "NOREG", "OUTREG"
parameter REGMODE_B = "NOREG";         // "NOREG", "OUTREG"
parameter WRITEMODE_A = "NORMAL";      // "NORMAL", "WRITETHROUGH"
parameter WRITEMODE_B = "NORMAL";      // "NORMAL", "WRITETHROUGH"
parameter SRMODE = "SYNC";          // "SYNC", "ASYNC"
parameter CENAMUX = "SIG"; //0, 1, INV, SIG
parameter CENBMUX = "SIG"; //0, 1, INV, SIG
parameter OCEAMUX = "SIG"; //0, 1, INV, SIG
parameter OCEBMUX = "SIG"; //0, 1, INV, SIG
parameter RSTAMUX = "SIG"; //0, 1, INV, SIG
parameter RSTBMUX = "SIG"; //0, 1, INV, SIG
parameter CLKAMUX = "SIG"; //0, 1, INV, SIG
parameter CLKBMUX = "SIG"; //0, 1, INV, SIG
parameter WENAMUX = "SIG"; //0, 1, INV, SIG
parameter WENBMUX = "SIG"; //0, 1, INV, SIG
parameter READBACK = "OFF"; // ON, OFF
endmodule
//****************
//****************
module AL_PHY_FIFO (
		rst, 
            // write port
            dia,dib, csw, we, clkw, 
            // read port
            dob,doa, csr, re, clkr, rprst, orea,oreb,
            // output flag
            empty_flag, aempty_flag, afull_flag, full_flag);



input    [8:0] dia,dib;
input    [2:0] csr;
input    [2:0] csw;
input    we, re, clkw, clkr, rst, rprst,orea,oreb;
output   [8:0] dob,doa;
output   empty_flag, aempty_flag, afull_flag, full_flag;
   parameter  MODE = "FIFO8K";
   parameter  DATA_WIDTH_A = "18"; // 1,2,4,9,18
   parameter  DATA_WIDTH_B = "18"; // 1,2,4,9,18
   parameter  READBACK = "OFF";
   parameter  REGMODE_A   = "NOREG"; //NOREG, OUTREG
   parameter  REGMODE_B   = "NOREG"; //NOREG, OUTREG   
   parameter  [13:0] AE   = 14'b00000001100000;
   parameter  [13:0] AF   = 14'b01111110010000;
   parameter  [13:0] F 	  = 14'b01111111110000;
   parameter  [13:0] AEP1 = 14'b00000001110000;
   parameter  [13:0] AFM1 = 14'b01111110000000;
   parameter  [13:0] FM1  = 14'b01111111100000;   
   parameter  [4:0] E   = 5'b00000;
   parameter  [5:0] EP1 = 6'b010000;
   parameter  GSR = "ENABLE";  //DISABLE, ENABLE
   parameter  RESETMODE = "ASYNC"; //SYNC, ASYNC
   parameter  ASYNC_RESET_RELEASE = "SYNC";  //SYNC, ASYNC
parameter CEA = "SIG"; //0, 1, INV, SIG
parameter CEB = "SIG"; //0, 1, INV, SIG
parameter OCEA = "SIG"; //0, 1, INV, SIG
parameter OCEB = "SIG"; //0, 1, INV, SIG
parameter RSTA = "SIG"; //0, 1, INV, SIG
parameter RSTB = "SIG"; //0, 1, INV, SIG
parameter CLKA = "SIG"; //0, 1, INV, SIG
parameter CLKB = "SIG"; //0, 1, INV, SIG
parameter WEA = "SIG"; //0, 1, INV, SIG
parameter WEB = "SIG"; //0, 1, INV, SIG
parameter CSA0 = "SIG" ; //1,0,INV,SIG
parameter CSA1 = "SIG" ;
parameter CSA2 = "SIG" ;
parameter CSB0 = "SIG" ;
parameter CSB1 = "SIG" ;
parameter CSB2 = "SIG" ;
endmodule
//****************
//****************
module AL_PHY_MULT18 (acout, bcout, p, signeda, signedb, a, b, acin, bcin, cea, ceb, cepd, clk, rstan, rstbn, rstpdn,
            sourcea, sourceb); 

    output [17:0] acout; 
    output [17:0] bcout; 
    output [35:0] p; 
    input signeda;
    input signedb;
    input [17:0] a;
    input [17:0] b;
    input [17:0] acin;
    input [17:0] bcin;
    input cea;
    input ceb;
    input cepd;
    input clk;
    input rstan;
    input rstbn;
    input rstpdn;
    input sourcea;
    input sourceb;
    parameter INPUTREGA = "ENABLE"; // ENABLE/DISABLE
    parameter INPUTREGB = "ENABLE"; // ENABLE/DISABLE
    parameter OUTPUTREG = "ENABLE"; // ENABLE/DISABLE
    parameter SRMODE    = "ASYNC";  // ASYNC/SYNC
    parameter MODE      = "MULT18X18C"; //MULT9X9C/MULT18X18C
    parameter CEAMUX = "SIG"; //0, 1, INV, SIG
    parameter CEBMUX = "SIG"; //0, 1, INV, SIG
    parameter CEPDMUX = "SIG"; //0, 1, INV, SIG
    parameter RSTANMUX = "SIG"; //0, 1, INV, SIG
    parameter RSTBNMUX = "SIG"; //0, 1, INV, SIG
    parameter RSTPDNMUX = "SIG"; //0, 1, INV, SIG
    parameter CLKMUX = "SIG"; //0, 1, INV, SIG
    parameter SIGNEDAMUX = "SIG"; //0, 1, INV, SIG
    parameter SIGNEDBMUX = "SIG"; //0, 1, INV, SIG
    parameter SOURCEAMUX = "SIG"; //0, 1, INV, SIG
    parameter SOURCEBMUX = "SIG"; //0, 1, INV, SIG
endmodule // dsp
//****************
//****************
module AL_PHY_OSC (osc_dis, osc_clk);		// osc clk

    input osc_dis;
    output osc_clk;
    parameter STDBY = "DISABLE"; 	// DISABLE/ENABLE 
endmodule
//****************
//****************
module AL_PHY_SDRAM_2M_32 (
    clk,
    ras_n,
    cas_n,
    we_n,
    addr,
    ba,
    dq,
    dm,     // aka dm2
    dm0,
    cke
);

input         clk;
input         ras_n;
input         cas_n;
input         we_n;
input  [10:0] addr;
input  [1:0]  ba;
inout  [31:0] dq;
input         dm;
input         dm0; //DEFAULT := '0'
input         cke; //DEFAULT := '0'
endmodule
//****************
//****************
module AL_PHY_GCLK (clki, clko);
    input clki;
    output clko;
endmodule
//****************
//****************
module AL_PHY_IOCLK (clki, stop, clko);
    input clki;
    input stop;
    output clko;
    parameter  STOPCLK = "DISABLE"; // ENABLE,DISABLE
endmodule
//****************
//****************
module AL_PHY_CLKDIV (clki, rst, rls, clkdiv1, clkdivx);

    output clkdiv1, clkdivx;
    input clki;
    input rst;
    input rls;
    parameter GSR = "DISABLE";     //  DISABLE/ENABLE 
    parameter DIV = 2;  //  2/4
endmodule
//****************
//****************
module AL_PHY_CSB (clko, ce, clki, drct, sel, ignr);

    output clko;
    input [1:0] ce;
    input [1:0] clki;
    input [1:0] drct;
    input [1:0] sel;
    input [1:0] ignr;
    parameter HOLD      = "YES";     //  YES/NO 
    parameter INITOUT   = "0";       //  0/1
    parameter PRESELECT = "NONE";    //  CLK0/CLK1/NONE
    parameter TESTMODE  = "DISABLE";  //  ENABLE/DISABLE
endmodule
//****************
//****************
module AL_PHY_CONFIG(jrstn,jrti, jshift, jtck,jtdi, 
	jupdate,jscanen, jtms,jtdo,jtag8_ipa,jtag8_ipb,
	done,highz,cclk,usr_gsrn,gsrn_sync_clk,mboot_rebootn,mboot_dynamic_addr);

  output       jrstn;
  output [0:1] jrti;
  output       jshift;
  output       jtck;
  output       jtdi;
  output       jupdate;
  output [0:1] jscanen;
  output       jtms;
  input [0:1]  jtdo;
  input [0:7]  jtag8_ipa;
  input [0:7]  jtag8_ipb;
  output       done;
  output       highz;
  output       cclk;
  input        usr_gsrn;
  input        gsrn_sync_clk;
  input          mboot_rebootn;
  input [7:0]    mboot_dynamic_addr;
  parameter MBOOT_AUTO_SEL = "DISABLE";   // DISABLE/ENABLE
  parameter ADDR_SOURCE_SEL = "STATIC";   // STATIC/DYNAMIC. STATIC:0, DYNAMIC:1
  parameter STATIC_ADDR = 8'b0;           // 0/1
  parameter GSRN_SYNC_SEL = "DISABLE";	// DISABLE/ENABLE set by wakeup register
  parameter USR_GSRN_EN = "DISABLE";	// DISABLE/ENABLE set by wakeup register
endmodule
//****************
//****************
module AL_PHY_MSLICE
  (
  a,
  b,
  c,
  ce,
  clk,
  d,
  mi,
  sr,
  fci,
  f,
  fx,
  q,
  fco,
  dpram_mode,
  dpram_di,
  dpram_we,
  dpram_waddr,
  dpram_wclk  
  );
  input [1:0] a,b,c,d,mi;
  input clk;
  input ce;
  input sr;
  input fci;
  output [1:0] f,fx,q;
  output fco;    
  input dpram_mode;
  input [1:0] dpram_di;
  input dpram_we;
  input dpram_wclk;
  input [3:0]dpram_waddr;
parameter INIT_LUT0 = 16'h0000 ;
parameter INIT_LUT1 = 16'h0000 ;
parameter MODE = "LOGIC" ;
parameter ALUTYPE = "";
parameter MSFXMUX = "OFF" ;
parameter GSR = "ENABLE" ;
parameter TESTMODE = "OFF" ;
parameter CEMUX = "CE" ;
parameter SRMUX = "SR" ;
parameter CLKMUX = "CLK" ;
parameter SRMODE = "ASYNC" ;
parameter DFFMODE = "FF" ;
parameter REG0_SD = "MI" ;
parameter REG1_SD = "MI" ;
parameter REG0_REGSET = "SET" ;
parameter REG1_REGSET = "SET" ;
endmodule
//****************
//****************
module AL_PHY_LSLICE
  (
  a,
  b,
  c,
  ce,
  clk,
  d,
  e,
  mi,
  sr,
  fci,
  f,
  fx,
  q,
  fco,
  dpram_mode,
  dpram_di,
  dpram_we,
  dpram_waddr,
  dpram_wclk    
  );
  input [1:0] a,b,c,d,e,mi;
  input clk;
  input ce;
  input sr;
  input fci;
  output [1:0] f,fx,q;
  output fco;    
  output [3:0] dpram_di;
  output [3:0] dpram_waddr;
  output dpram_wclk;
  output dpram_we;
  output dpram_mode;
parameter INIT_LUTF0 = 16'h0000 ;
parameter INIT_LUTG0 = 16'h0000 ;
parameter INIT_LUTF1 = 16'h0000 ;
parameter INIT_LUTG1 = 16'h0000 ;
parameter MODE = "LOGIC" ;
parameter GSR = "ENABLE" ;
parameter TESTMODE = "OFF";   //;
parameter CEMUX = "1" ;
parameter SRMUX = "SR" ; //
parameter CLKMUX = "CLK" ;
parameter SRMODE = "ASYNC" ;
parameter DFFMODE = "FF" ;
parameter REG0_SD = "MI" ;
parameter REG1_SD = "MI" ;
parameter REG0_REGSET = "SET" ;
parameter REG1_REGSET = "SET" ;
parameter DEMUX0 = "D" ;
parameter DEMUX1 = "D" ;
parameter CMIMUX0 = "C" ;
parameter CMIMUX1 = "C" ;
parameter LSFMUX0 = "LUTF";
parameter LSFXMUX0 = "LUTG";
parameter LSFMUX1 = "LUTF";
parameter LSFXMUX1 = "LUTG";
endmodule
//****************
//****************
module AL_LOGIC_BRAM (
  dia,
  addra, cea, ocea, clka, wea, rsta, bea,
  dib,
  addrb, ceb, oceb, clkb, web, rstb, beb,
  doa,
  dob);
  // 1kx9
  parameter DATA_WIDTH_A = 9;
  parameter DATA_WIDTH_B = DATA_WIDTH_A;
  parameter ADDR_WIDTH_A = 10;
  parameter ADDR_WIDTH_B = ADDR_WIDTH_A;
  parameter DATA_DEPTH_A = 2 ** ADDR_WIDTH_A;
  parameter DATA_DEPTH_B = 2 ** ADDR_WIDTH_B;
  parameter BYTE_ENABLE = 0;            // only interger {0, 8, 9} are valid
  parameter BYTE_A = BYTE_ENABLE == 0 ? 1 : DATA_WIDTH_A / BYTE_ENABLE;
  parameter BYTE_B = BYTE_ENABLE == 0 ? 1 : DATA_WIDTH_B / BYTE_ENABLE;
  output  [DATA_WIDTH_A-1:0] doa;
  output  [DATA_WIDTH_B-1:0] dob;
  input   [DATA_WIDTH_A-1:0] dia; //DEFAULT := (others => '0')
  input   [DATA_WIDTH_B-1:0] dib; //DEFAULT := (others => '0')
  input   cea, ocea, clka, wea, rsta; //DEFAULT := '0'
  input   ceb, oceb, clkb, web, rstb; //DEFAULT := '0'
  input [BYTE_A - 1 : 0] bea; //DEFAULT := (others => '0')
  input [BYTE_B - 1 : 0] beb; //DEFAULT := (others => '0')
  input   [ADDR_WIDTH_A-1:0] addra; //DEFAULT := (others => '0')
  input   [ADDR_WIDTH_B-1:0] addrb; //DEFAULT := (others => '0')
  parameter MODE = "DP";                 // DP,SP,PDPW,FIFO
  parameter REGMODE_A = "NOREG";         // "NOREG", "OUTREG"
  parameter REGMODE_B = "NOREG";         // "NOREG", "OUTREG"
  parameter WRITEMODE_A = "NORMAL";      // "NORMAL", "READBEFOREWRITE", "WRITETHROUGH"
  parameter WRITEMODE_B = "NORMAL";      // "NORMAL", "READBEFOREWRITE", "WRITETHROUGH"
  parameter RESETMODE = "SYNC";          // "SYNC", "ASYNC"
  parameter DEBUGGABLE = "NO";           // "YES", "NO"
  parameter PACKABLE = "NO";             // "YES", "NO"
  parameter FORCE_KEEP = "OFF";          // "ON", "OFF"
  parameter INIT_FILE = "NONE";
  parameter FILL_ALL = "NONE";          // "NONE" or some binary string like "0101", not wider than the wider port
  parameter IMPLEMENT = "9K";            // 9K | 9K(FAST) | 32K (all capitalized)
endmodule      
//****************
//****************
module AL_PHY_PLL(clkc,ext_lock,refclk,stdby,fbclk,pllreset,scanclk,phaseupdown,phasestep,phasedone,phcntsel);
output [4:0] clkc;      //DEFAULT := (others => '0')
output  ext_lock;       //DEFAULT := '0'
input  stdby;           //DEFAULT := '0'
input  refclk;          //DEFAULT := '0'
input  fbclk;           //DEFAULT := '0'
input  pllreset;        //DEFAULT := '0'
input  scanclk;         //DEFAULT := '0'
input  phaseupdown;     //DEFAULT := '0'
input  phasestep;       //DEFAULT := '0'
output phasedone;       //DEFAULT := '0'
input  [2:0] phcntsel;  //DEFAULT := (others => '0')
parameter REFCLK_SEL = "INTERNAL"; // INTERNAL/EXTRNAL //
parameter FIN = "100.0000"; //no use in fact  
parameter REFCLK_DIV    = 1;    //  1-128
parameter FBCLK_DIV     = 1;    //  1-128
parameter CLKC0_DIV     = 1;    //  1-128
parameter CLKC1_DIV     = 1;    //  1-128
parameter CLKC2_DIV     = 1;    //  1-128
parameter CLKC3_DIV     = 1;    //  1-128
parameter CLKC4_DIV     = 1;    //  1-128
parameter CLKC0_ENABLE  = "DISABLE";   //  ENABLE/DISABLE/SIGNAL
parameter CLKC1_ENABLE  = "DISABLE";   //  ENABLE/DISABLE/SIGNAL
parameter CLKC2_ENABLE  = "DISABLE";   //  ENABLE/DISABLE/SIGNAL
parameter CLKC3_ENABLE  = "DISABLE";   //  ENABLE/DISABLE/SIGNAL
parameter CLKC4_ENABLE  = "DISABLE";   //  ENABLE/DISABLE/SIGNAL
parameter FEEDBK_MODE   = "NORMAL";    // NORMAL/SOURCESYNC/NOCOMP/ZERODELAY
parameter FEEDBK_PATH   = "VCO_PHASE_0";  
parameter INT_FEEDBK_PATH   = "VCO_PHASE_0";  
parameter STDBY_ENABLE  = "ENABLE";   //  ENABLE/DISABLE
parameter CLKC0_FPHASE     = "0";    //  0-7
parameter CLKC1_FPHASE     = "0";    //  0-7
parameter CLKC2_FPHASE     = "0";    //  0-7
parameter CLKC3_FPHASE     = "0";    //  0-7
parameter CLKC4_FPHASE     = "0";    //  0-7
parameter CLKC0_CPHASE     = 1;      //  1-127  ,CLKC0_CPHASE <= CLKC0_DIV
parameter CLKC1_CPHASE     = 1;      //  1-127  ,CLKC1_CPHASE <= CLKC1_DIV
parameter CLKC2_CPHASE     = 1;      //  1-127  ,CLKC2_CPHASE <= CLKC2_DIV
parameter CLKC3_CPHASE     = 1;      //  1-127  ,CLKC3_CPHASE <= CLKC3_DIV
parameter CLKC4_CPHASE     = 1;      //  1-127  ,CLKC4_CPHASE <= CLKC4_DIV
parameter GMC_GAIN         = "7";  // 0-7
parameter GMC_TEST         = "14"; // 0-15
parameter ICP_CURRENT      = 14;   // 1-32
parameter KVCO             = "7";  // 0-7
parameter LPF_CAPACITOR    = "3";  // 0-3
parameter LPF_RESISTOR     = 1;    // 1-128
parameter PLLRST_ENA       = "ENABLE";    // ENABLE/DISABLE
parameter PLLMRST_ENA      = "DISABLE";   // ENABLE/DISABLE
parameter PLLC2RST_ENA     = "DISABLE";   // ENABLE/DISABLE
parameter PLLC34RST_ENA    = "DISABLE";   // ENABLE/DISABLE
parameter PREDIV_MUXC0 = "VCO";    // VCO/CLKMINUS1/CLKPLUS1/REFCLK
parameter PREDIV_MUXC1 = "VCO";    // VCO/CLKMINUS1/CLKPLUS1/REFCLK
parameter PREDIV_MUXC2 = "VCO";    // VCO/CLKMINUS1/CLKPLUS1/REFCLK
parameter PREDIV_MUXC3 = "VCO";    // VCO/CLKMINUS1/CLKPLUS1/REFCLK
parameter PREDIV_MUXC4 = "VCO";    // VCO/CLKMINUS1/CLKPLUS1/REFCLK
parameter ODIV_MUXC0 = "DIV";    // DIV/REFCLK
parameter ODIV_MUXC1 = "DIV";    // DIV/REFCLK
parameter ODIV_MUXC2 = "DIV";    // DIV/REFCLK
parameter ODIV_MUXC3 = "DIV";    // DIV/REFCLK
parameter ODIV_MUXC4 = "DIV";    // DIV/REFCLK
parameter FREQ_LOCK_ACCURACY = "2"; // 0-3
parameter PLL_LOCK_MODE = "0";    // 0-7, 2 => sticky, 0 => non-sticky
parameter INTFB_WAKE      = "DISABLE";  //  ENBALE/DISABLE
parameter DPHASE_SOURCE   = "DISABLE";  //  ENABLE/DISABLE  
parameter VCO_NORESET     = "DISABLE";  //  ENABLE/DISABLE 
parameter STDBY_VCO_ENA   = "DISABLE";  //  ENABLE/DISABLE 
parameter NORESET         = "DISABLE";  //  ENABLE/DISABLE 
parameter SYNC_ENABLE     = "ENABLE";   //  ENABLE/DISABLE                      
parameter DERIVE_PLL_CLOCKS = "DISABLE";  //  ENABLE/DISABLE
parameter GEN_BASIC_CLOCK   = "DISABLE";  //  ENABLE/DISABLE
endmodule
//****************
