------------------------------------------------
------------------------------------------------
--
--package AL_COMPONENTS :
-- It should be compiled into al_macro library.
--
------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

package AL_COMPONENTS is

component AL_LOGIC_RAMFIFO is 
generic (
    DATA_WIDTH : integer := 8;
    ADDR_WIDTH : integer := 8;
    SHOWAHEAD : integer := 0;
    IMPLEMENT : string := "AUTO"
    );
port (
    rst : in std_logic ;
    di : in std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    clkr : in std_logic ;
    re : in std_logic ;
    clkw : in std_logic ;
    we : in std_logic ;
    do : out std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    empty_flag : out std_logic ;
    full_flag : out std_logic ;
    rdusedw : out std_logic_vector(ADDR_WIDTH DOWNTO 0) ;
    wrusedw : out std_logic_vector(ADDR_WIDTH DOWNTO 0) 
    );
end component;


component gray2bin is 
generic (
    WIDTH : integer := 8
    );
port (
    din : in std_logic_vector(WIDTH-1 DOWNTO 0) ;
    dout : out std_logic_vector(WIDTH-1 DOWNTO 0) 
    );
end component;


component gray_count is 
generic (
    WIDTH : integer := 8;
    INIT : integer := 0
    );
port (
    clk : in std_logic ;
    enable : in std_logic ;
    reset : in std_logic ;
    cnt : out std_logic_vector(WIDTH-1 DOWNTO 0) 
    );
end component;


component showahead_buffer is 
generic (
    DATA_WIDTH : integer := 8
    );
port (
    rstn_i : in std_logic ;
    clk_i : in std_logic ;
    ena_i : in std_logic ;
    rd_i : in std_logic ;
    data_i : in std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    data_o : out std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    addr_inc_o : out std_logic ;
    empty_o : out std_logic 
    );
end component;


component AL_LOGIC_BUF is 
port (
    o : out std_logic ;
    i : in std_logic 
    );
end component;


component AL_LOGIC_GSRN is 
generic (
    USR_GSRN_EN : string := "ENABLE"
    );
port (
    gsrn : in std_logic 
    );
end component;


component AL_LOGIC_BUFG is 
port (
    o : out std_logic ;
    i : in std_logic 
    );
end component;


component AL_LOGIC_BUFIO is 
generic (
    GSR : string := "DISABLE";
    DIV : integer := 2;
    STOPCLK : string := "DISABLE"
    );
port (
    clki : in std_logic ;
    rst : in std_logic ;
    coe : in std_logic ;
    clko : out std_logic ;
    clkdiv1 : out std_logic ;
    clkdivx : out std_logic 
    );
end component;


component AL_LOGIC_BUFGMUX is 
generic (
    INIT_OUT : string := "0";
    PRESELECT_I0 : string := "TRUE";
    PRESELECT_I1 : string := "FALSE"
    );
port (
    o : out std_logic ;
    i0 : in std_logic ;
    i1 : in std_logic ;
    s : in std_logic 
    );
end component;


component AL_LOGIC_CCLK is 
port (
    cclk : out std_logic 
    );
end component;


component AL_LOGIC_IDELAY is 
generic (
    INDEL : integer := 0
    );
port (
    o : out std_logic ;
    i : in std_logic 
    );
end component;


component AL_LOGIC_IDDR is 
generic (
    SRMODE : string := "SYNC"
    );
port (
    q1 : out std_logic ;
    q2 : out std_logic ;
    clk : in std_logic ;
    d : in std_logic ;
    rst : in std_logic 
    );
end component;


component AL_LOGIC_ODDR is 
generic (
    SRMODE : string := "SYNC"
    );
port (
    q : out std_logic ;
    clk : in std_logic ;
    d1 : in std_logic ;
    d2 : in std_logic ;
    rst : in std_logic 
    );
end component;


component AL_LOGIC_PROGRAMN is 
port (
    programn : in std_logic 
    );
end component;


component AL_LOGIC_FIFO is 
generic (
    DATA_WIDTH_W : integer := 9;
    DATA_WIDTH_R : integer := 9;
    DATA_DEPTH_W : integer := 1024;
    DATA_DEPTH_R : integer := 1024;
    MODE : string := "FIFO8K";
    REGMODE_W : string := "NOREG";
    REGMODE_R : string := "NOREG";
    E : integer := 0;
    AE : integer := 6;
    AF : integer := 1018;
    F : integer := 1024;
    GSR : string := "DISABLE";
    RESETMODE : string := "ASYNC";
    ASYNC_RESET_RELEASE : string := "SYNC";
    ENDIAN : string := "LITTLE"
    );
port (
    rst : in std_logic ;
    di : in std_logic_vector(DATA_WIDTH_W-1 DOWNTO 0) ;
    do : out std_logic_vector(DATA_WIDTH_R-1 DOWNTO 0) ;
    clkw, we : in std_logic ;
    clkr, re, ore : in std_logic := '0';
    csw, csr : in std_logic_vector(2 DOWNTO 0) ;
    empty_flag, aempty_flag : out std_logic := '0';
    full_flag, afull_flag : out std_logic := '0'
    );
end component;


component AL_LOGIC_DRAM is 
generic (
    DATA_WIDTH_W : integer := 9;
    ADDR_WIDTH_W : integer := 10;
    DATA_DEPTH_W : integer := 1024;
    DATA_WIDTH_R : integer := 9;
    ADDR_WIDTH_R : integer := 10;
    DATA_DEPTH_R : integer := 1024;
    INIT_FILE : string := "NONE"
    );
port (
    di : in std_logic_vector(DATA_WIDTH_W-1 DOWNTO 0) ;
    waddr : in std_logic_vector(ADDR_WIDTH_W-1 DOWNTO 0) ;
    wclk : in std_logic ;
    we : in std_logic ;
    do : out std_logic_vector(DATA_WIDTH_R-1 DOWNTO 0) ;
    raddr : in std_logic_vector(ADDR_WIDTH_R-1 DOWNTO 0) 
    );
end component;


component AL_LOGIC_DRAM16X4 is 
generic (
    INIT_D0 : string := X"0000";
    INIT_D1 : string := X"0000";
    INIT_D2 : string := X"0000";
    INIT_D3 : string := X"0000"
    );
port (
    di : in std_logic_vector(3 DOWNTO 0) ;
    waddr : in std_logic_vector(3 DOWNTO 0) ;
    wclk : in std_logic ;
    we : in std_logic ;
    raddr : in std_logic_vector(3 DOWNTO 0) ;
    do : out std_logic_vector(3 DOWNTO 0) 
    );
end component;


component AL_LOGIC_MULT is 
generic (
    INPUT_WIDTH_A : integer := 18;
    INPUT_WIDTH_B : integer := 18;
    OUTPUT_WIDTH : integer := 36;
    INPUTFORMAT : string := "SIGNED";
    INPUTREGA : string := "ENABLE";
    INPUTREGB : string := "ENABLE";
    OUTPUTREG : string := "ENABLE";
    SRMODE : string := "ASYNC";
    IMPLEMENT : string := "AUTO"
    );
port (
    p : out std_logic_vector(OUTPUT_WIDTH-1 DOWNTO 0) ;
    a : in std_logic_vector(INPUT_WIDTH_A-1 DOWNTO 0) ;
    b : in std_logic_vector(INPUT_WIDTH_B-1 DOWNTO 0) ;
    cea : in std_logic := '0';
    ceb : in std_logic := '0';
    cepd : in std_logic := '0';
    clk : in std_logic := '0';
    rstan : in std_logic := '0';
    rstbn : in std_logic := '0';
    rstpdn : in std_logic := '0'
    );
end component;


component AL_LOGIC_SEQ_DIV is 
generic (
    NUMER_WIDTH : integer := 16;
    DENOM_WIDTH : integer := 16
    );
port (
    clk : in std_logic ;
    rst : in std_logic ;
    start : in std_logic ;
    numer : in std_logic_vector(NUMER_WIDTH-1 DOWNTO 0) ;
    denom : in std_logic_vector(DENOM_WIDTH-1 DOWNTO 0) ;
    quotient : out std_logic_vector(NUMER_WIDTH-1 DOWNTO 0) ;
    remain : out std_logic_vector(DENOM_WIDTH-1 DOWNTO 0) ;
    done : out std_logic 
    );
end component;


component AL_LOGIC_MBOOT is 
generic (
    ADDR_SOURCE_SEL : string := "STATIC";
    STATIC_ADDR : string := B"0";
    MBOOT_AUTO_SEL : string := "DISABLE"
    );
port (
    rebootn : in std_logic ;
    dynamic_addr : in std_logic_vector(7 DOWNTO 0) 
    );
end component;


component AL_LOGIC_DNA is 
generic (
    DNA : string := X"0000000000000000"
    );
port (
    dout : out std_logic ;
    clk : in std_logic ;
    din : in std_logic ;
    shift_en : in std_logic 
    );
end component;


component AL_PHY_BRAM is 
generic (
    MODE : string := "DP8K";
    DATA_WIDTH_A : string := "9";
    DATA_WIDTH_B : string := "9";
    READBACK : string := "OFF";
    REGMODE_A : string := "NOREG";
    REGMODE_B : string := "NOREG";
    WRITEMODE_A : string := "NORMAL";
    WRITEMODE_B : string := "NORMAL";
    GSR : string := "ENABLE";
    RESETMODE : string := "SYNC";
    ASYNC_RESET_RELEASE : string := "SYNC";
    CEAMUX : string := "SIG";
    CEBMUX : string := "SIG";
    OCEAMUX : string := "SIG";
    OCEBMUX : string := "SIG";
    RSTAMUX : string := "SIG";
    RSTBMUX : string := "SIG";
    CLKAMUX : string := "SIG";
    CLKBMUX : string := "SIG";
    WEAMUX : string := "SIG";
    WEBMUX : string := "SIG";
    CSA0 : string := "SIG";
    CSA1 : string := "SIG";
    CSA2 : string := "SIG";
    CSB0 : string := "SIG";
    CSB1 : string := "SIG";
    CSB2 : string := "SIG";
    INIT_FILE : string := "NONE";
    INITP_00 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INITP_01 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INITP_02 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INITP_03 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_00 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_01 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_02 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_03 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_04 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_05 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_06 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_07 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_08 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_09 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0A : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0B : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0C : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0D : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0E : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0F : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_10 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_11 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_12 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_13 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_14 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_15 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_16 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_17 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_18 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_19 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1A : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1B : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1C : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1D : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1E : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1F : string := X"0000000000000000000000000000000000000000000000000000000000000000"
    );
port (
    doa : out std_logic_vector(8 DOWNTO 0) ;
    dob : out std_logic_vector(8 DOWNTO 0) ;
    dia : in std_logic_vector(8 DOWNTO 0) ;
    dib : in std_logic_vector(8 DOWNTO 0) ;
    csa : in std_logic_vector(2 DOWNTO 0) ;
    csb : in std_logic_vector(2 DOWNTO 0) ;
    cea, ocea, clka, wea, rsta : in std_logic ;
    ceb, oceb, clkb, web, rstb : in std_logic ;
    addra : in std_logic_vector(12 DOWNTO 0) ;
    addrb : in std_logic_vector(12 DOWNTO 0) 
    );
end component;


component AL_PHY_PAD is 
generic (
    DEDCLK : string := "DISABLE";
    GSR : string := "ENABLE";
    SRMODE : string := "SYNC";
    TSMUX : string := "1";
    INCLKMUX : string := "0";
    INCEMUX : string := "CE";
    INRSTMUX : string := "0";
    IDDRMODE : string := "OFF";
    INDELMUX : string := "NODEL";
    INDEL : integer := 0;
    OUTCLKMUX : string := "0";
    OUTCEMUX : string := "CE";
    OUTRSTMUX : string := "0";
    ODDRMODE : string := "OFF";
    IN_DFFMODE : string := "NONE";
    IN_REGSET : string := "RESET";
    DO_DFFMODE : string := "NONE";
    DO_REGSET : string := "RESET";
    TO_DFFMODE : string := "NONE";
    TO_REGSET : string := "RESET";
    MODE : string := "IN";
    DRIVE : string := "NONE";
    IOTYPE : string := "LVCMOS25"
    );
port (
    opad : out std_logic ;
    di : out std_logic ;
    itrue : out std_logic ;
    icomp : out std_logic ;
    ce, clk, eninr_dyn, ipad, ocomp, otrue, rst, ts : in std_logic ;
    bpad : inout std_logic 
    );
end component;


component AL_PHY_BRAM32K is 
generic (
    MODE : string := "DP16K";
    DATA_WIDTH_A : string := "16";
    DATA_WIDTH_B : string := "16";
    REGMODE_A : string := "NOREG";
    REGMODE_B : string := "NOREG";
    WRITEMODE_A : string := "NORMAL";
    WRITEMODE_B : string := "NORMAL";
    SRMODE : string := "SYNC";
    CENAMUX : string := "SIG";
    CENBMUX : string := "SIG";
    OCEAMUX : string := "SIG";
    OCEBMUX : string := "SIG";
    RSTAMUX : string := "SIG";
    RSTBMUX : string := "SIG";
    CLKAMUX : string := "SIG";
    CLKBMUX : string := "SIG";
    WENAMUX : string := "SIG";
    WENBMUX : string := "SIG";
    READBACK : string := "OFF"
    );
port (
    doa : out std_logic_vector(15 DOWNTO 0) ;
    dob : out std_logic_vector(15 DOWNTO 0) ;
    dia : in std_logic_vector(15 DOWNTO 0) ;
    dib : in std_logic_vector(15 DOWNTO 0) ;
    addra : in std_logic_vector(10 DOWNTO 0) ;
    addrb : in std_logic_vector(10 DOWNTO 0) ;
    bytea : in std_logic ;
    byteb : in std_logic ;
    cena,wena : in std_logic ;
    clka,rsta : in std_logic ;
    cenb,wenb : in std_logic ;
    clkb,rstb : in std_logic ;
    ocea : in std_logic ;
    oceb : in std_logic 
    );
end component;


component AL_PHY_FIFO is 
generic (
    MODE : string := "FIFO8K";
    DATA_WIDTH_A : string := "18";
    DATA_WIDTH_B : string := "18";
    READBACK : string := "OFF";
    REGMODE_A : string := "NOREG";
    REGMODE_B : string := "NOREG";
     AE : string := B"00000001100000";
     AF : string := B"01111110010000";
     F : string := B"01111111110000";
     AEP1 : string := B"00000001110000";
     AFM1 : string := B"01111110000000";
     FM1 : string := B"01111111100000";
     E : string := B"00000";
     EP1 : string := B"010000";
    GSR : string := "ENABLE";
    RESETMODE : string := "ASYNC";
    ASYNC_RESET_RELEASE : string := "SYNC";
    CEA : string := "SIG";
    CEB : string := "SIG";
    OCEA : string := "SIG";
    OCEB : string := "SIG";
    RSTA : string := "SIG";
    RSTB : string := "SIG";
    CLKA : string := "SIG";
    CLKB : string := "SIG";
    WEA : string := "SIG";
    WEB : string := "SIG";
    CSA0 : string := "SIG";
    CSA1 : string := "SIG";
    CSA2 : string := "SIG";
    CSB0 : string := "SIG";
    CSB1 : string := "SIG";
    CSB2 : string := "SIG"
    );
port (
    flag : out std_logic ;
    dia,dib : in std_logic_vector(8 DOWNTO 0) ;
    csr : in std_logic_vector(2 DOWNTO 0) ;
    csw : in std_logic_vector(2 DOWNTO 0) ;
    we, re, clkw, clkr, rst, rprst,orea,oreb : in std_logic ;
    dob,doa : out std_logic_vector(8 DOWNTO 0) ;
    empty_flag, aempty_flag, afull_flag, full_flag : out std_logic 
    );
end component;


component AL_PHY_MULT18 is 
generic (
    INPUTREGA : string := "ENABLE";
    INPUTREGB : string := "ENABLE";
    OUTPUTREG : string := "ENABLE";
    SRMODE : string := "ASYNC";
    MODE : string := "MULT18X18C";
    CEAMUX : string := "SIG";
    CEBMUX : string := "SIG";
    CEPDMUX : string := "SIG";
    RSTANMUX : string := "SIG";
    RSTBNMUX : string := "SIG";
    RSTPDNMUX : string := "SIG";
    CLKMUX : string := "SIG";
    SIGNEDAMUX : string := "SIG";
    SIGNEDBMUX : string := "SIG";
    SOURCEAMUX : string := "SIG";
    SOURCEBMUX : string := "SIG"
    );
port (
    acout : out std_logic_vector(17 DOWNTO 0) ;
    bcout : out std_logic_vector(17 DOWNTO 0) ;
    p : out std_logic_vector(35 DOWNTO 0) ;
    signeda : in std_logic ;
    signedb : in std_logic ;
    a : in std_logic_vector(17 DOWNTO 0) ;
    b : in std_logic_vector(17 DOWNTO 0) ;
    acin : in std_logic_vector(17 DOWNTO 0) ;
    bcin : in std_logic_vector(17 DOWNTO 0) ;
    cea : in std_logic ;
    ceb : in std_logic ;
    cepd : in std_logic ;
    clk : in std_logic ;
    rstan : in std_logic ;
    rstbn : in std_logic ;
    rstpdn : in std_logic ;
    sourcea : in std_logic ;
    sourceb : in std_logic 
    );
end component;


component AL_PHY_OSC is 
generic (
    STDBY : string := "DISABLE"
    );
port (
    osc_dis : in std_logic ;
    osc_clk : out std_logic 
    );
end component;


component AL_PHY_SDRAM_2M_32 is 
port (
    clk : in std_logic ;
    ras_n : in std_logic ;
    cas_n : in std_logic ;
    we_n : in std_logic ;
    addr : in std_logic_vector(10 DOWNTO 0) ;
    ba : in std_logic_vector(1 DOWNTO 0) ;
    dq : inout std_logic_vector(31 DOWNTO 0) ;
    dm : in std_logic ;
    dm0 : in std_logic := '0';
    cke : in std_logic := '0'
    );
end component;


component AL_PHY_GCLK is 
port (
    clki : in std_logic ;
    clko : out std_logic 
    );
end component;


component AL_PHY_IOCLK is 
generic (
    STOPCLK : string := "DISABLE"
    );
port (
    clki : in std_logic ;
    stop : in std_logic ;
    clko : out std_logic 
    );
end component;


component AL_PHY_CLKDIV is 
generic (
    GSR : string := "DISABLE";
    DIV : integer := 2
    );
port (
    clkdiv1, clkdivx : out std_logic ;
    clki : in std_logic ;
    rst : in std_logic ;
    rls : in std_logic 
    );
end component;


component AL_PHY_CSB is 
generic (
    HOLD : string := "YES";
    INITOUT : string := "0";
    PRESELECT : string := "NONE";
    TESTMODE : string := "DISABLE"
    );
port (
    clko : out std_logic ;
    ce : in std_logic_vector(1 DOWNTO 0) ;
    clki : in std_logic_vector(1 DOWNTO 0) ;
    drct : in std_logic_vector(1 DOWNTO 0) ;
    sel : in std_logic_vector(1 DOWNTO 0) ;
    ignr : in std_logic_vector(1 DOWNTO 0) 
    );
end component;


component AL_PHY_CONFIG is 
generic (
    MBOOT_AUTO_SEL : string := "DISABLE";
    ADDR_SOURCE_SEL : string := "STATIC";
    STATIC_ADDR : string := B"0";
    GSRN_SYNC_SEL : string := "DISABLE";
    USR_GSRN_EN : string := "DISABLE"
    );
port (
    jrstn : out std_logic ;
    jrti : out std_logic_vector(0 DOWNTO 0) ;
    jshift : out std_logic ;
    jtck : out std_logic ;
    jtdi : out std_logic ;
    jupdate : out std_logic ;
    jscanen : out std_logic_vector(0 DOWNTO 0) ;
    jtms : out std_logic ;
    jtdo : in std_logic_vector(0 DOWNTO 0) ;
    jtag8_ipa : in std_logic_vector(0 DOWNTO 0) ;
    jtag8_ipb : in std_logic_vector(0 DOWNTO 0) ;
    done : out std_logic ;
    highz : out std_logic ;
    cclk : out std_logic ;
    usr_gsrn : in std_logic ;
    gsrn_sync_clk : in std_logic ;
    mboot_rebootn : in std_logic ;
    mboot_dynamic_addr : in std_logic_vector(7 DOWNTO 0) 
    );
end component;


component AL_PHY_MSLICE is 
generic (
    INIT_LUT0 : string := X"0000";
    INIT_LUT1 : string := X"0000";
    MODE : string := "LOGIC";
    ALUTYPE : string := "";
    MSFXMUX : string := "OFF";
    GSR : string := "ENABLE";
    TESTMODE : string := "OFF";
    CEMUX : string := "CE";
    SRMUX : string := "SR";
    CLKMUX : string := "CLK";
    SRMODE : string := "ASYNC";
    DFFMODE : string := "FF";
    REG0_SD : string := "MI";
    REG1_SD : string := "MI";
    REG0_REGSET : string := "SET";
    REG1_REGSET : string := "SET"
    );
port (
    a,b,c,d,mi : in std_logic_vector(1 DOWNTO 0) ;
    clk : in std_logic ;
    ce : in std_logic ;
    sr : in std_logic ;
    fci : in std_logic ;
    f,fx,q : out std_logic_vector(1 DOWNTO 0) ;
    fco : out std_logic ;
    dpram_mode : in std_logic ;
    dpram_di : in std_logic_vector(1 DOWNTO 0) ;
    dpram_we : in std_logic ;
    dpram_wclk : in std_logic ;
    dpram_waddr : in std_logic_vector(3 DOWNTO 0) 
    );
end component;


component AL_PHY_LSLICE is 
generic (
    INIT_LUTF0 : string := X"0000";
    INIT_LUTG0 : string := X"0000";
    INIT_LUTF1 : string := X"0000";
    INIT_LUTG1 : string := X"0000";
    MODE : string := "LOGIC";
    GSR : string := "ENABLE";
    TESTMODE : string := "OFF";
    CEMUX : string := "1";
    SRMUX : string := "SR";
    CLKMUX : string := "CLK";
    SRMODE : string := "ASYNC";
    DFFMODE : string := "FF";
    REG0_SD : string := "MI";
    REG1_SD : string := "MI";
    REG0_REGSET : string := "SET";
    REG1_REGSET : string := "SET";
    DEMUX0 : string := "D";
    DEMUX1 : string := "D";
    CMIMUX0 : string := "C";
    CMIMUX1 : string := "C";
    LSFMUX0 : string := "LUTF";
    LSFXMUX0 : string := "LUTG";
    LSFMUX1 : string := "LUTF";
    LSFXMUX1 : string := "LUTG"
    );
port (
    a,b,c,d,e,mi : in std_logic_vector(1 DOWNTO 0) ;
    clk : in std_logic ;
    ce : in std_logic ;
    sr : in std_logic ;
    fci : in std_logic ;
    f,fx,q : out std_logic_vector(1 DOWNTO 0) ;
    fco : out std_logic ;
    dpram_di : out std_logic_vector(3 DOWNTO 0) ;
    dpram_waddr : out std_logic_vector(3 DOWNTO 0) ;
    dpram_wclk : out std_logic ;
    dpram_we : out std_logic ;
    dpram_mode : out std_logic 
    );
end component;


component AL_LOGIC_BRAM is 
generic (
    DATA_WIDTH_A : integer := 9;
    DATA_WIDTH_B : integer := 9;
    ADDR_WIDTH_A : integer := 10;
    ADDR_WIDTH_B : integer := 10;
    DATA_DEPTH_A : integer := 1024;
    DATA_DEPTH_B : integer := 1024;
    BYTE_ENABLE : integer := 0;
    BYTE_A : integer := 1;
    BYTE_B : integer := 1;
    MODE : string := "DP";
    REGMODE_A : string := "NOREG";
    REGMODE_B : string := "NOREG";
    WRITEMODE_A : string := "NORMAL";
    WRITEMODE_B : string := "NORMAL";
    RESETMODE : string := "SYNC";
    DEBUGGABLE : string := "NO";
    PACKABLE : string := "NO";
    FORCE_KEEP : string := "OFF";
    INIT_FILE : string := "NONE";
    FILL_ALL : string := "NONE";
    IMPLEMENT : string := "9K"
    );
port (
    doa : out std_logic_vector(DATA_WIDTH_A-1 DOWNTO 0) ;
    dob : out std_logic_vector(DATA_WIDTH_B-1 DOWNTO 0) ;
    dia : in std_logic_vector(DATA_WIDTH_A-1 DOWNTO 0) := (others => '0');
    dib : in std_logic_vector(DATA_WIDTH_B-1 DOWNTO 0) := (others => '0');
    cea, ocea, clka, wea, rsta : in std_logic := '0';
    ceb, oceb, clkb, web, rstb : in std_logic := '0';
    bea : in std_logic_vector(BYTE_A - 1  DOWNTO 0) := (others => '0');
    beb : in std_logic_vector(BYTE_B - 1  DOWNTO 0) := (others => '0');
    addra : in std_logic_vector(ADDR_WIDTH_A-1 DOWNTO 0) := (others => '0');
    addrb : in std_logic_vector(ADDR_WIDTH_B-1 DOWNTO 0) := (others => '0')
    );
end component;


component AL_PHY_PLL is 
generic (
    REFCLK_SEL : string := "INTERNAL";
    FIN : string := "100.0000";
    REFCLK_DIV : integer := 1;
    FBCLK_DIV : integer := 1;
    CLKC0_DIV : integer := 1;
    CLKC1_DIV : integer := 1;
    CLKC2_DIV : integer := 1;
    CLKC3_DIV : integer := 1;
    CLKC4_DIV : integer := 1;
    CLKC0_ENABLE : string := "DISABLE";
    CLKC1_ENABLE : string := "DISABLE";
    CLKC2_ENABLE : string := "DISABLE";
    CLKC3_ENABLE : string := "DISABLE";
    CLKC4_ENABLE : string := "DISABLE";
    FEEDBK_MODE : string := "NORMAL";
    FEEDBK_PATH : string := "VCO_PHASE_0";
    INT_FEEDBK_PATH : string := "VCO_PHASE_0";
    STDBY_ENABLE : string := "ENABLE";
    CLKC0_FPHASE : string := "0";
    CLKC1_FPHASE : string := "0";
    CLKC2_FPHASE : string := "0";
    CLKC3_FPHASE : string := "0";
    CLKC4_FPHASE : string := "0";
    CLKC0_CPHASE : integer := 1;
    CLKC1_CPHASE : integer := 1;
    CLKC2_CPHASE : integer := 1;
    CLKC3_CPHASE : integer := 1;
    CLKC4_CPHASE : integer := 1;
    GMC_GAIN : string := "7";
    GMC_TEST : string := "14";
    ICP_CURRENT : integer := 14;
    KVCO : string := "7";
    LPF_CAPACITOR : string := "3";
    LPF_RESISTOR : integer := 1;
    PLLRST_ENA : string := "ENABLE";
    PLLMRST_ENA : string := "DISABLE";
    PLLC2RST_ENA : string := "DISABLE";
    PLLC34RST_ENA : string := "DISABLE";
    PREDIV_MUXC0 : string := "VCO";
    PREDIV_MUXC1 : string := "VCO";
    PREDIV_MUXC2 : string := "VCO";
    PREDIV_MUXC3 : string := "VCO";
    PREDIV_MUXC4 : string := "VCO";
    ODIV_MUXC0 : string := "DIV";
    ODIV_MUXC1 : string := "DIV";
    ODIV_MUXC2 : string := "DIV";
    ODIV_MUXC3 : string := "DIV";
    ODIV_MUXC4 : string := "DIV";
    FREQ_LOCK_ACCURACY : string := "2";
    PLL_LOCK_MODE : string := "0";
    INTFB_WAKE : string := "DISABLE";
    DPHASE_SOURCE : string := "DISABLE";
    VCO_NORESET : string := "DISABLE";
    STDBY_VCO_ENA : string := "DISABLE";
    NORESET : string := "DISABLE";
    SYNC_ENABLE : string := "ENABLE";
    DERIVE_PLL_CLOCKS : string := "DISABLE";
    GEN_BASIC_CLOCK : string := "DISABLE"
    );
port (
    clkc : out std_logic_vector(4 DOWNTO 0) := (others => '0');
    ext_lock : out std_logic := '0';
    stdby : in std_logic := '0';
    refclk : in std_logic := '0';
    fbclk : in std_logic := '0';
    pllreset : in std_logic := '0';
    scanclk : in std_logic := '0';
    phaseupdown : in std_logic := '0';
    phasestep : in std_logic := '0';
    phasedone : out std_logic := '0';
    phcntsel : in std_logic_vector(2 DOWNTO 0) := (others => '0')
    );
end component;


end AL_COMPONENTS;
------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_RAMFIFO is
generic (
    DATA_WIDTH : integer := 8;
    ADDR_WIDTH : integer := 8;
    SHOWAHEAD : integer := 0;
    IMPLEMENT : string := "AUTO"
    );
port (
    rst : in std_logic ;
    di : in std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    clkr : in std_logic ;
    re : in std_logic ;
    clkw : in std_logic ;
    we : in std_logic ;
    do : out std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    empty_flag : out std_logic ;
    full_flag : out std_logic ;
    rdusedw : out std_logic_vector(ADDR_WIDTH DOWNTO 0) ;
    wrusedw : out std_logic_vector(ADDR_WIDTH DOWNTO 0) 
    );
end entity;

architecture rtl of AL_LOGIC_RAMFIFO is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity gray2bin is
generic (
    WIDTH : integer := 8
    );
port (
    din : in std_logic_vector(WIDTH-1 DOWNTO 0) ;
    dout : out std_logic_vector(WIDTH-1 DOWNTO 0) 
    );
end entity;

architecture rtl of gray2bin is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity gray_count is
generic (
    WIDTH : integer := 8;
    INIT : integer := 0
    );
port (
    clk : in std_logic ;
    enable : in std_logic ;
    reset : in std_logic ;
    cnt : out std_logic_vector(WIDTH-1 DOWNTO 0) 
    );
end entity;

architecture rtl of gray_count is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity showahead_buffer is
generic (
    DATA_WIDTH : integer := 8
    );
port (
    rstn_i : in std_logic ;
    clk_i : in std_logic ;
    ena_i : in std_logic ;
    rd_i : in std_logic ;
    data_i : in std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    data_o : out std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    addr_inc_o : out std_logic ;
    empty_o : out std_logic 
    );
end entity;

architecture rtl of showahead_buffer is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_BUF is
port (
    o : out std_logic ;
    i : in std_logic 
    );
end entity;

architecture rtl of AL_LOGIC_BUF is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_GSRN is
generic (
    USR_GSRN_EN : string := "ENABLE"
    );
port (
    gsrn : in std_logic 
    );
end entity;

architecture rtl of AL_LOGIC_GSRN is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_BUFG is
port (
    o : out std_logic ;
    i : in std_logic 
    );
end entity;

architecture rtl of AL_LOGIC_BUFG is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_BUFIO is
generic (
    GSR : string := "DISABLE";
    DIV : integer := 2;
    STOPCLK : string := "DISABLE"
    );
port (
    clki : in std_logic ;
    rst : in std_logic ;
    coe : in std_logic ;
    clko : out std_logic ;
    clkdiv1 : out std_logic ;
    clkdivx : out std_logic 
    );
end entity;

architecture rtl of AL_LOGIC_BUFIO is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_BUFGMUX is
generic (
    INIT_OUT : string := "0";
    PRESELECT_I0 : string := "TRUE";
    PRESELECT_I1 : string := "FALSE"
    );
port (
    o : out std_logic ;
    i0 : in std_logic ;
    i1 : in std_logic ;
    s : in std_logic 
    );
end entity;

architecture rtl of AL_LOGIC_BUFGMUX is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_CCLK is
port (
    cclk : out std_logic 
    );
end entity;

architecture rtl of AL_LOGIC_CCLK is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_IDELAY is
generic (
    INDEL : integer := 0
    );
port (
    o : out std_logic ;
    i : in std_logic 
    );
end entity;

architecture rtl of AL_LOGIC_IDELAY is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_IDDR is
generic (
    SRMODE : string := "SYNC"
    );
port (
    q1 : out std_logic ;
    q2 : out std_logic ;
    clk : in std_logic ;
    d : in std_logic ;
    rst : in std_logic 
    );
end entity;

architecture rtl of AL_LOGIC_IDDR is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_ODDR is
generic (
    SRMODE : string := "SYNC"
    );
port (
    q : out std_logic ;
    clk : in std_logic ;
    d1 : in std_logic ;
    d2 : in std_logic ;
    rst : in std_logic 
    );
end entity;

architecture rtl of AL_LOGIC_ODDR is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_PROGRAMN is
port (
    programn : in std_logic 
    );
end entity;

architecture rtl of AL_LOGIC_PROGRAMN is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_FIFO is
generic (
    DATA_WIDTH_W : integer := 9;
    DATA_WIDTH_R : integer := 9;
    DATA_DEPTH_W : integer := 1024;
    DATA_DEPTH_R : integer := 1024;
    MODE : string := "FIFO8K";
    REGMODE_W : string := "NOREG";
    REGMODE_R : string := "NOREG";
    E : integer := 0;
    AE : integer := 6;
    AF : integer := 1018;
    F : integer := 1024;
    GSR : string := "DISABLE";
    RESETMODE : string := "ASYNC";
    ASYNC_RESET_RELEASE : string := "SYNC";
    ENDIAN : string := "LITTLE"
    );
port (
    rst : in std_logic ;
    di : in std_logic_vector(DATA_WIDTH_W-1 DOWNTO 0) ;
    do : out std_logic_vector(DATA_WIDTH_R-1 DOWNTO 0) ;
    clkw, we : in std_logic ;
    clkr, re, ore : in std_logic := '0';
    csw, csr : in std_logic_vector(2 DOWNTO 0) ;
    empty_flag, aempty_flag : out std_logic := '0';
    full_flag, afull_flag : out std_logic := '0'
    );
end entity;

architecture rtl of AL_LOGIC_FIFO is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_DRAM is
generic (
    DATA_WIDTH_W : integer := 9;
    ADDR_WIDTH_W : integer := 10;
    DATA_DEPTH_W : integer := 1024;
    DATA_WIDTH_R : integer := 9;
    ADDR_WIDTH_R : integer := 10;
    DATA_DEPTH_R : integer := 1024;
    INIT_FILE : string := "NONE"
    );
port (
    di : in std_logic_vector(DATA_WIDTH_W-1 DOWNTO 0) ;
    waddr : in std_logic_vector(ADDR_WIDTH_W-1 DOWNTO 0) ;
    wclk : in std_logic ;
    we : in std_logic ;
    do : out std_logic_vector(DATA_WIDTH_R-1 DOWNTO 0) ;
    raddr : in std_logic_vector(ADDR_WIDTH_R-1 DOWNTO 0) 
    );
end entity;

architecture rtl of AL_LOGIC_DRAM is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_DRAM16X4 is
generic (
    INIT_D0 : string := X"0000";
    INIT_D1 : string := X"0000";
    INIT_D2 : string := X"0000";
    INIT_D3 : string := X"0000"
    );
port (
    di : in std_logic_vector(3 DOWNTO 0) ;
    waddr : in std_logic_vector(3 DOWNTO 0) ;
    wclk : in std_logic ;
    we : in std_logic ;
    raddr : in std_logic_vector(3 DOWNTO 0) ;
    do : out std_logic_vector(3 DOWNTO 0) 
    );
end entity;

architecture rtl of AL_LOGIC_DRAM16X4 is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_MULT is
generic (
    INPUT_WIDTH_A : integer := 18;
    INPUT_WIDTH_B : integer := 18;
    OUTPUT_WIDTH : integer := 36;
    INPUTFORMAT : string := "SIGNED";
    INPUTREGA : string := "ENABLE";
    INPUTREGB : string := "ENABLE";
    OUTPUTREG : string := "ENABLE";
    SRMODE : string := "ASYNC";
    IMPLEMENT : string := "AUTO"
    );
port (
    p : out std_logic_vector(OUTPUT_WIDTH-1 DOWNTO 0) ;
    a : in std_logic_vector(INPUT_WIDTH_A-1 DOWNTO 0) ;
    b : in std_logic_vector(INPUT_WIDTH_B-1 DOWNTO 0) ;
    cea : in std_logic := '0';
    ceb : in std_logic := '0';
    cepd : in std_logic := '0';
    clk : in std_logic := '0';
    rstan : in std_logic := '0';
    rstbn : in std_logic := '0';
    rstpdn : in std_logic := '0'
    );
end entity;

architecture rtl of AL_LOGIC_MULT is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_SEQ_DIV is
generic (
    NUMER_WIDTH : integer := 16;
    DENOM_WIDTH : integer := 16
    );
port (
    clk : in std_logic ;
    rst : in std_logic ;
    start : in std_logic ;
    numer : in std_logic_vector(NUMER_WIDTH-1 DOWNTO 0) ;
    denom : in std_logic_vector(DENOM_WIDTH-1 DOWNTO 0) ;
    quotient : out std_logic_vector(NUMER_WIDTH-1 DOWNTO 0) ;
    remain : out std_logic_vector(DENOM_WIDTH-1 DOWNTO 0) ;
    done : out std_logic 
    );
end entity;

architecture rtl of AL_LOGIC_SEQ_DIV is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_MBOOT is
generic (
    ADDR_SOURCE_SEL : string := "STATIC";
    STATIC_ADDR : string := B"0";
    MBOOT_AUTO_SEL : string := "DISABLE"
    );
port (
    rebootn : in std_logic ;
    dynamic_addr : in std_logic_vector(7 DOWNTO 0) 
    );
end entity;

architecture rtl of AL_LOGIC_MBOOT is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_DNA is
generic (
    DNA : string := X"0000000000000000"
    );
port (
    dout : out std_logic ;
    clk : in std_logic ;
    din : in std_logic ;
    shift_en : in std_logic 
    );
end entity;

architecture rtl of AL_LOGIC_DNA is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_BRAM is
generic (
    MODE : string := "DP8K";
    DATA_WIDTH_A : string := "9";
    DATA_WIDTH_B : string := "9";
    READBACK : string := "OFF";
    REGMODE_A : string := "NOREG";
    REGMODE_B : string := "NOREG";
    WRITEMODE_A : string := "NORMAL";
    WRITEMODE_B : string := "NORMAL";
    GSR : string := "ENABLE";
    RESETMODE : string := "SYNC";
    ASYNC_RESET_RELEASE : string := "SYNC";
    CEAMUX : string := "SIG";
    CEBMUX : string := "SIG";
    OCEAMUX : string := "SIG";
    OCEBMUX : string := "SIG";
    RSTAMUX : string := "SIG";
    RSTBMUX : string := "SIG";
    CLKAMUX : string := "SIG";
    CLKBMUX : string := "SIG";
    WEAMUX : string := "SIG";
    WEBMUX : string := "SIG";
    CSA0 : string := "SIG";
    CSA1 : string := "SIG";
    CSA2 : string := "SIG";
    CSB0 : string := "SIG";
    CSB1 : string := "SIG";
    CSB2 : string := "SIG";
    INIT_FILE : string := "NONE";
    INITP_00 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INITP_01 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INITP_02 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INITP_03 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_00 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_01 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_02 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_03 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_04 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_05 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_06 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_07 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_08 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_09 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0A : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0B : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0C : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0D : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0E : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0F : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_10 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_11 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_12 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_13 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_14 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_15 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_16 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_17 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_18 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_19 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1A : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1B : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1C : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1D : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1E : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1F : string := X"0000000000000000000000000000000000000000000000000000000000000000"
    );
port (
    doa : out std_logic_vector(8 DOWNTO 0) ;
    dob : out std_logic_vector(8 DOWNTO 0) ;
    dia : in std_logic_vector(8 DOWNTO 0) ;
    dib : in std_logic_vector(8 DOWNTO 0) ;
    csa : in std_logic_vector(2 DOWNTO 0) ;
    csb : in std_logic_vector(2 DOWNTO 0) ;
    cea, ocea, clka, wea, rsta : in std_logic ;
    ceb, oceb, clkb, web, rstb : in std_logic ;
    addra : in std_logic_vector(12 DOWNTO 0) ;
    addrb : in std_logic_vector(12 DOWNTO 0) 
    );
end entity;

architecture rtl of AL_PHY_BRAM is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_PAD is
generic (
    DEDCLK : string := "DISABLE";
    GSR : string := "ENABLE";
    SRMODE : string := "SYNC";
    TSMUX : string := "1";
    INCLKMUX : string := "0";
    INCEMUX : string := "CE";
    INRSTMUX : string := "0";
    IDDRMODE : string := "OFF";
    INDELMUX : string := "NODEL";
    INDEL : integer := 0;
    OUTCLKMUX : string := "0";
    OUTCEMUX : string := "CE";
    OUTRSTMUX : string := "0";
    ODDRMODE : string := "OFF";
    IN_DFFMODE : string := "NONE";
    IN_REGSET : string := "RESET";
    DO_DFFMODE : string := "NONE";
    DO_REGSET : string := "RESET";
    TO_DFFMODE : string := "NONE";
    TO_REGSET : string := "RESET";
    MODE : string := "IN";
    DRIVE : string := "NONE";
    IOTYPE : string := "LVCMOS25"
    );
port (
    opad : out std_logic ;
    di : out std_logic ;
    itrue : out std_logic ;
    icomp : out std_logic ;
    ce, clk, eninr_dyn, ipad, ocomp, otrue, rst, ts : in std_logic ;
    bpad : inout std_logic 
    );
end entity;

architecture rtl of AL_PHY_PAD is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_BRAM32K is
generic (
    MODE : string := "DP16K";
    DATA_WIDTH_A : string := "16";
    DATA_WIDTH_B : string := "16";
    REGMODE_A : string := "NOREG";
    REGMODE_B : string := "NOREG";
    WRITEMODE_A : string := "NORMAL";
    WRITEMODE_B : string := "NORMAL";
    SRMODE : string := "SYNC";
    CENAMUX : string := "SIG";
    CENBMUX : string := "SIG";
    OCEAMUX : string := "SIG";
    OCEBMUX : string := "SIG";
    RSTAMUX : string := "SIG";
    RSTBMUX : string := "SIG";
    CLKAMUX : string := "SIG";
    CLKBMUX : string := "SIG";
    WENAMUX : string := "SIG";
    WENBMUX : string := "SIG";
    READBACK : string := "OFF"
    );
port (
    doa : out std_logic_vector(15 DOWNTO 0) ;
    dob : out std_logic_vector(15 DOWNTO 0) ;
    dia : in std_logic_vector(15 DOWNTO 0) ;
    dib : in std_logic_vector(15 DOWNTO 0) ;
    addra : in std_logic_vector(10 DOWNTO 0) ;
    addrb : in std_logic_vector(10 DOWNTO 0) ;
    bytea : in std_logic ;
    byteb : in std_logic ;
    cena,wena : in std_logic ;
    clka,rsta : in std_logic ;
    cenb,wenb : in std_logic ;
    clkb,rstb : in std_logic ;
    ocea : in std_logic ;
    oceb : in std_logic 
    );
end entity;

architecture rtl of AL_PHY_BRAM32K is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_FIFO is
generic (
    MODE : string := "FIFO8K";
    DATA_WIDTH_A : string := "18";
    DATA_WIDTH_B : string := "18";
    READBACK : string := "OFF";
    REGMODE_A : string := "NOREG";
    REGMODE_B : string := "NOREG";
     AE : string := B"00000001100000";
     AF : string := B"01111110010000";
     F : string := B"01111111110000";
     AEP1 : string := B"00000001110000";
     AFM1 : string := B"01111110000000";
     FM1 : string := B"01111111100000";
     E : string := B"00000";
     EP1 : string := B"010000";
    GSR : string := "ENABLE";
    RESETMODE : string := "ASYNC";
    ASYNC_RESET_RELEASE : string := "SYNC";
    CEA : string := "SIG";
    CEB : string := "SIG";
    OCEA : string := "SIG";
    OCEB : string := "SIG";
    RSTA : string := "SIG";
    RSTB : string := "SIG";
    CLKA : string := "SIG";
    CLKB : string := "SIG";
    WEA : string := "SIG";
    WEB : string := "SIG";
    CSA0 : string := "SIG";
    CSA1 : string := "SIG";
    CSA2 : string := "SIG";
    CSB0 : string := "SIG";
    CSB1 : string := "SIG";
    CSB2 : string := "SIG"
    );
port (
    flag : out std_logic ;
    dia,dib : in std_logic_vector(8 DOWNTO 0) ;
    csr : in std_logic_vector(2 DOWNTO 0) ;
    csw : in std_logic_vector(2 DOWNTO 0) ;
    we, re, clkw, clkr, rst, rprst,orea,oreb : in std_logic ;
    dob,doa : out std_logic_vector(8 DOWNTO 0) ;
    empty_flag, aempty_flag, afull_flag, full_flag : out std_logic 
    );
end entity;

architecture rtl of AL_PHY_FIFO is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_MULT18 is
generic (
    INPUTREGA : string := "ENABLE";
    INPUTREGB : string := "ENABLE";
    OUTPUTREG : string := "ENABLE";
    SRMODE : string := "ASYNC";
    MODE : string := "MULT18X18C";
    CEAMUX : string := "SIG";
    CEBMUX : string := "SIG";
    CEPDMUX : string := "SIG";
    RSTANMUX : string := "SIG";
    RSTBNMUX : string := "SIG";
    RSTPDNMUX : string := "SIG";
    CLKMUX : string := "SIG";
    SIGNEDAMUX : string := "SIG";
    SIGNEDBMUX : string := "SIG";
    SOURCEAMUX : string := "SIG";
    SOURCEBMUX : string := "SIG"
    );
port (
    acout : out std_logic_vector(17 DOWNTO 0) ;
    bcout : out std_logic_vector(17 DOWNTO 0) ;
    p : out std_logic_vector(35 DOWNTO 0) ;
    signeda : in std_logic ;
    signedb : in std_logic ;
    a : in std_logic_vector(17 DOWNTO 0) ;
    b : in std_logic_vector(17 DOWNTO 0) ;
    acin : in std_logic_vector(17 DOWNTO 0) ;
    bcin : in std_logic_vector(17 DOWNTO 0) ;
    cea : in std_logic ;
    ceb : in std_logic ;
    cepd : in std_logic ;
    clk : in std_logic ;
    rstan : in std_logic ;
    rstbn : in std_logic ;
    rstpdn : in std_logic ;
    sourcea : in std_logic ;
    sourceb : in std_logic 
    );
end entity;

architecture rtl of AL_PHY_MULT18 is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_OSC is
generic (
    STDBY : string := "DISABLE"
    );
port (
    osc_dis : in std_logic ;
    osc_clk : out std_logic 
    );
end entity;

architecture rtl of AL_PHY_OSC is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_SDRAM_2M_32 is
port (
    clk : in std_logic ;
    ras_n : in std_logic ;
    cas_n : in std_logic ;
    we_n : in std_logic ;
    addr : in std_logic_vector(10 DOWNTO 0) ;
    ba : in std_logic_vector(1 DOWNTO 0) ;
    dq : inout std_logic_vector(31 DOWNTO 0) ;
    dm : in std_logic ;
    dm0 : in std_logic := '0';
    cke : in std_logic := '0'
    );
end entity;

architecture rtl of AL_PHY_SDRAM_2M_32 is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_GCLK is
port (
    clki : in std_logic ;
    clko : out std_logic 
    );
end entity;

architecture rtl of AL_PHY_GCLK is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_IOCLK is
generic (
    STOPCLK : string := "DISABLE"
    );
port (
    clki : in std_logic ;
    stop : in std_logic ;
    clko : out std_logic 
    );
end entity;

architecture rtl of AL_PHY_IOCLK is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_CLKDIV is
generic (
    GSR : string := "DISABLE";
    DIV : integer := 2
    );
port (
    clkdiv1, clkdivx : out std_logic ;
    clki : in std_logic ;
    rst : in std_logic ;
    rls : in std_logic 
    );
end entity;

architecture rtl of AL_PHY_CLKDIV is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_CSB is
generic (
    HOLD : string := "YES";
    INITOUT : string := "0";
    PRESELECT : string := "NONE";
    TESTMODE : string := "DISABLE"
    );
port (
    clko : out std_logic ;
    ce : in std_logic_vector(1 DOWNTO 0) ;
    clki : in std_logic_vector(1 DOWNTO 0) ;
    drct : in std_logic_vector(1 DOWNTO 0) ;
    sel : in std_logic_vector(1 DOWNTO 0) ;
    ignr : in std_logic_vector(1 DOWNTO 0) 
    );
end entity;

architecture rtl of AL_PHY_CSB is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_CONFIG is
generic (
    MBOOT_AUTO_SEL : string := "DISABLE";
    ADDR_SOURCE_SEL : string := "STATIC";
    STATIC_ADDR : string := B"0";
    GSRN_SYNC_SEL : string := "DISABLE";
    USR_GSRN_EN : string := "DISABLE"
    );
port (
    jrstn : out std_logic ;
    jrti : out std_logic_vector(0 DOWNTO 0) ;
    jshift : out std_logic ;
    jtck : out std_logic ;
    jtdi : out std_logic ;
    jupdate : out std_logic ;
    jscanen : out std_logic_vector(0 DOWNTO 0) ;
    jtms : out std_logic ;
    jtdo : in std_logic_vector(0 DOWNTO 0) ;
    jtag8_ipa : in std_logic_vector(0 DOWNTO 0) ;
    jtag8_ipb : in std_logic_vector(0 DOWNTO 0) ;
    done : out std_logic ;
    highz : out std_logic ;
    cclk : out std_logic ;
    usr_gsrn : in std_logic ;
    gsrn_sync_clk : in std_logic ;
    mboot_rebootn : in std_logic ;
    mboot_dynamic_addr : in std_logic_vector(7 DOWNTO 0) 
    );
end entity;

architecture rtl of AL_PHY_CONFIG is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_MSLICE is
generic (
    INIT_LUT0 : string := X"0000";
    INIT_LUT1 : string := X"0000";
    MODE : string := "LOGIC";
    ALUTYPE : string := "";
    MSFXMUX : string := "OFF";
    GSR : string := "ENABLE";
    TESTMODE : string := "OFF";
    CEMUX : string := "CE";
    SRMUX : string := "SR";
    CLKMUX : string := "CLK";
    SRMODE : string := "ASYNC";
    DFFMODE : string := "FF";
    REG0_SD : string := "MI";
    REG1_SD : string := "MI";
    REG0_REGSET : string := "SET";
    REG1_REGSET : string := "SET"
    );
port (
    a,b,c,d,mi : in std_logic_vector(1 DOWNTO 0) ;
    clk : in std_logic ;
    ce : in std_logic ;
    sr : in std_logic ;
    fci : in std_logic ;
    f,fx,q : out std_logic_vector(1 DOWNTO 0) ;
    fco : out std_logic ;
    dpram_mode : in std_logic ;
    dpram_di : in std_logic_vector(1 DOWNTO 0) ;
    dpram_we : in std_logic ;
    dpram_wclk : in std_logic ;
    dpram_waddr : in std_logic_vector(3 DOWNTO 0) 
    );
end entity;

architecture rtl of AL_PHY_MSLICE is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_LSLICE is
generic (
    INIT_LUTF0 : string := X"0000";
    INIT_LUTG0 : string := X"0000";
    INIT_LUTF1 : string := X"0000";
    INIT_LUTG1 : string := X"0000";
    MODE : string := "LOGIC";
    GSR : string := "ENABLE";
    TESTMODE : string := "OFF";
    CEMUX : string := "1";
    SRMUX : string := "SR";
    CLKMUX : string := "CLK";
    SRMODE : string := "ASYNC";
    DFFMODE : string := "FF";
    REG0_SD : string := "MI";
    REG1_SD : string := "MI";
    REG0_REGSET : string := "SET";
    REG1_REGSET : string := "SET";
    DEMUX0 : string := "D";
    DEMUX1 : string := "D";
    CMIMUX0 : string := "C";
    CMIMUX1 : string := "C";
    LSFMUX0 : string := "LUTF";
    LSFXMUX0 : string := "LUTG";
    LSFMUX1 : string := "LUTF";
    LSFXMUX1 : string := "LUTG"
    );
port (
    a,b,c,d,e,mi : in std_logic_vector(1 DOWNTO 0) ;
    clk : in std_logic ;
    ce : in std_logic ;
    sr : in std_logic ;
    fci : in std_logic ;
    f,fx,q : out std_logic_vector(1 DOWNTO 0) ;
    fco : out std_logic ;
    dpram_di : out std_logic_vector(3 DOWNTO 0) ;
    dpram_waddr : out std_logic_vector(3 DOWNTO 0) ;
    dpram_wclk : out std_logic ;
    dpram_we : out std_logic ;
    dpram_mode : out std_logic 
    );
end entity;

architecture rtl of AL_PHY_LSLICE is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_LOGIC_BRAM is
generic (
    DATA_WIDTH_A : integer := 9;
    DATA_WIDTH_B : integer := 9;
    ADDR_WIDTH_A : integer := 10;
    ADDR_WIDTH_B : integer := 10;
    DATA_DEPTH_A : integer := 1024;
    DATA_DEPTH_B : integer := 1024;
    BYTE_ENABLE : integer := 0;
    BYTE_A : integer := 1;
    BYTE_B : integer := 1;
    MODE : string := "DP";
    REGMODE_A : string := "NOREG";
    REGMODE_B : string := "NOREG";
    WRITEMODE_A : string := "NORMAL";
    WRITEMODE_B : string := "NORMAL";
    RESETMODE : string := "SYNC";
    DEBUGGABLE : string := "NO";
    PACKABLE : string := "NO";
    FORCE_KEEP : string := "OFF";
    INIT_FILE : string := "NONE";
    FILL_ALL : string := "NONE";
    IMPLEMENT : string := "9K"
    );
port (
    doa : out std_logic_vector(DATA_WIDTH_A-1 DOWNTO 0) ;
    dob : out std_logic_vector(DATA_WIDTH_B-1 DOWNTO 0) ;
    dia : in std_logic_vector(DATA_WIDTH_A-1 DOWNTO 0) := (others => '0');
    dib : in std_logic_vector(DATA_WIDTH_B-1 DOWNTO 0) := (others => '0');
    cea, ocea, clka, wea, rsta : in std_logic := '0';
    ceb, oceb, clkb, web, rstb : in std_logic := '0';
    bea : in std_logic_vector(BYTE_A - 1  DOWNTO 0) := (others => '0');
    beb : in std_logic_vector(BYTE_B - 1  DOWNTO 0) := (others => '0');
    addra : in std_logic_vector(ADDR_WIDTH_A-1 DOWNTO 0) := (others => '0');
    addrb : in std_logic_vector(ADDR_WIDTH_B-1 DOWNTO 0) := (others => '0')
    );
end entity;

architecture rtl of AL_LOGIC_BRAM is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity AL_PHY_PLL is
generic (
    REFCLK_SEL : string := "INTERNAL";
    FIN : string := "100.0000";
    REFCLK_DIV : integer := 1;
    FBCLK_DIV : integer := 1;
    CLKC0_DIV : integer := 1;
    CLKC1_DIV : integer := 1;
    CLKC2_DIV : integer := 1;
    CLKC3_DIV : integer := 1;
    CLKC4_DIV : integer := 1;
    CLKC0_ENABLE : string := "DISABLE";
    CLKC1_ENABLE : string := "DISABLE";
    CLKC2_ENABLE : string := "DISABLE";
    CLKC3_ENABLE : string := "DISABLE";
    CLKC4_ENABLE : string := "DISABLE";
    FEEDBK_MODE : string := "NORMAL";
    FEEDBK_PATH : string := "VCO_PHASE_0";
    INT_FEEDBK_PATH : string := "VCO_PHASE_0";
    STDBY_ENABLE : string := "ENABLE";
    CLKC0_FPHASE : string := "0";
    CLKC1_FPHASE : string := "0";
    CLKC2_FPHASE : string := "0";
    CLKC3_FPHASE : string := "0";
    CLKC4_FPHASE : string := "0";
    CLKC0_CPHASE : integer := 1;
    CLKC1_CPHASE : integer := 1;
    CLKC2_CPHASE : integer := 1;
    CLKC3_CPHASE : integer := 1;
    CLKC4_CPHASE : integer := 1;
    GMC_GAIN : string := "7";
    GMC_TEST : string := "14";
    ICP_CURRENT : integer := 14;
    KVCO : string := "7";
    LPF_CAPACITOR : string := "3";
    LPF_RESISTOR : integer := 1;
    PLLRST_ENA : string := "ENABLE";
    PLLMRST_ENA : string := "DISABLE";
    PLLC2RST_ENA : string := "DISABLE";
    PLLC34RST_ENA : string := "DISABLE";
    PREDIV_MUXC0 : string := "VCO";
    PREDIV_MUXC1 : string := "VCO";
    PREDIV_MUXC2 : string := "VCO";
    PREDIV_MUXC3 : string := "VCO";
    PREDIV_MUXC4 : string := "VCO";
    ODIV_MUXC0 : string := "DIV";
    ODIV_MUXC1 : string := "DIV";
    ODIV_MUXC2 : string := "DIV";
    ODIV_MUXC3 : string := "DIV";
    ODIV_MUXC4 : string := "DIV";
    FREQ_LOCK_ACCURACY : string := "2";
    PLL_LOCK_MODE : string := "0";
    INTFB_WAKE : string := "DISABLE";
    DPHASE_SOURCE : string := "DISABLE";
    VCO_NORESET : string := "DISABLE";
    STDBY_VCO_ENA : string := "DISABLE";
    NORESET : string := "DISABLE";
    SYNC_ENABLE : string := "ENABLE";
    DERIVE_PLL_CLOCKS : string := "DISABLE";
    GEN_BASIC_CLOCK : string := "DISABLE"
    );
port (
    clkc : out std_logic_vector(4 DOWNTO 0) := (others => '0');
    ext_lock : out std_logic := '0';
    stdby : in std_logic := '0';
    refclk : in std_logic := '0';
    fbclk : in std_logic := '0';
    pllreset : in std_logic := '0';
    scanclk : in std_logic := '0';
    phaseupdown : in std_logic := '0';
    phasestep : in std_logic := '0';
    phasedone : out std_logic := '0';
    phcntsel : in std_logic_vector(2 DOWNTO 0) := (others => '0')
    );
end entity;

architecture rtl of AL_PHY_PLL is
    begin
    end;


