//****************
module EF3_LOGIC_RAMFIFO (/*autoarg*/
                   // Inputs
                   rst, di, clkr, re, clkw, we,
                   // Outputs
                   do, empty_flag, full_flag, rdusedw, wrusedw
                   );
   
   parameter DATA_WIDTH = 8;
   parameter ADDR_WIDTH = 8;
   parameter SHOWAHEAD = 0;
   parameter IMPLEMENT = "AUTO";  // 9K | 32K | AUTO (all capitalized)
   input                     rst;
   input   [DATA_WIDTH-1:0]  di;
   input                     clkr;
   input                     re;
   input                     clkw;
   input                     we;
   output  [DATA_WIDTH-1:0]  do;
   output                    empty_flag;
   output                    full_flag;
   output  [ADDR_WIDTH:0]    rdusedw;
   output  [ADDR_WIDTH:0]    wrusedw;
endmodule
//****************
//****************
module gray2bin (
                  // Inputs
                  din,
                  // Outputs
                  dout
                  );
                  
   parameter WIDTH = 8;
   input   [WIDTH-1:0]  din;
   output  [WIDTH-1:0]  dout;
endmodule
//****************
//****************
module gray_count (
                   // Inputs
                   clk, enable, reset,
                   // Outputs
                   cnt
                   );
                  
   parameter WIDTH = 8;
   parameter INIT  = 0;
   input                clk;
   input                enable;
   input                reset;
   output  [WIDTH-1:0]  cnt;
endmodule
//****************
//****************
module showahead_buffer (/*autoarg*/
                         // Inputs
                         rstn_i, clk_i, ena_i, rd_i, data_i,
                         // Outputs
                         data_o, addr_inc_o, empty_o
                         );
   
   parameter DATA_WIDTH = 8;
   input                     rstn_i;
   input                     clk_i;
   input                     ena_i;
   input                     rd_i;
   input   [DATA_WIDTH-1:0]  data_i;
   output  [DATA_WIDTH-1:0]  data_o;
   output                    addr_inc_o;
   output                    empty_o;
endmodule
//****************
//****************
module  EF3_LOGIC_BUF(o,i);     // user instantiated logic BUF
output o;
input i;
endmodule
//****************
//****************
module  EF3_LOGIC_BUFG(o,i);  // driver for global clk resource
output o;
input i;
endmodule
//****************
//****************
module  EF3_LOGIC_BUFIO(clki,rst,coe,clko,clkdiv1,clkdivx);  // driver for IO clk resource
input clki;
input rst;
input coe;
output clko;        // for pad sinks
output clkdiv1;     // for other sinks
output clkdivx;
parameter GSR     = "DISABLE";    //  DISABLE/ENABLE 
parameter DIV     = 2;            //  2/4
parameter STOPCLK = "DISABLE";    // ENABLE,DISABLE
endmodule
//****************
//****************
module  EF3_LOGIC_BUFGMUX(o,i0,i1,s); // global clk switch mux
output o;
input i0;
input i1;
input s;
parameter INIT_OUT = "0";
parameter PRESELECT_I0 = "TRUE";
parameter PRESELECT_I1 = "FALSE";
endmodule
//****************
//****************
module EF3_LOGIC_MBOOT(rebootn, dynamic_addr);   // config multiboot
input       rebootn;
input [7:0] dynamic_addr;    // high-8bit address
  parameter ADDR_SOURCE_SEL = "STATIC";  // STATIC/DYNAMIC 
  parameter STATIC_ADDR = 8'b00000000;    // high-8bit address
endmodule
//****************
//****************
module EF3_LOGIC_DNA(dout, clk, din, shift_en);    // config dna 
parameter DNA = 64'h0000000000000000;
output dout;
input  clk;
input  din;
input  shift_en;
endmodule
//****************
//****************
module EF3_LOGIC_GCTRL(done, highz);      // config global control
output done;
output highz;
endmodule
//****************
//****************
module  EF3_LOGIC_GSRN(gsrn, sync_clk);     // config gsrn
input gsrn;
input sync_clk; 
  parameter GSRN_SYNC_SEL = "DISABLE";  // DISABLE/ENABLE set by wakeup register
  parameter USR_GSRN_EN   = "DISABLE";  // DISABLE/ENABLE set by wakeup register
endmodule
//****************
//****************
module EF3_LOGIC_CCLK(cclk, en);        // config cclk
output cclk;
input  en;
parameter FREQ = "4.5";       // frequency set by wakeup register 
endmodule
//****************
//****************
module EF3_LOGIC_JTAGEN(jtagen);
input jtagen;
parameter LOCATION = "";
endmodule
//****************
//****************
module EF3_LOGIC_IDDR (q1, q0, clk, d, rst);
    output q1;
    output q0;
    input clk;
    input d;
    input rst;
    parameter ASYNCRST = "ENABLE"; //ENABLE, DISABLE
    parameter PIPEMODE = "PIPED"; //PIPED, NONE
endmodule
//****************
//****************
module EF3_LOGIC_ODDR (q, clk, d1, d0, rst);
    output q;
    input clk;
    input d1;
    input d0;
    input rst;
    parameter ASYNCRST = "ENABLE"; //ENABLE, DISABLE
endmodule
//****************
//****************
module EF3_LOGIC_IDDRx2 (q3, q2, q1, q0, pclk, sclk, d, rst);
    output q3;
    output q2;
    output q1;
    output q0;
    input pclk;
    input sclk;
    input d;
    input rst;
    parameter ASYNCRST = "ENABLE"; //ENABLE, DISABLE
endmodule
//****************
//****************
module EF3_LOGIC_ODDRx2 (q, pclk, sclk, d3, d2, d1, d0, rst);
    output q;
    input pclk;
    input sclk;
    input d3;
    input d2;
    input d1;
    input d0;
    input rst;
    parameter ASYNCRST = "ENABLE"; //ENABLE, DISABLE
endmodule
//****************
//****************
module EF3_LOGIC_ODDRx2l (q, sclk, d3, d2, d1, d0, rst);
    output q;
    input sclk;
    input d3;
    input d2;
    input d1;
    input d0;
    input rst;
    parameter ASYNCRST = "ENABLE"; //ENABLE, DISABLE
endmodule
//****************
//****************
module EF3_LOGIC_FIFO (
    rst,
    di, clkw, we, csw, 
    do, clkr, re, csr, ore,
    empty_flag, aempty_flag,
    full_flag, afull_flag
);
// 1kx9 by default
parameter DATA_WIDTH_W = 9;                 // 1 ... 9*48
parameter DATA_WIDTH_R = DATA_WIDTH_W;      // 1 ... 9*48
parameter DATA_DEPTH_W = 1024;                                        // 512, 1024, 2048, 4096, 8192
parameter DATA_DEPTH_R = DATA_WIDTH_W * DATA_DEPTH_W / DATA_WIDTH_R;  // 512, 1024, 2048, 4096, 8192
input   rst;
input   [DATA_WIDTH_W-1:0] di;
output  [DATA_WIDTH_R-1:0] do;
input   clkw, we;
input   clkr, re, ore; //DEFAULT := '0'
input [2:0] csw, csr;
output  empty_flag, aempty_flag; //DEFAULT := '0'
output  full_flag, afull_flag; //DEFAULT := '0'
parameter  MODE = "FIFO8K";
parameter  REGMODE_W   = "NOREG"; //NOREG, OUTREG
parameter  REGMODE_R   = "NOREG"; //NOREG, OUTREG   
parameter  E  = 0;
parameter  AE = 6;
parameter  AF = DATA_DEPTH_W-6;
parameter  F  = DATA_DEPTH_W;
parameter  GSR = "DISABLE";  //DISABLE, ENABLE
parameter  RESETMODE = "ASYNC"; //SYNC, ASYNC
parameter  ASYNC_RESET_RELEASE = "SYNC";  //SYNC, ASYNC
parameter ENDIAN = "LITTLE";    // LITTLE, BIG
endmodule
//****************
//****************
module EF3_LOGIC_DRAM (di, waddr, we, wclk, do, raddr);
parameter DATA_WIDTH_W = 9;
parameter ADDR_WIDTH_W = 10;
parameter DATA_DEPTH_W = 2 ** ADDR_WIDTH_W;
parameter DATA_WIDTH_R = 9;
parameter ADDR_WIDTH_R = 10;
parameter DATA_DEPTH_R = 2 ** ADDR_WIDTH_R;
input [DATA_WIDTH_W-1:0]di;
input [ADDR_WIDTH_W-1:0]waddr;
input wclk;
input we;
output [DATA_WIDTH_R-1:0]do;
input [ADDR_WIDTH_R-1:0]raddr;
parameter INIT_FILE = "NONE";
endmodule
//****************
//****************
module EF3_LOGIC_DRAM16X4 ( di, waddr, we, wclk, raddr, do);
parameter INIT_D0=16'h0000;
parameter INIT_D1=16'h0000;
parameter INIT_D2=16'h0000;
parameter INIT_D3=16'h0000;
  input [3:0]di;
  input [3:0]waddr;
  input wclk;
  input we;
  input [3:0]raddr;
  output [3:0]do;
endmodule
//****************
//****************
module EF3_LOGIC_MULT (p, a, b, cea, ceb, cepd, clk, rstan, rstbn, rstpdn); 
    parameter INPUT_WIDTH_A = 18;
    parameter INPUT_WIDTH_B = 18;
    parameter OUTPUT_WIDTH = 36;
    output [OUTPUT_WIDTH-1:0] p;
    input [INPUT_WIDTH_A-1:0] a;
    input [INPUT_WIDTH_B-1:0] b;
    input cea; //DEFAULT := '0'
    input ceb; //DEFAULT := '0'
    input cepd; //DEFAULT := '0'
    input clk; //DEFAULT := '0'
    input rstan; //DEFAULT := '0'
    input rstbn; //DEFAULT := '0'
    input rstpdn; //DEFAULT := '0'
    parameter INPUTFORMAT = "SIGNED"; // SIGNED, UNSIGNED
    parameter INPUTREGA = "ENABLE";   // ENABLE, DISABLE
    parameter INPUTREGB = "ENABLE";   // ENABLE, DISABLE
    parameter OUTPUTREG = "ENABLE";   // ENABLE, DISABLE
    parameter SRMODE    = "ASYNC";    // ASYNC, SYNC
    parameter IMPLEMENT = "AUTO";    // AUTO, DSP, GATE
endmodule
//****************
//****************
module EF3_LOGIC_SEQ_DIV (clk, rst, start, numer, denom, quotient, remain, done);

	parameter NUMER_WIDTH = 16;
  parameter DENOM_WIDTH = 16;
  input                     clk;
  input                     rst;  
  input                     start;  
  input  [NUMER_WIDTH-1:0]  numer;  
  input  [DENOM_WIDTH-1:0]  denom;  
  output [NUMER_WIDTH-1:0]  quotient;  
  output [DENOM_WIDTH-1:0]  remain;  
  output                    done;
endmodule
//****************
//****************
module EF3_PHY_BRAM (
  dia, csa,
  addra, cea, ocea, clka, wea, rsta,
  dib, csb,
  addrb, ceb, oceb, clkb, web, rstb,
  doa,
  dob);

output  [8:0] doa;
output  [8:0] dob;
input   [8:0] dia;
input   [8:0] dib;
input   [2:0] csa;
input   [2:0] csb;
input   cea, ocea, clka, wea, rsta;
input   ceb, oceb, clkb, web, rstb;
input   [12:0] addra;
input   [12:0] addrb;
parameter MODE = "DP8K";               // DP8K,SP8K,PDPW8K,FIFO8K : for DATA_WIDTH = 18, MODE must be "PDPW8K" or "FIFO8K" 
parameter DATA_WIDTH_A = "9";          // 1, 2, 4, 9, 18
parameter DATA_WIDTH_B = "9";          // 1, 2, 4, 9, 18
parameter READBACK = "OFF";
parameter REGMODE_A = "NOREG";         // "NOREG", "OUTREG"
parameter REGMODE_B = "NOREG";         // "NOREG", "OUTREG"
parameter WRITEMODE_A = "NORMAL";      // "NORMAL", "READBEFOREWRITE", "WRITETHROUGH"
parameter WRITEMODE_B = "NORMAL";      // "NORMAL", "READBEFOREWRITE", "WRITETHROUGH"
parameter GSR = "ENABLE";              // "DISABLE", "ENABLE"
parameter RESETMODE = "SYNC";          // "SYNC", "ASYNC"
parameter ASYNC_RESET_RELEASE = "SYNC";          // "SYNC", "ASYNC"
parameter CEAMUX = "SIG"; //0, 1, INV, SIG
parameter CEBMUX = "SIG"; //0, 1, INV, SIG
parameter OCEAMUX = "SIG"; //0, 1, INV, SIG
parameter OCEBMUX = "SIG"; //0, 1, INV, SIG
parameter RSTAMUX = "SIG"; //0, 1, INV, SIG
parameter RSTBMUX = "SIG"; //0, 1, INV, SIG
parameter CLKAMUX = "SIG"; //0, 1, INV, SIG
parameter CLKBMUX = "SIG"; //0, 1, INV, SIG
parameter WEAMUX = "SIG"; //0, 1, INV, SIG
parameter WEBMUX = "SIG"; //0, 1, INV, SIG
parameter CSA0 = "SIG" ; //1,0,INV,SIG
parameter CSA1 = "SIG" ;
parameter CSA2 = "SIG" ;
parameter CSB0 = "SIG" ;
parameter CSB1 = "SIG" ;
parameter CSB2 = "SIG" ;
    parameter INIT_FILE = "NONE";
    parameter INITP_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INITP_01 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INITP_02 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INITP_03 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_01 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_02 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_03 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_04 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_05 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_06 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_07 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_08 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_09 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_0A = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_0B = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_0C = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_0D = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_0E = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_0F = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_10 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_11 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_12 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_13 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_14 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_15 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_16 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_17 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_18 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_19 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_1A = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_1B = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_1C = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_1D = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_1E = 256'h0000000000000000000000000000000000000000000000000000000000000000;
    parameter INIT_1F = 256'h0000000000000000000000000000000000000000000000000000000000000000;
endmodule
//****************
//****************
module EF3_PHY_FIFO (
		rst, 
            // write port
            dia,dib, csw, we, clkw, 
            // read port
            dob,doa, csr, re, clkr, rprst, orea,oreb,
            // output flag
            empty_flag, aempty_flag, afull_flag, full_flag);



input    [8:0] dia,dib;
input    [2:0] csr;
input    [2:0] csw;
input    we, re, clkw, clkr, rst, rprst,orea,oreb;
output   [8:0] dob,doa;
output   empty_flag, aempty_flag, afull_flag, full_flag;
   parameter  MODE = "FIFO8K";
   parameter  DATA_WIDTH_A = "18"; // 1,2,4,9,18
   parameter  DATA_WIDTH_B = "18"; // 1,2,4,9,18
   parameter  READBACK = "OFF";
   parameter  REGMODE_A   = "NOREG"; //NOREG, OUTREG
   parameter  REGMODE_B   = "NOREG"; //NOREG, OUTREG   
   parameter  [13:0] AE   = 14'b00000001100000;
   parameter  [13:0] AF   = 14'b01111110010000;
   parameter  [13:0] F 	  = 14'b01111111110000;
   parameter  [13:0] AEP1 = 14'b00000001110000;
   parameter  [13:0] AFM1 = 14'b01111110000000;
   parameter  [13:0] FM1  = 14'b01111111100000;   
   parameter  [4:0] E   = 5'b00000;
   parameter  [5:0] EP1 = 6'b010000;
   parameter  GSR = "ENABLE";  //DISABLE, ENABLE
   parameter  RESETMODE = "ASYNC"; //SYNC, ASYNC
   parameter  ASYNC_RESET_RELEASE = "SYNC";  //SYNC, ASYNC
parameter CEA = "SIG"; //0, 1, INV, SIG
parameter CEB = "SIG"; //0, 1, INV, SIG
parameter OCEA = "SIG"; //0, 1, INV, SIG
parameter OCEB = "SIG"; //0, 1, INV, SIG
parameter RSTA = "SIG"; //0, 1, INV, SIG
parameter RSTB = "SIG"; //0, 1, INV, SIG
parameter CLKA = "SIG"; //0, 1, INV, SIG
parameter CLKB = "SIG"; //0, 1, INV, SIG
parameter WEA = "SIG"; //0, 1, INV, SIG
parameter WEB = "SIG"; //0, 1, INV, SIG
parameter CSA0 = "SIG" ; //1,0,INV,SIG
parameter CSA1 = "SIG" ;
parameter CSA2 = "SIG" ;
parameter CSB0 = "SIG" ;
parameter CSB1 = "SIG" ;
parameter CSB2 = "SIG" ;
endmodule
//****************
//****************
module EF3_PHY_MULT18 (acout, bcout, p, signeda, signedb, a, b, acin, bcin, cea, ceb, cepd, clk, rstan, rstbn, rstpdn,
            sourcea, sourceb); 

    output [17:0] acout; 
    output [17:0] bcout; 
    output [35:0] p; 
    input signeda;
    input signedb;
    input [17:0] a;
    input [17:0] b;
    input [17:0] acin;
    input [17:0] bcin;
    input cea;
    input ceb;
    input cepd;
    input clk;
    input rstan;
    input rstbn;
    input rstpdn;
    input sourcea;
    input sourceb;
    parameter INPUTREGA = "ENABLE"; // ENABLE/DISABLE
    parameter INPUTREGB = "ENABLE"; // ENABLE/DISABLE
    parameter OUTPUTREG = "ENABLE"; // ENABLE/DISABLE
    parameter SRMODE    = "ASYNC";  // ASYNC/SYNC
    parameter MODE      = "MULT18X18C"; //MULT9X9C/MULT18X18C
    parameter CEAMUX = "SIG"; //0, 1, INV, SIG
    parameter CEBMUX = "SIG"; //0, 1, INV, SIG
    parameter CEPDMUX = "SIG"; //0, 1, INV, SIG
    parameter RSTANMUX = "SIG"; //0, 1, INV, SIG
    parameter RSTBNMUX = "SIG"; //0, 1, INV, SIG
    parameter RSTPDNMUX = "SIG"; //0, 1, INV, SIG
    parameter CLKMUX = "SIG"; //0, 1, INV, SIG
    parameter SIGNEDAMUX = "SIG"; //0, 1, INV, SIG
    parameter SIGNEDBMUX = "SIG"; //0, 1, INV, SIG
    parameter SOURCEAMUX = "SIG"; //0, 1, INV, SIG
    parameter SOURCEBMUX = "SIG"; //0, 1, INV, SIG
endmodule // dsp
//****************
//****************
module EF3_PHY_GCLK (clki, clko);
    input clki;
    output clko;
endmodule
//****************
//****************
module EF3_PHY_IOCLK (clki, stop, clko);
    input clki;
    input stop;
    output clko;
    parameter  STOPCLK = "DISABLE"; // ENABLE,DISABLE
endmodule
//****************
//****************
module EF3_PHY_CLKDIV (clki, rst, rls, clkdiv1, clkdivx);

    output clkdiv1, clkdivx;
    input clki;
    input rst;
    input rls;
    parameter GSR = "DISABLE";  //  DISABLE/ENABLE 
    parameter DIV = 2;          //  2/4
endmodule
//****************
//****************
module EF3_PHY_CONFIG(
  //chip watcher
  jrstn, jrti, jshift, jtck, jtdi, jupdate, jscanen, jtms, jtdo, jtag8_ipa, jtag8_ipb,
  //multi boot
  mboot_rebootn, mboot_dynamic_addr,
  //DNA IP
  dna_dout, dna_clk, dna_din, dna_shift_en,
  //global control
  done, highz,
  //user gsrn
  usr_gsrn, gsrn_sync_clk,
  //cclk 
  cclk, cclk_en
);
  //chip watcher
  output       jrstn;
  output [1:0] jrti;
  output       jshift;
  output       jtck;
  output       jtdi;
  output       jupdate;
  output [1:0] jscanen;
  output       jtms;
  input [1:0]  jtdo;
  input [7:0]  jtag8_ipa;
  input [7:0]  jtag8_ipb;
  input        mboot_rebootn;
  input [7:0]  mboot_dynamic_addr;
  output       dna_dout;
  input        dna_clk;
  input        dna_din;
  input        dna_shift_en;
  output       done;
  output       highz;
  input        usr_gsrn;
  input        gsrn_sync_clk;
  output       cclk;
  input        cclk_en;  
  parameter DONE_PERSISTN = "ENABLE";  // ENABLE/DISABLE set in device setting 
  parameter INIT_PERSISTN = "ENABLE";  // ENABLE/DISABLE set in device setting
  parameter PROGRAMN_PERSISTN = "DISABLE";  // DISABLE/ENABLE set in device setting
  parameter JTAG_PERSISTN = "DISABLE";  // DISABLE/ENABLE set in device setting
  parameter MBOOT_AUTO_SEL = "DISABLE";  // DISABLE/ENABLE  
  parameter ADDR_SOURCE_SEL = "STATIC";  // STATIC/DYNAMIC
  parameter STATIC_ADDR = 8'b00000000;  // high-8bit address
  parameter GSRN_SYNC_SEL = "DISABLE";  // DISABLE/ENABLE set by wakeup register
  parameter USR_GSRN_EN = "DISABLE";  // DISABLE/ENABLE set by wakeup register
  parameter FREQ = "2.5";  // "2.5", "5.0", "10.0", "30.0", set by wakeup register
endmodule
//****************
//****************
module EF3_PHY_OSC (stdby, osc);		// osc clk

    input stdby;
    output osc;
    parameter STANDBY = "DISABLE"; 	// DISABLE/ENABLE 
endmodule
//****************
//****************
module EF3_PHY_OSCDIV (rstn, stdby, div, clko);

    input rstn;
    input stdby;
    input [6:0] div;   // Freq dividing coefficients, divided by 8: div = 7'b111
    output clko;
  parameter half_cycle = 1.9;
endmodule
//****************
//****************
module EF3_PHY_PWRMNT (sel_pwr, pwr_mnt_pd, pwr_dwn_n);
    output pwr_dwn_n;
    input sel_pwr;
    input pwr_mnt_pd;
  parameter MNT_LVL = 0; //1-7 1:1.86v 2:2.00v 3:2.17v 4:2.36v 5:2.60v 6:2.89v 7:3.25v
endmodule
//****************
//****************
module EF3_PHY_MCU(
		ppm_clk,   
		i2s_mst_clk,   
		ppm_rstn,   
		sleep_out,   
		gpio_h_out,   
		gpio_h_oe_n,   
		gpio_h_in,   
		ahbslave_dma_req,   
		ahbslave_dma_single,   
		ahbslave_dma_ack,   
		h2h_haddr,   
		h2h_htrans,   
		h2h_hwrite,   
		h2h_hsize,   
		h2h_hburst,   
		h2h_hprot,   
		h2h_hwdata,   
		h2h_hrdata,   
		h2h_hreadyout,   
		h2h_hresp,   
		h2h_mclk,  
		h2h_rstn,  
		qspi_rxd,
		qspi_txd,
		qspi_sclk,
		qspi_ss,
		qspi_oen,
		adc1_exttrg,   
		adc2_exttrg,   
		usr_reg);
	
	
		input           ppm_clk;
		input           i2s_mst_clk;
		input           ppm_rstn;
		output          sleep_out;
		output [15:0]   gpio_h_out;
		output [15:0]   gpio_h_oe_n;
		input  [15:0]   gpio_h_in;
		input           ahbslave_dma_req;
		input           ahbslave_dma_single;
		output          ahbslave_dma_ack;
		output  [31:0]  h2h_haddr;
		output  [1:0]   h2h_htrans;
		output          h2h_hwrite;
		output  [2:0]   h2h_hsize;
		output  [2:0]   h2h_hburst;
		output  [3:0]   h2h_hprot;
		output  [31:0]  h2h_hwdata;
		input   [31:0]  h2h_hrdata;
		input           h2h_hreadyout;
		input   [1:0]   h2h_hresp;
		input           h2h_mclk;
		input           h2h_rstn;
		input   [3:0]   qspi_rxd;   //   high routing priority
		output  [3:0]   qspi_txd;   //   high routing priority
		output          qspi_sclk;   //   high routing priority
		output          qspi_ss;   //   high routing priority
		output  [3:0]   qspi_oen;   //   high routing priority
		input           adc1_exttrg;
		input           adc2_exttrg;
		output  [15:0]  usr_reg;
		parameter GPIO_L0   = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L1   = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L2   = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L3   = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L4   = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L5   = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L6   = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L7   = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L8   = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L9   = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L10  = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L11  = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L12  = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L13  = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L14  = "DISABLE";  //  ENABLE/DISABLE
		parameter GPIO_L15  = "DISABLE";  //  ENABLE/DISABLEodule
endmodule
//****************
//****************
module EF3_PHY_PAD (ipad, opad, bpad, rst, ce, isclk, ipclk, osclk, opclk, ts, do, di, diq,
       lp_rx_en, lp_dir, lp_dout, lp_din, dynres, vcmpd_n 
    );
    parameter DEDCLK     = "DISABLE";   //DISABLE, ENABLE
    parameter GSR        = "ENABLE";    //ENABLE, DISABLE
    parameter SRMODE     = "SYNC";      //SYNC, ASYNC
    parameter TSMUX      = "1";         //1, 0, TS, INV
    parameter INSCLKMUX  = "0";         //CLK(ioclk[1:0],gclk[3:0],jclk,0(SW default)), INV
    parameter INPCLKMUX  = "CLK";       //CLK(ioclk[0],gclk[1:0],jclk(as SW default=0)), INV
    parameter INCEMUX    = "CE";        //CE, INV, 1
    parameter INRSTMUX   = "0";         //0, RST, INV
    parameter IN_REGSET  = "RESET";     //RESET, SET
    parameter IN_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter IDDRMODE   = "OFF";       //OFF, DDRX1, DDRX2
    parameter IDDRPIPEMODE  = "NONE";   //NONE, PIPED
    parameter INDELMUX   = "NODEL";     //NODEL, FIXDEL
    parameter INDEL      = 0;           // 0-31
    parameter OUTSCLKMUX = "0";         //CLK(ioclk[1:0],gclk[3:0],jclk,0(SW default)), INV
    parameter OUTPCLKMUX = "CLK";       //CLK(ioclk[0],gclk[1:0],jclk(as SW default=0)), INV
    parameter OUTCEMUX   = "CE";        //CE, INV, 1
    parameter OUTRSTMUX  = "0";         //0, RST, INV
    parameter DO_REGSET  = "RESET";     //RESET, SET
    parameter DO_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter ODDRMODE   = "OFF";       //OFF, DDRX1, DDRX2, DDRX2L
    parameter OUTDELMUX  = "NODEL";     //NODEL, FIXDEL
    parameter OUTDEL     = 0;           // 0-3
    parameter TO_REGSET  = "RESET";     //RESET, SET
    parameter TO_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter MODE       = "IN";        //IN, OUT, BI
    parameter DRIVE      = "NONE";      //NONE, 8, 12, 16, 20 24
    parameter IOTYPE     = "LVCMOS25";  //LVCMOS25, LVCMOS12, LVCMOS18, LVCMOS33, LVDS25 ...
    input  ipad;
    output opad;
    inout  bpad;
    input  rst, ce, isclk, ipclk, osclk, opclk;
    input  ts;
    input [3:0] do;
    output di;
    output [3:0] diq;
    input  lp_rx_en;
    input  lp_dir;
    input  lp_dout;
    input  dynres;
    input  vcmpd_n;
    output lp_din;
endmodule
//****************
//****************
module EF3_PHY_SPAD (ipad, opad, bpad, rst, ce, clk, ts, do, di, diq);
    parameter DEDCLK     = "DISABLE";   //DISABLE, ENABLE
    parameter GSR        = "ENABLE";    //ENABLE, DISABLE
    parameter SRMODE     = "SYNC";      //SYNC, ASYNC
    parameter TSMUX      = "TS";        //TS, INV
    parameter INCLKMUX   = "0";         //0, 1, CLK, INV
    parameter INCEMUX    = "CE";        //CE, INV
    parameter INRSTMUX   = "RST";       //RST, INV
    parameter IDDRMODE   = "OFF";       //OFF, ON
    parameter INDELMUX   = "NODEL";     //NODEL, FIXDEL
    parameter INDEL      = 0;           // 0-31
    parameter OUTCLKMUX  = "0";         //0, 1, CLK, INV
    parameter OUTCEMUX   = "CE";        //CE, INV
    parameter OUTRSTMUX  = "RST";       //RST, INV
    parameter ODDRMODE   = "OFF";       //OFF, ON
    parameter IN_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter IN_REGSET  = "RESET";     //RESET, SET
    parameter DO_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter DO_REGSET  = "RESET";     //RESET, SET
    parameter TO_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter TO_REGSET  = "RESET";     //RESET, SET
    parameter MODE       = "IN";        //IN, OUT, BI
    parameter DRIVE      = "NONE";      //NONE, 8, 12, 16, 20 24
    parameter IOTYPE     = "LVCMOS25";  //LVCMOS25, LVCMOS12, LVCMOS18, LVCMOS33, LVDS25 ...
    input  ipad;
    output opad;
    inout  bpad;
    input  rst, ce, clk;
    input  ts;
    input [1:0] do;
    output di;
    output [1:0] diq;
endmodule
//****************
//****************
module EF3_PHY_CSB (clko, ce, clki, drct, sel, ignr);

    output clko;
    input [1:0] ce;
    input [1:0] clki;
    input [1:0] drct;
    input [1:0] sel;
    input [1:0] ignr;
    parameter HOLD      = "YES";     //  YES/NO 
    parameter INITOUT   = "0";       //  0/1
    parameter PRESELECT = "NONE";    //  CLK0/CLK1/NONE
    parameter TESTMODE  = "DISABLE";  //  ENABLE/DISABLE
endmodule
//****************
//****************
module EF3_PHY_INTFLASH ( 
    mosi_io0,
    miso_io1,
    wp_n_io2,
    hold_n_io3,     
    cclk,
    cs_n
);

inout mosi_io0;    //PT6D
inout miso_io1;    //PT23B
inout wp_n_io2;    //PT20D
inout hold_n_io3;  // PT9D
input cclk;        //PT9B
input cs_n;        //PT11D
endmodule
//****************
//****************
module EF3_PHY_MSLICE
  (
  a,
  b,
  c,
  ce,
  clk,
  d,
  mi,
  sr,
  fci,
  f,
  fx,
  q,
  fco,
  dpram_mode,
  dpram_di,
  dpram_we,
  dpram_waddr,
  dpram_wclk  
  );
  input [1:0] a,b,c,d,mi;
  input clk;
  input ce;
  input sr;
  input fci;
  output [1:0] f,fx,q;
  output fco;    
  input dpram_mode;
  input [1:0] dpram_di;
  input dpram_we;
  input dpram_wclk;
  input [3:0]dpram_waddr;
parameter INIT_LUT0 = 16'h0000 ;
parameter INIT_LUT1 = 16'h0000 ;
parameter MODE = "LOGIC" ;
parameter ALUTYPE = "";
parameter MSFXMUX = "OFF" ;
parameter GSR = "ENABLE" ;
parameter TESTMODE = "OFF" ;
parameter CEMUX = "CE" ;
parameter SRMUX = "SR" ;
parameter CLKMUX = "CLK" ;
parameter SRMODE = "ASYNC" ;
parameter DFFMODE = "FF" ;
parameter REG0_SD = "MI" ;
parameter REG1_SD = "MI" ;
parameter REG0_REGSET = "SET" ;
parameter REG1_REGSET = "SET" ;
endmodule
//****************
//****************
module EF3_PHY_LSLICE
  (
  a,
  b,
  c,
  ce,
  clk,
  d,
  e,
  mi,
  sr,
  fci,
  f,
  fx,
  q,
  fco,
  dpram_mode,
  dpram_di,
  dpram_we,
  dpram_waddr,
  dpram_wclk    
  );
  input [1:0] a,b,c,d,e,mi;
  input clk;
  input ce;
  input sr;
  input fci;
  output [1:0] f,fx,q;
  output fco;    
  output [3:0] dpram_di;
  output [3:0] dpram_waddr;
  output dpram_wclk;
  output dpram_we;
  output dpram_mode;
parameter INIT_LUTF0 = 16'h0000 ;
parameter INIT_LUTG0 = 16'h0000 ;
parameter INIT_LUTF1 = 16'h0000 ;
parameter INIT_LUTG1 = 16'h0000 ;
parameter MODE = "LOGIC" ;
parameter GSR = "ENABLE" ;
parameter TESTMODE = "OFF";   //;
parameter CEMUX = "1" ;
parameter SRMUX = "SR" ; //
parameter CLKMUX = "CLK" ;
parameter SRMODE = "ASYNC" ;
parameter DFFMODE = "FF" ;
parameter REG0_SD = "MI" ;
parameter REG1_SD = "MI" ;
parameter REG0_REGSET = "SET" ;
parameter REG1_REGSET = "SET" ;
parameter DEMUX0 = "D" ;
parameter DEMUX1 = "D" ;
parameter CMIMUX0 = "C" ;
parameter CMIMUX1 = "C" ;
parameter LSFMUX0 = "LUTF";
parameter LSFXMUX0 = "LUTG";
parameter LSFMUX1 = "LUTF";
parameter LSFXMUX1 = "LUTG";
endmodule
//****************
//****************
module EF3_PHY_PLL(clkc,extlock,stdby,refclk,fbclk,reset,psdone,psclk,psdown,psstep,psclksel,
                  do,dclk,dcs,dwe,di,daddr,
                  dsm_refclk, dsm_rst, ssc_en, frac_offset_valid );

output [6:0] clkc;      //DEFAULT := (others => '0')
output extlock;         //DEFAULT := '0'
input  stdby;           //DEFAULT := '0'
input  refclk;          //DEFAULT := '0'
input  fbclk;           //DEFAULT := '0'
input  reset;           //DEFAULT := '0'
output psdone;          //DEFAULT := '0'
input  psclk;           //DEFAULT := '0'
input  psdown;          //DEFAULT := '0'
input  psstep;          //DEFAULT := '0'
input  [2:0] psclksel;  //DEFAULT := "000"
output [7:0]do;         //DEFAULT := (others => '0')
input  dclk;            //DEFAULT := '0'
input  dcs;             //DEFAULT := '0'
input  dwe;             //DEFAULT := '0'
input  [7:0] di;        //DEFAULT := (others => '0')
input  [5:0] daddr;     //DEFAULT := (others => '0')
input dsm_refclk;       //DEFAULT := '0'
input dsm_rst;          //DEFAULT := '0'
input ssc_en;           //DEFAULT := '0'
input frac_offset_valid;//DEFAULT := '0'
parameter DYNCFG = "DISABLE"; // Dynamic Phase Change InterFace DISABLE/ENABLE
parameter IF_ESCLKSTSW = "DISABLE"; // default Use dynamic clock switch to avoid glich, if_clk will delay 2 cycle after program done
parameter REFCLK_SEL = "INTERNAL"; // INTERNAL/EXTRNAL //
parameter FIN = "100.0000"; //no use in fact
parameter REFCLK_DIV    = 1;    //  1-128
parameter FBCLK_DIV     = 1;    //  1-128
parameter CLKC0_DIV     = 1;    //  1-128
parameter CLKC1_DIV     = 1;    //  1-128
parameter CLKC2_DIV     = 1;    //  1-128
parameter CLKC3_DIV     = 1;    //  1-128
parameter CLKC4_DIV     = 1;    //  1-128
parameter CLKC5_DIV     = 1;    //  1-128
parameter CLKC6_DIV     = 1;    //  1-128
parameter CLKC0_ENABLE  = "DISABLE";   //  ENABLE/DISABLE/SIGNAL
parameter CLKC1_ENABLE  = "DISABLE";   //  ENABLE/DISABLE/SIGNAL
parameter CLKC2_ENABLE  = "DISABLE";   //  ENABLE/DISABLE/SIGNAL
parameter CLKC3_ENABLE  = "DISABLE";   //  ENABLE/DISABLE/SIGNAL
parameter CLKC4_ENABLE  = "DISABLE";   //  ENABLE/DISABLE/SIGNAL
parameter CLKC5_ENABLE  = "DISABLE";   //  ENABLE/DISABLE/SIGNAL
parameter CLKC6_ENABLE  = "DISABLE";   //  ENABLE/DISABLE/SIGNAL
parameter CLKC0_DIV2_ENABLE = "DISABLE";   //  ENABLE/DISABLE;
parameter CLKC1_DIV2_ENABLE = "DISABLE";   //  ENABLE/DISABLE;
parameter CLKC2_DIV2_ENABLE = "DISABLE";   //  ENABLE/DISABLE;
parameter CLKC3_DIV2_ENABLE = "DISABLE";   //  ENABLE/DISABLE;
parameter CLKC4_DIV2_ENABLE = "DISABLE";   //  ENABLE/DISABLE;
parameter CLKC5_DIV2_ENABLE = "DISABLE";   //  ENABLE/DISABLE;
parameter CLKC6_DIV2_ENABLE = "DISABLE";   //  ENABLE/DISABLE;
parameter FEEDBK_MODE   = "NORMAL";    // NORMAL/SOURCESYNC/NOCOMP/ZERODELAY
parameter FEEDBK_PATH   = "VCO_PHASE_0";
parameter STDBY_ENABLE  = "ENABLE";   //  ENABLE/DISABLE
parameter CLKC0_FPHASE     = 0;    //  0-7
parameter CLKC1_FPHASE     = 0;    //  0-7
parameter CLKC2_FPHASE     = 0;    //  0-7
parameter CLKC3_FPHASE     = 0;    //  0-7
parameter CLKC4_FPHASE     = 0;    //  0-7
parameter CLKC0_CPHASE     = 1;      //  1-127  ,CLKC0_CPHASE <= CLKC0_DIV
parameter CLKC1_CPHASE     = 1;      //  1-127  ,CLKC1_CPHASE <= CLKC1_DIV
parameter CLKC2_CPHASE     = 1;      //  1-127  ,CLKC2_CPHASE <= CLKC2_DIV
parameter CLKC3_CPHASE     = 1;      //  1-127  ,CLKC3_CPHASE <= CLKC3_DIV
parameter CLKC4_CPHASE     = 1;      //  1-127  ,CLKC4_CPHASE <= CLKC4_DIV
parameter CLKC5_CPHASE     = 1;      //  1-127  ,CLKC3_CPHASE <= CLKC3_DIV
parameter CLKC6_CPHASE     = 1;      //  1-127  ,CLKC4_CPHASE <= CLKC4_DIV
parameter GMC_GAIN         = 7;  // 0-7
parameter GMC_TEST         = 14; // 0-15
parameter ICP_CURRENT      = 14;   // 1-32
parameter KVCO             = 7;  // 0-7
parameter LPF_CAPACITOR    = 3;  // 0-3
parameter LPF_RESISTOR     = 1;    // 1-128
parameter PLLRST_ENA       = "ENABLE";    // ENABLE/DISABLE
parameter PLLMRST_ENA      = "DISABLE";   // ENABLE/DISABLE
parameter PLLC2RST_ENA     = "DISABLE";   // ENABLE/DISABLE
parameter PLLC34RST_ENA    = "DISABLE";   // ENABLE/DISABLE
parameter PREDIV_MUXC0 = "VCO";    // VCO/CLKMINUS1/CLKPLUS1/REFCLK
parameter PREDIV_MUXC1 = "VCO";    // VCO/CLKMINUS1/SSCCLK/REFCLK
parameter PREDIV_MUXC2 = "VCO";    // VCO/CLKMINUS1/SSCCLK/REFCLK
parameter PREDIV_MUXC3 = "VCO";    // VCO/CLKMINUS1/SSCCLK/REFCLK
parameter PREDIV_MUXC4 = "VCO";    // VCO/CLKMINUS1/CLKPLUS1/REFCLK
parameter PREDIV_MUXC5 = "VCO";    // VCO/SSCCLK
parameter PREDIV_MUXC6 = "VCO";    // VCO/SSCCLK
parameter ODIV_MUXC0 = "DIV";    // DIV/REFCLK
parameter ODIV_MUXC1 = "DIV";    // DIV/REFCLK
parameter ODIV_MUXC2 = "DIV";    // DIV/REFCLK
parameter ODIV_MUXC3 = "DIV";    // DIV/REFCLK
parameter ODIV_MUXC4 = "DIV";    // DIV/REFCLK
parameter FREQ_LOCK_ACCURACY = 2; // 0-3
parameter PLL_LOCK_MODE = 0;    // 0-7, 2 => sticky, 0 => non-sticky
parameter INTFB_WAKE      = "DISABLE";  //  ENBALE/DISABLE
parameter DPHASE_SOURCE   = "DISABLE";  //  ENABLE/DISABLE
parameter VCO_NORESET     = "DISABLE";  //  ENABLE/DISABLE
parameter STDBY_VCO_ENA   = "DISABLE";  //  ENABLE/DISABLE
parameter NORESET         = "DISABLE";  //  ENABLE/DISABLE
parameter SYNC_ENABLE     = "ENABLE";   //  ENABLE/DISABLE
parameter DERIVE_PLL_CLOCKS = "DISABLE";  //  ENABLE/DISABLE
parameter GEN_BASIC_CLOCK   = "DISABLE";  //  ENABLE/DISABLE
parameter SSC_ENABLE        = "DISABLE";  //  ENBALE/DISABLE"
parameter SSC_MODE          = "Down";   //  1 => Down, 0 => Center
parameter SSC_AMP           = 0.0;      //  Center spread 0 -> 3%
parameter PU_INTP           = "DISABLE";  //  ENABLE/DISABLE, OR of SSC_EN | FRAC_EN
parameter INTPI             = 0;          //  0-6
parameter HIGH_SPEED_EN     = "DISABLE";  //  ENABLE/DISABLE
parameter SSC_FREQ_DIV      = 0;          //  [0, 2^15-1]
parameter SSC_RNGE          = 0;          //  [0, 2^11-1]
parameter FREQ_OFFSET    = 0.0;          //  +/-5%
parameter OFFSET_MODE    = "INT";  //  INT => CK_DIV4_OUT, EXT => FREQ_OFFSET_VALID from PIB
parameter FREQ_OFFSET_INT  = 0;          //  [0, 2^17-1]
parameter CLKC0_DUTY          = 0.0;       //  1/CLKC0_DIV * N, N is integer
parameter CLKC0_DUTY50        = "ENABLE";  //  ENABLE/DISABLE
parameter CLKC1_DUTY          = 0.0;       //  1/CLKC0_DIV * N, N is integer
parameter CLKC1_DUTY50        = "ENABLE";  //  ENABLE/DISABLE
parameter CLKC0_DUTY_INT      = 0;         //  1/CLKC0_DIV => (CLKC0_DIV-1)/CLKC0_DIV
parameter CLKC1_DUTY_INT      = 0;         //  1/CLKC1_DIV => (CLKC1_DIV-1)/CLKC1_DIV
endmodule
//****************
//****************
module EF3_LOGIC_BRAM (
  dia,
  addra, cea, ocea, clka, wea, rsta, bea,
  dib,
  addrb, ceb, oceb, clkb, web, rstb, beb,
  doa,
  dob);
  // 1kx9
  parameter DATA_WIDTH_A = 9;
  parameter DATA_WIDTH_B = DATA_WIDTH_A;
  parameter ADDR_WIDTH_A = 10;
  parameter ADDR_WIDTH_B = ADDR_WIDTH_A;
  parameter DATA_DEPTH_A = 2 ** ADDR_WIDTH_A;
  parameter DATA_DEPTH_B = 2 ** ADDR_WIDTH_B;
  parameter BYTE_ENABLE = 0;            // only interger {0, 8, 9} are valid
  parameter BYTE_A = BYTE_ENABLE == 0 ? 1 : DATA_WIDTH_A / BYTE_ENABLE;
  parameter BYTE_B = BYTE_ENABLE == 0 ? 1 : DATA_WIDTH_B / BYTE_ENABLE;
  output  [DATA_WIDTH_A-1:0] doa;
  output  [DATA_WIDTH_B-1:0] dob;
  input   [DATA_WIDTH_A-1:0] dia; //DEFAULT := (others => '0')
  input   [DATA_WIDTH_B-1:0] dib; //DEFAULT := (others => '0')
  input   cea, ocea, clka, wea, rsta; //DEFAULT := '0'
  input   ceb, oceb, clkb, web, rstb; //DEFAULT := '0'
  input [BYTE_A - 1 : 0] bea; //DEFAULT := (others => '0')
  input [BYTE_B - 1 : 0] beb; //DEFAULT := (others => '0')
  input   [ADDR_WIDTH_A-1:0] addra; //DEFAULT := (others => '0')
  input   [ADDR_WIDTH_B-1:0] addrb; //DEFAULT := (others => '0')
  parameter MODE = "DP";                 // DP,SP,PDPW,FIFO
  parameter REGMODE_A = "NOREG";         // "NOREG", "OUTREG"
  parameter REGMODE_B = "NOREG";         // "NOREG", "OUTREG"
  parameter WRITEMODE_A = "NORMAL";      // "NORMAL", "READBEFOREWRITE", "WRITETHROUGH"
  parameter WRITEMODE_B = "NORMAL";      // "NORMAL", "READBEFOREWRITE", "WRITETHROUGH"
  parameter RESETMODE = "SYNC";          // "SYNC", "ASYNC"
  parameter DEBUGGABLE = "NO";           // "YES", "NO"
  parameter PACKABLE = "NO";             // "YES", "NO"
  parameter FORCE_KEEP = "OFF";          // "ON", "OFF"
  parameter INIT_FILE = "NONE";
  parameter FILL_ALL = "NONE";          // "NONE" or some binary string like "0101", not wider than the wider port
  parameter IMPLEMENT = "9K";            // 9K | 9K(FAST)
endmodule      
//****************
//****************
module EF3_PHY_ADC(clk, pd, s, soc, eoc, dout);
input clk;          // clock
input pd;           // power Down signal.
input [2:0] s;      // channel Select Signal.
input soc;          // start of Conversion.
output eoc;         // end of Conversion.
output [11:0] dout; // AD conversion results
parameter TEMPERATURE = "DISABLE" ;
parameter CH0 = "DISABLE" ;
parameter CH1 = "DISABLE" ;
parameter CH2 = "DISABLE" ;
parameter CH3 = "DISABLE" ;
parameter CH4 = "DISABLE" ;
parameter CH5 = "DISABLE" ;
parameter CH6 = "DISABLE" ;
parameter CH7 = "DISABLE" ;
parameter VREF = "DISABLE" ;
endmodule
//****************
