------------------------------------------------
------------------------------------------------
--
--package EF3_COMPONENTS :
-- It should be compiled into ef3_macro library.
--
------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

package EF3_COMPONENTS is

component EF3_LOGIC_RAMFIFO is 
generic (
    DATA_WIDTH : integer := 8;
    ADDR_WIDTH : integer := 8;
    SHOWAHEAD : integer := 0;
    IMPLEMENT : string := "AUTO"
    );
port (
    rst : in std_logic ;
    di : in std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    clkr : in std_logic ;
    re : in std_logic ;
    clkw : in std_logic ;
    we : in std_logic ;
    do : out std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    empty_flag : out std_logic ;
    full_flag : out std_logic ;
    rdusedw : out std_logic_vector(ADDR_WIDTH DOWNTO 0) ;
    wrusedw : out std_logic_vector(ADDR_WIDTH DOWNTO 0) 
    );
end component;


component gray2bin is 
generic (
    WIDTH : integer := 8
    );
port (
    din : in std_logic_vector(WIDTH-1 DOWNTO 0) ;
    dout : out std_logic_vector(WIDTH-1 DOWNTO 0) 
    );
end component;


component gray_count is 
generic (
    WIDTH : integer := 8;
    INIT : integer := 0
    );
port (
    clk : in std_logic ;
    enable : in std_logic ;
    reset : in std_logic ;
    cnt : out std_logic_vector(WIDTH-1 DOWNTO 0) 
    );
end component;


component showahead_buffer is 
generic (
    DATA_WIDTH : integer := 8
    );
port (
    rstn_i : in std_logic ;
    clk_i : in std_logic ;
    ena_i : in std_logic ;
    rd_i : in std_logic ;
    data_i : in std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    data_o : out std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    addr_inc_o : out std_logic ;
    empty_o : out std_logic 
    );
end component;


component EF3_LOGIC_BUF is 
port (
    o : out std_logic ;
    i : in std_logic 
    );
end component;


component EF3_LOGIC_BUFG is 
port (
    o : out std_logic ;
    i : in std_logic 
    );
end component;


component EF3_LOGIC_BUFIO is 
generic (
    GSR : string := "DISABLE";
    DIV : integer := 2;
    STOPCLK : string := "DISABLE"
    );
port (
    clki : in std_logic ;
    rst : in std_logic ;
    coe : in std_logic ;
    clko : out std_logic ;
    clkdiv1 : out std_logic ;
    clkdivx : out std_logic 
    );
end component;


component EF3_LOGIC_BUFGMUX is 
generic (
    INIT_OUT : string := "0";
    PRESELECT_I0 : string := "TRUE";
    PRESELECT_I1 : string := "FALSE"
    );
port (
    o : out std_logic ;
    i0 : in std_logic ;
    i1 : in std_logic ;
    s : in std_logic 
    );
end component;


component EF3_LOGIC_MBOOT is 
generic (
    ADDR_SOURCE_SEL : string := "STATIC";
    STATIC_ADDR : string := B"00000000"
    );
port (
    rebootn : in std_logic ;
    dynamic_addr : in std_logic_vector(7 DOWNTO 0) 
    );
end component;


component EF3_LOGIC_DNA is 
generic (
    DNA : string := X"0000000000000000"
    );
port (
    dout : out std_logic ;
    clk : in std_logic ;
    din : in std_logic ;
    shift_en : in std_logic 
    );
end component;


component EF3_LOGIC_GCTRL is 
port (
    done : out std_logic ;
    highz : out std_logic 
    );
end component;


component EF3_LOGIC_GSRN is 
generic (
    GSRN_SYNC_SEL : string := "DISABLE";
    USR_GSRN_EN : string := "DISABLE"
    );
port (
    gsrn : in std_logic ;
    sync_clk : in std_logic 
    );
end component;


component EF3_LOGIC_CCLK is 
generic (
    FREQ : string := "4.5"
    );
port (
    cclk : out std_logic ;
    en : in std_logic 
    );
end component;


component EF3_LOGIC_JTAGEN is 
generic (
    LOCATION : string := ""
    );
port (
    jtagen : in std_logic 
    );
end component;


component EF3_LOGIC_IDDR is 
generic (
    ASYNCRST : string := "ENABLE";
    PIPEMODE : string := "PIPED"
    );
port (
    q1 : out std_logic ;
    q0 : out std_logic ;
    clk : in std_logic ;
    d : in std_logic ;
    rst : in std_logic 
    );
end component;


component EF3_LOGIC_ODDR is 
generic (
    ASYNCRST : string := "ENABLE"
    );
port (
    q : out std_logic ;
    clk : in std_logic ;
    d1 : in std_logic ;
    d0 : in std_logic ;
    rst : in std_logic 
    );
end component;


component EF3_LOGIC_IDDRx2 is 
generic (
    ASYNCRST : string := "ENABLE"
    );
port (
    q3 : out std_logic ;
    q2 : out std_logic ;
    q1 : out std_logic ;
    q0 : out std_logic ;
    pclk : in std_logic ;
    sclk : in std_logic ;
    d : in std_logic ;
    rst : in std_logic 
    );
end component;


component EF3_LOGIC_ODDRx2 is 
generic (
    ASYNCRST : string := "ENABLE"
    );
port (
    q : out std_logic ;
    pclk : in std_logic ;
    sclk : in std_logic ;
    d3 : in std_logic ;
    d2 : in std_logic ;
    d1 : in std_logic ;
    d0 : in std_logic ;
    rst : in std_logic 
    );
end component;


component EF3_LOGIC_ODDRx2l is 
generic (
    ASYNCRST : string := "ENABLE"
    );
port (
    q : out std_logic ;
    sclk : in std_logic ;
    d3 : in std_logic ;
    d2 : in std_logic ;
    d1 : in std_logic ;
    d0 : in std_logic ;
    rst : in std_logic 
    );
end component;


component EF3_LOGIC_FIFO is 
generic (
    DATA_WIDTH_W : integer := 9;
    DATA_WIDTH_R : integer := 9;
    DATA_DEPTH_W : integer := 1024;
    DATA_DEPTH_R : integer := 1024;
    MODE : string := "FIFO8K";
    REGMODE_W : string := "NOREG";
    REGMODE_R : string := "NOREG";
    E : integer := 0;
    AE : integer := 6;
    AF : integer := 1018;
    F : integer := 1024;
    GSR : string := "DISABLE";
    RESETMODE : string := "ASYNC";
    ASYNC_RESET_RELEASE : string := "SYNC";
    ENDIAN : string := "LITTLE"
    );
port (
    rst : in std_logic ;
    di : in std_logic_vector(DATA_WIDTH_W-1 DOWNTO 0) ;
    do : out std_logic_vector(DATA_WIDTH_R-1 DOWNTO 0) ;
    clkw, we : in std_logic ;
    clkr, re, ore : in std_logic := '0';
    csw, csr : in std_logic_vector(2 DOWNTO 0) ;
    empty_flag, aempty_flag : out std_logic := '0';
    full_flag, afull_flag : out std_logic := '0'
    );
end component;


component EF3_LOGIC_DRAM is 
generic (
    DATA_WIDTH_W : integer := 9;
    ADDR_WIDTH_W : integer := 10;
    DATA_DEPTH_W : integer := 1024;
    DATA_WIDTH_R : integer := 9;
    ADDR_WIDTH_R : integer := 10;
    DATA_DEPTH_R : integer := 1024;
    INIT_FILE : string := "NONE"
    );
port (
    di : in std_logic_vector(DATA_WIDTH_W-1 DOWNTO 0) ;
    waddr : in std_logic_vector(ADDR_WIDTH_W-1 DOWNTO 0) ;
    wclk : in std_logic ;
    we : in std_logic ;
    do : out std_logic_vector(DATA_WIDTH_R-1 DOWNTO 0) ;
    raddr : in std_logic_vector(ADDR_WIDTH_R-1 DOWNTO 0) 
    );
end component;


component EF3_LOGIC_DRAM16X4 is 
generic (
    INIT_D0 : string := X"0000";
    INIT_D1 : string := X"0000";
    INIT_D2 : string := X"0000";
    INIT_D3 : string := X"0000"
    );
port (
    di : in std_logic_vector(3 DOWNTO 0) ;
    waddr : in std_logic_vector(3 DOWNTO 0) ;
    wclk : in std_logic ;
    we : in std_logic ;
    raddr : in std_logic_vector(3 DOWNTO 0) ;
    do : out std_logic_vector(3 DOWNTO 0) 
    );
end component;


component EF3_LOGIC_MULT is 
generic (
    INPUT_WIDTH_A : integer := 18;
    INPUT_WIDTH_B : integer := 18;
    OUTPUT_WIDTH : integer := 36;
    INPUTFORMAT : string := "SIGNED";
    INPUTREGA : string := "ENABLE";
    INPUTREGB : string := "ENABLE";
    OUTPUTREG : string := "ENABLE";
    SRMODE : string := "ASYNC";
    IMPLEMENT : string := "AUTO"
    );
port (
    p : out std_logic_vector(OUTPUT_WIDTH-1 DOWNTO 0) ;
    a : in std_logic_vector(INPUT_WIDTH_A-1 DOWNTO 0) ;
    b : in std_logic_vector(INPUT_WIDTH_B-1 DOWNTO 0) ;
    cea : in std_logic := '0';
    ceb : in std_logic := '0';
    cepd : in std_logic := '0';
    clk : in std_logic := '0';
    rstan : in std_logic := '0';
    rstbn : in std_logic := '0';
    rstpdn : in std_logic := '0'
    );
end component;


component EF3_LOGIC_SEQ_DIV is 
generic (
    NUMER_WIDTH : integer := 16;
    DENOM_WIDTH : integer := 16
    );
port (
    clk : in std_logic ;
    rst : in std_logic ;
    start : in std_logic ;
    numer : in std_logic_vector(NUMER_WIDTH-1 DOWNTO 0) ;
    denom : in std_logic_vector(DENOM_WIDTH-1 DOWNTO 0) ;
    quotient : out std_logic_vector(NUMER_WIDTH-1 DOWNTO 0) ;
    remain : out std_logic_vector(DENOM_WIDTH-1 DOWNTO 0) ;
    done : out std_logic 
    );
end component;


component EF3_PHY_BRAM is 
generic (
    MODE : string := "DP8K";
    DATA_WIDTH_A : string := "9";
    DATA_WIDTH_B : string := "9";
    READBACK : string := "OFF";
    REGMODE_A : string := "NOREG";
    REGMODE_B : string := "NOREG";
    WRITEMODE_A : string := "NORMAL";
    WRITEMODE_B : string := "NORMAL";
    GSR : string := "ENABLE";
    RESETMODE : string := "SYNC";
    ASYNC_RESET_RELEASE : string := "SYNC";
    CEAMUX : string := "SIG";
    CEBMUX : string := "SIG";
    OCEAMUX : string := "SIG";
    OCEBMUX : string := "SIG";
    RSTAMUX : string := "SIG";
    RSTBMUX : string := "SIG";
    CLKAMUX : string := "SIG";
    CLKBMUX : string := "SIG";
    WEAMUX : string := "SIG";
    WEBMUX : string := "SIG";
    CSA0 : string := "SIG";
    CSA1 : string := "SIG";
    CSA2 : string := "SIG";
    CSB0 : string := "SIG";
    CSB1 : string := "SIG";
    CSB2 : string := "SIG";
    INIT_FILE : string := "NONE";
    INITP_00 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INITP_01 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INITP_02 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INITP_03 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_00 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_01 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_02 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_03 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_04 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_05 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_06 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_07 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_08 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_09 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0A : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0B : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0C : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0D : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0E : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0F : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_10 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_11 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_12 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_13 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_14 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_15 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_16 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_17 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_18 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_19 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1A : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1B : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1C : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1D : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1E : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1F : string := X"0000000000000000000000000000000000000000000000000000000000000000"
    );
port (
    doa : out std_logic_vector(8 DOWNTO 0) ;
    dob : out std_logic_vector(8 DOWNTO 0) ;
    dia : in std_logic_vector(8 DOWNTO 0) ;
    dib : in std_logic_vector(8 DOWNTO 0) ;
    csa : in std_logic_vector(2 DOWNTO 0) ;
    csb : in std_logic_vector(2 DOWNTO 0) ;
    cea, ocea, clka, wea, rsta : in std_logic ;
    ceb, oceb, clkb, web, rstb : in std_logic ;
    addra : in std_logic_vector(12 DOWNTO 0) ;
    addrb : in std_logic_vector(12 DOWNTO 0) 
    );
end component;


component EF3_PHY_FIFO is 
generic (
    MODE : string := "FIFO8K";
    DATA_WIDTH_A : string := "18";
    DATA_WIDTH_B : string := "18";
    READBACK : string := "OFF";
    REGMODE_A : string := "NOREG";
    REGMODE_B : string := "NOREG";
     AE : string := B"00000001100000";
     AF : string := B"01111110010000";
     F : string := B"01111111110000";
     AEP1 : string := B"00000001110000";
     AFM1 : string := B"01111110000000";
     FM1 : string := B"01111111100000";
     E : string := B"00000";
     EP1 : string := B"010000";
    GSR : string := "ENABLE";
    RESETMODE : string := "ASYNC";
    ASYNC_RESET_RELEASE : string := "SYNC";
    CEA : string := "SIG";
    CEB : string := "SIG";
    OCEA : string := "SIG";
    OCEB : string := "SIG";
    RSTA : string := "SIG";
    RSTB : string := "SIG";
    CLKA : string := "SIG";
    CLKB : string := "SIG";
    WEA : string := "SIG";
    WEB : string := "SIG";
    CSA0 : string := "SIG";
    CSA1 : string := "SIG";
    CSA2 : string := "SIG";
    CSB0 : string := "SIG";
    CSB1 : string := "SIG";
    CSB2 : string := "SIG"
    );
port (
    flag : out std_logic ;
    dia,dib : in std_logic_vector(8 DOWNTO 0) ;
    csr : in std_logic_vector(2 DOWNTO 0) ;
    csw : in std_logic_vector(2 DOWNTO 0) ;
    we, re, clkw, clkr, rst, rprst,orea,oreb : in std_logic ;
    dob,doa : out std_logic_vector(8 DOWNTO 0) ;
    empty_flag, aempty_flag, afull_flag, full_flag : out std_logic 
    );
end component;


component EF3_PHY_MULT18 is 
generic (
    INPUTREGA : string := "ENABLE";
    INPUTREGB : string := "ENABLE";
    OUTPUTREG : string := "ENABLE";
    SRMODE : string := "ASYNC";
    MODE : string := "MULT18X18C";
    CEAMUX : string := "SIG";
    CEBMUX : string := "SIG";
    CEPDMUX : string := "SIG";
    RSTANMUX : string := "SIG";
    RSTBNMUX : string := "SIG";
    RSTPDNMUX : string := "SIG";
    CLKMUX : string := "SIG";
    SIGNEDAMUX : string := "SIG";
    SIGNEDBMUX : string := "SIG";
    SOURCEAMUX : string := "SIG";
    SOURCEBMUX : string := "SIG"
    );
port (
    acout : out std_logic_vector(17 DOWNTO 0) ;
    bcout : out std_logic_vector(17 DOWNTO 0) ;
    p : out std_logic_vector(35 DOWNTO 0) ;
    signeda : in std_logic ;
    signedb : in std_logic ;
    a : in std_logic_vector(17 DOWNTO 0) ;
    b : in std_logic_vector(17 DOWNTO 0) ;
    acin : in std_logic_vector(17 DOWNTO 0) ;
    bcin : in std_logic_vector(17 DOWNTO 0) ;
    cea : in std_logic ;
    ceb : in std_logic ;
    cepd : in std_logic ;
    clk : in std_logic ;
    rstan : in std_logic ;
    rstbn : in std_logic ;
    rstpdn : in std_logic ;
    sourcea : in std_logic ;
    sourceb : in std_logic 
    );
end component;


component EF3_PHY_GCLK is 
port (
    clki : in std_logic ;
    clko : out std_logic 
    );
end component;


component EF3_PHY_IOCLK is 
generic (
    STOPCLK : string := "DISABLE"
    );
port (
    clki : in std_logic ;
    stop : in std_logic ;
    clko : out std_logic 
    );
end component;


component EF3_PHY_CLKDIV is 
generic (
    GSR : string := "DISABLE";
    DIV : integer := 2
    );
port (
    clkdiv1, clkdivx : out std_logic ;
    clki : in std_logic ;
    rst : in std_logic ;
    rls : in std_logic 
    );
end component;


component EF3_PHY_CONFIG is 
generic (
    DONE_PERSISTN : string := "ENABLE";
    INIT_PERSISTN : string := "ENABLE";
    PROGRAMN_PERSISTN : string := "DISABLE";
    JTAG_PERSISTN : string := "DISABLE";
    MBOOT_AUTO_SEL : string := "DISABLE";
    ADDR_SOURCE_SEL : string := "STATIC";
    STATIC_ADDR : string := B"00000000";
    GSRN_SYNC_SEL : string := "DISABLE";
    USR_GSRN_EN : string := "DISABLE";
    FREQ : string := "2.5"
    );
port (
    jrstn : out std_logic ;
    jrti : out std_logic_vector(1 DOWNTO 0) ;
    jshift : out std_logic ;
    jtck : out std_logic ;
    jtdi : out std_logic ;
    jupdate : out std_logic ;
    jscanen : out std_logic_vector(1 DOWNTO 0) ;
    jtms : out std_logic ;
    jtdo : in std_logic_vector(1 DOWNTO 0) ;
    jtag8_ipa : in std_logic_vector(7 DOWNTO 0) ;
    jtag8_ipb : in std_logic_vector(7 DOWNTO 0) ;
    mboot_rebootn : in std_logic ;
    mboot_dynamic_addr : in std_logic_vector(7 DOWNTO 0) ;
    dna_dout : out std_logic ;
    dna_clk : in std_logic ;
    dna_din : in std_logic ;
    dna_shift_en : in std_logic ;
    done : out std_logic ;
    highz : out std_logic ;
    usr_gsrn : in std_logic ;
    gsrn_sync_clk : in std_logic ;
    cclk : out std_logic ;
    cclk_en : in std_logic 
    );
end component;


component EF3_PHY_OSC is 
generic (
    STANDBY : string := "DISABLE"
    );
port (
    stdby : in std_logic ;
    osc : out std_logic 
    );
end component;


component EF3_PHY_OSCDIV is 
generic (
    half_cycle : real := 1.9
    );
port (
    rstn : in std_logic ;
    stdby : in std_logic ;
    div : in std_logic_vector(6 DOWNTO 0) ;
    clko : out std_logic 
    );
end component;


component EF3_PHY_PWRMNT is 
generic (
    MNT_LVL : integer := 0
    );
port (
    pwr_dwn_n : out std_logic ;
    sel_pwr : in std_logic ;
    pwr_mnt_pd : in std_logic 
    );
end component;


component EF3_PHY_MCU is 
generic (
    GPIO_L0 : string := "DISABLE";
    GPIO_L1 : string := "DISABLE";
    GPIO_L2 : string := "DISABLE";
    GPIO_L3 : string := "DISABLE";
    GPIO_L4 : string := "DISABLE";
    GPIO_L5 : string := "DISABLE";
    GPIO_L6 : string := "DISABLE";
    GPIO_L7 : string := "DISABLE";
    GPIO_L8 : string := "DISABLE";
    GPIO_L9 : string := "DISABLE";
    GPIO_L10 : string := "DISABLE";
    GPIO_L11 : string := "DISABLE";
    GPIO_L12 : string := "DISABLE";
    GPIO_L13 : string := "DISABLE";
    GPIO_L14 : string := "DISABLE";
    GPIO_L15 : string := "DISABLE"
    );
port (
    ppm_clk : in std_logic ;
    i2s_mst_clk : in std_logic ;
    ppm_rstn : in std_logic ;
    sleep_out : out std_logic ;
    gpio_h_out : out std_logic_vector(15 DOWNTO 0) ;
    gpio_h_oe_n : out std_logic_vector(15 DOWNTO 0) ;
    gpio_h_in : in std_logic_vector(15 DOWNTO 0) ;
    ahbslave_dma_req : in std_logic ;
    ahbslave_dma_single : in std_logic ;
    ahbslave_dma_ack : out std_logic ;
    h2h_haddr : out std_logic_vector(31 DOWNTO 0) ;
    h2h_htrans : out std_logic_vector(1 DOWNTO 0) ;
    h2h_hwrite : out std_logic ;
    h2h_hsize : out std_logic_vector(2 DOWNTO 0) ;
    h2h_hburst : out std_logic_vector(2 DOWNTO 0) ;
    h2h_hprot : out std_logic_vector(3 DOWNTO 0) ;
    h2h_hwdata : out std_logic_vector(31 DOWNTO 0) ;
    h2h_hrdata : in std_logic_vector(31 DOWNTO 0) ;
    h2h_hreadyout : in std_logic ;
    h2h_hresp : in std_logic_vector(1 DOWNTO 0) ;
    h2h_mclk : in std_logic ;
    h2h_rstn : in std_logic ;
    qspi_rxd : in std_logic_vector(3 DOWNTO 0) ;
    qspi_txd : out std_logic_vector(3 DOWNTO 0) ;
    qspi_sclk : out std_logic ;
    qspi_ss : out std_logic ;
    qspi_oen : out std_logic_vector(3 DOWNTO 0) ;
    adc1_exttrg : in std_logic ;
    adc2_exttrg : in std_logic ;
    usr_reg : out std_logic_vector(15 DOWNTO 0) 
    );
end component;


component EF3_PHY_PAD is 
generic (
    DEDCLK : string := "DISABLE";
    GSR : string := "ENABLE";
    SRMODE : string := "SYNC";
    TSMUX : string := "1";
    INSCLKMUX : string := "0";
    INPCLKMUX : string := "CLK";
    INCEMUX : string := "CE";
    INRSTMUX : string := "0";
    IN_REGSET : string := "RESET";
    IN_DFFMODE : string := "NONE";
    IDDRMODE : string := "OFF";
    IDDRPIPEMODE : string := "NONE";
    INDELMUX : string := "NODEL";
    INDEL : integer := 0;
    OUTSCLKMUX : string := "0";
    OUTPCLKMUX : string := "CLK";
    OUTCEMUX : string := "CE";
    OUTRSTMUX : string := "0";
    DO_REGSET : string := "RESET";
    DO_DFFMODE : string := "NONE";
    ODDRMODE : string := "OFF";
    OUTDELMUX : string := "NODEL";
    OUTDEL : integer := 0;
    TO_REGSET : string := "RESET";
    TO_DFFMODE : string := "NONE";
    MODE : string := "IN";
    DRIVE : string := "NONE";
    IOTYPE : string := "LVCMOS25"
    );
port (
    ipad : in std_logic ;
    opad : out std_logic ;
    bpad : inout std_logic ;
    rst, ce, isclk, ipclk, osclk, opclk : in std_logic ;
    ts : in std_logic ;
    do : in std_logic_vector(3 DOWNTO 0) ;
    di : out std_logic ;
    diq : out std_logic_vector(3 DOWNTO 0) ;
    lp_rx_en : in std_logic ;
    lp_dir : in std_logic ;
    lp_dout : in std_logic ;
    dynres : in std_logic ;
    vcmpd_n : in std_logic ;
    lp_din : out std_logic 
    );
end component;


component EF3_PHY_SPAD is 
generic (
    DEDCLK : string := "DISABLE";
    GSR : string := "ENABLE";
    SRMODE : string := "SYNC";
    TSMUX : string := "TS";
    INCLKMUX : string := "0";
    INCEMUX : string := "CE";
    INRSTMUX : string := "RST";
    IDDRMODE : string := "OFF";
    INDELMUX : string := "NODEL";
    INDEL : integer := 0;
    OUTCLKMUX : string := "0";
    OUTCEMUX : string := "CE";
    OUTRSTMUX : string := "RST";
    ODDRMODE : string := "OFF";
    IN_DFFMODE : string := "NONE";
    IN_REGSET : string := "RESET";
    DO_DFFMODE : string := "NONE";
    DO_REGSET : string := "RESET";
    TO_DFFMODE : string := "NONE";
    TO_REGSET : string := "RESET";
    MODE : string := "IN";
    DRIVE : string := "NONE";
    IOTYPE : string := "LVCMOS25"
    );
port (
    ipad : in std_logic ;
    opad : out std_logic ;
    bpad : inout std_logic ;
    rst, ce, clk : in std_logic ;
    ts : in std_logic ;
    do : in std_logic_vector(1 DOWNTO 0) ;
    di : out std_logic ;
    diq : out std_logic_vector(1 DOWNTO 0) 
    );
end component;


component EF3_PHY_CSB is 
generic (
    HOLD : string := "YES";
    INITOUT : string := "0";
    PRESELECT : string := "NONE";
    TESTMODE : string := "DISABLE"
    );
port (
    clko : out std_logic ;
    ce : in std_logic_vector(1 DOWNTO 0) ;
    clki : in std_logic_vector(1 DOWNTO 0) ;
    drct : in std_logic_vector(1 DOWNTO 0) ;
    sel : in std_logic_vector(1 DOWNTO 0) ;
    ignr : in std_logic_vector(1 DOWNTO 0) 
    );
end component;


component EF3_PHY_INTFLASH is 
port (
    mosi_io0 : inout std_logic ;
    miso_io1 : inout std_logic ;
    wp_n_io2 : inout std_logic ;
    hold_n_io3 : inout std_logic ;
    cclk : in std_logic ;
    cs_n : in std_logic 
    );
end component;


component EF3_PHY_MSLICE is 
generic (
    INIT_LUT0 : string := X"0000";
    INIT_LUT1 : string := X"0000";
    MODE : string := "LOGIC";
    ALUTYPE : string := "";
    MSFXMUX : string := "OFF";
    GSR : string := "ENABLE";
    TESTMODE : string := "OFF";
    CEMUX : string := "CE";
    SRMUX : string := "SR";
    CLKMUX : string := "CLK";
    SRMODE : string := "ASYNC";
    DFFMODE : string := "FF";
    REG0_SD : string := "MI";
    REG1_SD : string := "MI";
    REG0_REGSET : string := "SET";
    REG1_REGSET : string := "SET"
    );
port (
    a,b,c,d,mi : in std_logic_vector(1 DOWNTO 0) ;
    clk : in std_logic ;
    ce : in std_logic ;
    sr : in std_logic ;
    fci : in std_logic ;
    f,fx,q : out std_logic_vector(1 DOWNTO 0) ;
    fco : out std_logic ;
    dpram_mode : in std_logic ;
    dpram_di : in std_logic_vector(1 DOWNTO 0) ;
    dpram_we : in std_logic ;
    dpram_wclk : in std_logic ;
    dpram_waddr : in std_logic_vector(3 DOWNTO 0) 
    );
end component;


component EF3_PHY_LSLICE is 
generic (
    INIT_LUTF0 : string := X"0000";
    INIT_LUTG0 : string := X"0000";
    INIT_LUTF1 : string := X"0000";
    INIT_LUTG1 : string := X"0000";
    MODE : string := "LOGIC";
    GSR : string := "ENABLE";
    TESTMODE : string := "OFF";
    CEMUX : string := "1";
    SRMUX : string := "SR";
    CLKMUX : string := "CLK";
    SRMODE : string := "ASYNC";
    DFFMODE : string := "FF";
    REG0_SD : string := "MI";
    REG1_SD : string := "MI";
    REG0_REGSET : string := "SET";
    REG1_REGSET : string := "SET";
    DEMUX0 : string := "D";
    DEMUX1 : string := "D";
    CMIMUX0 : string := "C";
    CMIMUX1 : string := "C";
    LSFMUX0 : string := "LUTF";
    LSFXMUX0 : string := "LUTG";
    LSFMUX1 : string := "LUTF";
    LSFXMUX1 : string := "LUTG"
    );
port (
    a,b,c,d,e,mi : in std_logic_vector(1 DOWNTO 0) ;
    clk : in std_logic ;
    ce : in std_logic ;
    sr : in std_logic ;
    fci : in std_logic ;
    f,fx,q : out std_logic_vector(1 DOWNTO 0) ;
    fco : out std_logic ;
    dpram_di : out std_logic_vector(3 DOWNTO 0) ;
    dpram_waddr : out std_logic_vector(3 DOWNTO 0) ;
    dpram_wclk : out std_logic ;
    dpram_we : out std_logic ;
    dpram_mode : out std_logic 
    );
end component;


component EF3_PHY_PLL is 
generic (
    DYNCFG : string := "DISABLE";
    IF_ESCLKSTSW : string := "DISABLE";
    REFCLK_SEL : string := "INTERNAL";
    FIN : string := "100.0000";
    REFCLK_DIV : integer := 1;
    FBCLK_DIV : integer := 1;
    CLKC0_DIV : integer := 1;
    CLKC1_DIV : integer := 1;
    CLKC2_DIV : integer := 1;
    CLKC3_DIV : integer := 1;
    CLKC4_DIV : integer := 1;
    CLKC5_DIV : integer := 1;
    CLKC6_DIV : integer := 1;
    CLKC0_ENABLE : string := "DISABLE";
    CLKC1_ENABLE : string := "DISABLE";
    CLKC2_ENABLE : string := "DISABLE";
    CLKC3_ENABLE : string := "DISABLE";
    CLKC4_ENABLE : string := "DISABLE";
    CLKC5_ENABLE : string := "DISABLE";
    CLKC6_ENABLE : string := "DISABLE";
    CLKC0_DIV2_ENABLE : string := "DISABLE";
    CLKC1_DIV2_ENABLE : string := "DISABLE";
    CLKC2_DIV2_ENABLE : string := "DISABLE";
    CLKC3_DIV2_ENABLE : string := "DISABLE";
    CLKC4_DIV2_ENABLE : string := "DISABLE";
    CLKC5_DIV2_ENABLE : string := "DISABLE";
    CLKC6_DIV2_ENABLE : string := "DISABLE";
    FEEDBK_MODE : string := "NORMAL";
    FEEDBK_PATH : string := "VCO_PHASE_0";
    STDBY_ENABLE : string := "ENABLE";
    CLKC0_FPHASE : integer := 0;
    CLKC1_FPHASE : integer := 0;
    CLKC2_FPHASE : integer := 0;
    CLKC3_FPHASE : integer := 0;
    CLKC4_FPHASE : integer := 0;
    CLKC0_CPHASE : integer := 1;
    CLKC1_CPHASE : integer := 1;
    CLKC2_CPHASE : integer := 1;
    CLKC3_CPHASE : integer := 1;
    CLKC4_CPHASE : integer := 1;
    CLKC5_CPHASE : integer := 1;
    CLKC6_CPHASE : integer := 1;
    GMC_GAIN : integer := 7;
    GMC_TEST : integer := 14;
    ICP_CURRENT : integer := 14;
    KVCO : integer := 7;
    LPF_CAPACITOR : integer := 3;
    LPF_RESISTOR : integer := 1;
    PLLRST_ENA : string := "ENABLE";
    PLLMRST_ENA : string := "DISABLE";
    PLLC2RST_ENA : string := "DISABLE";
    PLLC34RST_ENA : string := "DISABLE";
    PREDIV_MUXC0 : string := "VCO";
    PREDIV_MUXC1 : string := "VCO";
    PREDIV_MUXC2 : string := "VCO";
    PREDIV_MUXC3 : string := "VCO";
    PREDIV_MUXC4 : string := "VCO";
    PREDIV_MUXC5 : string := "VCO";
    PREDIV_MUXC6 : string := "VCO";
    ODIV_MUXC0 : string := "DIV";
    ODIV_MUXC1 : string := "DIV";
    ODIV_MUXC2 : string := "DIV";
    ODIV_MUXC3 : string := "DIV";
    ODIV_MUXC4 : string := "DIV";
    FREQ_LOCK_ACCURACY : integer := 2;
    PLL_LOCK_MODE : integer := 0;
    INTFB_WAKE : string := "DISABLE";
    DPHASE_SOURCE : string := "DISABLE";
    VCO_NORESET : string := "DISABLE";
    STDBY_VCO_ENA : string := "DISABLE";
    NORESET : string := "DISABLE";
    SYNC_ENABLE : string := "ENABLE";
    DERIVE_PLL_CLOCKS : string := "DISABLE";
    GEN_BASIC_CLOCK : string := "DISABLE";
    SSC_ENABLE : string := "DISABLE";
    SSC_MODE : string := "Down";
    SSC_AMP : real := 0.0;
    PU_INTP : string := "DISABLE";
    INTPI : integer := 0;
    HIGH_SPEED_EN : string := "DISABLE";
    SSC_FREQ_DIV : integer := 0;
    SSC_RNGE : integer := 0;
    FREQ_OFFSET : real := 0.0;
    OFFSET_MODE : string := "INT";
    FREQ_OFFSET_INT : integer := 0;
    CLKC0_DUTY : real := 0.0;
    CLKC0_DUTY50 : string := "ENABLE";
    CLKC1_DUTY : real := 0.0;
    CLKC1_DUTY50 : string := "ENABLE";
    CLKC0_DUTY_INT : integer := 0;
    CLKC1_DUTY_INT : integer := 0
    );
port (
    clkc : out std_logic_vector(6 DOWNTO 0) := (others => '0');
    extlock : out std_logic := '0';
    stdby : in std_logic := '0';
    refclk : in std_logic := '0';
    fbclk : in std_logic := '0';
    reset : in std_logic := '0';
    psdone : out std_logic := '0';
    psclk : in std_logic := '0';
    psdown : in std_logic := '0';
    psstep : in std_logic := '0';
    psclksel : in std_logic_vector(2 DOWNTO 0) := "000";
    do : out std_logic_vector(7 DOWNTO 0) := (others => '0');
    dclk : in std_logic := '0';
    dcs : in std_logic := '0';
    dwe : in std_logic := '0';
    di : in std_logic_vector(7 DOWNTO 0) := (others => '0');
    daddr : in std_logic_vector(5 DOWNTO 0) := (others => '0');
    dsm_refclk : in std_logic := '0';
    dsm_rst : in std_logic := '0';
    ssc_en : in std_logic := '0';
    frac_offset_valid : in std_logic := '0'
    );
end component;


component EF3_LOGIC_BRAM is 
generic (
    DATA_WIDTH_A : integer := 9;
    DATA_WIDTH_B : integer := 9;
    ADDR_WIDTH_A : integer := 10;
    ADDR_WIDTH_B : integer := 10;
    DATA_DEPTH_A : integer := 1024;
    DATA_DEPTH_B : integer := 1024;
    BYTE_ENABLE : integer := 0;
    BYTE_A : integer := 1;
    BYTE_B : integer := 1;
    MODE : string := "DP";
    REGMODE_A : string := "NOREG";
    REGMODE_B : string := "NOREG";
    WRITEMODE_A : string := "NORMAL";
    WRITEMODE_B : string := "NORMAL";
    RESETMODE : string := "SYNC";
    DEBUGGABLE : string := "NO";
    PACKABLE : string := "NO";
    FORCE_KEEP : string := "OFF";
    INIT_FILE : string := "NONE";
    FILL_ALL : string := "NONE";
    IMPLEMENT : string := "9K"
    );
port (
    doa : out std_logic_vector(DATA_WIDTH_A-1 DOWNTO 0) ;
    dob : out std_logic_vector(DATA_WIDTH_B-1 DOWNTO 0) ;
    dia : in std_logic_vector(DATA_WIDTH_A-1 DOWNTO 0) := (others => '0');
    dib : in std_logic_vector(DATA_WIDTH_B-1 DOWNTO 0) := (others => '0');
    cea, ocea, clka, wea, rsta : in std_logic := '0';
    ceb, oceb, clkb, web, rstb : in std_logic := '0';
    bea : in std_logic_vector(BYTE_A - 1  DOWNTO 0) := (others => '0');
    beb : in std_logic_vector(BYTE_B - 1  DOWNTO 0) := (others => '0');
    addra : in std_logic_vector(ADDR_WIDTH_A-1 DOWNTO 0) := (others => '0');
    addrb : in std_logic_vector(ADDR_WIDTH_B-1 DOWNTO 0) := (others => '0')
    );
end component;


component EF3_PHY_ADC is 
generic (
    TEMPERATURE : string := "DISABLE";
    CH0 : string := "DISABLE";
    CH1 : string := "DISABLE";
    CH2 : string := "DISABLE";
    CH3 : string := "DISABLE";
    CH4 : string := "DISABLE";
    CH5 : string := "DISABLE";
    CH6 : string := "DISABLE";
    CH7 : string := "DISABLE";
    VREF : string := "DISABLE"
    );
port (
    clk : in std_logic ;
    pd : in std_logic ;
    s : in std_logic_vector(2 DOWNTO 0) ;
    soc : in std_logic ;
    eoc : out std_logic ;
    dout : out std_logic_vector(11 DOWNTO 0) 
    );
end component;


end EF3_COMPONENTS;
------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_RAMFIFO is
generic (
    DATA_WIDTH : integer := 8;
    ADDR_WIDTH : integer := 8;
    SHOWAHEAD : integer := 0;
    IMPLEMENT : string := "AUTO"
    );
port (
    rst : in std_logic ;
    di : in std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    clkr : in std_logic ;
    re : in std_logic ;
    clkw : in std_logic ;
    we : in std_logic ;
    do : out std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    empty_flag : out std_logic ;
    full_flag : out std_logic ;
    rdusedw : out std_logic_vector(ADDR_WIDTH DOWNTO 0) ;
    wrusedw : out std_logic_vector(ADDR_WIDTH DOWNTO 0) 
    );
end entity;

architecture rtl of EF3_LOGIC_RAMFIFO is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity gray2bin is
generic (
    WIDTH : integer := 8
    );
port (
    din : in std_logic_vector(WIDTH-1 DOWNTO 0) ;
    dout : out std_logic_vector(WIDTH-1 DOWNTO 0) 
    );
end entity;

architecture rtl of gray2bin is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity gray_count is
generic (
    WIDTH : integer := 8;
    INIT : integer := 0
    );
port (
    clk : in std_logic ;
    enable : in std_logic ;
    reset : in std_logic ;
    cnt : out std_logic_vector(WIDTH-1 DOWNTO 0) 
    );
end entity;

architecture rtl of gray_count is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity showahead_buffer is
generic (
    DATA_WIDTH : integer := 8
    );
port (
    rstn_i : in std_logic ;
    clk_i : in std_logic ;
    ena_i : in std_logic ;
    rd_i : in std_logic ;
    data_i : in std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    data_o : out std_logic_vector(DATA_WIDTH-1 DOWNTO 0) ;
    addr_inc_o : out std_logic ;
    empty_o : out std_logic 
    );
end entity;

architecture rtl of showahead_buffer is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_BUF is
port (
    o : out std_logic ;
    i : in std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_BUF is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_BUFG is
port (
    o : out std_logic ;
    i : in std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_BUFG is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_BUFIO is
generic (
    GSR : string := "DISABLE";
    DIV : integer := 2;
    STOPCLK : string := "DISABLE"
    );
port (
    clki : in std_logic ;
    rst : in std_logic ;
    coe : in std_logic ;
    clko : out std_logic ;
    clkdiv1 : out std_logic ;
    clkdivx : out std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_BUFIO is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_BUFGMUX is
generic (
    INIT_OUT : string := "0";
    PRESELECT_I0 : string := "TRUE";
    PRESELECT_I1 : string := "FALSE"
    );
port (
    o : out std_logic ;
    i0 : in std_logic ;
    i1 : in std_logic ;
    s : in std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_BUFGMUX is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_MBOOT is
generic (
    ADDR_SOURCE_SEL : string := "STATIC";
    STATIC_ADDR : string := B"00000000"
    );
port (
    rebootn : in std_logic ;
    dynamic_addr : in std_logic_vector(7 DOWNTO 0) 
    );
end entity;

architecture rtl of EF3_LOGIC_MBOOT is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_DNA is
generic (
    DNA : string := X"0000000000000000"
    );
port (
    dout : out std_logic ;
    clk : in std_logic ;
    din : in std_logic ;
    shift_en : in std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_DNA is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_GCTRL is
port (
    done : out std_logic ;
    highz : out std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_GCTRL is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_GSRN is
generic (
    GSRN_SYNC_SEL : string := "DISABLE";
    USR_GSRN_EN : string := "DISABLE"
    );
port (
    gsrn : in std_logic ;
    sync_clk : in std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_GSRN is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_CCLK is
generic (
    FREQ : string := "4.5"
    );
port (
    cclk : out std_logic ;
    en : in std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_CCLK is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_JTAGEN is
generic (
    LOCATION : string := ""
    );
port (
    jtagen : in std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_JTAGEN is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_IDDR is
generic (
    ASYNCRST : string := "ENABLE";
    PIPEMODE : string := "PIPED"
    );
port (
    q1 : out std_logic ;
    q0 : out std_logic ;
    clk : in std_logic ;
    d : in std_logic ;
    rst : in std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_IDDR is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_ODDR is
generic (
    ASYNCRST : string := "ENABLE"
    );
port (
    q : out std_logic ;
    clk : in std_logic ;
    d1 : in std_logic ;
    d0 : in std_logic ;
    rst : in std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_ODDR is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_IDDRx2 is
generic (
    ASYNCRST : string := "ENABLE"
    );
port (
    q3 : out std_logic ;
    q2 : out std_logic ;
    q1 : out std_logic ;
    q0 : out std_logic ;
    pclk : in std_logic ;
    sclk : in std_logic ;
    d : in std_logic ;
    rst : in std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_IDDRx2 is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_ODDRx2 is
generic (
    ASYNCRST : string := "ENABLE"
    );
port (
    q : out std_logic ;
    pclk : in std_logic ;
    sclk : in std_logic ;
    d3 : in std_logic ;
    d2 : in std_logic ;
    d1 : in std_logic ;
    d0 : in std_logic ;
    rst : in std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_ODDRx2 is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_ODDRx2l is
generic (
    ASYNCRST : string := "ENABLE"
    );
port (
    q : out std_logic ;
    sclk : in std_logic ;
    d3 : in std_logic ;
    d2 : in std_logic ;
    d1 : in std_logic ;
    d0 : in std_logic ;
    rst : in std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_ODDRx2l is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_FIFO is
generic (
    DATA_WIDTH_W : integer := 9;
    DATA_WIDTH_R : integer := 9;
    DATA_DEPTH_W : integer := 1024;
    DATA_DEPTH_R : integer := 1024;
    MODE : string := "FIFO8K";
    REGMODE_W : string := "NOREG";
    REGMODE_R : string := "NOREG";
    E : integer := 0;
    AE : integer := 6;
    AF : integer := 1018;
    F : integer := 1024;
    GSR : string := "DISABLE";
    RESETMODE : string := "ASYNC";
    ASYNC_RESET_RELEASE : string := "SYNC";
    ENDIAN : string := "LITTLE"
    );
port (
    rst : in std_logic ;
    di : in std_logic_vector(DATA_WIDTH_W-1 DOWNTO 0) ;
    do : out std_logic_vector(DATA_WIDTH_R-1 DOWNTO 0) ;
    clkw, we : in std_logic ;
    clkr, re, ore : in std_logic := '0';
    csw, csr : in std_logic_vector(2 DOWNTO 0) ;
    empty_flag, aempty_flag : out std_logic := '0';
    full_flag, afull_flag : out std_logic := '0'
    );
end entity;

architecture rtl of EF3_LOGIC_FIFO is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_DRAM is
generic (
    DATA_WIDTH_W : integer := 9;
    ADDR_WIDTH_W : integer := 10;
    DATA_DEPTH_W : integer := 1024;
    DATA_WIDTH_R : integer := 9;
    ADDR_WIDTH_R : integer := 10;
    DATA_DEPTH_R : integer := 1024;
    INIT_FILE : string := "NONE"
    );
port (
    di : in std_logic_vector(DATA_WIDTH_W-1 DOWNTO 0) ;
    waddr : in std_logic_vector(ADDR_WIDTH_W-1 DOWNTO 0) ;
    wclk : in std_logic ;
    we : in std_logic ;
    do : out std_logic_vector(DATA_WIDTH_R-1 DOWNTO 0) ;
    raddr : in std_logic_vector(ADDR_WIDTH_R-1 DOWNTO 0) 
    );
end entity;

architecture rtl of EF3_LOGIC_DRAM is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_DRAM16X4 is
generic (
    INIT_D0 : string := X"0000";
    INIT_D1 : string := X"0000";
    INIT_D2 : string := X"0000";
    INIT_D3 : string := X"0000"
    );
port (
    di : in std_logic_vector(3 DOWNTO 0) ;
    waddr : in std_logic_vector(3 DOWNTO 0) ;
    wclk : in std_logic ;
    we : in std_logic ;
    raddr : in std_logic_vector(3 DOWNTO 0) ;
    do : out std_logic_vector(3 DOWNTO 0) 
    );
end entity;

architecture rtl of EF3_LOGIC_DRAM16X4 is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_MULT is
generic (
    INPUT_WIDTH_A : integer := 18;
    INPUT_WIDTH_B : integer := 18;
    OUTPUT_WIDTH : integer := 36;
    INPUTFORMAT : string := "SIGNED";
    INPUTREGA : string := "ENABLE";
    INPUTREGB : string := "ENABLE";
    OUTPUTREG : string := "ENABLE";
    SRMODE : string := "ASYNC";
    IMPLEMENT : string := "AUTO"
    );
port (
    p : out std_logic_vector(OUTPUT_WIDTH-1 DOWNTO 0) ;
    a : in std_logic_vector(INPUT_WIDTH_A-1 DOWNTO 0) ;
    b : in std_logic_vector(INPUT_WIDTH_B-1 DOWNTO 0) ;
    cea : in std_logic := '0';
    ceb : in std_logic := '0';
    cepd : in std_logic := '0';
    clk : in std_logic := '0';
    rstan : in std_logic := '0';
    rstbn : in std_logic := '0';
    rstpdn : in std_logic := '0'
    );
end entity;

architecture rtl of EF3_LOGIC_MULT is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_SEQ_DIV is
generic (
    NUMER_WIDTH : integer := 16;
    DENOM_WIDTH : integer := 16
    );
port (
    clk : in std_logic ;
    rst : in std_logic ;
    start : in std_logic ;
    numer : in std_logic_vector(NUMER_WIDTH-1 DOWNTO 0) ;
    denom : in std_logic_vector(DENOM_WIDTH-1 DOWNTO 0) ;
    quotient : out std_logic_vector(NUMER_WIDTH-1 DOWNTO 0) ;
    remain : out std_logic_vector(DENOM_WIDTH-1 DOWNTO 0) ;
    done : out std_logic 
    );
end entity;

architecture rtl of EF3_LOGIC_SEQ_DIV is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_BRAM is
generic (
    MODE : string := "DP8K";
    DATA_WIDTH_A : string := "9";
    DATA_WIDTH_B : string := "9";
    READBACK : string := "OFF";
    REGMODE_A : string := "NOREG";
    REGMODE_B : string := "NOREG";
    WRITEMODE_A : string := "NORMAL";
    WRITEMODE_B : string := "NORMAL";
    GSR : string := "ENABLE";
    RESETMODE : string := "SYNC";
    ASYNC_RESET_RELEASE : string := "SYNC";
    CEAMUX : string := "SIG";
    CEBMUX : string := "SIG";
    OCEAMUX : string := "SIG";
    OCEBMUX : string := "SIG";
    RSTAMUX : string := "SIG";
    RSTBMUX : string := "SIG";
    CLKAMUX : string := "SIG";
    CLKBMUX : string := "SIG";
    WEAMUX : string := "SIG";
    WEBMUX : string := "SIG";
    CSA0 : string := "SIG";
    CSA1 : string := "SIG";
    CSA2 : string := "SIG";
    CSB0 : string := "SIG";
    CSB1 : string := "SIG";
    CSB2 : string := "SIG";
    INIT_FILE : string := "NONE";
    INITP_00 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INITP_01 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INITP_02 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INITP_03 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_00 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_01 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_02 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_03 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_04 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_05 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_06 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_07 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_08 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_09 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0A : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0B : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0C : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0D : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0E : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_0F : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_10 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_11 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_12 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_13 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_14 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_15 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_16 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_17 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_18 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_19 : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1A : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1B : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1C : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1D : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1E : string := X"0000000000000000000000000000000000000000000000000000000000000000";
    INIT_1F : string := X"0000000000000000000000000000000000000000000000000000000000000000"
    );
port (
    doa : out std_logic_vector(8 DOWNTO 0) ;
    dob : out std_logic_vector(8 DOWNTO 0) ;
    dia : in std_logic_vector(8 DOWNTO 0) ;
    dib : in std_logic_vector(8 DOWNTO 0) ;
    csa : in std_logic_vector(2 DOWNTO 0) ;
    csb : in std_logic_vector(2 DOWNTO 0) ;
    cea, ocea, clka, wea, rsta : in std_logic ;
    ceb, oceb, clkb, web, rstb : in std_logic ;
    addra : in std_logic_vector(12 DOWNTO 0) ;
    addrb : in std_logic_vector(12 DOWNTO 0) 
    );
end entity;

architecture rtl of EF3_PHY_BRAM is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_FIFO is
generic (
    MODE : string := "FIFO8K";
    DATA_WIDTH_A : string := "18";
    DATA_WIDTH_B : string := "18";
    READBACK : string := "OFF";
    REGMODE_A : string := "NOREG";
    REGMODE_B : string := "NOREG";
     AE : string := B"00000001100000";
     AF : string := B"01111110010000";
     F : string := B"01111111110000";
     AEP1 : string := B"00000001110000";
     AFM1 : string := B"01111110000000";
     FM1 : string := B"01111111100000";
     E : string := B"00000";
     EP1 : string := B"010000";
    GSR : string := "ENABLE";
    RESETMODE : string := "ASYNC";
    ASYNC_RESET_RELEASE : string := "SYNC";
    CEA : string := "SIG";
    CEB : string := "SIG";
    OCEA : string := "SIG";
    OCEB : string := "SIG";
    RSTA : string := "SIG";
    RSTB : string := "SIG";
    CLKA : string := "SIG";
    CLKB : string := "SIG";
    WEA : string := "SIG";
    WEB : string := "SIG";
    CSA0 : string := "SIG";
    CSA1 : string := "SIG";
    CSA2 : string := "SIG";
    CSB0 : string := "SIG";
    CSB1 : string := "SIG";
    CSB2 : string := "SIG"
    );
port (
    flag : out std_logic ;
    dia,dib : in std_logic_vector(8 DOWNTO 0) ;
    csr : in std_logic_vector(2 DOWNTO 0) ;
    csw : in std_logic_vector(2 DOWNTO 0) ;
    we, re, clkw, clkr, rst, rprst,orea,oreb : in std_logic ;
    dob,doa : out std_logic_vector(8 DOWNTO 0) ;
    empty_flag, aempty_flag, afull_flag, full_flag : out std_logic 
    );
end entity;

architecture rtl of EF3_PHY_FIFO is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_MULT18 is
generic (
    INPUTREGA : string := "ENABLE";
    INPUTREGB : string := "ENABLE";
    OUTPUTREG : string := "ENABLE";
    SRMODE : string := "ASYNC";
    MODE : string := "MULT18X18C";
    CEAMUX : string := "SIG";
    CEBMUX : string := "SIG";
    CEPDMUX : string := "SIG";
    RSTANMUX : string := "SIG";
    RSTBNMUX : string := "SIG";
    RSTPDNMUX : string := "SIG";
    CLKMUX : string := "SIG";
    SIGNEDAMUX : string := "SIG";
    SIGNEDBMUX : string := "SIG";
    SOURCEAMUX : string := "SIG";
    SOURCEBMUX : string := "SIG"
    );
port (
    acout : out std_logic_vector(17 DOWNTO 0) ;
    bcout : out std_logic_vector(17 DOWNTO 0) ;
    p : out std_logic_vector(35 DOWNTO 0) ;
    signeda : in std_logic ;
    signedb : in std_logic ;
    a : in std_logic_vector(17 DOWNTO 0) ;
    b : in std_logic_vector(17 DOWNTO 0) ;
    acin : in std_logic_vector(17 DOWNTO 0) ;
    bcin : in std_logic_vector(17 DOWNTO 0) ;
    cea : in std_logic ;
    ceb : in std_logic ;
    cepd : in std_logic ;
    clk : in std_logic ;
    rstan : in std_logic ;
    rstbn : in std_logic ;
    rstpdn : in std_logic ;
    sourcea : in std_logic ;
    sourceb : in std_logic 
    );
end entity;

architecture rtl of EF3_PHY_MULT18 is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_GCLK is
port (
    clki : in std_logic ;
    clko : out std_logic 
    );
end entity;

architecture rtl of EF3_PHY_GCLK is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_IOCLK is
generic (
    STOPCLK : string := "DISABLE"
    );
port (
    clki : in std_logic ;
    stop : in std_logic ;
    clko : out std_logic 
    );
end entity;

architecture rtl of EF3_PHY_IOCLK is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_CLKDIV is
generic (
    GSR : string := "DISABLE";
    DIV : integer := 2
    );
port (
    clkdiv1, clkdivx : out std_logic ;
    clki : in std_logic ;
    rst : in std_logic ;
    rls : in std_logic 
    );
end entity;

architecture rtl of EF3_PHY_CLKDIV is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_CONFIG is
generic (
    DONE_PERSISTN : string := "ENABLE";
    INIT_PERSISTN : string := "ENABLE";
    PROGRAMN_PERSISTN : string := "DISABLE";
    JTAG_PERSISTN : string := "DISABLE";
    MBOOT_AUTO_SEL : string := "DISABLE";
    ADDR_SOURCE_SEL : string := "STATIC";
    STATIC_ADDR : string := B"00000000";
    GSRN_SYNC_SEL : string := "DISABLE";
    USR_GSRN_EN : string := "DISABLE";
    FREQ : string := "2.5"
    );
port (
    jrstn : out std_logic ;
    jrti : out std_logic_vector(1 DOWNTO 0) ;
    jshift : out std_logic ;
    jtck : out std_logic ;
    jtdi : out std_logic ;
    jupdate : out std_logic ;
    jscanen : out std_logic_vector(1 DOWNTO 0) ;
    jtms : out std_logic ;
    jtdo : in std_logic_vector(1 DOWNTO 0) ;
    jtag8_ipa : in std_logic_vector(7 DOWNTO 0) ;
    jtag8_ipb : in std_logic_vector(7 DOWNTO 0) ;
    mboot_rebootn : in std_logic ;
    mboot_dynamic_addr : in std_logic_vector(7 DOWNTO 0) ;
    dna_dout : out std_logic ;
    dna_clk : in std_logic ;
    dna_din : in std_logic ;
    dna_shift_en : in std_logic ;
    done : out std_logic ;
    highz : out std_logic ;
    usr_gsrn : in std_logic ;
    gsrn_sync_clk : in std_logic ;
    cclk : out std_logic ;
    cclk_en : in std_logic 
    );
end entity;

architecture rtl of EF3_PHY_CONFIG is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_OSC is
generic (
    STANDBY : string := "DISABLE"
    );
port (
    stdby : in std_logic ;
    osc : out std_logic 
    );
end entity;

architecture rtl of EF3_PHY_OSC is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_OSCDIV is
generic (
    half_cycle : real := 1.9
    );
port (
    rstn : in std_logic ;
    stdby : in std_logic ;
    div : in std_logic_vector(6 DOWNTO 0) ;
    clko : out std_logic 
    );
end entity;

architecture rtl of EF3_PHY_OSCDIV is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_PWRMNT is
generic (
    MNT_LVL : integer := 0
    );
port (
    pwr_dwn_n : out std_logic ;
    sel_pwr : in std_logic ;
    pwr_mnt_pd : in std_logic 
    );
end entity;

architecture rtl of EF3_PHY_PWRMNT is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_MCU is
generic (
    GPIO_L0 : string := "DISABLE";
    GPIO_L1 : string := "DISABLE";
    GPIO_L2 : string := "DISABLE";
    GPIO_L3 : string := "DISABLE";
    GPIO_L4 : string := "DISABLE";
    GPIO_L5 : string := "DISABLE";
    GPIO_L6 : string := "DISABLE";
    GPIO_L7 : string := "DISABLE";
    GPIO_L8 : string := "DISABLE";
    GPIO_L9 : string := "DISABLE";
    GPIO_L10 : string := "DISABLE";
    GPIO_L11 : string := "DISABLE";
    GPIO_L12 : string := "DISABLE";
    GPIO_L13 : string := "DISABLE";
    GPIO_L14 : string := "DISABLE";
    GPIO_L15 : string := "DISABLE"
    );
port (
    ppm_clk : in std_logic ;
    i2s_mst_clk : in std_logic ;
    ppm_rstn : in std_logic ;
    sleep_out : out std_logic ;
    gpio_h_out : out std_logic_vector(15 DOWNTO 0) ;
    gpio_h_oe_n : out std_logic_vector(15 DOWNTO 0) ;
    gpio_h_in : in std_logic_vector(15 DOWNTO 0) ;
    ahbslave_dma_req : in std_logic ;
    ahbslave_dma_single : in std_logic ;
    ahbslave_dma_ack : out std_logic ;
    h2h_haddr : out std_logic_vector(31 DOWNTO 0) ;
    h2h_htrans : out std_logic_vector(1 DOWNTO 0) ;
    h2h_hwrite : out std_logic ;
    h2h_hsize : out std_logic_vector(2 DOWNTO 0) ;
    h2h_hburst : out std_logic_vector(2 DOWNTO 0) ;
    h2h_hprot : out std_logic_vector(3 DOWNTO 0) ;
    h2h_hwdata : out std_logic_vector(31 DOWNTO 0) ;
    h2h_hrdata : in std_logic_vector(31 DOWNTO 0) ;
    h2h_hreadyout : in std_logic ;
    h2h_hresp : in std_logic_vector(1 DOWNTO 0) ;
    h2h_mclk : in std_logic ;
    h2h_rstn : in std_logic ;
    qspi_rxd : in std_logic_vector(3 DOWNTO 0) ;
    qspi_txd : out std_logic_vector(3 DOWNTO 0) ;
    qspi_sclk : out std_logic ;
    qspi_ss : out std_logic ;
    qspi_oen : out std_logic_vector(3 DOWNTO 0) ;
    adc1_exttrg : in std_logic ;
    adc2_exttrg : in std_logic ;
    usr_reg : out std_logic_vector(15 DOWNTO 0) 
    );
end entity;

architecture rtl of EF3_PHY_MCU is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_PAD is
generic (
    DEDCLK : string := "DISABLE";
    GSR : string := "ENABLE";
    SRMODE : string := "SYNC";
    TSMUX : string := "1";
    INSCLKMUX : string := "0";
    INPCLKMUX : string := "CLK";
    INCEMUX : string := "CE";
    INRSTMUX : string := "0";
    IN_REGSET : string := "RESET";
    IN_DFFMODE : string := "NONE";
    IDDRMODE : string := "OFF";
    IDDRPIPEMODE : string := "NONE";
    INDELMUX : string := "NODEL";
    INDEL : integer := 0;
    OUTSCLKMUX : string := "0";
    OUTPCLKMUX : string := "CLK";
    OUTCEMUX : string := "CE";
    OUTRSTMUX : string := "0";
    DO_REGSET : string := "RESET";
    DO_DFFMODE : string := "NONE";
    ODDRMODE : string := "OFF";
    OUTDELMUX : string := "NODEL";
    OUTDEL : integer := 0;
    TO_REGSET : string := "RESET";
    TO_DFFMODE : string := "NONE";
    MODE : string := "IN";
    DRIVE : string := "NONE";
    IOTYPE : string := "LVCMOS25"
    );
port (
    ipad : in std_logic ;
    opad : out std_logic ;
    bpad : inout std_logic ;
    rst, ce, isclk, ipclk, osclk, opclk : in std_logic ;
    ts : in std_logic ;
    do : in std_logic_vector(3 DOWNTO 0) ;
    di : out std_logic ;
    diq : out std_logic_vector(3 DOWNTO 0) ;
    lp_rx_en : in std_logic ;
    lp_dir : in std_logic ;
    lp_dout : in std_logic ;
    dynres : in std_logic ;
    vcmpd_n : in std_logic ;
    lp_din : out std_logic 
    );
end entity;

architecture rtl of EF3_PHY_PAD is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_SPAD is
generic (
    DEDCLK : string := "DISABLE";
    GSR : string := "ENABLE";
    SRMODE : string := "SYNC";
    TSMUX : string := "TS";
    INCLKMUX : string := "0";
    INCEMUX : string := "CE";
    INRSTMUX : string := "RST";
    IDDRMODE : string := "OFF";
    INDELMUX : string := "NODEL";
    INDEL : integer := 0;
    OUTCLKMUX : string := "0";
    OUTCEMUX : string := "CE";
    OUTRSTMUX : string := "RST";
    ODDRMODE : string := "OFF";
    IN_DFFMODE : string := "NONE";
    IN_REGSET : string := "RESET";
    DO_DFFMODE : string := "NONE";
    DO_REGSET : string := "RESET";
    TO_DFFMODE : string := "NONE";
    TO_REGSET : string := "RESET";
    MODE : string := "IN";
    DRIVE : string := "NONE";
    IOTYPE : string := "LVCMOS25"
    );
port (
    ipad : in std_logic ;
    opad : out std_logic ;
    bpad : inout std_logic ;
    rst, ce, clk : in std_logic ;
    ts : in std_logic ;
    do : in std_logic_vector(1 DOWNTO 0) ;
    di : out std_logic ;
    diq : out std_logic_vector(1 DOWNTO 0) 
    );
end entity;

architecture rtl of EF3_PHY_SPAD is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_CSB is
generic (
    HOLD : string := "YES";
    INITOUT : string := "0";
    PRESELECT : string := "NONE";
    TESTMODE : string := "DISABLE"
    );
port (
    clko : out std_logic ;
    ce : in std_logic_vector(1 DOWNTO 0) ;
    clki : in std_logic_vector(1 DOWNTO 0) ;
    drct : in std_logic_vector(1 DOWNTO 0) ;
    sel : in std_logic_vector(1 DOWNTO 0) ;
    ignr : in std_logic_vector(1 DOWNTO 0) 
    );
end entity;

architecture rtl of EF3_PHY_CSB is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_INTFLASH is
port (
    mosi_io0 : inout std_logic ;
    miso_io1 : inout std_logic ;
    wp_n_io2 : inout std_logic ;
    hold_n_io3 : inout std_logic ;
    cclk : in std_logic ;
    cs_n : in std_logic 
    );
end entity;

architecture rtl of EF3_PHY_INTFLASH is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_MSLICE is
generic (
    INIT_LUT0 : string := X"0000";
    INIT_LUT1 : string := X"0000";
    MODE : string := "LOGIC";
    ALUTYPE : string := "";
    MSFXMUX : string := "OFF";
    GSR : string := "ENABLE";
    TESTMODE : string := "OFF";
    CEMUX : string := "CE";
    SRMUX : string := "SR";
    CLKMUX : string := "CLK";
    SRMODE : string := "ASYNC";
    DFFMODE : string := "FF";
    REG0_SD : string := "MI";
    REG1_SD : string := "MI";
    REG0_REGSET : string := "SET";
    REG1_REGSET : string := "SET"
    );
port (
    a,b,c,d,mi : in std_logic_vector(1 DOWNTO 0) ;
    clk : in std_logic ;
    ce : in std_logic ;
    sr : in std_logic ;
    fci : in std_logic ;
    f,fx,q : out std_logic_vector(1 DOWNTO 0) ;
    fco : out std_logic ;
    dpram_mode : in std_logic ;
    dpram_di : in std_logic_vector(1 DOWNTO 0) ;
    dpram_we : in std_logic ;
    dpram_wclk : in std_logic ;
    dpram_waddr : in std_logic_vector(3 DOWNTO 0) 
    );
end entity;

architecture rtl of EF3_PHY_MSLICE is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_LSLICE is
generic (
    INIT_LUTF0 : string := X"0000";
    INIT_LUTG0 : string := X"0000";
    INIT_LUTF1 : string := X"0000";
    INIT_LUTG1 : string := X"0000";
    MODE : string := "LOGIC";
    GSR : string := "ENABLE";
    TESTMODE : string := "OFF";
    CEMUX : string := "1";
    SRMUX : string := "SR";
    CLKMUX : string := "CLK";
    SRMODE : string := "ASYNC";
    DFFMODE : string := "FF";
    REG0_SD : string := "MI";
    REG1_SD : string := "MI";
    REG0_REGSET : string := "SET";
    REG1_REGSET : string := "SET";
    DEMUX0 : string := "D";
    DEMUX1 : string := "D";
    CMIMUX0 : string := "C";
    CMIMUX1 : string := "C";
    LSFMUX0 : string := "LUTF";
    LSFXMUX0 : string := "LUTG";
    LSFMUX1 : string := "LUTF";
    LSFXMUX1 : string := "LUTG"
    );
port (
    a,b,c,d,e,mi : in std_logic_vector(1 DOWNTO 0) ;
    clk : in std_logic ;
    ce : in std_logic ;
    sr : in std_logic ;
    fci : in std_logic ;
    f,fx,q : out std_logic_vector(1 DOWNTO 0) ;
    fco : out std_logic ;
    dpram_di : out std_logic_vector(3 DOWNTO 0) ;
    dpram_waddr : out std_logic_vector(3 DOWNTO 0) ;
    dpram_wclk : out std_logic ;
    dpram_we : out std_logic ;
    dpram_mode : out std_logic 
    );
end entity;

architecture rtl of EF3_PHY_LSLICE is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_PLL is
generic (
    DYNCFG : string := "DISABLE";
    IF_ESCLKSTSW : string := "DISABLE";
    REFCLK_SEL : string := "INTERNAL";
    FIN : string := "100.0000";
    REFCLK_DIV : integer := 1;
    FBCLK_DIV : integer := 1;
    CLKC0_DIV : integer := 1;
    CLKC1_DIV : integer := 1;
    CLKC2_DIV : integer := 1;
    CLKC3_DIV : integer := 1;
    CLKC4_DIV : integer := 1;
    CLKC5_DIV : integer := 1;
    CLKC6_DIV : integer := 1;
    CLKC0_ENABLE : string := "DISABLE";
    CLKC1_ENABLE : string := "DISABLE";
    CLKC2_ENABLE : string := "DISABLE";
    CLKC3_ENABLE : string := "DISABLE";
    CLKC4_ENABLE : string := "DISABLE";
    CLKC5_ENABLE : string := "DISABLE";
    CLKC6_ENABLE : string := "DISABLE";
    CLKC0_DIV2_ENABLE : string := "DISABLE";
    CLKC1_DIV2_ENABLE : string := "DISABLE";
    CLKC2_DIV2_ENABLE : string := "DISABLE";
    CLKC3_DIV2_ENABLE : string := "DISABLE";
    CLKC4_DIV2_ENABLE : string := "DISABLE";
    CLKC5_DIV2_ENABLE : string := "DISABLE";
    CLKC6_DIV2_ENABLE : string := "DISABLE";
    FEEDBK_MODE : string := "NORMAL";
    FEEDBK_PATH : string := "VCO_PHASE_0";
    STDBY_ENABLE : string := "ENABLE";
    CLKC0_FPHASE : integer := 0;
    CLKC1_FPHASE : integer := 0;
    CLKC2_FPHASE : integer := 0;
    CLKC3_FPHASE : integer := 0;
    CLKC4_FPHASE : integer := 0;
    CLKC0_CPHASE : integer := 1;
    CLKC1_CPHASE : integer := 1;
    CLKC2_CPHASE : integer := 1;
    CLKC3_CPHASE : integer := 1;
    CLKC4_CPHASE : integer := 1;
    CLKC5_CPHASE : integer := 1;
    CLKC6_CPHASE : integer := 1;
    GMC_GAIN : integer := 7;
    GMC_TEST : integer := 14;
    ICP_CURRENT : integer := 14;
    KVCO : integer := 7;
    LPF_CAPACITOR : integer := 3;
    LPF_RESISTOR : integer := 1;
    PLLRST_ENA : string := "ENABLE";
    PLLMRST_ENA : string := "DISABLE";
    PLLC2RST_ENA : string := "DISABLE";
    PLLC34RST_ENA : string := "DISABLE";
    PREDIV_MUXC0 : string := "VCO";
    PREDIV_MUXC1 : string := "VCO";
    PREDIV_MUXC2 : string := "VCO";
    PREDIV_MUXC3 : string := "VCO";
    PREDIV_MUXC4 : string := "VCO";
    PREDIV_MUXC5 : string := "VCO";
    PREDIV_MUXC6 : string := "VCO";
    ODIV_MUXC0 : string := "DIV";
    ODIV_MUXC1 : string := "DIV";
    ODIV_MUXC2 : string := "DIV";
    ODIV_MUXC3 : string := "DIV";
    ODIV_MUXC4 : string := "DIV";
    FREQ_LOCK_ACCURACY : integer := 2;
    PLL_LOCK_MODE : integer := 0;
    INTFB_WAKE : string := "DISABLE";
    DPHASE_SOURCE : string := "DISABLE";
    VCO_NORESET : string := "DISABLE";
    STDBY_VCO_ENA : string := "DISABLE";
    NORESET : string := "DISABLE";
    SYNC_ENABLE : string := "ENABLE";
    DERIVE_PLL_CLOCKS : string := "DISABLE";
    GEN_BASIC_CLOCK : string := "DISABLE";
    SSC_ENABLE : string := "DISABLE";
    SSC_MODE : string := "Down";
    SSC_AMP : real := 0.0;
    PU_INTP : string := "DISABLE";
    INTPI : integer := 0;
    HIGH_SPEED_EN : string := "DISABLE";
    SSC_FREQ_DIV : integer := 0;
    SSC_RNGE : integer := 0;
    FREQ_OFFSET : real := 0.0;
    OFFSET_MODE : string := "INT";
    FREQ_OFFSET_INT : integer := 0;
    CLKC0_DUTY : real := 0.0;
    CLKC0_DUTY50 : string := "ENABLE";
    CLKC1_DUTY : real := 0.0;
    CLKC1_DUTY50 : string := "ENABLE";
    CLKC0_DUTY_INT : integer := 0;
    CLKC1_DUTY_INT : integer := 0
    );
port (
    clkc : out std_logic_vector(6 DOWNTO 0) := (others => '0');
    extlock : out std_logic := '0';
    stdby : in std_logic := '0';
    refclk : in std_logic := '0';
    fbclk : in std_logic := '0';
    reset : in std_logic := '0';
    psdone : out std_logic := '0';
    psclk : in std_logic := '0';
    psdown : in std_logic := '0';
    psstep : in std_logic := '0';
    psclksel : in std_logic_vector(2 DOWNTO 0) := "000";
    do : out std_logic_vector(7 DOWNTO 0) := (others => '0');
    dclk : in std_logic := '0';
    dcs : in std_logic := '0';
    dwe : in std_logic := '0';
    di : in std_logic_vector(7 DOWNTO 0) := (others => '0');
    daddr : in std_logic_vector(5 DOWNTO 0) := (others => '0');
    dsm_refclk : in std_logic := '0';
    dsm_rst : in std_logic := '0';
    ssc_en : in std_logic := '0';
    frac_offset_valid : in std_logic := '0'
    );
end entity;

architecture rtl of EF3_PHY_PLL is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_LOGIC_BRAM is
generic (
    DATA_WIDTH_A : integer := 9;
    DATA_WIDTH_B : integer := 9;
    ADDR_WIDTH_A : integer := 10;
    ADDR_WIDTH_B : integer := 10;
    DATA_DEPTH_A : integer := 1024;
    DATA_DEPTH_B : integer := 1024;
    BYTE_ENABLE : integer := 0;
    BYTE_A : integer := 1;
    BYTE_B : integer := 1;
    MODE : string := "DP";
    REGMODE_A : string := "NOREG";
    REGMODE_B : string := "NOREG";
    WRITEMODE_A : string := "NORMAL";
    WRITEMODE_B : string := "NORMAL";
    RESETMODE : string := "SYNC";
    DEBUGGABLE : string := "NO";
    PACKABLE : string := "NO";
    FORCE_KEEP : string := "OFF";
    INIT_FILE : string := "NONE";
    FILL_ALL : string := "NONE";
    IMPLEMENT : string := "9K"
    );
port (
    doa : out std_logic_vector(DATA_WIDTH_A-1 DOWNTO 0) ;
    dob : out std_logic_vector(DATA_WIDTH_B-1 DOWNTO 0) ;
    dia : in std_logic_vector(DATA_WIDTH_A-1 DOWNTO 0) := (others => '0');
    dib : in std_logic_vector(DATA_WIDTH_B-1 DOWNTO 0) := (others => '0');
    cea, ocea, clka, wea, rsta : in std_logic := '0';
    ceb, oceb, clkb, web, rstb : in std_logic := '0';
    bea : in std_logic_vector(BYTE_A - 1  DOWNTO 0) := (others => '0');
    beb : in std_logic_vector(BYTE_B - 1  DOWNTO 0) := (others => '0');
    addra : in std_logic_vector(ADDR_WIDTH_A-1 DOWNTO 0) := (others => '0');
    addrb : in std_logic_vector(ADDR_WIDTH_B-1 DOWNTO 0) := (others => '0')
    );
end entity;

architecture rtl of EF3_LOGIC_BRAM is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity EF3_PHY_ADC is
generic (
    TEMPERATURE : string := "DISABLE";
    CH0 : string := "DISABLE";
    CH1 : string := "DISABLE";
    CH2 : string := "DISABLE";
    CH3 : string := "DISABLE";
    CH4 : string := "DISABLE";
    CH5 : string := "DISABLE";
    CH6 : string := "DISABLE";
    CH7 : string := "DISABLE";
    VREF : string := "DISABLE"
    );
port (
    clk : in std_logic ;
    pd : in std_logic ;
    s : in std_logic_vector(2 DOWNTO 0) ;
    soc : in std_logic ;
    eoc : out std_logic ;
    dout : out std_logic_vector(11 DOWNTO 0) 
    );
end entity;

architecture rtl of EF3_PHY_ADC is
    begin
    end;


