------------------------------------------------
------------------------------------------------
--
--package ELF_COMPONENTS :
-- It should be compiled into elf_macro library.
--
------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

package ELF_COMPONENTS is

component ELF_LOGIC_BUF is 
port (
    o : out std_logic ;
    i : in std_logic 
    );
end component;


component ELF_LOGIC_BUFG is 
port (
    o : out std_logic ;
    i : in std_logic 
    );
end component;


component ELF_LOGIC_CCLK is 
port (
    cclk : out std_logic 
    );
end component;


component ELF_LOGIC_IDELAY is 
generic (
    INDEL : integer := 0
    );
port (
    o : out std_logic ;
    i : in std_logic 
    );
end component;


component ELF_LOGIC_IDDR is 
generic (
    SRMODE : string := "SYNC"
    );
port (
    q1 : out std_logic ;
    q2 : out std_logic ;
    clk : in std_logic ;
    d : in std_logic ;
    rst : in std_logic 
    );
end component;


component ELF_LOGIC_ODDR is 
generic (
    SRMODE : string := "SYNC"
    );
port (
    q : out std_logic ;
    clk : in std_logic ;
    d1 : in std_logic ;
    d2 : in std_logic ;
    rst : in std_logic 
    );
end component;


component ELF_LOGIC_DRAM is 
generic (
    DATA_WIDTH_W : integer := 9;
    ADDR_WIDTH_W : integer := 10;
    DATA_DEPTH_W : integer := 1024;
    DATA_WIDTH_R : integer := 9;
    ADDR_WIDTH_R : integer := 10;
    DATA_DEPTH_R : integer := 1024;
    INIT_FILE : string := "NONE"
    );
port (
    di : in std_logic_vector(DATA_WIDTH_W-1 DOWNTO 0) ;
    waddr : in std_logic_vector(ADDR_WIDTH_W-1 DOWNTO 0) ;
    wclk : in std_logic ;
    we : in std_logic ;
    do : out std_logic_vector(DATA_WIDTH_R-1 DOWNTO 0) ;
    raddr : in std_logic_vector(ADDR_WIDTH_R-1 DOWNTO 0) 
    );
end component;


component ELF_LOGIC_SEQ_DIV is 
generic (
    NUMER_WIDTH : integer := 16;
    DENOM_WIDTH : integer := 16
    );
port (
    clk : in std_logic ;
    rst : in std_logic ;
    start : in std_logic ;
    numer : in std_logic_vector(NUMER_WIDTH-1 DOWNTO 0) ;
    denom : in std_logic_vector(DENOM_WIDTH-1 DOWNTO 0) ;
    quotient : out std_logic_vector(NUMER_WIDTH-1 DOWNTO 0) ;
    remain : out std_logic_vector(DENOM_WIDTH-1 DOWNTO 0) ;
    done : out std_logic 
    );
end component;


component ELF_LOGIC_DRAM32X4 is 
generic (
    INIT_D0 : string := X"0000";
    INIT_D1 : string := X"0000";
    INIT_D2 : string := X"0000";
    INIT_D3 : string := X"0000";
    INIT_D4 : string := X"0000";
    INIT_D5 : string := X"0000";
    INIT_D6 : string := X"0000";
    INIT_D7 : string := X"0000"
    );
port (
    di : in std_logic_vector(3 DOWNTO 0) ;
    waddr : in std_logic_vector(4 DOWNTO 0) ;
    wclk : in std_logic ;
    we : in std_logic ;
    raddr : in std_logic_vector(4 DOWNTO 0) ;
    do : out std_logic_vector(3 DOWNTO 0) 
    );
end component;


component ELF_PHY_DRAMHARDCON is 
generic (
    MODE : string := "DPRAM"
    );
port (
    b : in std_logic_vector(7 DOWNTO 0) ;
    sload : in std_logic ;
    clk0 : in std_logic ;
    mi7 : in std_logic ;
    dpram_di : out std_logic_vector(3 DOWNTO 0) ;
    dpram_we : out std_logic ;
    dpram_waddr : out std_logic_vector(4 DOWNTO 0) ;
    dpram_wclk : out std_logic ;
    dpram_mode : out std_logic 
    );
end component;


component ELF_LOGIC_DRAM32X4W4R8 is 
generic (
    INIT_D0 : string := X"0000";
    INIT_D1 : string := X"0000";
    INIT_D2 : string := X"0000";
    INIT_D3 : string := X"0000";
    INIT_D4 : string := X"0000";
    INIT_D5 : string := X"0000";
    INIT_D6 : string := X"0000";
    INIT_D7 : string := X"0000"
    );
port (
    di : in std_logic_vector(3 DOWNTO 0) ;
    waddr : in std_logic_vector(4 DOWNTO 0) ;
    wclk : in std_logic ;
    we : in std_logic ;
    raddr : in std_logic_vector(3 DOWNTO 0) ;
    do : out std_logic_vector(7 DOWNTO 0) 
    );
end component;


component ELF_PHY_OSC is 
generic (
    FREQ : string := "60"
    );
port (
    osc_dis : in std_logic ;
    osc_clk : out std_logic 
    );
end component;


component ELF_PHY_MSLICE is 
generic (
    INIT_LUT0 : string := X"0000";
    INIT_LUT1 : string := X"0000";
    MODE : string := "LOGIC";
    FMUX0 : string := "LUT4";
    FMUX1 : string := "LUT4";
    GSR : string := "ENABLE";
    CEMUX : string := "CE";
    ALUTYPE : string := "";
    ASRMUX : string := "ASR";
    CLKMUX : string := "CLK";
    DFFMODE : string := "FF";
    SHIFT : string := "DISABLE";
    REG0_SD : string := "F";
    REG1_SD : string := "F";
    REG0_REGSET : string := "RESET";
    REG1_REGSET : string := "RESET";
    SSRMUX : string := "0";
    SSRSTATUS : string := "DISABLE";
    SSRMODE : string := "OFF";
    SELGLB : string := "DISABLE"
    );
port (
    a,b,c,d,mi : in std_logic_vector(1 DOWNTO 0) ;
    clk : in std_logic ;
    ce : in std_logic ;
    ssr : in std_logic ;
    asr : in std_logic ;
    sload : in std_logic ;
    fci : in std_logic ;
    shift_in : in std_logic ;
    shift_out : out std_logic ;
    f,q : out std_logic_vector(1 DOWNTO 0) ;
    fco : out std_logic ;
    dpram_mode : in std_logic ;
    dpram_di : in std_logic ;
    dpram_we : in std_logic ;
    dpram_wclk : in std_logic ;
    dpram_waddr : in std_logic_vector(4 DOWNTO 0) ;
    glb : in std_logic 
    );
end component;


component ELF_PHY_PAD is 
generic (
    DEDCLK : string := "DISABLE";
    GSR : string := "ENABLE";
    SRMODE : string := "SYNC";
    TSMUX : string := "1";
    INCLKMUX : string := "0";
    INCEMUX : string := "CE";
    INRSTMUX : string := "0";
    IDDRMODE : string := "OFF";
    INDELMUX : string := "NODEL";
    INDEL : integer := 0;
    OUTCLKMUX : string := "0";
    OUTCEMUX : string := "CE";
    OUTRSTMUX : string := "0";
    ODDRMODE : string := "OFF";
    IN_DFFMODE : string := "NONE";
    IN_REGSET : string := "RESET";
    DO_DFFMODE : string := "NONE";
    DO_REGSET : string := "RESET";
    TO_DFFMODE : string := "NONE";
    TO_REGSET : string := "RESET";
    MODE : string := "IN";
    DRIVE : string := "NONE";
    IOTYPE : string := "LVCMOS25"
    );
port (
    ipad : in std_logic ;
    opad : out std_logic ;
    bpad : inout std_logic ;
    rst, ce, clk : in std_logic ;
    eninr_dyn, ts : in std_logic ;
    otrue : in std_logic ;
    ocomp : in std_logic ;
    itrue : out std_logic ;
    icomp : out std_logic 
    );
end component;


component ELF_PHY_CONFIG is 
generic (
    DISOSC : string := "DISABLE";
    CFGSTDBY : string := "DISABLE";
    I2CWAKEUP : string := "DISABLE";
    TSALL : string := "DISABLE";
    TSINV : string := "DISABLE";
    ISP1 : string := "DISABLE";
    ISP2 : string := "DISABLE";
    DONE_PHASE : string := "0";
    GOE_PHASE : string := "0";
    GSR_PHASE : string := "0";
    GWE_PHASE : string := "0";
    SYNSRC : string := "DISABLE";
    SYNEXTDONE : string := "DISABLE";
    FORCEGSRN : string := "DISABLE";
    FORCEGWE : string := "DISABLE";
    USR_GSRN : string := "DISABLE";
    GSRMODE : string := "ASYNC";
    I2C_PERSISTN : string := "DISABLE";
    MSPI_PERSIST : string := "DISABLE";
    PLLCHECK : string := "DISABLE";
    PIBMCLK : string := "DISABLE";
    PIBMCLKDIV : integer := 1;
    PWRDWN : string := "DISABLE";
    mc1_i2c_mspi : string := "0";
    mc1_i2c_persistn : string := "0"
    );
port (
    wb_dat_o : out std_logic_vector(7 DOWNTO 0) ;
    wb_wt_en : in std_logic ;
    wb_cyc_en : in std_logic ;
    wb_stb_en : in std_logic ;
    wb_rst : in std_logic ;
    wb_dat_i : in std_logic_vector(7 DOWNTO 0) ;
    wb_adr_i : in std_logic_vector(3 DOWNTO 0) ;
    wb_clk_i : in std_logic ;
    ufm_tbit_o : out std_logic ;
    pib_ufm_cs : in std_logic ;
    pib_ufm_ae : in std_logic ;
    pib_ufm_oe : in std_logic ;
    pib_ufm_prog : in std_logic ;
    pib_ufm_nvstr : in std_logic ;
    pib_ufm_sera : in std_logic ;
    pib_done : out std_logic ;
    pib_freeze_io : out std_logic ;
    cclk : out std_logic ;
    pib_programn : in std_logic ;
    pib_ts_all : in std_logic ;
    pib_user_clk : in std_logic ;
    pib_user_gsr : in std_logic ;
    gsrn_sync_clk : in std_logic ;
    jce : out std_logic_vector(1 DOWNTO 0) ;
    jrstn : out std_logic ;
    jrti : out std_logic_vector(1 DOWNTO 0) ;
    jshift : out std_logic ;
    jtck : out std_logic ;
    jtdi : out std_logic ;
    jtms : out std_logic ;
    jupdate : out std_logic ;
    jtdo : in std_logic_vector(1 DOWNTO 0) 
    );
end component;


component ELF_PHY_GCLK is 
port (
    clki : in std_logic ;
    clko : out std_logic 
    );
end component;


end ELF_COMPONENTS;
------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_LOGIC_BUF is
port (
    o : out std_logic ;
    i : in std_logic 
    );
end entity;

architecture rtl of ELF_LOGIC_BUF is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_LOGIC_BUFG is
port (
    o : out std_logic ;
    i : in std_logic 
    );
end entity;

architecture rtl of ELF_LOGIC_BUFG is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_LOGIC_CCLK is
port (
    cclk : out std_logic 
    );
end entity;

architecture rtl of ELF_LOGIC_CCLK is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_LOGIC_IDELAY is
generic (
    INDEL : integer := 0
    );
port (
    o : out std_logic ;
    i : in std_logic 
    );
end entity;

architecture rtl of ELF_LOGIC_IDELAY is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_LOGIC_IDDR is
generic (
    SRMODE : string := "SYNC"
    );
port (
    q1 : out std_logic ;
    q2 : out std_logic ;
    clk : in std_logic ;
    d : in std_logic ;
    rst : in std_logic 
    );
end entity;

architecture rtl of ELF_LOGIC_IDDR is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_LOGIC_ODDR is
generic (
    SRMODE : string := "SYNC"
    );
port (
    q : out std_logic ;
    clk : in std_logic ;
    d1 : in std_logic ;
    d2 : in std_logic ;
    rst : in std_logic 
    );
end entity;

architecture rtl of ELF_LOGIC_ODDR is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_LOGIC_DRAM is
generic (
    DATA_WIDTH_W : integer := 9;
    ADDR_WIDTH_W : integer := 10;
    DATA_DEPTH_W : integer := 1024;
    DATA_WIDTH_R : integer := 9;
    ADDR_WIDTH_R : integer := 10;
    DATA_DEPTH_R : integer := 1024;
    INIT_FILE : string := "NONE"
    );
port (
    di : in std_logic_vector(DATA_WIDTH_W-1 DOWNTO 0) ;
    waddr : in std_logic_vector(ADDR_WIDTH_W-1 DOWNTO 0) ;
    wclk : in std_logic ;
    we : in std_logic ;
    do : out std_logic_vector(DATA_WIDTH_R-1 DOWNTO 0) ;
    raddr : in std_logic_vector(ADDR_WIDTH_R-1 DOWNTO 0) 
    );
end entity;

architecture rtl of ELF_LOGIC_DRAM is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_LOGIC_SEQ_DIV is
generic (
    NUMER_WIDTH : integer := 16;
    DENOM_WIDTH : integer := 16
    );
port (
    clk : in std_logic ;
    rst : in std_logic ;
    start : in std_logic ;
    numer : in std_logic_vector(NUMER_WIDTH-1 DOWNTO 0) ;
    denom : in std_logic_vector(DENOM_WIDTH-1 DOWNTO 0) ;
    quotient : out std_logic_vector(NUMER_WIDTH-1 DOWNTO 0) ;
    remain : out std_logic_vector(DENOM_WIDTH-1 DOWNTO 0) ;
    done : out std_logic 
    );
end entity;

architecture rtl of ELF_LOGIC_SEQ_DIV is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_LOGIC_DRAM32X4 is
generic (
    INIT_D0 : string := X"0000";
    INIT_D1 : string := X"0000";
    INIT_D2 : string := X"0000";
    INIT_D3 : string := X"0000";
    INIT_D4 : string := X"0000";
    INIT_D5 : string := X"0000";
    INIT_D6 : string := X"0000";
    INIT_D7 : string := X"0000"
    );
port (
    di : in std_logic_vector(3 DOWNTO 0) ;
    waddr : in std_logic_vector(4 DOWNTO 0) ;
    wclk : in std_logic ;
    we : in std_logic ;
    raddr : in std_logic_vector(4 DOWNTO 0) ;
    do : out std_logic_vector(3 DOWNTO 0) 
    );
end entity;

architecture rtl of ELF_LOGIC_DRAM32X4 is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_PHY_DRAMHARDCON is
generic (
    MODE : string := "DPRAM"
    );
port (
    b : in std_logic_vector(7 DOWNTO 0) ;
    sload : in std_logic ;
    clk0 : in std_logic ;
    mi7 : in std_logic ;
    dpram_di : out std_logic_vector(3 DOWNTO 0) ;
    dpram_we : out std_logic ;
    dpram_waddr : out std_logic_vector(4 DOWNTO 0) ;
    dpram_wclk : out std_logic ;
    dpram_mode : out std_logic 
    );
end entity;

architecture rtl of ELF_PHY_DRAMHARDCON is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_LOGIC_DRAM32X4W4R8 is
generic (
    INIT_D0 : string := X"0000";
    INIT_D1 : string := X"0000";
    INIT_D2 : string := X"0000";
    INIT_D3 : string := X"0000";
    INIT_D4 : string := X"0000";
    INIT_D5 : string := X"0000";
    INIT_D6 : string := X"0000";
    INIT_D7 : string := X"0000"
    );
port (
    di : in std_logic_vector(3 DOWNTO 0) ;
    waddr : in std_logic_vector(4 DOWNTO 0) ;
    wclk : in std_logic ;
    we : in std_logic ;
    raddr : in std_logic_vector(3 DOWNTO 0) ;
    do : out std_logic_vector(7 DOWNTO 0) 
    );
end entity;

architecture rtl of ELF_LOGIC_DRAM32X4W4R8 is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_PHY_OSC is
generic (
    FREQ : string := "60"
    );
port (
    osc_dis : in std_logic ;
    osc_clk : out std_logic 
    );
end entity;

architecture rtl of ELF_PHY_OSC is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_PHY_MSLICE is
generic (
    INIT_LUT0 : string := X"0000";
    INIT_LUT1 : string := X"0000";
    MODE : string := "LOGIC";
    FMUX0 : string := "LUT4";
    FMUX1 : string := "LUT4";
    GSR : string := "ENABLE";
    CEMUX : string := "CE";
    ALUTYPE : string := "";
    ASRMUX : string := "ASR";
    CLKMUX : string := "CLK";
    DFFMODE : string := "FF";
    SHIFT : string := "DISABLE";
    REG0_SD : string := "F";
    REG1_SD : string := "F";
    REG0_REGSET : string := "RESET";
    REG1_REGSET : string := "RESET";
    SSRMUX : string := "0";
    SSRSTATUS : string := "DISABLE";
    SSRMODE : string := "OFF";
    SELGLB : string := "DISABLE"
    );
port (
    a,b,c,d,mi : in std_logic_vector(1 DOWNTO 0) ;
    clk : in std_logic ;
    ce : in std_logic ;
    ssr : in std_logic ;
    asr : in std_logic ;
    sload : in std_logic ;
    fci : in std_logic ;
    shift_in : in std_logic ;
    shift_out : out std_logic ;
    f,q : out std_logic_vector(1 DOWNTO 0) ;
    fco : out std_logic ;
    dpram_mode : in std_logic ;
    dpram_di : in std_logic ;
    dpram_we : in std_logic ;
    dpram_wclk : in std_logic ;
    dpram_waddr : in std_logic_vector(4 DOWNTO 0) ;
    glb : in std_logic 
    );
end entity;

architecture rtl of ELF_PHY_MSLICE is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_PHY_PAD is
generic (
    DEDCLK : string := "DISABLE";
    GSR : string := "ENABLE";
    SRMODE : string := "SYNC";
    TSMUX : string := "1";
    INCLKMUX : string := "0";
    INCEMUX : string := "CE";
    INRSTMUX : string := "0";
    IDDRMODE : string := "OFF";
    INDELMUX : string := "NODEL";
    INDEL : integer := 0;
    OUTCLKMUX : string := "0";
    OUTCEMUX : string := "CE";
    OUTRSTMUX : string := "0";
    ODDRMODE : string := "OFF";
    IN_DFFMODE : string := "NONE";
    IN_REGSET : string := "RESET";
    DO_DFFMODE : string := "NONE";
    DO_REGSET : string := "RESET";
    TO_DFFMODE : string := "NONE";
    TO_REGSET : string := "RESET";
    MODE : string := "IN";
    DRIVE : string := "NONE";
    IOTYPE : string := "LVCMOS25"
    );
port (
    ipad : in std_logic ;
    opad : out std_logic ;
    bpad : inout std_logic ;
    rst, ce, clk : in std_logic ;
    eninr_dyn, ts : in std_logic ;
    otrue : in std_logic ;
    ocomp : in std_logic ;
    itrue : out std_logic ;
    icomp : out std_logic 
    );
end entity;

architecture rtl of ELF_PHY_PAD is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_PHY_CONFIG is
generic (
    DISOSC : string := "DISABLE";
    CFGSTDBY : string := "DISABLE";
    I2CWAKEUP : string := "DISABLE";
    TSALL : string := "DISABLE";
    TSINV : string := "DISABLE";
    ISP1 : string := "DISABLE";
    ISP2 : string := "DISABLE";
    DONE_PHASE : string := "0";
    GOE_PHASE : string := "0";
    GSR_PHASE : string := "0";
    GWE_PHASE : string := "0";
    SYNSRC : string := "DISABLE";
    SYNEXTDONE : string := "DISABLE";
    FORCEGSRN : string := "DISABLE";
    FORCEGWE : string := "DISABLE";
    USR_GSRN : string := "DISABLE";
    GSRMODE : string := "ASYNC";
    I2C_PERSISTN : string := "DISABLE";
    MSPI_PERSIST : string := "DISABLE";
    PLLCHECK : string := "DISABLE";
    PIBMCLK : string := "DISABLE";
    PIBMCLKDIV : integer := 1;
    PWRDWN : string := "DISABLE";
    mc1_i2c_mspi : string := "0";
    mc1_i2c_persistn : string := "0"
    );
port (
    wb_dat_o : out std_logic_vector(7 DOWNTO 0) ;
    wb_wt_en : in std_logic ;
    wb_cyc_en : in std_logic ;
    wb_stb_en : in std_logic ;
    wb_rst : in std_logic ;
    wb_dat_i : in std_logic_vector(7 DOWNTO 0) ;
    wb_adr_i : in std_logic_vector(3 DOWNTO 0) ;
    wb_clk_i : in std_logic ;
    ufm_tbit_o : out std_logic ;
    pib_ufm_cs : in std_logic ;
    pib_ufm_ae : in std_logic ;
    pib_ufm_oe : in std_logic ;
    pib_ufm_prog : in std_logic ;
    pib_ufm_nvstr : in std_logic ;
    pib_ufm_sera : in std_logic ;
    pib_done : out std_logic ;
    pib_freeze_io : out std_logic ;
    cclk : out std_logic ;
    pib_programn : in std_logic ;
    pib_ts_all : in std_logic ;
    pib_user_clk : in std_logic ;
    pib_user_gsr : in std_logic ;
    gsrn_sync_clk : in std_logic ;
    jce : out std_logic_vector(1 DOWNTO 0) ;
    jrstn : out std_logic ;
    jrti : out std_logic_vector(1 DOWNTO 0) ;
    jshift : out std_logic ;
    jtck : out std_logic ;
    jtdi : out std_logic ;
    jtms : out std_logic ;
    jupdate : out std_logic ;
    jtdo : in std_logic_vector(1 DOWNTO 0) 
    );
end entity;

architecture rtl of ELF_PHY_CONFIG is
    begin
    end;


------------------------------------------------
------------------------------------------------
library ieee;
library work;
use ieee.std_logic_1164.all;

entity ELF_PHY_GCLK is
port (
    clki : in std_logic ;
    clko : out std_logic 
    );
end entity;

architecture rtl of ELF_PHY_GCLK is
    begin
    end;


