//---------------------------------------------//
// $File Name : cwc_top.v
// $Author    : hsun
// $Company   : Anlogic
// $Version   : v1.5
// $Date      : 2018/11/15
//---------------------------------------------//

module cwc_top(
//configure interface
  jtdi,
  jtck,
  jrstn,
  jscan,
  jshift,
  jupdate,
  jtdo,
//internal trigger interface
  non_bus_din,
  bus_din,
  trig_clk,
//to bram block
  wt_ce,
  wt_en,
  wt_addr
);
//parameters  
  parameter   CTRL_REG_LEN = 93;      //control register length
  parameter   STAT_REG_LEN = 18;      //status register length
  parameter   STOP_LEN = 100;         //(1-trig_position)*trig_length //trig_postion 1/3 1/2 2/3 
  parameter   NON_BUS_NODE_NUM = 1;   //non-bus trigger node number
  parameter   BUS_NODE_NUM = 8;       //bus trigger node number
  parameter   BUS_NUM = 1;            //bus number
  parameter   BUS1_WIDTH = 8;         //1st bus width
  parameter   BUS2_WIDTH = 8;         //2nd bus width
  parameter   BUS3_WIDTH = 8;         //3rd bus width
  parameter   BUS4_WIDTH = 8;         //4th bus width
  parameter   BUS5_WIDTH = 8;         //5th bus width
  parameter   BUS6_WIDTH = 8;         //6th bus width
  parameter   BUS7_WIDTH = 8;         //7th bus width
  parameter   BUS8_WIDTH = 8;         //8th bus width


//configure interface
  input        jtdi;
  input        jtck;
  input        jrstn;
  input [1:0]  jscan;
  input        jshift;
  input        jupdate;

  output [1:0] jtdo;

//internal trigger interface
  input [NON_BUS_NODE_NUM-1:0]  non_bus_din;
  input [BUS_NODE_NUM-1:0]      bus_din;  
  input                         trig_clk;
//to bram block
  output        wt_ce;
  output        wt_en;
  output [15:0] wt_addr;
//
  wire [CTRL_REG_LEN-1:0]   control;
  wire [STAT_REG_LEN-1:0]   status;


  cfg_int #(
      .CTRL_REG_LEN(CTRL_REG_LEN),
      .STAT_REG_LEN(STAT_REG_LEN)
  ) cfg_int_inst (
      .jtdi(jtdi),
      .jtck(jtck),
      .jrstn(jrstn),
      .jscan(jscan),
      .jshift(jshift),
      .jupdate(jupdate),
      .jtdo(jtdo),
      .status(status),
      .control(control)
  );

  trigger #(
      .CTRL_REG_LEN(CTRL_REG_LEN),
      .NON_BUS_NODE_NUM(NON_BUS_NODE_NUM),
      .BUS_NODE_NUM(BUS_NODE_NUM),
      .BUS_NUM(BUS_NUM),
      .BUS1_WIDTH(BUS1_WIDTH),
      .BUS2_WIDTH(BUS2_WIDTH),
      .BUS3_WIDTH(BUS3_WIDTH),
      .BUS4_WIDTH(BUS4_WIDTH),
      .BUS5_WIDTH(BUS5_WIDTH),
      .BUS6_WIDTH(BUS6_WIDTH),
      .BUS7_WIDTH(BUS7_WIDTH),
      .BUS8_WIDTH(BUS8_WIDTH),
      .STOP_LEN(STOP_LEN)
  ) trigger_inst (
      .trig_clk(trig_clk),
      .non_bus_din(non_bus_din),
      .bus_din(bus_din),
      .control(control),
      .overflow_flag(status[16]),
      .stop_flag(status[17]),
      .stop_addr(status[15:0]),
      .wt_ce(wt_ce),
      .wt_en(wt_en),
      .wt_addr(wt_addr)
  );
    
endmodule
