//-----------------------------------------------------//
// $File Name : detect_bus.v
// $Author    : hsun
// $Company   : Anlogic
// $Version   : v1.5
// $Date      : 2019/07/13
//-----------------------------------------------------//
//
// bus_logic          description
//   000                   ==
//   001                   !=
//   010                    <
//   011                   <=
//   100                    >
//   101                   >=
//   110                  true  --> dont care in AND logic
//   111                 false  --> dont care in OR logic
//----------------------------------------------------//

module detect_bus(
  clk,
  rstn,
  force_acq_fin,
  bus_din,       //bus signal 
  bus_logic,
  bus_value,
  trigger_bus_en
);
  
  parameter    BUS_WIDTH = 8;

  //inputs
  input	     clk;
  input      rstn;
  input      force_acq_fin;
  
  input [2:0]   bus_logic;
  
  input [BUS_WIDTH-1:0]  bus_value;
  input [BUS_WIDTH-1:0]  bus_din;

  //output
  output     trigger_bus_en;

  reg                   trigger_bus_en;
  reg [BUS_WIDTH-1:0]   bus_din_reg;

  always @(posedge clk or negedge rstn)
  begin
	if(~rstn) begin
             bus_din_reg <= 0;
        end
        else begin
             bus_din_reg <= bus_din;
        end	
  end

  always @(bus_logic or bus_din_reg or bus_value or force_acq_fin) 
  begin
        if(force_acq_fin == 1'b0) begin
             trigger_bus_en = 1'b0;
        end
        else begin
	     case(bus_logic)
             3'b000: begin
                       if(bus_din_reg == bus_value) begin
                             trigger_bus_en = 1'b1;
                       end
                       else begin
                             trigger_bus_en = 1'b0;
                       end
                     end
             3'b001: begin
                       if(bus_din_reg != bus_value) begin
                             trigger_bus_en = 1'b1;
                       end
                       else begin
                             trigger_bus_en = 1'b0;
                       end
                     end
             3'b010: begin
                       if(bus_din_reg < bus_value) begin
                             trigger_bus_en = 1'b1;
                       end
                       else begin
                             trigger_bus_en = 1'b0;
                       end 
                     end
             3'b011: begin
                       if(bus_din_reg <= bus_value) begin
                            trigger_bus_en = 1'b1;
                       end
                       else begin
                            trigger_bus_en = 1'b0;
                       end
                     end
             3'b100: begin
                       if(bus_din_reg > bus_value) begin
                            trigger_bus_en = 1'b1;
                       end
                       else begin
                            trigger_bus_en = 1'b0;
                       end
                     end
             3'b101: begin
                       if(bus_din_reg >= bus_value) begin
                            trigger_bus_en = 1'b1;
                       end
                       else begin
                            trigger_bus_en = 1'b0;
                       end
                     end
             3'b110: begin
                            trigger_bus_en = 1'b1;
                     end
			 3'b111: begin
                            trigger_bus_en = 1'b0;
			         end
             default:begin
                            trigger_bus_en = 1'b0;
                     end
             endcase
        end
  end

endmodule

