//----------------------------------------------//
// $File Name   : detect_non_bus.v
// $Author      : hsun
// $Company     : Anlogic
// $Version     : v1.5
// $Date        : 2019/07/26
//----------------------------------------------//
//  node_logic      description
//      0               ==(equal)
//      1             !=(not equal)
//
//  edge_mode          description
//     000               posedge
//     001               negedge
//     010              low level
//     011              high level
//     100              double edge
//     101                 true    --> dont care in AND logic
//     110                false    --> dont care in OR logic
//     111               reserved
//----------------------------------------------//

module detect_non_bus(
   clk,
   rstn,
   node_din,
   force_acq_fin,
   node_logic,
   edge_mode,
   trigger_node_en
);

   input   clk;
   input   rstn;
   input   node_din;
   input   force_acq_fin;

   input        node_logic;
   input [2:0]  edge_mode;
   
   output  trigger_node_en;

   reg     trigger_node_en;
   reg     ctl_this;
   reg     ctl_last;
   reg     level_0_r;
   reg     level_1_r;
   reg     true_r;
   reg     false_r;


  always @(posedge clk or negedge rstn)
  begin
        if(~rstn) begin
             ctl_this  <= 1'b0;
             ctl_last  <= 1'b0;
             level_0_r <= 1'b1;
             level_1_r <= 1'b0;
             true_r <= 1'b0;
			 false_r <= 1'b1;
        end
        else begin
             ctl_this  <= node_din;
             ctl_last  <= ctl_this;
             level_0_r <= node_din;
             level_1_r <= node_din;
             true_r <= 1'b1;
			 false_r <= 1'b0;
        end
  end

  always @(*)
  begin
       if(force_acq_fin == 1'b0) begin
            trigger_node_en = 1'b0;
       end
       else begin
            case(edge_mode)
              3'b000: begin
                           trigger_node_en = (node_logic == 1'b0) ? (ctl_this & (!ctl_last)) : !(ctl_this & (!ctl_last));
                      end  
              3'b001: begin
                           trigger_node_en = (node_logic == 1'b0) ? ((!ctl_this) & ctl_last) : !((!ctl_this) & ctl_last);
                      end
              3'b010: begin
                           trigger_node_en = (node_logic == 1'b0) ? !level_0_r : level_0_r;
                      end
              3'b011: begin
                           trigger_node_en = (node_logic == 1'b0) ? level_1_r : !level_1_r;
                      end
              3'b100: begin
                           trigger_node_en = (node_logic == 1'b0) ? (ctl_this & !ctl_last) | (!ctl_this & ctl_last) : !((ctl_this & !ctl_last) | (!ctl_this & ctl_last));
                      end
              3'b101: begin
                           trigger_node_en = (node_logic == 1'b0) ? true_r : !true_r;
                      end
			  3'b110: begin
						   trigger_node_en = (node_logic == 1'b0) ? false_r : !false_r;
			          end
              default:begin
                           trigger_node_en = 1'b0;
                      end
            endcase        
       end
  end


endmodule
