//-------------------------------------------------//
// $Module Name: emb_ctrl.v
// $Author: hsun
// $Date: 2018/11/15
//-------------------------------------------------//

module emb_ctrl(
  clk,     //triggger clk
  rstn,
  emb_store_en,
  pause,
  trig_length,
  overflow,
  wt_addr,
  wt_ce,
  wt_en,
  stop 
);
 
  parameter    STOP_LEN = 100;    //(1-trig_position)*trig_length
  
  input         clk;
  input         rstn;
  input         emb_store_en;
  input         pause;
  input [15:0]  trig_length;
  
  output        overflow;
  output [15:0] wt_addr;
  output        wt_ce;
  output        wt_en;
  output        stop;

  reg [15:0]    addr_tmp;
  reg [19:0]    addr_long;
  
  reg           wt_ce;
  reg           wt_en;
  reg           stop;
  reg           overflow;
  reg [15:0]    stop_count;

  always @(posedge clk or negedge rstn)
  begin
       if(~rstn)
       begin
            wt_ce <= 1'b0;
            wt_en <= 1'b0;
       end
       else if( stop | pause)
       begin
            wt_ce <= 1'b0;
            wt_en <= 1'b0;
       end
       else begin
            wt_ce <= 1'b1;
            wt_en <= 1'b1;
       end
  end

  assign   wt_addr = addr_long[15:0];

  always @(posedge clk or negedge rstn)
  begin
       if(~rstn) begin
           addr_tmp <= 16'd0;
       end
       else begin
           addr_tmp <= wt_addr;
       end
  end

  always @(posedge clk or negedge rstn)
  begin
       if(~rstn) begin
            overflow <= 1'b0;
       end
       else if((addr_tmp == (trig_length - 1'b1)) & (~stop))
       begin
             overflow <= 1'b1;
       end
  end
  
  always @(posedge clk or negedge rstn)
  begin
       if(~rstn) begin
            addr_long <= 20'hfffff;
       end
       else if((addr_long[15:0] == (trig_length - 1'b1)) & (~stop))
       begin
            addr_long <= 20'b0;
       end
       else if(~stop & ~pause)
       begin
            addr_long <= addr_long + 1'b1;
       end
  end

  always @(posedge clk or negedge rstn)
  begin
       if(~rstn) begin
            stop_count <= 16'b0;
       end
       else if(stop)
       begin
            stop_count <= 16'b0;
       end
       else if(emb_store_en)
       begin
            stop_count <= stop_count + 16'b1;
       end 
  end

  always @(posedge clk or negedge rstn)
  begin
       if(~rstn) begin
            stop <= 1'b0;
       end
       else if( stop_count >= STOP_LEN-5 )
       begin
            stop <= 1'b1;
       end
  end

endmodule
