//----------------------------------------------//
// $Module Name: register.v
// $Author: hsun
// $Date: 2018/11/15
//----------------------------------------------//

module register(
  jtdi,
  jtck,
  rst,
  scan_1,
  shift_0,
  shift_1,
  update_0,
  status,
  jtdo,
  ctrl_r
);

 parameter CTRL_REG_LEN = 92;
 parameter STAT_REG_LEN = 18;

 output [1:0]              jtdo;
 output [CTRL_REG_LEN-1:0] ctrl_r;

 input         jtdi;
 input         jtck;
 input         rst;
 input         scan_1;
 input         shift_0;
 input         shift_1;
 input         update_0;

 input [STAT_REG_LEN-1:0]   status;

 reg   [CTRL_REG_LEN-1:0]   ctrl_r;
 reg   [CTRL_REG_LEN-1:0]   cshift_r;
 reg   [STAT_REG_LEN-1:0]   sshift_r2;
 
 wire          do_0;
 wire          do_1;

 always @(posedge jtck or posedge rst)
 begin
      if(rst)
      begin
	   cshift_r <= 0;
      end
      else if(shift_0)//ctrl reg shift in shift_0 = 1, but data will not go to tdo of cfg, because cfg prohibit
      begin
           cshift_r <= {jtdi, cshift_r[CTRL_REG_LEN-1:1]};
      end
 end

 always @(posedge jtck or posedge rst)
 begin
      if(rst)
      begin
	   sshift_r2 <= 18'b0;
      end
      else if(scan_1)
      begin
	   sshift_r2 <= status;
      end
      else if(shift_1)//status reg shift in shift_1 = 1
      begin
	   sshift_r2 <= {1'b0, sshift_r2[STAT_REG_LEN-1:1]};
      end
 end
 
//ctrl reg
 always @(posedge jtck or posedge rst)
 begin
      if(rst)
      begin
	   ctrl_r <= 0;
      end
      else if(update_0)
      begin
	   ctrl_r <= cshift_r;
      end
 end

 assign do_0 = cshift_r[0];
 assign do_1 = sshift_r2[0];
 assign jtdo[0] = do_0 & shift_0; 
 assign jtdo[1] = do_1 & shift_1;    

endmodule

 
