//   Vendor : Anlogic
//   Version : 0.1
//   Description : Anlogic Functional Simulation Library Component
//            block RAM
//   Filename : al_phy_bram.v
//   Timestamp : Mon Nov  4 14:20:41 CST 2013
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    11/04/13 - Initial version.
//    04/04/14 - Update RAM initial parameter.	
//    04/23/14 - Update parameter : GSRMODE to GSR
///////////////////////////////////////////////////////////////////////////////
`timescale 1ns / 1ps
module AL_PHY_BRAM32K ( 
  doa, dia, addra, bytea, cena, wena, clka, rsta, ocea,
  dob, dib, addrb, byteb, cenb, wenb, clkb, rstb, oceb
  );

output  [15:0] doa;
output  [15:0] dob;
input   [15:0] dia;
input   [15:0] dib;
input   [10:0] addra;
input   [10:0] addrb;
input   bytea; // port a byte mode, lowest address
input   byteb; // port b byte mode, lowest address
input   cena,wena; // 0 is valid
input   clka,rsta;
input   cenb,wenb; // 0 is valid
input   clkb,rstb;
input   ocea;  // port a output register ce input when REGMODE_A=="OUTREG"
input   oceb;  // port B output register ce input when REGMODE_B=="OUTREG"

tri1 	[15:0] dia;
tri1 	[15:0] dib;
tri1    cena, ocea, clka, wena;
tri1    cenb, oceb, clkb, wenb;
tri1  	rsta,rstb;         
tri1 	[10:0] addra;
tri1 	[10:0] addrb;
tri1 	bytea;
tri1 	byteb;


parameter MODE = "DP16K";               // DP16K, SP16K
parameter DATA_WIDTH_A = "16";          // 8, 16
parameter DATA_WIDTH_B = "16";          // 8, 16

parameter REGMODE_A = "NOREG";         // "NOREG", "OUTREG"
parameter REGMODE_B = "NOREG";         // "NOREG", "OUTREG"
parameter WRITEMODE_A = "NORMAL";      // "NORMAL", "WRITETHROUGH"
parameter WRITEMODE_B = "NORMAL";      // "NORMAL", "WRITETHROUGH"
parameter SRMODE = "SYNC";          // "SYNC", "ASYNC"

parameter CENAMUX = "SIG"; //0, 1, INV, SIG
parameter CENBMUX = "SIG"; //0, 1, INV, SIG
parameter OCEAMUX = "SIG"; //0, 1, INV, SIG
parameter OCEBMUX = "SIG"; //0, 1, INV, SIG
parameter RSTAMUX = "SIG"; //0, 1, INV, SIG
parameter RSTBMUX = "SIG"; //0, 1, INV, SIG
parameter CLKAMUX = "SIG"; //0, 1, INV, SIG
parameter CLKBMUX = "SIG"; //0, 1, INV, SIG
parameter WENAMUX = "SIG"; //0, 1, INV, SIG
parameter WENBMUX = "SIG"; //0, 1, INV, SIG
parameter READBACK = "OFF"; // ON, OFF

  reg rbyte1_reg_a,rbyte1_reg_b;
  wire asyncrst_a,asyncrst_b;
  wire syncrst_a,syncrst_b;
  reg [15:0] doa_reg, dob_reg;
  reg rd_reg_a, rd_reg_b; 
  reg bytemode_a, bytemode_b;
  reg oregmode_a, oregmode_b;
  reg syncrst;
         
  wire  bwena_odd;
  wire  bwena_even; 
  wire  [15:0]  BWENAMUX ;
  wire  bwenb_odd ;
  wire  bwenb_even;    
  wire  [15:0]  BWENBMUX;  
  reg [15:0] bramdi_a,bramdi_b;  
  wire [15:0] doa_tmp,dob_tmp;
  reg  CLKAMUX_int;
  reg  CLKBMUX_int;
  reg  CENAMUX_int;
  reg  CENBMUX_int;
  reg  WENAMUX_int;
  reg  WENBMUX_int;
  reg  OCEAMUX_int;
  reg  OCEBMUX_int;  
  reg  rsta_int;
  reg  rstb_int; 
  reg  f_wta;
  reg  f_wtb;
  localparam	Bits = 16;
  localparam	Word_Depth = 2048;
  localparam	Add_Width = 11;
  localparam    Wen_Width = 16;
  localparam    Word_Pt = 1;
  reg [Bits-1:0] 	QA_slip;
  reg [Bits-1:0] 	QB_slip; 
  wire [Bits-1:0] 	QA_int;
  wire [Bits-1:0] 	QB_int;
  wire [Add_Width-1:0] 	AA_int;
  wire [Add_Width-1:0] 	AB_int;

  wire [Wen_Width-1:0]  BWENAMUX_int;
  wire [Wen_Width-1:0]  BWENBMUX_int;
  wire [Bits-1:0] 	DA_int;
  wire [Bits-1:0] 	DB_int;

  reg  [Bits-1:0] 	QA_latched;
  reg  [Bits-1:0] 	QB_latched;
  reg  [Add_Width-1:0] 	AA_latched;
  reg  [Add_Width-1:0] 	AB_latched;
  reg  [Bits-1:0] 	DA_latched;
  reg  [Bits-1:0] 	DB_latched;
  reg                  	CENAMUX_latched;
  reg                  	CENBMUX_latched;
  reg                  	LAST_CLKAMUX;
  reg                  	LAST_CLKBMUX;
  reg                  	WENAMUX_latched;
  reg                  	WENBMUX_latched;
  reg [Wen_Width-1:0]  BWENAMUX_latched;
  reg [Wen_Width-1:0]  BWENBMUX_latched;

  reg [Bits-1:0]        data_tmpa;
  reg [Bits-1:0]        data_tmpb;
  reg [Bits-1:0] 	mem_array[Word_Depth-1:0];

  wire [15:0] doa_t,dob_t;
  
  buf (doa[0],  doa_t[0]);
  buf (doa[1],  doa_t[1]);
  buf (doa[2],  doa_t[2]);
  buf (doa[3],  doa_t[3]);
  buf (doa[4],  doa_t[4]);
  buf (doa[5],  doa_t[5]);
  buf (doa[6],  doa_t[6]);
  buf (doa[7],  doa_t[7]);
  buf (doa[8],  doa_t[8]);
  buf (doa[9],  doa_t[9]);
  buf (doa[10], doa_t[10]);
  buf (doa[11], doa_t[11]);
  buf (doa[12], doa_t[12]);
  buf (doa[13], doa_t[13]);
  buf (doa[14], doa_t[14]);
  buf (doa[15], doa_t[15]);

  buf (dob[0],  dob_t[0]);
  buf (dob[1],  dob_t[1]);
  buf (dob[2],  dob_t[2]);
  buf (dob[3],  dob_t[3]);
  buf (dob[4],  dob_t[4]);
  buf (dob[5],  dob_t[5]);
  buf (dob[6],  dob_t[6]);
  buf (dob[7],  dob_t[7]);
  buf (dob[8],  dob_t[8]);
  buf (dob[9],  dob_t[9]);
  buf (dob[10], dob_t[10]);
  buf (dob[11], dob_t[11]);
  buf (dob[12], dob_t[12]);
  buf (dob[13], dob_t[13]);
  buf (dob[14], dob_t[14]);
  buf (dob[15], dob_t[15]);  
  
  

  integer      i,j,wenn,lb,hb;
  integer      n;

   reg param_trig;
   initial begin
   param_trig=0;
   param_trig=1;
   end
 
   always @(param_trig)
   begin
        case (CENAMUX)
            "SIG"   	: assign CENAMUX_int = cena;
            "0" 	: assign CENAMUX_int = 0;
            "1" 	: assign CENAMUX_int = 1;
            "INV"   	: assign CENAMUX_int = ~cena;
            default 	: assign CENAMUX_int = 1;
        endcase

        case (CENBMUX)
            "SIG"   	: assign CENBMUX_int = cenb;
            "0" 	: assign CENBMUX_int = 0;
            "1" 	: assign CENBMUX_int = 1;
            "INV"   	: assign CENBMUX_int = ~cenb;
            default 	: assign CENBMUX_int = 1;
        endcase

        case (WENAMUX)
            "SIG"   	: assign WENAMUX_int = wena;
            "0" 	: assign WENAMUX_int = 0;
            "1" 	: assign WENAMUX_int = 1;
            "INV"   	: assign WENAMUX_int = ~wena;
            default 	: assign WENAMUX_int = 1;
        endcase

        case (WENBMUX)
            "SIG"   	: assign WENBMUX_int = wenb;
            "0" 	: assign WENBMUX_int = 0;
            "1" 	: assign WENBMUX_int = 1;
            "INV"   	: assign WENBMUX_int = ~wenb;
            default 	: assign WENBMUX_int = 1;
        endcase

        case (OCEAMUX)
            "SIG"   	: assign OCEAMUX_int = ocea;
            "0" 	: assign OCEAMUX_int = 0;
            "1" 	: assign OCEAMUX_int = 1;
            "INV"   	: assign OCEAMUX_int = ~ocea;
            default 	: assign OCEAMUX_int = 1;
        endcase

        case (OCEBMUX)
            "SIG"   	: assign OCEBMUX_int = oceb;
            "0" 	: assign OCEBMUX_int = 0;
            "1" 	: assign OCEBMUX_int = 1;
            "INV"   	: assign OCEBMUX_int = ~oceb;
            default 	: assign OCEBMUX_int = 1;
        endcase


        case (RSTAMUX)
            "SIG"   	: assign rsta_int = rsta;
            "0" 	: assign rsta_int = 0;
            "1" 	: assign rsta_int = 1;
            "INV"   	: assign rsta_int = ~rsta;
            default 	: assign rsta_int = 1;
        endcase

        case (RSTBMUX)
            "SIG"   	: assign rstb_int = rstb;
            "0" 	: assign rstb_int = 0;
            "1" 	: assign rstb_int = 1;
            "INV"   	: assign rstb_int = ~rstb;
            default 	: assign rstb_int = 1;
        endcase

        case (CLKAMUX)
            "SIG"   	: assign CLKAMUX_int = clka;
            "0" 	: assign CLKAMUX_int = 0;
            "1" 	: assign CLKAMUX_int = 1;
            "INV"   	: assign CLKAMUX_int = ~clka;
            default 	: assign CLKAMUX_int = 1;
        endcase

        case (CLKBMUX)
            "SIG"   	: assign CLKBMUX_int = clkb;
            "0" 	: assign CLKBMUX_int = 0;
            "1" 	: assign CLKBMUX_int = 1;
            "INV"   	: assign CLKBMUX_int = ~clkb;
            default 	: assign CLKBMUX_int = 1;
        endcase


    end
    
    
   initial 
   begin
   rbyte1_reg_a=0;
   rbyte1_reg_b=0;
   doa_reg=0;
   dob_reg=0;
   rd_reg_a=1;
   rd_reg_b=1;   
   case(DATA_WIDTH_A)
    "8" :  bytemode_a = 1;
    "16" : bytemode_a = 0;
    default : bytemode_a = 0;                   
   endcase
   case(DATA_WIDTH_B)
    "8" :  bytemode_b = 1;
    "16" : bytemode_b = 0;
    default : bytemode_b = 0;                   
   endcase
   case(REGMODE_A)
    "OUTREG": oregmode_a = 1;
    "NOREG" : oregmode_a = 0;
    default : oregmode_a = 0;
   endcase
   case(REGMODE_B)
    "OUTREG": oregmode_b = 1;
    "NOREG" : oregmode_b = 0;
    default : oregmode_b = 0;
   endcase 
   case(SRMODE)
    "SYNC":   syncrst = 1;
    "ASYNC" : syncrst = 0;
    default : syncrst = 1;
   endcase    
   case(WRITEMODE_A)
    "NORMAL":   f_wta = 0;
    "WRITETHROUGH" : f_wta = 1;
    default : f_wta = 0;
   endcase    
   case(WRITEMODE_B)
    "NORMAL":   f_wtb = 0;
    "WRITETHROUGH" : f_wtb = 1;
    default : f_wtb = 0;
   endcase    
   end
   
  assign asyncrst_a= rsta_int & !syncrst;
  assign asyncrst_b= rstb_int & !syncrst;
  assign syncrst_a= rsta_int & syncrst;
  assign syncrst_b= rstb_int & syncrst; 
//  assign bramdi_a[7:0] = dia[7:0];
//  assign bramdi_a[15:8]= bytemode_a ? dia[7:0] : dia[15:8] ;
//  assign bramdi_b[7:0] = dib[7:0];
//  assign bramdi_b[15:8]= bytemode_b ? dib[7:0] : dib[15:8] ; 
  
  always @(dia or bytemode_a)
  begin
//    genvar i;
//    generate
      for(i=0; i<8; i=i+1)
      begin
       bramdi_a[2*i]=dia[i];
       if(bytemode_a)
           bramdi_a[2*i+1]=dia[i];
       else
           bramdi_a[2*i+1]=dia[i+8];   
      end      
  end
  always @(dib or bytemode_b)
  begin
//    genvar i;
//    generate
      for(i=0; i<8; i=i+1)
      begin
       bramdi_b[2*i]=dib[i];
       if(bytemode_b)
           bramdi_b[2*i+1]=dib[i];
       else
           bramdi_b[2*i+1]=dib[i+8];   
      end      
  end  
  
  
  assign  bwena_odd = !((!WENAMUX_int) & !(bytemode_a & !bytea));
  assign  bwena_even= !((!WENAMUX_int) & !(bytemode_a & bytea)); 
  assign  BWENAMUX = {8{bwena_odd,bwena_even}};
  assign  bwenb_odd = !((!WENBMUX_int) & !(bytemode_b & !byteb));
  assign  bwenb_even= !((!WENBMUX_int) & !(bytemode_b & byteb));    
  assign  BWENBMUX = {8{bwenb_odd,bwenb_even}};    

  buf bwena_buf[Wen_Width-1:0] (BWENAMUX_int, BWENAMUX);
  buf bwenb_buf[Wen_Width-1:0] (BWENBMUX_int, BWENBMUX);
  buf aa_buf[Add_Width-1:0] (AA_int, addra);
  buf ab_buf[Add_Width-1:0] (AB_int, addrb);
  buf da_buf[Bits-1:0] (DA_int, bramdi_a);   
  buf db_buf[Bits-1:0] (DB_int, bramdi_b);   

  
  assign QA_int=QA_latched;
  assign QB_int=QB_latched;
  
  wire    clkconfA_flag;
  wire    clkconfB_flag;
  wire    clkconf_flag;
  assign clkconfA_flag=(AA_int===AB_latched) && (CENAMUX_int!==1'b1) && (CENBMUX_latched!==1'b1);
  assign clkconfB_flag=(AB_int===AA_latched) && (CENBMUX_int!==1'b1) && (CENAMUX_latched!==1'b1);
  assign clkconf_flag=(AA_int===AB_int) && (CENAMUX_int!==1'b1) && (CENBMUX_int!==1'b1);


  
   always @(CLKAMUX_int)
    begin
      casez({LAST_CLKAMUX, CLKAMUX_int})
        2'b01: begin
          CENAMUX_latched = CENAMUX_int;
          WENAMUX_latched = WENAMUX_int;
          BWENAMUX_latched = BWENAMUX_int;
          AA_latched = AA_int;
          DA_latched = DA_int;
          rw_memA;
        end
        2'b10,
        2'bx?,
        2'b00,
        2'b11: ;
        2'b?x: begin
	  for(i=0;i<Word_Depth;i=i+1)
    	    mem_array[i]={Bits{1'bx}};
    	  QA_latched={Bits{1'bx}};
          rw_memA;
          end
      endcase
    LAST_CLKAMUX=CLKAMUX_int;
   end

always @(CLKBMUX_int)
    begin
      casez({LAST_CLKBMUX, CLKBMUX_int})
        2'b01: begin
          CENBMUX_latched = CENBMUX_int;
          WENBMUX_latched = WENBMUX_int;
          BWENBMUX_latched = BWENBMUX_int;
          AB_latched = AB_int;
          DB_latched = DB_int;
          rw_memB;
        end
        2'b10,
        2'bx?,
        2'b00,
        2'b11: ;
        2'b?x: begin
          for(i=0;i<Word_Depth;i=i+1)
    	    mem_array[i]={Bits{1'bx}};
	    QB_latched={Bits{1'bx}};
            rw_memA;
          end
      endcase
    LAST_CLKBMUX=CLKBMUX_int;
   end


   always @(posedge CLKAMUX_int)
   begin
      if (CENAMUX_int == 0)
         rbyte1_reg_a <= bytea & bytemode_a;
   end
   always @(posedge CLKBMUX_int)
   begin
      if (CENBMUX_int == 0)
         rbyte1_reg_b <= byteb & bytemode_b;
   end
   always @(posedge CLKAMUX_int)
   begin
      if (CENAMUX_int == 0)
         rd_reg_a <= WENAMUX_int;
   end
   always @(posedge CLKBMUX_int)
   begin
      if (CENBMUX_int == 0)
         rd_reg_b <= WENBMUX_int;
   end   

   wire oprcea = (oregmode_a ? OCEAMUX_int : rd_reg_a) & (rd_reg_a|f_wta);
   wire oprceb = (oregmode_b ? OCEBMUX_int : rd_reg_b) & (rd_reg_b|f_wtb);
   wire osela = (rd_reg_a|f_wta) & !oregmode_a;
   wire oselb = (rd_reg_b|f_wtb) & !oregmode_b;
   

   
   always @(QA_int)
   begin
//    genvar i;
//    generate
      for(i=0; i<8; i=i+1)
      begin
       QA_slip[i]=QA_int[2*i];
       QA_slip[i+8]=QA_int[2*i+1];
      end      
   end

   always @(QB_int)
   begin
//    genvar i;
//    generate
      for(i=0; i<8; i=i+1)
      begin
       QB_slip[i]=QB_int[2*i];
       QB_slip[i+8]=QB_int[2*i+1];
      end      
   end
      
   always @(posedge asyncrst_a or posedge CLKAMUX_int)
   begin
      
      if (asyncrst_a == 1)
         doa_reg <= 0;
      else
      begin
         if (oprcea)
            begin
               if(syncrst_a) doa_reg <= 0;
               else
                    doa_reg[15:0] <= doa_tmp[15:0];               
            end
	 else
	   doa_reg <= doa_reg ;
      end
   end



   always @(posedge asyncrst_b or posedge CLKBMUX_int)
   begin
      if (asyncrst_b == 1)
         dob_reg <= 0;
      else
      begin
         if (oprceb)
            begin
               if(syncrst_b) dob_reg <= 0;
               else
               dob_reg[15:0] <= dob_tmp[15:0];                
            end
         else
            dob_reg <= dob_reg ;
      end
   end
   
   assign doa_tmp[15:8]  = {8{(!bytemode_a)}} & QA_slip[15:8];
   assign doa_tmp[7:0] =  rbyte1_reg_a ? QA_slip[15:8] : QA_slip[7:0] ;  

   assign dob_tmp[15:8]  = {8{(!bytemode_b)}} & QB_slip[15:8];
   assign dob_tmp[7:0] =  rbyte1_reg_b ? QB_slip[15:8] : QB_slip[7:0] ;     
   
   assign doa_t = osela ? doa_tmp : doa_reg;   
   assign dob_t = oselb ? dob_tmp : dob_reg;
   
  task rw_memA;
    begin
      if(CENAMUX_latched==1'b0)
        begin
          if (WENAMUX_latched==1'b1)
            begin
              if(^(AA_latched)==1'bx)
                QA_latched={Bits{1'bx}};
              else
                QA_latched=mem_array[AA_latched];
            end
          else if (WENAMUX_latched==1'b0)
          begin
            for (wenn=0; wenn<Wen_Width; wenn=wenn+1)
              begin
                lb=wenn*Word_Pt;
                if ( (lb+Word_Pt) >= Bits) hb=Bits-1;
                else hb=lb+Word_Pt-1;
                if (BWENAMUX_latched[wenn]==1'b1)
                  begin
                    if(^(AA_latched)==1'bx)
                      for (i=lb; i<=hb; i=i+1) QA_latched[i]=1'bx;
                    else
                      begin
                      data_tmpa=mem_array[AA_latched];
                      for (i=lb; i<=hb; i=i+1) QA_latched[i]=data_tmpa[i];
                      end
                  end
                else if (BWENAMUX_latched[wenn]==1'b0)
                  begin
                    if (^(AA_latched)==1'bx)
                      begin
                        for (i=0; i<Word_Depth; i=i+1)
                          begin
                            data_tmpa=mem_array[i];
                            for (j=lb; j<=hb; j=j+1) data_tmpa[j]=1'bx;
                            mem_array[i]=data_tmpa;
                          end
                        for (i=lb; i<=hb; i=i+1) QA_latched[i]=1'bx;
                      end
                    else
                      begin
                        data_tmpa=mem_array[AA_latched];
                        for (i=lb; i<=hb; i=i+1) data_tmpa[i]=DA_latched[i];
                        mem_array[AA_latched]=data_tmpa;
                        for (i=lb; i<=hb; i=i+1) QA_latched[i]=data_tmpa[i];
                      end
                  end
                else
                  begin
                    for (i=lb; i<=hb;i=i+1) QA_latched[i]=1'bx;
                    if (^(AA_latched)==1'bx)
                      begin
                        for (i=0; i<Word_Depth; i=i+1)
                          begin
                            data_tmpa=mem_array[i];
                            for (j=lb; j<=hb; j=j+1) data_tmpa[j]=1'bx;
                            mem_array[i]=data_tmpa;
                          end
                      end
                    else
                      begin
                        data_tmpa=mem_array[AA_latched];
                        for (i=lb; i<=hb; i=i+1) data_tmpa[i]=1'bx;
                        mem_array[AA_latched]=data_tmpa;
                      end
                 end
               end
             end
           else
             begin
               for (wenn=0; wenn<Wen_Width; wenn=wenn+1)
               begin
                 lb=wenn*Word_Pt;
                 if ( (lb+Word_Pt) >= Bits) hb=Bits-1;
                 else hb=lb+Word_Pt-1;
                 if (BWENAMUX_latched[wenn]==1'b1)
                  begin
                    if(^(AA_latched)==1'bx)
                      for (i=lb; i<=hb; i=i+1) QA_latched[i]=1'bx;
                    else
                      begin
                      data_tmpa=mem_array[AA_latched];
                      for (i=lb; i<=hb; i=i+1) QA_latched[i]=data_tmpa[i];
                      end
                  end
                else
                  begin
                    for (i=lb; i<=hb;i=i+1) QA_latched[i]=1'bx;
                    if (^(AA_latched)==1'bx)
                      begin
                        for (i=0; i<Word_Depth; i=i+1)
                          begin
                            data_tmpa=mem_array[i];
                            for (j=lb; j<=hb; j=j+1) data_tmpa[j]=1'bx;
                            mem_array[i]=data_tmpa;
                          end
                      end
                    else
                      begin
                        data_tmpa=mem_array[AA_latched];
                        for (i=lb; i<=hb; i=i+1) data_tmpa[i]=1'bx;
                        mem_array[AA_latched]=data_tmpa;
                      end
                 end
               end
             end
           end
         else if (CENAMUX_latched==1'bx)
           begin
             for (wenn=0;wenn<Wen_Width;wenn=wenn+1)
            begin
              lb=wenn*Word_Pt;
              if ((lb+Word_Pt)>=Bits) hb=Bits-1;
              else hb=lb+Word_Pt-1;
              if(WENAMUX_latched==1'b1 || BWENAMUX_latched[wenn]==1'b1)
                for (i=lb;i<=hb;i=i+1) QA_latched[i]=1'bx;
              else
                begin
                  for (i=lb;i<=hb;i=i+1) QA_latched[i]=1'bx;
                  if(^(AA_latched)==1'bx)
                    begin
                      for (i=0;i<Word_Depth;i=i+1)
                        begin
                          data_tmpa=mem_array[i];
                          for (j=lb;j<=hb;j=j+1) data_tmpa[j]=1'bx;
                          mem_array[i]=data_tmpa;
                        end
                    end
                  else
                    begin
                      data_tmpa=mem_array[AA_latched];
                      for (i=lb;i<=hb;i=i+1) data_tmpa[i]=1'bx;
                      mem_array[AA_latched]=data_tmpa;
                    end
                end
            end
        end
    end
  endtask
  
task rw_memB;
    begin
      if(CENBMUX_latched==1'b0)
        begin
          if (WENBMUX_latched==1'b1)
            begin
              if(^(AB_latched)==1'bx)
                QB_latched={Bits{1'bx}};
              else
                QB_latched=mem_array[AB_latched];
            end
          else if (WENBMUX_latched==1'b0)
          begin
            for (wenn=0; wenn<Wen_Width; wenn=wenn+1)
              begin
                lb=wenn*Word_Pt;
                if ( (lb+Word_Pt) >= Bits) hb=Bits-1;
                else hb=lb+Word_Pt-1;
                if (BWENBMUX_latched[wenn]==1'b1)
                  begin
                    if(^(AB_latched)==1'bx)
                      for (i=lb; i<=hb; i=i+1) QB_latched[i]=1'bx;
                    else
                      begin
                      data_tmpb=mem_array[AB_latched];
                      for (i=lb; i<=hb; i=i+1) QB_latched[i]=data_tmpb[i];
                      end
                  end
                else if (BWENBMUX_latched[wenn]==1'b0)
                  begin
                    if (^(AB_latched)==1'bx)
                      begin
                        for (i=0; i<Word_Depth; i=i+1)
                          begin
                            data_tmpb=mem_array[i];
                            for (j=lb; j<=hb; j=j+1) data_tmpb[j]=1'bx;
                            mem_array[i]=data_tmpb;
                          end
                        for (i=lb; i<=hb; i=i+1) QB_latched[i]=1'bx;
                      end
                    else
                      begin
                        data_tmpb=mem_array[AB_latched];
                        for (i=lb; i<=hb; i=i+1) data_tmpb[i]=DB_latched[i];
                        mem_array[AB_latched]=data_tmpb;
                        for (i=lb; i<=hb; i=i+1) QB_latched[i]=data_tmpb[i];
                      end
                  end
                else
                  begin
                    for (i=lb; i<=hb;i=i+1) QB_latched[i]=1'bx;
                    if (^(AB_latched)==1'bx)
                      begin
                        for (i=0; i<Word_Depth; i=i+1)
                          begin
                            data_tmpb=mem_array[i];
                            for (j=lb; j<=hb; j=j+1) data_tmpb[j]=1'bx;
                            mem_array[i]=data_tmpb;
                          end
                      end
                    else
                      begin
                        data_tmpb=mem_array[AB_latched];
                        for (i=lb; i<=hb; i=i+1) data_tmpb[i]=1'bx;
                        mem_array[AB_latched]=data_tmpb;
                      end
                 end
               end
             end
           else
             begin
               for (wenn=0; wenn<Wen_Width; wenn=wenn+1)
               begin
                 lb=wenn*Word_Pt;
                 if ( (lb+Word_Pt) >= Bits) hb=Bits-1;
                 else hb=lb+Word_Pt-1;
                 if (BWENBMUX_latched[wenn]==1'b1)
                  begin
                    if(^(AB_latched)==1'bx)
                      for (i=lb; i<=hb; i=i+1) QB_latched[i]=1'bx;
                    else
                      begin
                      data_tmpb=mem_array[AB_latched];
                      for (i=lb; i<=hb; i=i+1) QB_latched[i]=data_tmpb[i];
                      end
                  end
                else
                  begin
                    for (i=lb; i<=hb;i=i+1) QB_latched[i]=1'bx;
                    if (^(AB_latched)==1'bx)
                      begin
                        for (i=0; i<Word_Depth; i=i+1)
                          begin
                            data_tmpb=mem_array[i];
                            for (j=lb; j<=hb; j=j+1) data_tmpb[j]=1'bx;
                            mem_array[i]=data_tmpb;
                          end
                      end
                    else
                      begin
                        data_tmpb=mem_array[AB_latched];
                        for (i=lb; i<=hb; i=i+1) data_tmpb[i]=1'bx;
                        mem_array[AB_latched]=data_tmpb;
                      end
                 end
               end
             end
           end
         else if (CENBMUX_latched==1'bx)
           begin
             for (wenn=0;wenn<Wen_Width;wenn=wenn+1)
            begin
              lb=wenn*Word_Pt;
              if ((lb+Word_Pt)>=Bits) hb=Bits-1;
              else hb=lb+Word_Pt-1;
              if(WENBMUX_latched==1'b1 || BWENBMUX_latched[wenn]==1'b1)
                for (i=lb;i<=hb;i=i+1) QB_latched[i]=1'bx;
              else
                begin
                  for (i=lb;i<=hb;i=i+1) QB_latched[i]=1'bx;
                  if(^(AB_latched)==1'bx)
                    begin
                      for (i=0;i<Word_Depth;i=i+1)
                        begin
                          data_tmpb=mem_array[i];
                          for (j=lb;j<=hb;j=j+1) data_tmpb[j]=1'bx;
                          mem_array[i]=data_tmpb;
                        end
                    end
                  else
                    begin
                      data_tmpb=mem_array[AB_latched];
                      for (i=lb;i<=hb;i=i+1) data_tmpb[i]=1'bx;
                      mem_array[AB_latched]=data_tmpb;
                    end
                end
            end
        end
    end
  endtask

   task x_mem;
   begin
     for(i=0;i<Word_Depth;i=i+1)
     mem_array[i]={Bits{1'bx}};
   end
   endtask
wire done_gwe = glbl.done_gwe;
and (dia_check, !CENAMUX_int, !WENAMUX_int,done_gwe);
and (dib_check, !CENBMUX_int, !WENBMUX_int,done_gwe);
and (addra_check, !CENAMUX_int,done_gwe);
and (addrb_check, !CENBMUX_int,done_gwe);
 
 specify                                      
 
        (clka  => doa[0]) = (0:0:0, 0:0:0);
        (clka  => doa[1]) = (0:0:0, 0:0:0);
        (clka  => doa[2]) = (0:0:0, 0:0:0);
        (clka  => doa[3]) = (0:0:0, 0:0:0);
        (clka  => doa[4]) = (0:0:0, 0:0:0);
        (clka  => doa[5]) = (0:0:0, 0:0:0);
        (clka  => doa[6]) = (0:0:0, 0:0:0);
        (clka  => doa[7]) = (0:0:0, 0:0:0);
        (clka  => doa[8]) = (0:0:0, 0:0:0);
        (clka  => doa[9]) = (0:0:0, 0:0:0);                
        (clka  => doa[10]) = (0:0:0, 0:0:0);
        (clka  => doa[11]) = (0:0:0, 0:0:0);
        (clka  => doa[12]) = (0:0:0, 0:0:0);
        (clka  => doa[13]) = (0:0:0, 0:0:0);
        (clka  => doa[14]) = (0:0:0, 0:0:0);
        (clka  => doa[15]) = (0:0:0, 0:0:0);
        (clkb  => dob[0]) = (0:0:0, 0:0:0);
        (clkb  => dob[1]) = (0:0:0, 0:0:0);
        (clkb  => dob[2]) = (0:0:0, 0:0:0);
        (clkb  => dob[3]) = (0:0:0, 0:0:0);
        (clkb  => dob[4]) = (0:0:0, 0:0:0);
        (clkb  => dob[5]) = (0:0:0, 0:0:0);
        (clkb  => dob[6]) = (0:0:0, 0:0:0);
        (clkb  => dob[7]) = (0:0:0, 0:0:0);
        (clkb  => dob[8]) = (0:0:0, 0:0:0); 
        (clkb  => dob[9]) = (0:0:0, 0:0:0);
        (clkb  => dob[10]) = (0:0:0, 0:0:0);
        (clkb  => dob[11]) = (0:0:0, 0:0:0);
        (clkb  => dob[12]) = (0:0:0, 0:0:0);
        (clkb  => dob[13]) = (0:0:0, 0:0:0);
        (clkb  => dob[14]) = (0:0:0, 0:0:0);
        (clkb  => dob[15]) = (0:0:0, 0:0:0);        
        $recovery (rsta, posedge clka &&& (done_gwe!=0), 0:0:0 );
        $removal  (rsta, posedge clka &&& (done_gwe!=0), 0:0:0 );
        $recovery (rstb, posedge clkb &&& (done_gwe!=0), 0:0:0 );
        $removal  (rstb, posedge clkb &&& (done_gwe!=0), 0:0:0 );
        $recovery (rsta, negedge clka &&& (done_gwe!=0), 0:0:0 );
        $removal  (rsta, negedge clka &&& (done_gwe!=0), 0:0:0 );
        $recovery (rstb, negedge clkb &&& (done_gwe!=0), 0:0:0 );
        $removal  (rstb, negedge clkb &&& (done_gwe!=0), 0:0:0 ); 
        
                    
	$setuphold (posedge clka, posedge dia[0] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge dia[1] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge dia[2] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, posedge dia[3] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge dia[4] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge dia[5] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge dia[6] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, posedge dia[7] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, posedge dia[8] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, posedge dia[9] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge dia[10]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge dia[11]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge dia[12]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, posedge dia[13]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge dia[14]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge dia[15]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
			
	$setuphold (posedge clkb, posedge dib[0] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge dib[1] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge dib[2] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, posedge dib[3] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge dib[4] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge dib[5] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge dib[6] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, posedge dib[7] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, posedge dib[8] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, posedge dib[9] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, posedge dib[10]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge dib[11]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge dib[12]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, posedge dib[13]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge dib[14]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge dib[15]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);
                                       
	$setuphold (posedge clka, posedge addra[0]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge addra[1]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge addra[2]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, posedge addra[3]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge addra[4]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge addra[5]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge addra[6]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, posedge addra[7]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, posedge addra[8]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, posedge addra[9]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge addra[10] &&& (addra_check!=1'b0) 	, 0:0:0, 0:0:0);
                                                         
	$setuphold (posedge clkb, posedge addrb[0]  &&& (addrb_check!=1'b0), 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge addrb[1]  &&& (addrb_check!=1'b0), 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge addrb[2]  &&& (addrb_check!=1'b0), 0:0:0, 0:0:0);
	$setuphold (posedge clkb, posedge addrb[3]  &&& (addrb_check!=1'b0), 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge addrb[4]  &&& (addrb_check!=1'b0), 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge addrb[5]  &&& (addrb_check!=1'b0), 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge addrb[6]  &&& (addrb_check!=1'b0), 0:0:0, 0:0:0);
	$setuphold (posedge clkb, posedge addrb[7]  &&& (addrb_check!=1'b0), 0:0:0, 0:0:0);
	$setuphold (posedge clkb, posedge addrb[8]  &&& (addrb_check!=1'b0), 0:0:0, 0:0:0);
	$setuphold (posedge clkb, posedge addrb[9]  &&& (addrb_check!=1'b0), 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge addrb[10] &&& (addrb_check!=1'b0), 0:0:0, 0:0:0);
	
	$setuphold (posedge clka, posedge cena 	&&& (done_gwe!=0), 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge wena 	&&& (done_gwe!=0), 0:0:0, 0:0:0);	
	$setuphold (posedge clka, posedge rsta 	&&& (done_gwe!=0), 0:0:0, 0:0:0);
	$setuphold (posedge clka, posedge ocea 	&&& (done_gwe!=0), 0:0:0, 0:0:0);
	$setuphold (posedge clka, posedge bytea &&& (done_gwe!=0), 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge cenb 	&&& (done_gwe!=0), 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge wenb 	&&& (done_gwe!=0), 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, posedge rstb 	&&& (done_gwe!=0), 0:0:0, 0:0:0);
	$setuphold (posedge clkb, posedge oceb 	&&& (done_gwe!=0), 0:0:0, 0:0:0);
	$setuphold (posedge clkb, posedge byteb &&& (done_gwe!=0), 0:0:0, 0:0:0);	

	$setuphold (posedge clka, negedge dia[0] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge dia[1] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge dia[2] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, negedge dia[3] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge dia[4] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge dia[5] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge dia[6] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, negedge dia[7] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, negedge dia[8] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, negedge dia[9] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge dia[10]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge dia[11]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge dia[12]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, negedge dia[13]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge dia[14]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge dia[15]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
			
	$setuphold (posedge clkb, negedge dib[0] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge dib[1] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge dib[2] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, negedge dib[3] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge dib[4] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge dib[5] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge dib[6] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, negedge dib[7] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, negedge dib[8] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, negedge dib[9] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, negedge dib[10]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge dib[11]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge dib[12]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, negedge dib[13]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge dib[14]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge dib[15]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);
                                       
	$setuphold (posedge clka, negedge addra[0]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge addra[1]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge addra[2]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, negedge addra[3]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge addra[4]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge addra[5]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge addra[6]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, negedge addra[7]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, negedge addra[8]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clka, negedge addra[9]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge addra[10] &&& (addra_check!=1'b0) 	, 0:0:0, 0:0:0);

	$setuphold (posedge clkb, negedge addrb[0]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge addrb[1]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge addrb[2]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, negedge addrb[3]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge addrb[4]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge addrb[5]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge addrb[6]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, negedge addrb[7]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, negedge addrb[8]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clkb, negedge addrb[9]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge addrb[10] &&& (addrb_check!=1'b0) 	, 0:0:0, 0:0:0);
	
	$setuphold (posedge clka, negedge cena 	&&& (done_gwe!=0), 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge wena 	&&& (done_gwe!=0), 0:0:0, 0:0:0);	
	$setuphold (posedge clka, negedge rsta 	&&& (done_gwe!=0), 0:0:0, 0:0:0);
	$setuphold (posedge clka, negedge ocea 	&&& (done_gwe!=0), 0:0:0, 0:0:0);
	$setuphold (posedge clka, negedge bytea &&& (done_gwe!=0), 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge cenb 	&&& (done_gwe!=0), 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge wenb 	&&& (done_gwe!=0), 0:0:0, 0:0:0);	
	$setuphold (posedge clkb, negedge rstb 	&&& (done_gwe!=0), 0:0:0, 0:0:0);
	$setuphold (posedge clkb, negedge oceb 	&&& (done_gwe!=0), 0:0:0, 0:0:0);
	$setuphold (posedge clkb, negedge byteb &&& (done_gwe!=0), 0:0:0, 0:0:0);





	$setuphold (negedge clka, posedge dia[0] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge dia[1] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge dia[2] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, posedge dia[3] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge dia[4] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge dia[5] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge dia[6] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, posedge dia[7] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, posedge dia[8] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, posedge dia[9] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge dia[10]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge dia[11]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge dia[12]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, posedge dia[13]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge dia[14]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge dia[15]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
			
	$setuphold (negedge clkb, posedge dib[0] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge dib[1] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge dib[2] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, posedge dib[3] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge dib[4] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge dib[5] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge dib[6] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, posedge dib[7] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, posedge dib[8] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, posedge dib[9] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, posedge dib[10]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge dib[11]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge dib[12]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, posedge dib[13]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge dib[14]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge dib[15]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);
                                       
	$setuphold (negedge clka, posedge addra[0]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge addra[1]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge addra[2]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, posedge addra[3]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge addra[4]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge addra[5]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge addra[6]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, posedge addra[7]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, posedge addra[8]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, posedge addra[9]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge addra[10] &&& (addra_check!=1'b0) 	, 0:0:0, 0:0:0);

	$setuphold (negedge clkb, posedge addrb[0]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge addrb[1]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge addrb[2]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, posedge addrb[3]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge addrb[4]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge addrb[5]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge addrb[6]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, posedge addrb[7]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, posedge addrb[8]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, posedge addrb[9]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge addrb[10] &&& (addrb_check!=1'b0) 	, 0:0:0, 0:0:0);
	
	$setuphold (negedge clka, posedge cena 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge wena 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (negedge clka, posedge rsta 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clka, posedge ocea 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clka, posedge bytea &&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge cenb 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge wenb 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, posedge rstb 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clkb, posedge oceb 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clkb, posedge byteb &&& (done_gwe!=0) , 0:0:0, 0:0:0);	

	$setuphold (negedge clka, negedge dia[0] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge dia[1] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge dia[2] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, negedge dia[3] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge dia[4] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge dia[5] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge dia[6] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, negedge dia[7] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, negedge dia[8] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, negedge dia[9] &&& (dia_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge dia[10]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge dia[11]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge dia[12]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, negedge dia[13]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge dia[14]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge dia[15]&&& (dia_check!=1'b0) 	, 0:0:0, 0:0:0);	
			
	$setuphold (negedge clkb, negedge dib[0] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge dib[1] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge dib[2] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, negedge dib[3] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge dib[4] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge dib[5] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge dib[6] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, negedge dib[7] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, negedge dib[8] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, negedge dib[9] &&& (dib_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, negedge dib[10]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge dib[11]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge dib[12]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, negedge dib[13]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge dib[14]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge dib[15]&&& (dib_check!=1'b0) 	, 0:0:0, 0:0:0);
                                       
	$setuphold (negedge clka, negedge addra[0]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge addra[1]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge addra[2]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, negedge addra[3]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge addra[4]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge addra[5]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge addra[6]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, negedge addra[7]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, negedge addra[8]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clka, negedge addra[9]  &&& (addra_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge addra[10] &&& (addra_check!=1'b0) 	, 0:0:0, 0:0:0);

	$setuphold (negedge clkb, negedge addrb[0]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge addrb[1]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge addrb[2]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, negedge addrb[3]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge addrb[4]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge addrb[5]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge addrb[6]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, negedge addrb[7]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, negedge addrb[8]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clkb, negedge addrb[9]  &&& (addrb_check!=1'b0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge addrb[10] &&& (addrb_check!=1'b0) 	, 0:0:0, 0:0:0);
	
	$setuphold (negedge clka, negedge cena 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge wena 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (negedge clka, negedge rsta 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clka, negedge ocea 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clka, negedge bytea &&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge cenb 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge wenb 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (negedge clkb, negedge rstb 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clkb, negedge oceb 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clkb, negedge byteb &&& (done_gwe!=0) , 0:0:0, 0:0:0);
 endspecify    

endmodule

