///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011-2021 Anlogic, Inc.
// All Right Reserved.
///////////////////////////////////////////////////////////////////////////////
//
//
//   Vendor : Anlogic
//   Version : 0.1
//   Description : Anlogic Functional Simulation Library Component
//            bram
//   Filename : ef2_logic_arith.v
//   Timestamp : Fri Jan  12 14:20:41 CST 2018
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    01/12/18 - Initial version.
//
///////////////////////////////////////////////////////////////////////////////
`timescale 1 ns / 1 ps

module EF2_LOGIC_SEQ_DIV (clk, rst, start, numer, denom, quotient, remain, done);

	parameter NUMER_WIDTH = 16;
  parameter DENOM_WIDTH = 16;

  input                     clk;
  input                     rst;  
  input                     start;  
  input  [NUMER_WIDTH-1:0]  numer;  
  input  [DENOM_WIDTH-1:0]  denom;  
  output [NUMER_WIDTH-1:0]  quotient;  
  output [DENOM_WIDTH-1:0]  remain;  
  output                    done;

  reg    [NUMER_WIDTH-1:0]  quotient;
  reg                       done;
  reg    [NUMER_WIDTH-1:0]  state;
  reg    [NUMER_WIDTH-1:0]  remain_r;
  reg    [NUMER_WIDTH+DENOM_WIDTH-1:0]  denomtemp;
  
  assign remain = remain_r;
  wire [NUMER_WIDTH+DENOM_WIDTH-1:0] accum_minus_bpower = remain_r - denomtemp;

  always @(posedge clk or posedge rst) begin
    if (rst) begin
      quotient  <= 0;
      remain_r  <= 0;
      state     <= 0;
      denomtemp <= 0;
      done      <= 1;
    end
    else begin
      if (start) begin
        quotient             <= 0;
        remain_r             <= numer;
        state[NUMER_WIDTH-1] <= 1'b1; 
        denomtemp            <= denom << (NUMER_WIDTH-1); 
        done                 <= 0;
      end
      else begin
        if (state != 0) begin
          if ( ~accum_minus_bpower[NUMER_WIDTH+DENOM_WIDTH-1])
          begin  // 'remain_r - denomtemp' is positive
            quotient <= quotient | state;
            remain_r <= accum_minus_bpower;
          end
          state      <= state >> 1;
          denomtemp  <= denomtemp >> 1;
        end
        else begin
          done <= 1;
        end
      end
    end
  end

endmodule
