///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011-2021 Anlogic, Inc.
// All Right Reserved.
///////////////////////////////////////////////////////////////////////////////
//
//
//   Vendor : Anlogic
//   Version : 0.1
//   Description : Anlogic Functional Simulation Library Component
//               pad
//   Filename : ef2_phy_pad.v
//   Timestamp : Fri Feb 19 15:54:16 CST 2016
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    02/19/16 - Initial version.
//    11/15/17 - Updated to support MIPI IO
//    12/27/28 - Remove idly and odly cell
//
///////////////////////////////////////////////////////////////////////////////
`timescale 1ns / 1ps
module EF2_PHY_PAD (ipad, opad, bpad, rst, ce, isclk, ipclk, osclk, opclk, ts, do, di, diq,
       lp_rx_en, lp_dir, lp_dout, lp_din, dynres, vcmpd_n 
    );
    parameter DEDCLK     = "DISABLE";   //DISABLE, ENABLE
    parameter GSR        = "ENABLE";    //ENABLE, DISABLE
    parameter SRMODE     = "SYNC";      //SYNC, ASYNC
    parameter TSMUX      = "1";         //1, 0, TS, INV
    parameter INSCLKMUX  = "0";         //CLK(ioclk[1:0],gclk[3:0],jclk,0(SW default)), INV
    parameter INPCLKMUX  = "CLK";       //CLK(ioclk[0],gclk[1:0],jclk(as SW default=0)), INV
    parameter INCEMUX    = "CE";        //CE, INV, 1
    parameter INRSTMUX   = "0";         //0, RST, INV
    parameter IN_REGSET  = "RESET";     //RESET, SET
    parameter IN_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter IDDRMODE   = "OFF";       //OFF, DDRX1, DDRX2
    parameter IDDRPIPEMODE  = "NONE";   //NONE, PIPED
    parameter INDELMUX   = "NODEL";     //NODEL, FIXDEL
    parameter INDEL      = 0;           // 0-31
    parameter OUTSCLKMUX = "0";         //CLK(ioclk[1:0],gclk[3:0],jclk,0(SW default)), INV
    parameter OUTPCLKMUX = "CLK";       //CLK(ioclk[0],gclk[1:0],jclk(as SW default=0)), INV
    parameter OUTCEMUX   = "CE";        //CE, INV, 1
    parameter OUTRSTMUX  = "0";         //0, RST, INV
    parameter DO_REGSET  = "RESET";     //RESET, SET
    parameter DO_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter ODDRMODE   = "OFF";       //OFF, DDRX1, DDRX2, DDRX2L
    parameter OUTDELMUX  = "NODEL";     //NODEL, FIXDEL
    parameter OUTDEL     = 0;           // 0-3
    parameter TO_REGSET  = "RESET";     //RESET, SET
    parameter TO_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter MODE       = "IN";        //IN, OUT, BI
    parameter DRIVE      = "NONE";      //NONE, 8, 12, 16, 20 24
    parameter IOTYPE     = "LVCMOS25";  //LVCMOS25, LVCMOS12, LVCMOS18, LVCMOS33, LVDS25 ...

    input  ipad;
    output opad;
    inout  bpad;
    input  rst, ce, isclk, ipclk, osclk, opclk;
    input  ts;
    input [3:0] do;
    output di;
    output [3:0] diq;

    // New ports for MIPI
    input  lp_rx_en;
    input  lp_dir;
    input  lp_dout;
    input  dynres;
    input  vcmpd_n;
    output lp_din;

    reg IN_INIT = 1'b0;
    reg DO_INIT = 1'b0;
    reg TO_INIT = 1'b0;

    reg opad;
    reg dim, osclk_div2;
    reg ince, outce, inrst, outrst;
    reg in_reg, do_reg, ts_reg, ts_reg_s;
    reg in_latch, do_latch, ts_latch;
    reg do_out, do_t, data_in, ts_out;
    // reg inrst_asyn, outrst_asyn, gsr;
    reg inrst_asyn, outrst_asyn;

    reg  ts_in, ts_t;
    reg  [3:0] diq;
    wire do_pad, data_delay, do_x1, do_x2; 
    wire [1:0] diqx1;
    wire [3:0] diqx2;

    wire bpad = ts_out ? 1'bz : do_pad;
    wire di = dim;

    wire isclkii = (INSCLKMUX  == "INV")? ~isclk : isclk;
    wire ipclkii = (INPCLKMUX  == "INV")? ~ipclk : ipclk;

    wire osclkii = (OUTSCLKMUX  == "INV")? ~osclk : osclk;
    wire opclkii = (OUTPCLKMUX  == "INV")? ~opclk : opclk;

    wire gsr = (GSR == "DISABLE" ) ? 1'b0 : glbl.gsr;
    //initial begin
    always @(*) begin
      case (MODE)
        "BI"  : begin
                  if (DRIVE == "NONE" && IOTYPE != "LVDS25") begin
                    $display ("in BI MODE, DRIVE strength cannot be set to NONE\n");
                    //$finish;
                  end
                  //if (ts_out === 1'b1)
                //data_in = ts_out? ipad : bpad;
                  data_in = bpad;
                  opad = ts_out ? (lp_dir ? 1'bz : lp_dout) : do_pad;
                end
        "OUT" : begin
                  if (TSMUX == "1") begin
                    $display ("in OUT MODE, TS cannot be set to 1\n");
                    $finish;
                  end
                  if (DRIVE == "NONE" && IOTYPE != "LVDS25") begin
                    $display ("in OUT MODE, DRIVE strength cannot be set to NONE\n");
                    //$finish;
                  end
                  data_in = 1'b0;
                  opad = ts_out ? (lp_dir ? 1'bz : lp_dout) : do_pad;
                end
        "IN"  : begin
                  if (TSMUX == "0") begin
                    $display ("in IN MODE, TS cannot be set to 0\n");
                    $finish;
                  end
                  data_in = ipad;
                  opad = ts_out ? (lp_dir ? 1'bz : lp_dout) : do_pad;
                end
        default  : begin
                  data_in = ipad;
                  opad = ts_out ? (lp_dir ? 1'bz : lp_dout) : do_pad;
                end
      endcase

        case (TSMUX)
            "TS"    : ts_in = ts;
            "INV"   : ts_in = ~ts;
            "0"     : ts_in = 1'b0;
            "1"     : ts_in = 1'b1;
            default : ts_in = 1'b1;
        endcase

        case (INCEMUX)
            "1"     : ince = 1'b1;
            "CE"    : ince = ce;
            "INV"   : ince = ~ce;
            default : ince = ce;
        endcase

        case (OUTCEMUX)
            "1"     : outce = 1'b1;
            "CE"    : outce = ce;
            "INV"   : outce = ~ce;
            default : outce = ce;
        endcase

        case (INRSTMUX)
            "RST"   : inrst = rst;
            "INV"   : inrst = ~rst;
            "0"     : inrst = 1'b0;
            default : inrst = 1'b0;
        endcase

        case (OUTRSTMUX)
            "RST"   : outrst = rst;
            "INV"   : outrst = ~rst;
            "0"     : outrst = 1'b0;
            default : outrst = 1'b0;
        endcase

        case (INDELMUX)
           "FIXDEL" : dim = data_delay;
           "NODEL"  : dim = data_in;
           default  : dim = data_in;
        endcase

        // case (GSR)
        //   "DISABLE" : gsr = 1'b0;
        //   "ENABLE"  : gsr = glbl_gsr;
        //   default   : gsr = glbl_gsr;
        // endcase

        case (SRMODE)
          "ASYNC" : begin
                      inrst_asyn = gsr | inrst;
                      outrst_asyn = gsr | outrst;
                    end
          "SYNC"  : begin
                      inrst_asyn = gsr;
                      outrst_asyn = gsr;
                    end
          default : begin
                      inrst_asyn = gsr;
                      outrst_asyn = gsr;
                    end
        endcase

        case (IN_REGSET)
            "SET"   : begin
                       if (IDDRMODE == "ON") begin
                          $display ("in ddr mode, register can only be set to RESET\n");
                          $finish;
                       end
                       IN_INIT = 1'b1;
                     end
            "RESET" : IN_INIT = 1'b0;
            default : IN_INIT = 1'b0;
        endcase

        case (DO_REGSET)
            "SET"   : begin
                        if (IDDRMODE == "ON") begin
                          $display ("in ddr mode, register can only be set to RESET\n");
                          $finish;
                        end
                        DO_INIT = 1'b1;
                      end
            "RESET" : DO_INIT = 1'b0;
            default : DO_INIT = 1'b0;
        endcase

        case (TO_REGSET)
            "SET"   : TO_INIT = 1'b1;
            "RESET" : TO_INIT = 1'b0;
            default : TO_INIT = 1'b0;
        endcase

        case (IN_DFFMODE)
            "FF"    : begin
                        diq[3:0] <= {3'b0,in_reg};
                      end
            "LATCH" : begin
                        if (IDDRMODE == "ON") begin
                          $display ("in ddr mode, register can only be set to FF\n");
                          $finish;
                        end
                        diq[3:0] <= {3'b0,in_latch};
                      end
            "NONE"  : begin
                        if (IDDRMODE == "ON") begin
                          $display ("in ddr mode, register can only be set to FF\n");
                          $finish;
                        end
                        diq[3:0] <= 4'b0;
                      end
            default : begin
                        if (IDDRMODE == "ON") begin
                          $display ("in ddr mode, register can only be set to FF\n");
                          $finish;
                        end
                        diq[3:0] <= 4'b0;
                      end
        endcase

        case (IDDRMODE)
            "DDRX1" : begin
                       diq[3:0] <= {2'b0,diqx1[1:0]};
                      end
            "DDRX2" : begin
                       diq[3:0] <= diqx2[3:0];
                      end
            "OFF"   : diq[3:1] <= 3'b0;
            default : diq[3:1] <= 3'b0;
        endcase


        case (DO_DFFMODE)
            "FF"    : do_t <= do_reg;
            "LATCH" : begin
                        if (ODDRMODE == "ON") begin
                          $display ("in ddr mode, register can only be set to FF\n");
                          $finish;
                        end
                        do_t <= do_latch;
                      end
            "NONE"  : begin
                        if (ODDRMODE == "ON") begin
                          $display ("in ddr mode, register can only be set to FF\n");
                          $finish;
                        end
                        do_t <= do[0];
                      end
            default : do_t <= do[0];
        endcase

        case (TO_DFFMODE)
            "FF"    : ts_t <= ts_reg;
            "LATCH" : ts_t <= ts_latch;
            "NONE"  : ts_t <= ts_in;
            default : ts_t <= ts_in;
        endcase

        case (ODDRMODE)
            "DDRX1"  : begin do_out <= do_x1; ts_out = ts_t; end
            "DDRX2"  : begin do_out <= do_x2; ts_out = ts_reg_s; end
            "DDRX2L" : begin do_out <= do_x2; ts_out = ts_reg_s; end
            "OFF"    : begin do_out <= do_t;  ts_out = ts_t; end
            default  : begin do_out <= do_t;  ts_out = ts_t; end
        endcase

    end

    wire in_ce = ince & ~inrst;

// Input DFF
    always @(posedge ipclkii or posedge inrst_asyn) begin
      if (inrst_asyn || inrst)
        in_reg <= IN_INIT;
      else if(in_ce)
        in_reg <= dim;
    end

// Input Latch
    always @(*) begin
      if (inrst_asyn)
        in_latch <= IN_INIT;
      else if (inrst && !ipclkii)
        in_latch <= IN_INIT;
      else if (!ipclkii && in_ce)
        in_latch <= dim;
    end

    wire out_ce = outce & ~outrst;

// Output DO DFF
    always @(posedge osclkii or posedge outrst_asyn) begin
      if (outrst_asyn || outrst)
        do_reg <= DO_INIT;
      else if(out_ce)
        do_reg <= do[0];
    end

// Output DO Latch
    always @(*) begin
      if (outrst_asyn)
        do_latch <= DO_INIT;
      else if (outrst && !opclkii)
        do_latch <= DO_INIT;
      else if (!opclkii && out_ce)
        do_latch <= do[0];
    end

// TS Clock

   wire ts_pclk = (ODDRMODE == "DDRX2" || ODDRMODE == "DDRX2L")? ((ODDRMODE == "DDRX2L")? osclk_div2 : opclkii ) : osclkii;
   wire ts_sclk = ((ODDRMODE == "DDRX2L")? ~osclk_div2 : osclk_div2) & osclkii;


// Output TS DFF
    always @(posedge ts_pclk or posedge outrst_asyn) begin
      if (outrst_asyn || outrst)
        ts_reg <= TO_INIT;
      else if(out_ce)
        ts_reg <= ts_in;
    end

    always @(posedge ts_sclk or posedge outrst_asyn) begin
      if (outrst_asyn)
        ts_reg_s <= 1'b0;
      else 
        ts_reg_s <= ts_reg;
    end

// Output TS Latch
    always @(*) begin
      if (outrst_asyn)
        ts_latch <= TO_INIT;
      else if (outrst && !opclkii)
        ts_latch <= TO_INIT;
      else if (!opclkii && out_ce)
        ts_latch <= ts_in;
    end


// osclk div2
    always @(posedge osclkii or posedge outrst_asyn) begin
      if (outrst_asyn)
        osclk_div2 <= 1'b0;
      else 
        osclk_div2 <= ~osclk_div2;
    end
//---------------------------------------------------------------
// Use assign instead of EF2_LOGIC_IDELAY to insert delay
// Updated by YS.Zhao 
// Date: Thu Dec 27 16:21:11 CST 2018
//---------------------------------------------------------------
   localparam xidl = 0.32 + INDEL*0.035; 
   assign #xidl data_delay = data_in;
//   EF2_LOGIC_IDELAY xidelay (.o(data_delay), .i(data_in));
//   defparam xidelay.INDEL = INDEL; 

   // IDDRx1
   EF2_LOGIC_IDDR xiddrx1 (.q0(diqx1[0]), .q1(diqx1[1]), .clk(ipclkii), .d(di), .rst(inrst_asyn));
   defparam xiddrx1.PIPEMODE = IDDRPIPEMODE;
   // IDDRx2
   EF2_LOGIC_IDDRx2 xiddrx2 (.q0(diqx2[0]), .q1(diqx2[1]), .q2(diqx2[2]), .q3(diqx2[3]),
                            .pclk(ipclkii), .sclk(isclkii), .d(dim), .rst(inrst_asyn));


   wire opclki = (ODDRMODE == "DDRX2L")? osclk_div2 : opclkii;
   
//---------------------------------------------------------------
// Use assign instead of //EF2_LOGIC_ODELAY to insert delay
// Updated by YS.Zhao 
// Date: Thu Dec 27 16:21:11 CST 2018
//---------------------------------------------------------------
   localparam xodl = 0.07 + OUTDEL*0.07; 
   assign #xodl do_pad = do_out;
   //EF2_LOGIC_ODELAY xodelay (.o(do_pad), .i(do_out));
   // ODDRx1
  // defparam xodelay.OUTDEL = OUTDEL; 
   EF2_LOGIC_ODDR xoddrx1 (.q(do_x1), .clk(osclkii), .d0(do[0]), .d1(do[1]), .rst(outrst_asyn));
   // ODDRx2
   EF2_LOGIC_ODDRx2 xoddrx2 (.q(do_x2), .pclk(opclki), .sclk(osclkii), .rst(outrst_asyn),
                            .d0(do[0]), .d1(do[1]), .d2(do[2]), .d3(do[3]));

    // MIPI Logic
    wire lp_din = ipad & lp_rx_en;

wire done_gwe = glbl.done_gwe;
and (incheck,ince,done_gwe);
and (outcheck,outce,done_gwe);

 specify
        (do[0]  => opad) = (0:0:0, 0:0:0);
        (do[1]  => opad) = (0:0:0, 0:0:0);
        (do[2]  => opad) = (0:0:0, 0:0:0);
        (do[3]  => opad) = (0:0:0, 0:0:0);
        (do[0]  => bpad) = (0:0:0, 0:0:0);
        (do[1]  => bpad) = (0:0:0, 0:0:0);
        (do[2]  => bpad) = (0:0:0, 0:0:0);
        (do[3]  => bpad) = (0:0:0, 0:0:0);
        (ts     => opad) = (0:0:0, 0:0:0);
        (ts     => bpad) = (0:0:0, 0:0:0);        
        (ipad   *> di) 	= (0:0:0, 0:0:0);
        (bpad   *> di) 	= (0:0:0, 0:0:0);        

        (osclk    => opad) = (0:0:0, 0:0:0);
        (osclk    => bpad) = (0:0:0, 0:0:0);      
        (isclk   => diq[0]) = (0:0:0, 0:0:0);
        (isclk   => diq[1]) = (0:0:0, 0:0:0);
        (isclk   => diq[2]) = (0:0:0, 0:0:0);
        (isclk   => diq[3]) = (0:0:0, 0:0:0);  
        (opclk    => opad) = (0:0:0, 0:0:0);
        (opclk    => bpad) = (0:0:0, 0:0:0);      
        (ipclk   => diq[0]) = (0:0:0, 0:0:0);
        (ipclk   => diq[1]) = (0:0:0, 0:0:0);
        (ipclk   => diq[2]) = (0:0:0, 0:0:0);
        (ipclk   => diq[3]) = (0:0:0, 0:0:0);       

        $recovery (rst, posedge isclk &&& (done_gwe!=0) , 0:0:0 );
        $recovery (rst, negedge isclk &&& (done_gwe!=0) , 0:0:0 );
        $recovery (rst, posedge ipclk &&& (done_gwe!=0) , 0:0:0 );
        $recovery (rst, negedge ipclk &&& (done_gwe!=0) , 0:0:0 );
        $recovery (rst, posedge osclk &&& (done_gwe!=0) , 0:0:0 );
        $recovery (rst, negedge osclk &&& (done_gwe!=0) , 0:0:0 );
        $recovery (rst, posedge opclk &&& (done_gwe!=0) , 0:0:0 );
        $recovery (rst, negedge opclk &&& (done_gwe!=0) , 0:0:0 );        

        $removal (rst, posedge isclk &&& (done_gwe!=0) , 0:0:0 );
        $removal (rst, negedge isclk &&& (done_gwe!=0) , 0:0:0 );
        $removal (rst, posedge ipclk &&& (done_gwe!=0) , 0:0:0 );
        $removal (rst, negedge ipclk &&& (done_gwe!=0) , 0:0:0 );
        $removal (rst, posedge osclk &&& (done_gwe!=0) , 0:0:0 );
        $removal (rst, negedge osclk &&& (done_gwe!=0) , 0:0:0 );
        $removal (rst, posedge opclk &&& (done_gwe!=0) , 0:0:0 );
        $removal (rst, negedge opclk &&& (done_gwe!=0) , 0:0:0 ); 
                        
        $setuphold (posedge isclk, posedge ipad &&& (incheck!=0) 	, 0:0:0, 0:0:0);                        
	$setuphold (posedge isclk, posedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge isclk, posedge ce 	&&& (done_gwe!=0)	, 0:0:0, 0:0:0);
	$setuphold (posedge isclk, posedge rst 	&&& (done_gwe!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge isclk, posedge ts 	&&& (done_gwe!=0)	, 0:0:0, 0:0:0);	
        $setuphold (posedge isclk, negedge ipad &&& (incheck!=0) 	, 0:0:0, 0:0:0);                        
	$setuphold (posedge isclk, negedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge isclk, negedge ce 	&&& (done_gwe!=0)	, 0:0:0, 0:0:0);
	$setuphold (posedge isclk, negedge rst 	&&& (done_gwe!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge isclk, negedge ts 	&&& (done_gwe!=0)	, 0:0:0, 0:0:0);	
        $setuphold (negedge isclk, posedge ipad &&& (incheck!=0)        , 0:0:0, 0:0:0);                        
	$setuphold (negedge isclk, posedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge isclk, posedge ce 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge isclk, posedge rst  &&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
	$setuphold (negedge isclk, posedge ts 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
        $setuphold (negedge isclk, negedge ipad &&& (incheck!=0)        , 0:0:0, 0:0:0);                        
	$setuphold (negedge isclk, negedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge isclk, negedge ce 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge isclk, negedge rst  &&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
	$setuphold (negedge isclk, negedge ts 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
        $setuphold (posedge ipclk, posedge ipad &&& (incheck!=0)        , 0:0:0, 0:0:0);                        
	$setuphold (posedge ipclk, posedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge ipclk, posedge ce 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
	$setuphold (posedge ipclk, posedge rst 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
	$setuphold (posedge ipclk, posedge ts 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
        $setuphold (posedge ipclk, posedge ipad &&& (incheck!=0)        , 0:0:0, 0:0:0);                        		
	$setuphold (posedge ipclk, posedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge ipclk, posedge ce 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
	$setuphold (posedge ipclk, posedge rst 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
	$setuphold (posedge ipclk, posedge ts 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
        $setuphold (negedge ipclk, posedge ipad &&& (incheck!=0)        , 0:0:0, 0:0:0);                        
	$setuphold (negedge ipclk, posedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge ipclk, posedge ce 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge ipclk, posedge rst 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
	$setuphold (negedge ipclk, posedge ts 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
        $setuphold (negedge ipclk, negedge ipad &&& (incheck!=0)        , 0:0:0, 0:0:0);                        
	$setuphold (negedge ipclk, negedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge ipclk, negedge ce 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge ipclk, negedge rst 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
	$setuphold (negedge ipclk, negedge ts 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
        $setuphold (posedge osclk, posedge ipad &&& (incheck!=0)        , 0:0:0, 0:0:0);                        
	$setuphold (posedge osclk, posedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge osclk, posedge ce 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
	$setuphold (posedge osclk, posedge rst 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
	$setuphold (posedge osclk, posedge ts 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
        $setuphold (posedge osclk, negedge ipad &&& (incheck!=0)        , 0:0:0, 0:0:0);                        
	$setuphold (posedge osclk, negedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge osclk, negedge ce 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
	$setuphold (posedge osclk, negedge rst 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
	$setuphold (posedge osclk, negedge ts 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
        $setuphold (negedge osclk, posedge ipad &&& (incheck!=0)        , 0:0:0, 0:0:0);                        
	$setuphold (negedge osclk, posedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge osclk, posedge ce 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge osclk, posedge rst 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
	$setuphold (negedge osclk, posedge ts 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
        $setuphold (negedge osclk, negedge ipad &&& (incheck!=0)        , 0:0:0, 0:0:0);                        
	$setuphold (negedge osclk, negedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge osclk, negedge ce 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge osclk, negedge rst 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
	$setuphold (negedge osclk, negedge ts 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
        $setuphold (posedge opclk, posedge ipad &&& (incheck!=0)        , 0:0:0, 0:0:0);                        
	$setuphold (posedge opclk, posedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge opclk, posedge ce 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
	$setuphold (posedge opclk, posedge rst 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
	$setuphold (posedge opclk, posedge ts 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
        $setuphold (posedge opclk, negedge ipad &&& (incheck!=0)        , 0:0:0, 0:0:0);                        
	$setuphold (posedge opclk, negedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge opclk, negedge ce 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
	$setuphold (posedge opclk, negedge rst 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
	$setuphold (posedge opclk, negedge ts 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
        $setuphold (negedge opclk, posedge ipad &&& (incheck!=0)        , 0:0:0, 0:0:0);                        
	$setuphold (negedge opclk, posedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge opclk, posedge ce 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge opclk, posedge rst 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
	$setuphold (negedge opclk, posedge ts 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
        $setuphold (negedge opclk, negedge ipad &&& (incheck!=0)        , 0:0:0, 0:0:0);                        
	$setuphold (negedge opclk, negedge bpad &&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge opclk, negedge ce 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge opclk, negedge rst 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	
	$setuphold (negedge opclk, negedge ts 	&&& (done_gwe!=0)       , 0:0:0, 0:0:0);	

	$setuphold (posedge isclk, posedge do[0] &&& (outcheck!=0)	, 0:0:0, 0:0:0);
	$setuphold (posedge isclk, negedge do[0] &&& (outcheck!=0)	, 0:0:0, 0:0:0);
	$setuphold (negedge isclk, posedge do[0] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge isclk, negedge do[0] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (posedge ipclk, posedge do[0] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (posedge ipclk, posedge do[0] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge ipclk, posedge do[0] &&& (outcheck!=0)       , 0:0:0, 0:0:0);	
	$setuphold (negedge ipclk, negedge do[0] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (posedge osclk, posedge do[0] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (posedge osclk, negedge do[0] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge osclk, posedge do[0] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge osclk, negedge do[0] &&& (outcheck!=0)       , 0:0:0, 0:0:0);	
	$setuphold (posedge opclk, posedge do[0] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (posedge opclk, negedge do[0] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge opclk, posedge do[0] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge opclk, negedge do[0] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	
	$setuphold (posedge isclk, posedge do[1] &&& (outcheck!=0)	, 0:0:0, 0:0:0);
	$setuphold (posedge isclk, negedge do[1] &&& (outcheck!=0)	, 0:0:0, 0:0:0);
	$setuphold (negedge isclk, posedge do[1] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge isclk, negedge do[1] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (posedge ipclk, posedge do[1] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (posedge ipclk, posedge do[1] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge ipclk, posedge do[1] &&& (outcheck!=0)       , 0:0:0, 0:0:0);	
	$setuphold (negedge ipclk, negedge do[1] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (posedge osclk, posedge do[1] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (posedge osclk, negedge do[1] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge osclk, posedge do[1] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge osclk, negedge do[1] &&& (outcheck!=0)       , 0:0:0, 0:0:0);	
	$setuphold (posedge opclk, posedge do[1] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (posedge opclk, negedge do[1] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge opclk, posedge do[1] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge opclk, negedge do[1] &&& (outcheck!=0)       , 0:0:0, 0:0:0);

	$setuphold (posedge isclk, posedge do[2] &&& (outcheck!=0)	, 0:0:0, 0:0:0);
	$setuphold (posedge isclk, negedge do[2] &&& (outcheck!=0)	, 0:0:0, 0:0:0);
	$setuphold (negedge isclk, posedge do[2] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge isclk, negedge do[2] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (posedge ipclk, posedge do[2] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (posedge ipclk, posedge do[2] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge ipclk, posedge do[2] &&& (outcheck!=0)       , 0:0:0, 0:0:0);	
	$setuphold (negedge ipclk, negedge do[2] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (posedge osclk, posedge do[2] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (posedge osclk, negedge do[2] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge osclk, posedge do[2] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge osclk, negedge do[2] &&& (outcheck!=0)       , 0:0:0, 0:0:0);	
	$setuphold (posedge opclk, posedge do[2] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (posedge opclk, negedge do[2] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge opclk, posedge do[2] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge opclk, negedge do[2] &&& (outcheck!=0)       , 0:0:0, 0:0:0);

	$setuphold (posedge isclk, posedge do[3] &&& (outcheck!=0)	, 0:0:0, 0:0:0);
	$setuphold (posedge isclk, negedge do[3] &&& (outcheck!=0)	, 0:0:0, 0:0:0);
	$setuphold (negedge isclk, posedge do[3] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge isclk, negedge do[3] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (posedge ipclk, posedge do[3] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (posedge ipclk, posedge do[3] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge ipclk, posedge do[3] &&& (outcheck!=0)       , 0:0:0, 0:0:0);	
	$setuphold (negedge ipclk, negedge do[3] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (posedge osclk, posedge do[3] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (posedge osclk, negedge do[3] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge osclk, posedge do[3] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge osclk, negedge do[3] &&& (outcheck!=0)       , 0:0:0, 0:0:0);	
	$setuphold (posedge opclk, posedge do[3] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (posedge opclk, negedge do[3] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
	$setuphold (negedge opclk, posedge do[3] &&& (outcheck!=0)       , 0:0:0, 0:0:0);
	$setuphold (negedge opclk, negedge do[3] &&& (outcheck!=0)       , 0:0:0, 0:0:0);		
												
 endspecify 

endmodule

