///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011-2021 Anlogic, Inc.
// All Right Reserved.
///////////////////////////////////////////////////////////////////////////////
//
//
//   Vendor : Anlogic
//   Version : 0.1
//   Description : Anlogic Functional Simulation Library Component
//               simple pad
//   Filename : ef2_phy_spad.v
//   Timestamp : Mon Nov  4 14:20:41 CST 2013
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    11/04/13 - Initial version.
//    02/03/17 - Modify port name for EF2 SIMPLE IO
//    12/27/28 - Remove idly cell
//
///////////////////////////////////////////////////////////////////////////////
`timescale 1ns / 1ps
module EF2_PHY_SPAD (ipad, opad, bpad, rst, ce, clk, ts, do, di, diq);
    parameter DEDCLK     = "DISABLE";   //DISABLE, ENABLE
    parameter GSR        = "ENABLE";    //ENABLE, DISABLE
    parameter SRMODE     = "SYNC";      //SYNC, ASYNC
    parameter TSMUX      = "TS";        //TS, INV
    parameter INCLKMUX   = "0";         //0, 1, CLK, INV
    parameter INCEMUX    = "CE";        //CE, INV
    parameter INRSTMUX   = "RST";       //RST, INV
    parameter IDDRMODE   = "OFF";       //OFF, ON
    parameter INDELMUX   = "NODEL";     //NODEL, FIXDEL
    parameter INDEL      = 0;           // 0-31
    parameter OUTCLKMUX  = "0";         //0, 1, CLK, INV
    parameter OUTCEMUX   = "CE";        //CE, INV
    parameter OUTRSTMUX  = "RST";       //RST, INV
    parameter ODDRMODE   = "OFF";       //OFF, ON
    parameter IN_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter IN_REGSET  = "RESET";     //RESET, SET
    parameter DO_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter DO_REGSET  = "RESET";     //RESET, SET
    parameter TO_DFFMODE = "NONE";      //NONE, FF, LATCH
    parameter TO_REGSET  = "RESET";     //RESET, SET
    parameter MODE       = "IN";        //IN, OUT, BI
    parameter DRIVE      = "NONE";      //NONE, 8, 12, 16, 20 24
    parameter IOTYPE     = "LVCMOS25";  //LVCMOS25, LVCMOS12, LVCMOS18, LVCMOS33, LVDS25 ...

    input  ipad;
    output opad;
    inout  bpad;
    input  rst, ce, clk;
    input  ts;
    input [1:0] do;
    output di;
    output [1:0] diq;
    //
    reg [1:0] diq_tmp;
    reg di_tmp;
    buf (di,di_tmp);
    buf (diq[0],diq_tmp[0]);
    buf (diq[1],diq_tmp[1]);    
    

    

    reg IN_INIT = 1'b0;
    reg DO_INIT = 1'b0;
    reg TO_INIT = 1'b0;

    reg inclk, outclk, ince, outce, inrst, outrst;
    reg ts_in;
    reg data_delay;


    reg in_reg, do_reg, ts_reg;
    reg in_latch, do_latch, ts_latch;
    reg do_out, do_t, data_in, ts_out;
    reg inlsr, outlsr, gsr;
    reg in_ddr_reg_t2, in_ddr_reg_c1, in_ddr_reg_c2;
    reg do_ddr_reg_c1, do_ddr_reg_c2;
    wire do_ddr;

    reg    opad_tmp;
    bufif0 ( opad,opad_tmp,ts_out);
    bufif0 ( bpad,do_out,ts_out);    
   
//---------------------------------------------------------------
// Use actual delay in input path
// Updated by YS.Zhao
// Date: Thu Dec 27 16:21:11 CST 2018
//---------------------------------------------------------------
    localparam delay_time = 0.52 + INDEL*0.12;

    //assign bpad = ts_out ? 1'bz : do_out;

    //initial begin
    always @(*) begin
        case (MODE)
            "BI"    : begin
                           if (DRIVE == "NONE") begin
                               $display ("in BI MODE, DRIVE strength cannot be set to NONE\n");
                               //$finish;
                           end
                           //if (ts_out === 1'b1)
                               data_in <= bpad;
                       end
            "OUT"   : begin
                           if (TSMUX == "1") begin
                               $display ("in OUT MODE, TS cannot be set to 1\n");
                               $finish;
                           end
                           if (DRIVE == "NONE") begin
                               $display ("in OUT MODE, DRIVE strength cannot be set to NONE\n");
                               //$finish;
                           end
                           opad_tmp = ts_out ? 1'bz : do_out;
                       end
            "IN"    : begin
                           if (TSMUX == "0") begin
                               $display ("in IN MODE, TS cannot be set to 0\n");
                               $finish;
                           end
                           data_in = ipad;
                       end
            default  : data_in = ipad;
        endcase

        case (INCLKMUX)
            "CLK"   : inclk = clk;
            "INV"   : inclk = ~clk;
            "0"     : inclk = 1'b0;
            default : inclk = 1'b0;
        endcase

        case (OUTCLKMUX)
            "CLK"   : outclk = clk;
            "INV"   : outclk = ~clk;
            "0"     : outclk = 1'b0;
            default : outclk = 1'b0;
        endcase

        case (TSMUX)
            "TS"    : ts_in = ts;
            "INV"   : ts_in = ~ts;


            default : ts_in = ts;
        endcase

        case (INCEMUX)

            "CE"    : ince = ce;
            "INV"   : ince = ~ce;
            default : ince = ce;
        endcase

        case (OUTCEMUX)

            "CE"    : outce = ce;
            "INV"   : outce = ~ce;
            default : outce = ce;
        endcase

        case (INRSTMUX)
            "RST"   : inrst = rst;
            "INV"   : inrst = ~rst;

            default : inrst = rst;
        endcase

        case (OUTRSTMUX)
            "RST"   : outrst = rst;
            "INV"   : outrst = ~rst;

            default : outrst = rst;
        endcase

        case (INDELMUX)
            "FIXDEL"   : begin
                              #delay_time;
                              data_delay = data_in;
                          end
            "NODEL"    : data_delay = data_in;
            default    : data_delay = data_in;
        endcase

        case (GSR)
            "DISABLE"  : gsr = 1'b0;
            "ENABLE"   : gsr = glbl.gsr;
            default    : gsr = glbl.gsr;
        endcase

        case (SRMODE)
            "ASYNC"    : begin
                              inlsr = gsr | inrst;
                              outlsr = gsr | outrst;
                          end
            "SYNC"     : begin
                              inlsr = gsr;
                              outlsr = gsr;
                          end
            default    : begin
                              inlsr = gsr;
                              outlsr = gsr;
                          end
        endcase

        case (IN_REGSET)
            "SET"      : begin
                              if (IDDRMODE == "ON") begin
                                  $display ("in ddr mode, register can only be set to RESET\n");
                                  $finish;
                              end
                              IN_INIT = 1'b1;
                          end
            "RESET"    : IN_INIT = 1'b0;
            default    : IN_INIT = 1'b0;
        endcase

        case (DO_REGSET)
            "SET"      : begin
                              if (IDDRMODE == "ON") begin
                                  $display ("in ddr mode, register can only be set to RESET\n");
                                  $finish;
                              end
                              DO_INIT = 1'b1;
                          end
            "RESET"    : DO_INIT = 1'b0;
            default    : DO_INIT = 1'b0;
        endcase

        case (TO_REGSET)
            "SET"      : TO_INIT = 1'b1;
            "RESET"    : TO_INIT = 1'b0;
            default    : TO_INIT = 1'b0;
        endcase

        case (IN_DFFMODE)
            "FF"       : begin
                              diq_tmp[0] <= in_reg;
                              di_tmp <= data_delay;
                          end
            "LATCH"    : begin
                              if (IDDRMODE == "ON") begin
                                  $display ("in ddr mode, register can only be set to FF\n");
                                  $finish;
                              end
                              diq_tmp[0] <= in_latch;
                              di_tmp <= data_delay;
                          end
            "NONE"     : begin
                              if (IDDRMODE == "ON") begin
                                  $display ("in ddr mode, register can only be set to FF\n");
                                  $finish;
                              end
                              di_tmp <= data_delay;
                          end
            default    : begin
                              if (IDDRMODE == "ON") begin
                                  $display ("in ddr mode, register can only be set to FF\n");
                                  $finish;
                              end
                              di_tmp <= data_delay;
                          end
        endcase

        case (DO_DFFMODE)
            "FF"       : do_t <= do_reg;
            "LATCH"    : begin
                              if (ODDRMODE == "ON") begin
                                  $display ("in ddr mode, register can only be set to FF\n");
                                  $finish;
                              end
                              do_t <= do_latch;
                          end
            "NONE"     : begin
                              if (ODDRMODE == "ON") begin
                                  $display ("in ddr mode, register can only be set to FF\n");
                                  $finish;
                              end
                              do_t <= do[0];
                          end
            default    : do_t <= do[0];
        endcase

        case (TO_DFFMODE)
            "FF"       : ts_out <= ts_reg;
            "LATCH"    : ts_out <= ts_latch;
            "NONE"     : ts_out <= ts_in;
            default    : ts_out <= ts_in;
        endcase

        case (IDDRMODE)
            "ON"       : begin
                              diq_tmp[0] <= in_ddr_reg_t2;
                              diq_tmp[1] <= in_ddr_reg_c2;
                          end
            "OFF"      : diq_tmp[1] <= 1'b0;
            default    : diq_tmp[1] <= 1'b0;
        endcase

        case (ODDRMODE)
            "ON"       : do_out <= do_ddr;
            "OFF"      : do_out <= do_t;
            default    : do_out <= do_t;
        endcase
    end

    assign do_ddr = outclk ? do_t : do_ddr_reg_c2;
      wire in_ce = ince & ~inrst;

    always @(posedge inclk or posedge inlsr) begin
        if (inlsr || inrst)
            in_reg <= IN_INIT;
        else if(in_ce)
            in_reg <= data_delay;
    end

    always @(data_delay or posedge inlsr) begin
        if (inlsr)
            in_latch <= IN_INIT;
        else if (inrst && !inclk)
            in_latch <= IN_INIT;
        else if (!inclk & in_ce)
            in_latch <= data_delay;
    end

    wire out_ce = outce & ~outrst;

    always @(posedge outclk or posedge outlsr) begin
        if (outlsr || outrst)
            do_reg <= DO_INIT;
        else if(out_ce)
            do_reg <= do[0];
    end

    always @(do[0] or posedge outlsr) begin
        if (outlsr)
            do_latch <= DO_INIT;
        else if (outrst && !outclk)
            do_latch <= DO_INIT;
        else if (!outclk & out_ce)
            do_latch <= do[0];
    end

    always @(posedge outclk or posedge outlsr) begin
        if (outlsr || outrst)
            ts_reg <= TO_INIT;
        else if(out_ce)
            ts_reg <= ts_in;
    end

    always @(ts_in or posedge outlsr) begin
        if (outlsr)
            ts_latch <= TO_INIT;
        else if (outrst && !outclk)
            ts_latch <= TO_INIT;
        else if (!outclk & out_ce)
            ts_latch <= ts_in;
    end

    //for DDRMODE
    always @(negedge inclk or posedge inlsr) begin
        if (inlsr || inrst)
            in_ddr_reg_c1 <= IN_INIT;
        else
            in_ddr_reg_c1 <= data_delay;
    end

    always @(posedge inclk or posedge inlsr) begin
        if (inlsr || inrst)
            in_ddr_reg_c2 <= IN_INIT;
        else
            in_ddr_reg_c2 <= in_ddr_reg_c1;
    end

    always @(posedge inclk or posedge inlsr) begin
        if (inlsr || inrst)
            in_ddr_reg_t2 <= IN_INIT;
        else
            in_ddr_reg_t2 <= in_reg;
    end

    always @(posedge outclk or posedge outlsr) begin
        if (outlsr || outrst)
            do_ddr_reg_c1 <= DO_INIT;
        else
            do_ddr_reg_c1 <= do[1];
    end

    always @(negedge outclk or posedge outlsr) begin
        if (outlsr || outrst)
            do_ddr_reg_c2 <= DO_INIT;
        else
            do_ddr_reg_c2 <= do_ddr_reg_c1;
    end
wire done_gwe = glbl.done_gwe;
and (incheck,ince,done_gwe);
and (outcheck,outce,done_gwe);
 specify
        (do[0]  => opad) = (0:0:0, 0:0:0);
        (do[1]  => opad) = (0:0:0, 0:0:0);
        (do[0]  => bpad) = (0:0:0, 0:0:0);
        (do[1]  => bpad) = (0:0:0, 0:0:0);
        (clk    => opad) = (0:0:0, 0:0:0);
        (clk    => bpad) = (0:0:0, 0:0:0);        
        (ts     => opad) = (0:0:0, 0:0:0);
        (ts     => bpad) = (0:0:0, 0:0:0);        
        (ipad   => di) 	= (0:0:0, 0:0:0);
        (bpad   => di) 	= (0:0:0, 0:0:0);        
        (clk   => diq[0]) = (0:0:0, 0:0:0);
        (clk   => diq[1]) = (0:0:0, 0:0:0);
        $recovery (rst, posedge clk &&& (done_gwe!=0), 0:0:0 );
        $recovery (rst, negedge clk &&& (done_gwe!=0), 0:0:0 );
        $removal  (rst, posedge clk &&& (done_gwe!=0), 0:0:0 );
        $removal  (rst, negedge clk &&& (done_gwe!=0), 0:0:0 ); 
                
        $setuphold (posedge clk, posedge ipad 	&&& (incheck!=0)    , 0:0:0, 0:0:0);                        
	$setuphold (posedge clk, posedge do[0] 	&&& (outcheck!=0)   , 0:0:0, 0:0:0);
	$setuphold (posedge clk, posedge do[1] 	&&& (outcheck!=0)   , 0:0:0, 0:0:0);	
	$setuphold (posedge clk, posedge bpad 	&&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clk, posedge ce 	&&& (done_gwe!=0)   , 0:0:0, 0:0:0);
	$setuphold (posedge clk, posedge rst 	&&& (done_gwe!=0)   , 0:0:0, 0:0:0);	
	$setuphold (posedge clk, posedge ts 	&&& (done_gwe!=0)   , 0:0:0, 0:0:0);	
	
        $setuphold (negedge clk, posedge ipad 	&&& (incheck!=0)    , 0:0:0, 0:0:0);                        
	$setuphold (negedge clk, posedge do[0] 	&&& (outcheck!=0)   , 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge do[1] 	&&& (outcheck!=0)   , 0:0:0, 0:0:0);	
	$setuphold (negedge clk, posedge bpad 	&&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clk, posedge ce 	&&& (done_gwe!=0)   , 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge rst 	&&& (done_gwe!=0)   , 0:0:0, 0:0:0);	
	$setuphold (negedge clk, posedge ts 	&&& (done_gwe!=0)   , 0:0:0, 0:0:0);

        $setuphold (posedge clk, negedge ipad 	&&& (incheck!=0)    , 0:0:0, 0:0:0);                        
	$setuphold (posedge clk, negedge do[0] 	&&& (outcheck!=0)   , 0:0:0, 0:0:0);
	$setuphold (posedge clk, negedge do[1] 	&&& (outcheck!=0)   , 0:0:0, 0:0:0);	
	$setuphold (posedge clk, negedge bpad 	&&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clk, negedge ce 	&&& (done_gwe!=0)   , 0:0:0, 0:0:0);
	$setuphold (posedge clk, negedge rst 	&&& (done_gwe!=0)   , 0:0:0, 0:0:0);	
	$setuphold (posedge clk, negedge ts 	&&& (done_gwe!=0)   , 0:0:0, 0:0:0);		
        $setuphold (negedge clk, negedge ipad 	&&& (incheck!=0)    , 0:0:0, 0:0:0);                        
	$setuphold (negedge clk, negedge do[0] 	&&& (outcheck!=0)   , 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge do[1] 	&&& (outcheck!=0)   , 0:0:0, 0:0:0);	
	$setuphold (negedge clk, negedge bpad 	&&& (incheck!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clk, negedge ce 	&&& (done_gwe!=0)   , 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge rst 	&&& (done_gwe!=0)   , 0:0:0, 0:0:0);	
	$setuphold (negedge clk, negedge ts 	&&& (done_gwe!=0)   , 0:0:0, 0:0:0);	
 endspecify 

endmodule
