///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011-2021 Anlogic, Inc.
// All Right Reserved.
///////////////////////////////////////////////////////////////////////////////
//
//
//   Vendor : Anlogic
//   Version : 0.1
//   Description : Anlogic Functional Simulation Library Component
//            config multiboot 
//            config dna 
//            config gctrl 
//            config gsrn
//            config cclk 
//   Filename : ef3_logic_config.v
//   Timestamp : Wed Nov  2 17:27:07 CST 2016
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    11/02/16 - Initial version.
//
///////////////////////////////////////////////////////////////////////////////

`timescale  1 ns / 1 ps

module EF3_LOGIC_MBOOT(rebootn, dynamic_addr);   // config multiboot
input       rebootn;
input [7:0] dynamic_addr;    // high-8bit address

  //parameter 
  parameter ADDR_SOURCE_SEL = "STATIC";  // STATIC/DYNAMIC 
  parameter STATIC_ADDR = 8'b00000000;    // high-8bit address

endmodule

module EF3_LOGIC_DNA(dout, clk, din, shift_en);    // config dna 
parameter DNA = 64'h0000000000000000;
output dout;
input  clk;
input  din;
input  shift_en;

tri0 shift_en, dout;

reg [63:0] dna_reg;
reg shift_en_r;
wire load_dna;

initial
begin
dna_reg = 64'h0000000000000000;
shift_en_r = 1'b0;
end


always @(posedge clk)
  shift_en_r <= shift_en;

assign load_dna = shift_en & ~shift_en_r;

always @(posedge clk)
  if(load_dna)
    dna_reg <= DNA;
  else if(shift_en & ~load_dna)
    dna_reg <= {din, dna_reg[63:1]};

assign dout = dna_reg[0];

endmodule

module EF3_LOGIC_GCTRL(done, highz);      // config global control
output done;
output highz;

endmodule

module  EF3_LOGIC_GSRN(gsrn, sync_clk);     // config gsrn
input gsrn;
input sync_clk; 

  //parameter 
  parameter GSRN_SYNC_SEL = "DISABLE";  // DISABLE/ENABLE set by wakeup register
  parameter USR_GSRN_EN   = "DISABLE";  // DISABLE/ENABLE set by wakeup register
  
  reg [1:0] usr_gsrn_sync;

  initial
  begin
    usr_gsrn_sync = 2'b00;
  end

  always @(posedge sync_clk)
    usr_gsrn_sync <= {usr_gsrn_sync[1], gsrn};

  always @(*)
  begin
    #0.1 glbl.usr_gsrn_en <= (USR_GSRN_EN == "ENABLE") ? 1'b1 : 1'b0;
    #0.1 glbl.gsrn <= (USR_GSRN_EN == "ENABLE") ? ((GSRN_SYNC_SEL == "ENABLE") ? usr_gsrn_sync[1] : gsrn) : 1'b1;
    #0.1 glbl.gsr <= ~glbl.gsrn;
  end

endmodule

module EF3_LOGIC_CCLK(cclk, en);        // config cclk
output cclk;
input  en;

parameter FREQ = "4.5";       // frequency set by wakeup register 

realtime half_period;
reg int_clk;

initial
begin
  int_clk = 1'b0;
end

initial
begin
  case (FREQ)
    "4.5"   : half_period = 111.1; //111.1ns (4.5MHz)
    "6.5"   : half_period = 76.9; 
    "10.0"  : half_period = 50; 
    "20.0"  : half_period = 25; 
    "30.0"  : half_period = 16.67; 
    "40.0"  : half_period = 12.5; 
    "60.0"  : half_period = 8.3; 
    "130.0" : half_period = 3.8; 
    default : half_period = 111.1;   //default frequency is 4.5MHz
  endcase
end

always
  #half_period int_clk = ~int_clk;

assign cclk = en ? int_clk : 1'b0;

endmodule

