///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011-2021 Anlogic, Inc.
// All Right Reserved.
///////////////////////////////////////////////////////////////////////////////
//
//   Vendor : Anlogic
//   Version : 1.1
//   Description : Anlogic Hard IP Library Component
//            CONFIG block
//   Filename : ef3_phy_config.v
//   Timestamp : Fri July  8 09:30:30 CST 2016
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    8/2/16 - Initial version.      
//    11/3/16 - v1.0      
///////////////////////////////////////////////////////////////////////////////

`timescale 1 ns / 10 ps

module EF3_PHY_CONFIG(
  //chip watcher
  jrstn, jrti, jshift, jtck, jtdi, jupdate, jscanen, jtms, jtdo, jtag8_ipa, jtag8_ipb,
  //multi boot
  mboot_rebootn, mboot_dynamic_addr,
  //DNA IP
  dna_dout, dna_clk, dna_din, dna_shift_en,
  //global control
  done, highz,
  //user gsrn
  usr_gsrn, gsrn_sync_clk,
  //cclk 
  cclk, cclk_en
);
  //chip watcher
  output       jrstn;
  output [1:0] jrti;
  output       jshift;
  output       jtck;
  output       jtdi;
  output       jupdate;
  output [1:0] jscanen;
  output       jtms;
  input [1:0]  jtdo;
  input [7:0]  jtag8_ipa;
  input [7:0]  jtag8_ipb;

  //multi boot
  input        mboot_rebootn;
  input [7:0]  mboot_dynamic_addr;

  //DNA IP
  output       dna_dout;
  input        dna_clk;
  input        dna_din;
  input        dna_shift_en;

  //global control
  output       done;
  output       highz;

  //user gsrn
  input        usr_gsrn;
  input        gsrn_sync_clk;

  //cclk 
  output       cclk;
  input        cclk_en;  

  tri0 cclk_en;

  //Parameter
  //io
  parameter DONE_PERSISTN = "ENABLE";  // ENABLE/DISABLE set in device setting 
  parameter INIT_PERSISTN = "ENABLE";  // ENABLE/DISABLE set in device setting
  parameter PROGRAMN_PERSISTN = "DISABLE";  // DISABLE/ENABLE set in device setting
  parameter JTAG_PERSISTN = "DISABLE";  // DISABLE/ENABLE set in device setting
  //multiboot
  parameter MBOOT_AUTO_SEL = "DISABLE";  // DISABLE/ENABLE  
  parameter ADDR_SOURCE_SEL = "STATIC";  // STATIC/DYNAMIC
  parameter STATIC_ADDR = 8'b00000000;  // high-8bit address
  //parameter(register)
  //gsrn
  parameter GSRN_SYNC_SEL = "DISABLE";  // DISABLE/ENABLE set by wakeup register
  parameter USR_GSRN_EN = "DISABLE";  // DISABLE/ENABLE set by wakeup register
  //cclk
  parameter FREQ = "2.5";  // "2.5", "5.0", "10.0", "30.0", set by wakeup register

  // GSR implementation
  reg [1:0] usr_gsrn_sync;

  initial
  begin
    usr_gsrn_sync = 2'b00;
  end

  always @(posedge gsrn_sync_clk)
    usr_gsrn_sync <= {usr_gsrn_sync[1], usr_gsrn};

  always @(*)
  begin
    #0.1 glbl.usr_gsrn_en <= (USR_GSRN_EN == "ENABLE") ? 1'b1 : 1'b0;
    #0.1 glbl.gsrn <= (USR_GSRN_EN == "ENABLE") ? ((GSRN_SYNC_SEL == "ENABLE") ? usr_gsrn_sync[1] : usr_gsrn) : 1'b1;
    #0.1 glbl.gsr <= ~glbl.gsrn;
  end

endmodule

