///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011-2021 Anlogic, Inc.
// All Right Reserved.
///////////////////////////////////////////////////////////////////////////////
//
//
//   Vendor : Anlogic
//   Version : 1.0
//   Description : Anlogic Functional Simulation Library Component
//            OSC block 
//   Filename : ef2_phy_osc.v
//   Timestamp : Mon Nov 13 10:58:49 CST 2017
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    13/11/17 - Initial version.
//
///////////////////////////////////////////////////////////////////////////////

`timescale  1 ns / 10 ps

module EF3_PHY_OSC (stdby, osc);		// osc clk

    input stdby;
    output osc;

    parameter STANDBY = "DISABLE"; 	// DISABLE/ENABLE 

    reg int_osc_clk;

    initial
      begin
        int_osc_clk = 1'b0;
      end

    wire osc_dis = (STANDBY == "ENABLE")? stdby : 0;

    always
      #1.9 int_osc_clk = ~int_osc_clk & ~osc_dis;

    buf (osc, int_osc_clk);

endmodule


///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    13/11/17 - Initial version.
//   Module Name : OSCDIV
//   Version : 1.0
//   Description : Anlogic Functional Simulation Library Component divider
//   Timestamp : Mon Nov 13 10:58:49 CST 2017
//
///////////////////////////////////////////////////////////////////////////////

//`timescale  1ns/ 10ps

module EF3_PHY_OSCDIV (rstn, stdby, div, clko);

    input rstn;
    input stdby;
    input [6:0] div;   // Freq dividing coefficients, divided by 8: div = 7'b111
    output clko;

  reg [7:0] count;
  reg rstn_sync;
  reg clki;

  parameter half_cycle = 1.9;

  initial begin
    clki = 1'b0;
    rstn_sync = 1'b0;
    count = 8'b0000_0000;
  end

  wire [6:0] m_half = div;
  wire [7:0] m_prd  = div*2+1;
 
  always #(half_cycle) clki = ~clki & ~stdby;

  wire osc_en = glbl.done_gwe & ~stdby;
  //wire ck = clki & osc_en;
  wire #0.1 clko = (count<=m_half);
 
  always @(posedge clki)
    #0.1 rstn_sync <= rstn & osc_en;

  always @(posedge clki or negedge clki)
    begin
      if(~rstn_sync) begin
        count <= 0;
       end
      else if(count==m_prd) begin
        count <= 0;
       end
      else count <= count+1;
   end

endmodule

