///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011-2021 Anlogic, Inc.
// All Right Reserved.
///////////////////////////////////////////////////////////////////////////////
//
//
//   Vendor : Anlogic
//   Version : 0.1
//   Description : Anlogic Functional Simulation Library Component
//            idelay, iddr, oddr 
//   Filename : eg_logic_io.v
//   Timestamp : Mon Jul 29 14:20:41 CST 2016
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    29/07/16 - Initial version.
///////////////////////////////////////////////////////////////////////////////
`timescale 1 ns / 1 ps
module  EG_LOGIC_IDELAY(o,i);
output o;
input i;

parameter INDEL = 0;  // 0-31

localparam SIM_TAPDELAY_VALUE = 0.12;
reg [4:0] delay_count;
reg o_out;

initial begin
  delay_count = 0;
  #1;
  delay_count = INDEL;
end

    wire delay_chain_0,  delay_chain_1,  delay_chain_2,  delay_chain_3,
         delay_chain_4,  delay_chain_5,  delay_chain_6,  delay_chain_7,
         delay_chain_8,  delay_chain_9,  delay_chain_10, delay_chain_11,
         delay_chain_12, delay_chain_13, delay_chain_14, delay_chain_15,
         delay_chain_16, delay_chain_17, delay_chain_18, delay_chain_19,
         delay_chain_20, delay_chain_21, delay_chain_22, delay_chain_23,
         delay_chain_24, delay_chain_25, delay_chain_26, delay_chain_27,
         delay_chain_28, delay_chain_29, delay_chain_30, delay_chain_31;


// delay chain

    assign #0.52               delay_chain_0  = i;
    assign #SIM_TAPDELAY_VALUE delay_chain_1  = delay_chain_0;
    assign #SIM_TAPDELAY_VALUE delay_chain_2  = delay_chain_1;
    assign #SIM_TAPDELAY_VALUE delay_chain_3  = delay_chain_2;
    assign #SIM_TAPDELAY_VALUE delay_chain_4  = delay_chain_3;
    assign #SIM_TAPDELAY_VALUE delay_chain_5  = delay_chain_4;
    assign #SIM_TAPDELAY_VALUE delay_chain_6  = delay_chain_5;
    assign #SIM_TAPDELAY_VALUE delay_chain_7  = delay_chain_6;
    assign #SIM_TAPDELAY_VALUE delay_chain_8  = delay_chain_7;
    assign #SIM_TAPDELAY_VALUE delay_chain_9  = delay_chain_8;
    assign #SIM_TAPDELAY_VALUE delay_chain_10 = delay_chain_9;
    assign #SIM_TAPDELAY_VALUE delay_chain_11 = delay_chain_10;
    assign #SIM_TAPDELAY_VALUE delay_chain_12 = delay_chain_11;
    assign #SIM_TAPDELAY_VALUE delay_chain_13 = delay_chain_12;
    assign #SIM_TAPDELAY_VALUE delay_chain_14 = delay_chain_13;
    assign #SIM_TAPDELAY_VALUE delay_chain_15 = delay_chain_14;
    assign #SIM_TAPDELAY_VALUE delay_chain_16 = delay_chain_15;
    assign #SIM_TAPDELAY_VALUE delay_chain_17 = delay_chain_16;
    assign #SIM_TAPDELAY_VALUE delay_chain_18 = delay_chain_17;
    assign #SIM_TAPDELAY_VALUE delay_chain_19 = delay_chain_18;
    assign #SIM_TAPDELAY_VALUE delay_chain_20 = delay_chain_19;
    assign #SIM_TAPDELAY_VALUE delay_chain_21 = delay_chain_20;
    assign #SIM_TAPDELAY_VALUE delay_chain_22 = delay_chain_21;
    assign #SIM_TAPDELAY_VALUE delay_chain_23 = delay_chain_22;
    assign #SIM_TAPDELAY_VALUE delay_chain_24 = delay_chain_23;
    assign #SIM_TAPDELAY_VALUE delay_chain_25 = delay_chain_24;
    assign #SIM_TAPDELAY_VALUE delay_chain_26 = delay_chain_25;
    assign #SIM_TAPDELAY_VALUE delay_chain_27 = delay_chain_26;
    assign #SIM_TAPDELAY_VALUE delay_chain_28 = delay_chain_27;
    assign #SIM_TAPDELAY_VALUE delay_chain_29 = delay_chain_28;
    assign #SIM_TAPDELAY_VALUE delay_chain_30 = delay_chain_29;
    assign #SIM_TAPDELAY_VALUE delay_chain_31 = delay_chain_30;
    assign #SIM_TAPDELAY_VALUE delay_chain_32 = delay_chain_31;
    assign #SIM_TAPDELAY_VALUE delay_chain_33 = delay_chain_32;


    always @(*) begin

	case (delay_count)
            0:  o_out = delay_chain_0;
            1:  o_out = delay_chain_1;
            2:  o_out = delay_chain_2;
            3:  o_out = delay_chain_3;
            4:  o_out = delay_chain_4;
            5:  o_out = delay_chain_5;
            6:  o_out = delay_chain_6;
            7:  o_out = delay_chain_7;
            8:  o_out = delay_chain_8;
            9:  o_out = delay_chain_9;
            10: o_out = delay_chain_10;
            11: o_out = delay_chain_11;
            12: o_out = delay_chain_12;
            13: o_out = delay_chain_13;
            14: o_out = delay_chain_14;
            15: o_out = delay_chain_15;
            16: o_out = delay_chain_16;
            17: o_out = delay_chain_17;
            18: o_out = delay_chain_18;
            19: o_out = delay_chain_19;
            20: o_out = delay_chain_20;
            21: o_out = delay_chain_21;
            22: o_out = delay_chain_22;
            23: o_out = delay_chain_23;
            24: o_out = delay_chain_24;
            25: o_out = delay_chain_25;
            26: o_out = delay_chain_26;
            27: o_out = delay_chain_27;
            28: o_out = delay_chain_28;
            29: o_out = delay_chain_29;
            30: o_out = delay_chain_30;
            31: o_out = delay_chain_31;
            default:
		assign o_out = delay_chain_0;

	endcase
    end 

  buf(o,o_out);

endmodule


//--------------------------------------------------------------------
// Verilog HDL for "EG_LOGIC_IDDR", "functional"
//--------------------------------------------------------------------
`timescale 1 ns / 1 ps
module EG_LOGIC_IDDR (q1, q0, clk, d, rst);
    output q1;
    output q0;
    input clk;
    input d;
    input rst;

    parameter ASYNCRST = "ENABLE"; //ENABLE, DISABLE
    parameter PIPEMODE = "PIPED"; //PIPED, NONE

    tri1 gsr_n = glbl.gsrn;
    wire rst_node = (ASYNCRST=="ENABLE")? rst:1'b0;
    wire async_rst = ~gsr_n | rst_node;
    wire clk_node = clk; 
    reg [1:0] comp_reg,true_reg;

    
    always @(negedge clk_node or posedge async_rst)
    begin
     if(async_rst)
     comp_reg[0] <= 1'b0 ;
     else
     comp_reg[0] <= d ;
    end
    always @(posedge clk_node or posedge async_rst)
    begin
     if(async_rst)
     comp_reg[1] = 1'b0 ;
     else
     comp_reg[1] <= comp_reg[0] ;
    end
    
    always @(posedge clk_node or posedge async_rst)
    begin
     if(async_rst)
     true_reg[0] <= 1'b0 ;
     else
     true_reg[0] <= d ;
    end
    always @(posedge clk_node or posedge async_rst)
    begin
     if(async_rst)
     true_reg[1] = 1'b0 ;
     else
     true_reg[1] <= true_reg[0] ;
    end    
    
    wire true_dff = (PIPEMODE == "PIPED")? true_reg[1]:true_reg[0];
    buf (q1, comp_reg[1]);
    buf (q0, true_dff);          	

endmodule

//--------------------------------------------------------------------
// Verilog HDL for "EG_LOGIC_ODDR", "functional"
//--------------------------------------------------------------------
`timescale 1 ns / 1 ps
module EG_LOGIC_ODDR (q, clk, d1, d0, rst);
    output q;
    input clk;
    input d1;
    input d0;
    input rst;
    
    parameter ASYNCRST = "ENABLE"; //ENABLE, DISABLE

    tri1 gsr_n = glbl.gsrn;
    wire rst_node = (ASYNCRST=="ENABLE")? rst:1'b0;
    wire async_rst = ~gsr_n | rst_node ;   
    wire clk_node = clk;
    reg [1:0] comp_reg;
    reg true_reg;   
    wire ddr;
    buf (d_comp,d1);
    buf (d_true,d0);
    buf (q,ddr);
    always @(posedge clk_node or posedge async_rst)
    begin
     if(async_rst)
     comp_reg[0] = 1'b0 ;
     else
     comp_reg[0] = d_comp ;
    end
    always @(negedge clk_node or posedge async_rst)
    begin
     if(async_rst)
     comp_reg[1] = 1'b0 ;
     else
     comp_reg[1] = comp_reg[0] ;
    end

    always @(posedge clk_node or posedge async_rst)
    begin
     if(async_rst)
     true_reg = 1'b0 ;
     else
     true_reg = d_true ;
    end

    reg clk_mux;
    always @(clk_node)  clk_mux <= #0.1 clk_node; 
    assign   ddr = clk_mux ? true_reg : comp_reg[1] ;    

endmodule


//--------------------------------------------------------------------
// Verilog HDL for "EG_LOGIC_IDDRx2", "functional"
//--------------------------------------------------------------------
`timescale 1 ns / 1 ps
module EG_LOGIC_IDDRx2 (q3, q2, q1, q0, pclk, sclk, d, rst);
    output q3;
    output q2;
    output q1;
    output q0;
    input pclk;
    input sclk;
    input d;
    input rst;

    parameter ASYNCRST = "ENABLE"; //ENABLE, DISABLE

    tri1 gsr_n = glbl.gsrn;
    wire rst_node = (ASYNCRST=="ENABLE")? rst:1'b0;
    wire async_rst = ~gsr_n | rst_node;
    reg cap_reg;
    reg [1:0] cap_reg_1, cap_reg_2;
    reg [1:0] des_reg;
    reg [3:0] des_reg_1;

// Capture to sclk domain    
    always @(posedge sclk or posedge async_rst)
    begin  
     if(async_rst)
       cap_reg <= 1'b0;
     else
       cap_reg <= d;
    end
    
// Sync to sclk domain    
    always @(negedge sclk or posedge async_rst)
    begin
     if(async_rst)
       cap_reg_1[1:0] <= 2'b00 ;
     else
       cap_reg_1[1:0] <= {d,cap_reg};
    end

    always @(posedge sclk or posedge async_rst)
    begin
     if(async_rst)
       cap_reg_2 <= 2'b00 ;
     else
       cap_reg_2 <= cap_reg_1;
    end


// Capture to pclk domain    
    always @(negedge pclk or posedge async_rst)
    begin
     if(async_rst)
       des_reg[1:0] <= 2'b00 ;
     else
       des_reg[1:0] <= cap_reg_2 ;
    end    
    
// Deserialize in pclk domain
    always @(posedge pclk or posedge async_rst)
    begin
     if(async_rst)
       des_reg_1[3:0] <= 4'b0000 ;
     else
       des_reg_1[3:0] <= {cap_reg_2[1:0], des_reg[1:0]} ;
    end    

    buf (q3, des_reg_1[3]);
    buf (q2, des_reg_1[2]);          	
    buf (q1, des_reg_1[1]);
    buf (q0, des_reg_1[0]);          	

endmodule

//--------------------------------------------------------------------
// Verilog HDL for "EG_LOGIC_ODELAY", "functional"
//--------------------------------------------------------------------
`timescale 1 ns / 1 ps
module  EG_LOGIC_ODELAY(o,i);
output o;
input i;

parameter OUTDEL = 0;  // 0-3

localparam SIM_TAPDELAY_VALUE = 0.11;
reg [1:0] delay_count ;
reg o_out;

initial begin
  delay_count = OUTDEL;
end

    wire delay_chain_0,  delay_chain_1,  delay_chain_2,  delay_chain_3;

// delay chain

    assign #0.175              delay_chain_0  = i;
    assign #SIM_TAPDELAY_VALUE delay_chain_1  = delay_chain_0;
    assign #SIM_TAPDELAY_VALUE delay_chain_2  = delay_chain_1;
    assign #SIM_TAPDELAY_VALUE delay_chain_3  = delay_chain_2;


    always @(*) begin
	case (delay_count)
            0:  o_out = delay_chain_0;
            1:  o_out = delay_chain_1;
            2:  o_out = delay_chain_2;
            3:  o_out = delay_chain_3;
            default:
		o_out = delay_chain_0;
	endcase
    end 

    wire o_outm = o_out;

  buf(o,o_outm);

endmodule

//--------------------------------------------------------------------
// Verilog HDL for "EG_LOGIC_ODDRx2", "functional"
//--------------------------------------------------------------------
`timescale 1 ns / 1 ps
module EG_LOGIC_ODDRx2 (q, pclk, sclk, d3, d2, d1, d0, rst);
    output q;
    input pclk;
    input sclk;
    input d3;
    input d2;
    input d1;
    input d0;
    input rst;
    
    parameter ASYNCRST = "ENABLE"; //ENABLE, DISABLE

    tri1 gsr_n = glbl.gsrn;
    wire rst_node = (ASYNCRST=="ENABLE")? rst:1'b0;
    wire async_rst = ~gsr_n | rst_node;

    EG_LOGIC_ODDR x2to1_20 (.q(q20), .clk(pclk), .d1(d2), .d0(d0), .rst(async_rst));
    EG_LOGIC_ODDR x2to1_31 (.q(q31), .clk(pclk), .d1(d3), .d0(d1), .rst(async_rst));

    EG_LOGIC_ODDR x2to1_4to1 (.q(q), .clk(sclk), .d1(q31), .d0(q20), .rst(async_rst));

endmodule

//--------------------------------------------------------------------
// Verilog HDL for "EG_LOGIC_ODDRx2l", "functional"
//--------------------------------------------------------------------
`timescale 1 ns / 1 ps
module EG_LOGIC_ODDRx2l (q, sclk, d3, d2, d1, d0, rst);
    output q;
    input sclk;
    input d3;
    input d2;
    input d1;
    input d0;
    input rst;
    
    parameter ASYNCRST = "ENABLE"; //ENABLE, DISABLE

    tri1 gsr_n = glbl.gsrn;
    wire rst_node = (ASYNCRST=="ENABLE")? rst:1'b0;
    wire async_rst = ~gsr_n | rst_node;

    reg ckd2;

    initial begin
      ckd2 = 1'b0;
    end

    always @(posedge sclk or posedge async_rst)
      begin
        if(async_rst)
          ckd2 <= 1'b0;
        else
          ckd2 <= ~ckd2;
      end


    EG_LOGIC_ODDR x2to1_20 (.q(q20), .clk(ckd2), .d1(d2), .d0(d0), .rst(async_rst));
    EG_LOGIC_ODDR x2to1_31 (.q(q31), .clk(ckd2), .d1(d3), .d0(d1), .rst(async_rst));

    EG_LOGIC_ODDR x2to1_4to1 (.q(q), .clk(sclk), .d1(q31), .d0(q20), .rst(async_rst));

endmodule

/*
//--------------------------------------------------------------------
// Verilog HDL for "EG_LOGIC_BI_DDR", "functional"
//--------------------------------------------------------------------
`timescale 1 ns / 1 ps
***********module EG_LOGIC_BI_DDR (q, d1, d0, rst, osclk,
                        q0, q1, d, ipclk);
// oDDR ports    
    output q;
    input d1, d0;
    input osclk;
    input rst;

// iDDR ports    
    input d;
    output q1, q0;
    input ipclk;
    
    parameter ASYNCRST = "ENABLE"; //ENABLE, DISABLE
    parameter PIPEMODE = "PIPED"; //PIPED, NONE

    tri1 gsr_n = glbl.gsrn;
    wire rst_node = (ASYNCRST=="ENABLE")? rst:1'b0;
    wire async_rst = ~gsr_n | rst_node;

    EG_LOGIC_IDDR xiddr (.q1(q1), .q0(q0), .clk(ipclk), .d(d), .rst(async_rst));
//    defparam xiddr.ASYNCRST = ASYNCRST;
//    defparam xiddr.PIPEMODE = PIPEMODE;
    EG_LOGIC_ODDR xoddr (.q(q), .clk(osclk), .d1(d1), .d0(d0), .rst(async_rst));
//    defparam xoddr.ASYNCRST = ASYNCRST;

***********endmodule


//--------------------------------------------------------------------
// Verilog HDL for "EG_LOGIC_BI_DDRx2", "functional"
//--------------------------------------------------------------------
`timescale 1 ns / 1 ps
***********module EG_LOGIC_BI_DDRx2 (q, pclk, osclk, d3, d2, d1, d0, rst,
                          q3, q2, q1, q0, isclk, d);

// oDDRx2 ports    
    output q;
    input d3, d2, d1, d0;
    input rst;
    input pclk;
    input osclk;

// iDDRx2 ports    
    output q3, q2, q1, q0;
    input isclk;
    input d;

    parameter ASYNCRST = "ENABLE"; //ENABLE, DISABLE

    tri1 gsr_n = glbl.gsrn;
    wire rst_node = (ASYNCRST=="ENABLE")? rst:1'b0;
    wire async_rst = ~gsr_n | rst_node;

    // IDDRx2
    EG_LOGIC_IDDRx2 xiddrx2 (.q3(q3), .q2(q2), .q1(q1), .q0(q0), 
                             .pclk(pclk), .sclk(isclk), .d(d), .rst(async_rst));

    // ODDRx2
    EG_LOGIC_ODDRx2 xoddrx2 (.q(q), .pclk(pclk), .sclk(osclk), .rst(async_rst),
                             .d3(d3), .d2(d2), .d1(d1), .d0(d0));

***********endmodule
*/
