///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011-2021 Anlogic, Inc.
// All Right Reserved.
///////////////////////////////////////////////////////////////////////////////
//
//   Vendor : Anlogic
//   Version : 1.1
//   Description : Anlogic Hard IP Library Component
//            CONFIG block
//   Filename : eg_phy_config.v
//   Timestamp : Fri July  8 09:30:30 CST 2016
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    8/2/16 - Initial version.      
//    11/3/16 - v1.0      
///////////////////////////////////////////////////////////////////////////////

`timescale 1 ns / 10 ps

module EG_PHY_CONFIG(
  jrstn, jrti, jshift, jtck, jtdi, jupdate, jscanen, jtms, jtdo, jtag8_ipa, jtag8_ipb,
  done, highz,
  cclk, cclk_en,
  gsrn_sync_clk, usr_gsrn,
  dna_dout, dna_clk, dna_din, dna_shift_en,
  mboot_rebootn, mboot_dynamic_addr
);
  //chip watcher(customer should not see chip watcher interface)
  output       jrstn;
  output [1:0] jrti;
  output       jshift;
  output       jtck;
  output       jtdi;
  output       jupdate;
  output [1:0] jscanen;
  output       jtms;

  input [1:0]  jtdo;
  input [7:0]  jtag8_ipa;
  input [7:0]  jtag8_ipb;

  //global control
  output       	done;
  output       	highz;

  //cclk 
  output       	cclk;
  input	      	cclk_en;	

  //user gsrn
  input        	gsrn_sync_clk;
  input        	usr_gsrn;

  //DNA IP
  output		dna_dout;
  input		dna_clk;
  input		dna_din;
  input		dna_shift_en;

  //Multi boot
  input		mboot_rebootn;
  input [7:0] 	mboot_dynamic_addr;

  tri0 cclk_en;

  //Parameter
  //multiboot
  parameter MBOOT_AUTO_SEL = "DISABLE";	// DISABLE/ENABLE	
  parameter ADDR_SOURCE_SEL = "STATIC";	// STATIC/DYNAMIC. STATIC:0, DYNAMIC:1
  parameter STATIC_ADDR = 8'b0;		// 0/1	
  //io
  parameter DONE_PERSISTN = "ENABLE";	// ENABLE/DISABLE	
  parameter INIT_PERSISTN = "ENABLE";	// ENABLE/DISABLE
  parameter PROGRAMN_PERSISTN = "DISABLE";// DISABLE/ENABLE
  parameter JTAG_PERSISTN = "DISABLE";	// DISABLE/ENABLE
  //parameter(register)
  parameter GSRN_SYNC_SEL = "DISABLE";	// DISABLE/ENABLE set by wakeup register
  parameter FREQ = "2.5";			// frequency set by wakeup register
  parameter USR_GSRN_EN = "DISABLE";	// DISABLE/ENABLE set by wakeup register

  // GSR implementation
  reg [1:0] usr_gsrn_sync;

  initial
  begin
    usr_gsrn_sync = 2'b00;
  end

  always @(posedge gsrn_sync_clk)
    usr_gsrn_sync <= {usr_gsrn_sync[1], usr_gsrn};

  always @(*)
  begin
    glbl.usr_gsrn_en <= (USR_GSRN_EN == "ENABLE") ? 1'b1 : 1'b0;
    glbl.gsrn <= (USR_GSRN_EN == "ENABLE") ? ((GSRN_SYNC_SEL == "ENABLE") ? usr_gsrn_sync[1] : usr_gsrn) : 1'b1;
    glbl.gsr <= ~glbl.gsrn;
  end

endmodule

