///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011-2021 Anlogic, Inc.
// All Right Reserved.
///////////////////////////////////////////////////////////////////////////////
//
//
//   Vendor : Anlogic
//   Version : 0.1
//   Description : Anlogic Functional Simulation Library Component
//            18X18/9x9 Signed Multiplier
//   Filename : eg_phy_mult18.v
//   Timestamp : Mon Nov  4 14:20:41 CST 2013
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    11/04/13 - Initial version.
//    06/20/14 - update INVMUX parameter
///////////////////////////////////////////////////////////////////////////////

`timescale  1 ns / 10 ps

module EG_PHY_MULT18 (acout, bcout, p, signeda, signedb, a, b, acin, bcin, cea, ceb, cepd, clk, rstan, rstbn, rstpdn,
            sourcea, sourceb); 

    output [17:0] acout; 
    output [17:0] bcout; 
    output [35:0] p; 

    input signeda;
    input signedb;
    input [17:0] a;
    input [17:0] b;
    input [17:0] acin;
    input [17:0] bcin;
    input cea;
    input ceb;
    input cepd;
    input clk;
    input rstan;
    input rstbn;
    input rstpdn;
    input sourcea;
    input sourceb;


    tri1 sourcea, sourceb;
    tri1 signeda, signedb;
    tri1 cea, ceb;
    tri1 cepd, rstan, rstbn, rstpdn;
    tri1 [17:0] a,b,acin,bcin;

    tri1 gsrn = glbl.gsrn;
    parameter INPUTREGA = "ENABLE"; // ENABLE/DISABLE
    parameter INPUTREGB = "ENABLE"; // ENABLE/DISABLE
    parameter OUTPUTREG = "ENABLE"; // ENABLE/DISABLE
    parameter SRMODE    = "ASYNC";  // ASYNC/SYNC
    parameter MODE      = "MULT18X18C"; //MULT9X9C/MULT18X18C
    
    parameter CEAMUX = "SIG"; //0, 1, INV, SIG
    parameter CEBMUX = "SIG"; //0, 1, INV, SIG
    parameter CEPDMUX = "SIG"; //0, 1, INV, SIG
    parameter RSTANMUX = "SIG"; //0, 1, INV, SIG
    parameter RSTBNMUX = "SIG"; //0, 1, INV, SIG
    parameter RSTPDNMUX = "SIG"; //0, 1, INV, SIG
    parameter CLKMUX = "SIG"; //0, 1, INV, SIG
    parameter SIGNEDAMUX = "SIG"; //0, 1, INV, SIG
    parameter SIGNEDBMUX = "SIG"; //0, 1, INV, SIG
    parameter SOURCEAMUX = "SIG"; //0, 1, INV, SIG
    parameter SOURCEBMUX = "SIG"; //0, 1, INV, SIG

// for SDF
  reg [35:0] p_tmp;
  reg [17:0] ir_a,ir_b;
  buf (p[0],  p_tmp[0]);
  buf (p[1],  p_tmp[1]);
  buf (p[2],  p_tmp[2]);
  buf (p[3],  p_tmp[3]);
  buf (p[4],  p_tmp[4]);
  buf (p[5],  p_tmp[5]);
  buf (p[6],  p_tmp[6]);
  buf (p[7],  p_tmp[7]);
  buf (p[8],  p_tmp[8]);
  buf (p[9],  p_tmp[9]);
  buf (p[10], p_tmp[10]);
  buf (p[11], p_tmp[11]);
  buf (p[12], p_tmp[12]);
  buf (p[13], p_tmp[13]);
  buf (p[14], p_tmp[14]);
  buf (p[15], p_tmp[15]);
  buf (p[16], p_tmp[16]);
  buf (p[17], p_tmp[17]);
  buf (p[18], p_tmp[18]);
  buf (p[19], p_tmp[19]);  
  buf (p[20], p_tmp[20]);
  buf (p[21], p_tmp[21]);
  buf (p[22], p_tmp[22]);
  buf (p[23], p_tmp[23]);
  buf (p[24], p_tmp[24]);
  buf (p[25], p_tmp[25]);
  buf (p[26], p_tmp[26]);
  buf (p[27], p_tmp[27]);
  buf (p[28], p_tmp[28]);
  buf (p[29], p_tmp[29]);    
  buf (p[30], p_tmp[30]);
  buf (p[31], p_tmp[31]);
  buf (p[32], p_tmp[32]);
  buf (p[33], p_tmp[33]);
  buf (p[34], p_tmp[34]);
  buf (p[35], p_tmp[35]);  
 
  buf (acout[0],  ir_a[0]);
  buf (acout[1],  ir_a[1]);
  buf (acout[2],  ir_a[2]);
  buf (acout[3],  ir_a[3]);
  buf (acout[4],  ir_a[4]);
  buf (acout[5],  ir_a[5]);
  buf (acout[6],  ir_a[6]);
  buf (acout[7],  ir_a[7]);
  buf (acout[8],  ir_a[8]);
  buf (acout[9],  ir_a[9]);
  buf (acout[10], ir_a[10]);
  buf (acout[11], ir_a[11]);
  buf (acout[12], ir_a[12]);
  buf (acout[13], ir_a[13]);
  buf (acout[14], ir_a[14]);
  buf (acout[15], ir_a[15]);
  buf (acout[16], ir_a[16]);
  buf (acout[17], ir_a[17]);

  buf (bcout[0],  ir_b[0]);
  buf (bcout[1],  ir_b[1]);
  buf (bcout[2],  ir_b[2]);
  buf (bcout[3],  ir_b[3]);
  buf (bcout[4],  ir_b[4]);
  buf (bcout[5],  ir_b[5]);
  buf (bcout[6],  ir_b[6]);
  buf (bcout[7],  ir_b[7]);
  buf (bcout[8],  ir_b[8]);
  buf (bcout[9],  ir_b[9]);
  buf (bcout[10], ir_b[10]);
  buf (bcout[11], ir_b[11]);
  buf (bcout[12], ir_b[12]);
  buf (bcout[13], ir_b[13]);
  buf (bcout[14], ir_b[14]);
  buf (bcout[15], ir_b[15]);
  buf (bcout[16], ir_b[16]);
  buf (bcout[17], ir_b[17]);

//sdf end

    reg [35:0] pd_r;
    reg [35:0] mux_pt;
    reg [17:0] multi_a;
    reg [17:0] multi_b;

    reg [17:0] a_se;
    reg [17:0] b_se;
    reg [17:0] imux_a;
    reg [17:0] imux_b;

    reg rsta_asyn_n;
    reg rstb_asyn_n;
    reg rstp_asyn_n;
    reg rsta_sync_n;
    reg rstb_sync_n;
    reg rstp_sync_n;

    wire [17:0] m9_a0;
    wire [17:0] m9_b0;
    wire [17:0] m9_a1;
    wire [17:0] m9_b1;
    wire [17:0] m9_pt0;
    wire [17:0] m9_pt1;
    wire [35:0] m18_a;
    wire [35:0] m18_b;
    wire [35:0] m18_pt;

    reg rstan_node,rstbn_node,rstpdn_node;
    reg cea_node,ceb_node,cepd_node;
    reg sourcea_node,sourceb_node;
    reg signeda_node,signedb_node;
    reg clk_node;
    
reg param_trig;
initial begin
param_trig=0;
param_trig=1;
end

   always @(param_trig)
   begin
        case (RSTANMUX)
            "SIG"   	: assign rstan_node = rstan;
            "0" 	: assign rstan_node = 0;
            "1" 	: assign rstan_node = 1;
            "INV"   	: assign rstan_node = ~rstan;
            default 	: assign rstan_node = rstan;
        endcase
        case (RSTBNMUX)
            "SIG"   	: assign rstbn_node = rstbn;
            "0" 	: assign rstbn_node = 0;
            "1" 	: assign rstbn_node = 1;
            "INV"   	: assign rstbn_node = ~rstbn;
            default 	: assign rstbn_node = rstbn;
        endcase          
        case (RSTPDNMUX)
            "SIG"   	: assign rstpdn_node = rstpdn;
            "0" 	: assign rstpdn_node = 0;
            "1" 	: assign rstpdn_node = 1;
            "INV"   	: assign rstpdn_node = ~rstpdn;
            default 	: assign rstpdn_node = rstpdn;
        endcase          
        case (CEAMUX)
            "SIG"   	: assign cea_node = cea;
            "0" 	: assign cea_node = 0;
            "1" 	: assign cea_node = 1;
            "INV"   	: assign cea_node = ~cea;
            default 	: assign cea_node = cea;
        endcase
        case (CEBMUX)
            "SIG"   	: assign ceb_node = ceb;
            "0" 	: assign ceb_node = 0;
            "1" 	: assign ceb_node = 1;
            "INV"   	: assign ceb_node = ~ceb;
            default 	: assign ceb_node = ceb;
        endcase        
        case (CEPDMUX)
            "SIG"   	: assign cepd_node = cepd;
            "0" 	: assign cepd_node = 0;
            "1" 	: assign cepd_node = 1;
            "INV"   	: assign cepd_node = ~cepd;
            default 	: assign cepd_node = cepd;
        endcase
        case (SOURCEAMUX)
            "SIG"   	: assign sourcea_node = sourcea;
            "0" 	: assign sourcea_node = 0;
            "1" 	: assign sourcea_node = 1;
            "INV"   	: assign sourcea_node = ~sourcea;
            default 	: assign sourcea_node = sourcea;
        endcase
        case (SOURCEBMUX)
            "SIG"   	: assign sourceb_node = sourceb; 
            "0" 	: assign sourceb_node = 0;       
            "1" 	: assign sourceb_node = 1;       
            "INV"   	: assign sourceb_node = ~sourceb;
            default 	: assign sourceb_node = sourceb; 
        endcase        
        case (SIGNEDAMUX)
            "SIG"   	: assign signeda_node = signeda;
            "0" 	: assign signeda_node = 0;
            "1" 	: assign signeda_node = 1;
            "INV"   	: assign signeda_node = ~signeda;
            default 	: assign signeda_node = signeda;
        endcase
        case (SIGNEDBMUX)
            "SIG"   	: assign signedb_node = signedb; 
            "0" 	: assign signedb_node = 0;       
            "1" 	: assign signedb_node = 1;       
            "INV"   	: assign signedb_node = ~signedb;
            default 	: assign signedb_node = signedb; 
        endcase        
        case (CLKMUX)
            "SIG"   	: assign clk_node = clk;
            "0" 	: assign clk_node = 0;
            "1" 	: assign clk_node = 1;
            "INV"   	: assign clk_node = ~clk;
            default 	: assign clk_node = clk;
        endcase
       
   end // always



//*** GSRN pin
    always @(gsrn) begin
        if (!gsrn) begin
            assign ir_a = 18'b0;
            assign ir_b = 18'b0;
            assign pd_r = 36'b0;
        end
        else begin
            deassign ir_a;
            deassign ir_b;
            deassign pd_r;
        end
    end

    always @(rstan_node or rstbn_node or rstpdn_node) begin
      case(SRMODE)
        "ASYNC": begin
               rsta_asyn_n <= rstan_node;
               rstb_asyn_n <= rstbn_node;
               rstp_asyn_n <= rstpdn_node;
               rsta_sync_n <= rstan_node;
               rstb_sync_n <= rstbn_node;
               rstp_sync_n <= rstpdn_node;
          end
        "SYNC": begin
               rsta_asyn_n <= 1'b1;
               rstb_asyn_n <= 1'b1;
               rstp_asyn_n <= 1'b1;
               rsta_sync_n <= rstan_node;
               rstb_sync_n <= rstbn_node;
               rstp_sync_n <= rstpdn_node;
          end
        default: begin
              $display("attribute syntax error : the attribute SRMODE on dsp instance %m is set to %s. legal values for this attribute are ASYNC or SYNC.", SRMODE);
	      $finish;
          end
      endcase
    end

    always @(a or acin or sourcea_node) begin
      case(sourcea_node)
         0: imux_a <= acin;
         1: imux_a <= a;
      endcase
    end

    always @(b or bcin or sourceb_node) begin
      case(sourceb_node)
         0: imux_b <= bcin;
         1: imux_b <= b;
      endcase
    end

// Latch data
    always @(posedge clk_node or negedge rsta_asyn_n) begin
      if(!rsta_asyn_n) ir_a <= 18'b0;
      else if(!rsta_sync_n) ir_a <= 18'b0;
      else if(cea_node) ir_a <= imux_a;
    end
       
    always @(posedge clk_node or negedge rstb_asyn_n) begin
      if(!rstb_asyn_n) ir_b <= 18'b0;
      else if(!rstb_sync_n) ir_b <= 18'b0;
      else if(ceb_node) ir_b <= imux_b;
    end

    always @(posedge clk_node or negedge rstp_asyn_n) begin
      if(!rstp_asyn_n) pd_r <= 36'b0;
      else if(!rstp_sync_n) pd_r <= 36'b0;
      else if(cepd_node) pd_r <= mux_pt;
    end

// Bypass Register
   always @(ir_a or imux_a) begin
     case(INPUTREGA)
         "ENABLE": multi_a <= ir_a;
         "DISABLE": multi_a <= imux_a;
         default: begin
              $display("attribute syntax error : the attribute INPUTREGA on dsp instance %m is set to %s. legal values for this attribute are ENABLE or DISABLE.", INPUTREGA);
	      $finish;
          end
     endcase
   end

   always @(ir_b or imux_b) begin
     case(INPUTREGB)
         "ENABLE": multi_b <= ir_b;
         "DISABLE": multi_b <= imux_b;
         default: begin
              $display("attribute syntax error : the attribute INPUTREGB on dsp instance %m is set to %s. legal values for this attribute are ENABLE or DISABLE.", INPUTREGB);
	      $finish;
          end
     endcase
   end

   always @(pd_r or mux_pt) begin
     case(OUTPUTREG)
         "ENABLE": p_tmp <= pd_r;
         "DISABLE": p_tmp <= mux_pt;
         default: begin
              $display("attribute syntax error : the attribute OUTPUTREG on dsp instance %m is set to %s. legal values for this attribute are ENABLE or DISABLE.", OUTPUTREG);
	      $finish;
          end
     endcase
   end

// Sign extention   
   
     assign m9_a0 = {{9{signeda_node&multi_a[8]}},multi_a[8:0]};
     assign m9_b0 = {{9{signedb_node&multi_b[8]}},multi_b[8:0]};

     assign m9_a1 = {{9{signeda_node&multi_a[17]}},multi_a[17:9]};
     assign m9_b1 = {{9{signedb_node&multi_b[17]}},multi_b[17:9]};

     assign m18_a = {{18{signeda_node&multi_a[17]}},multi_a[17:0]};
     assign m18_b = {{18{signedb_node&multi_b[17]}},multi_b[17:0]};

     assign m9_pt0 = m9_a0 * m9_b0;
     assign m9_pt1 = m9_a1 * m9_b1;

     assign m18_pt = m18_a * m18_b;
   
     assign acout = ir_a;
     assign bcout = ir_b;

// M18x18/M9x9
   always @(m9_pt0 or m9_pt1 or m18_pt) begin
     case(MODE)
       "MULT9X9C":   mux_pt = {m9_pt1[17:0],m9_pt0[17:0]};
       "MULT18X18C": mux_pt = m18_pt;
       default: begin
            $display("attribute syntax error : the attribute MODE on dsp instance %m is set to %s. legal values for this attribute are MULT9X9C or mult18x18.", MODE); $finish;
          end
     endcase
   end

wire done_gwe = glbl.done_gwe;
and ( cea_check, done_gwe,cea_node);
and ( ceb_check, done_gwe,ceb_node);
 specify                                      
        (clk  *> acout) = (0:0:0, 0:0:0);
        (clk  *> bcout) = (0:0:0, 0:0:0);
        (clk  *> p) = (0:0:0, 0:0:0);
        (a *> p) = (0:0:0, 0:0:0);
        (b *> p) = (0:0:0, 0:0:0);        
        (acin *> p) = (0:0:0, 0:0:0);
        (bcin *> p) = (0:0:0, 0:0:0);  
        $recovery (rstan,  posedge clk &&& (done_gwe!=0), 0:0:0 );
        $recovery (rstbn,  posedge clk &&& (done_gwe!=0), 0:0:0 );
        $recovery (rstpdn, posedge clk &&& (done_gwe!=0), 0:0:0 );
        $recovery (rstan,  negedge clk &&& (done_gwe!=0), 0:0:0 );
        $recovery (rstbn,  negedge clk &&& (done_gwe!=0), 0:0:0 );
        $recovery (rstpdn, negedge clk &&& (done_gwe!=0), 0:0:0 );
        
        $removal (rstan,  posedge clk &&& (done_gwe!=0), 0:0:0 );
        $removal (rstbn,  posedge clk &&& (done_gwe!=0), 0:0:0 );
        $removal (rstpdn, posedge clk &&& (done_gwe!=0), 0:0:0 );
        $removal (rstan,  negedge clk &&& (done_gwe!=0), 0:0:0 );
        $removal (rstbn,  negedge clk &&& (done_gwe!=0), 0:0:0 );
        $removal (rstpdn, negedge clk &&& (done_gwe!=0), 0:0:0 ); 

	$setuphold (posedge clk, posedge rstan 	&&& (done_gwe!=0), 0:0:0, 0:0:0);
        $setuphold (posedge clk, posedge cea 	&&& (done_gwe!=0), 0:0:0, 0:0:0);                   
	$setuphold (posedge clk, posedge signeda &&& (cea_check!=0)  	, 0:0:0, 0:0:0);
	$setuphold (posedge clk, posedge a 	 &&& (cea_check!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge clk, posedge acin 	 &&& (cea_check!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge clk, posedge sourcea &&& (cea_check!=0)  	, 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge rstbn 	&&& (done_gwe!=0), 0:0:0, 0:0:0);		
	$setuphold (posedge clk, posedge ceb 	&&& (done_gwe!=0), 0:0:0, 0:0:0);	
	$setuphold (posedge clk, posedge signedb &&& (ceb_check!=0) 	, 0:0:0, 0:0:0);		
	$setuphold (posedge clk, posedge b 	 &&& (ceb_check!=0), 0:0:0, 0:0:0);			
	$setuphold (posedge clk, posedge bcin 	 &&& (ceb_check!=0), 0:0:0, 0:0:0);	
	$setuphold (posedge clk, posedge sourceb &&& (ceb_check!=0) 	, 0:0:0, 0:0:0);		
	$setuphold (posedge clk, posedge cepd 	&&& (done_gwe!=0), 0:0:0, 0:0:0);

	$setuphold (posedge clk, negedge rstan 	&&& (done_gwe!=0), 0:0:0, 0:0:0);
        $setuphold (posedge clk, negedge cea 	&&& (done_gwe!=0), 0:0:0, 0:0:0);                   
	$setuphold (posedge clk, negedge signeda &&& (cea_check!=0)  	, 0:0:0, 0:0:0);
	$setuphold (posedge clk, negedge a 	 &&& (cea_check!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge clk, negedge acin 	 &&& (cea_check!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge clk, negedge sourcea &&& (cea_check!=0)  	, 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge rstbn 	&&& (done_gwe!=0), 0:0:0, 0:0:0);		
	$setuphold (posedge clk, negedge ceb 	&&& (done_gwe!=0), 0:0:0, 0:0:0);	
	$setuphold (posedge clk, negedge signedb &&& (ceb_check!=0) 	, 0:0:0, 0:0:0);		
	$setuphold (posedge clk, negedge b 	 &&& (ceb_check!=0), 0:0:0, 0:0:0);			
	$setuphold (posedge clk, negedge bcin 	 &&& (ceb_check!=0), 0:0:0, 0:0:0);	
	$setuphold (posedge clk, negedge sourceb &&& (ceb_check!=0) 	, 0:0:0, 0:0:0);		
	$setuphold (posedge clk, negedge cepd 	&&& (done_gwe!=0), 0:0:0, 0:0:0);					

	$setuphold (negedge clk, posedge rstan 	&&& (done_gwe!=0), 0:0:0, 0:0:0);
        $setuphold (negedge clk, posedge cea 	&&& (done_gwe!=0), 0:0:0, 0:0:0);                   
	$setuphold (negedge clk, posedge signeda &&& (cea_check!=0)  	, 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge a 	 &&& (cea_check!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge acin 	 &&& (cea_check!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge sourcea &&& (cea_check!=0)  	, 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge rstbn &&& (done_gwe!=0)	, 0:0:0, 0:0:0);		
	$setuphold (negedge clk, posedge ceb 	&&& (done_gwe!=0), 0:0:0, 0:0:0);	
	$setuphold (negedge clk, posedge signedb &&& (ceb_check!=0) 	, 0:0:0, 0:0:0);		
	$setuphold (negedge clk, posedge b 	 &&& (ceb_check!=0), 0:0:0, 0:0:0);			
	$setuphold (negedge clk, posedge bcin 	 &&& (ceb_check!=0), 0:0:0, 0:0:0);	
	$setuphold (negedge clk, posedge sourceb &&& (ceb_check!=0) 	, 0:0:0, 0:0:0);		
	$setuphold (negedge clk, posedge cepd 	&&& (done_gwe!=0), 0:0:0, 0:0:0);

	$setuphold (negedge clk, negedge rstan 	&&& (done_gwe!=0), 0:0:0, 0:0:0);
        $setuphold (negedge clk, negedge cea 	&&& (done_gwe!=0), 0:0:0, 0:0:0);                   
	$setuphold (negedge clk, negedge signeda &&& (cea_check!=0)  	, 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge a 	 &&& (cea_check!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge acin 	 &&& (cea_check!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge sourcea &&& (cea_check!=0)  	, 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge rstbn 	&&& (done_gwe!=0), 0:0:0, 0:0:0);		
	$setuphold (negedge clk, negedge ceb 	&&& (done_gwe!=0), 0:0:0, 0:0:0);	
	$setuphold (negedge clk, negedge signedb &&& (ceb_check!=0) 	, 0:0:0, 0:0:0);		
	$setuphold (negedge clk, negedge b 	 &&& (ceb_check!=0), 0:0:0, 0:0:0);			
	$setuphold (negedge clk, negedge bcin 	 &&& (ceb_check!=0), 0:0:0, 0:0:0);	
	$setuphold (negedge clk, negedge sourceb &&& (ceb_check!=0) 	, 0:0:0, 0:0:0);		
	$setuphold (negedge clk, negedge cepd 	&&& (done_gwe!=0), 0:0:0, 0:0:0);		

 endspecify

endmodule // dsp
