///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011-2021 Anlogic, Inc.
// All Right Reserved.
///////////////////////////////////////////////////////////////////////////////
//
//
//   Vendor : Anlogic
//   Version : 0.1
//   Description : Anlogic Functional Simulation Library Component
//            config gsrn as user gsrn,
//   Filename : elf_logic_config.v
//   Timestamp : Wed Nov  2 17:27:07 CST 2016
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    11/02/16 - Initial version.
//
///////////////////////////////////////////////////////////////////////////////

`timescale  1 ns / 1 ps

module  ELF_LOGIC_GSRN(gsrn, sync_clk);   	// config gsrn
input gsrn;
input sync_clk; 

  //parameter 
  parameter USR_GSRN = "DISABLE";     //DISABLE, ENABLE
  parameter GSRMODE = "ASYNC";        //ASYNC, SYNC
  
  wire cfg_local_gsrn_out;

  reg cfg_local_gsrn;
  reg [2:0] gsrn_out_reg;

  initial
  begin
    cfg_local_gsrn = 1'b0;
    #10;
    cfg_local_gsrn = 1'b1;
  end

  always @(posedge sync_clk or negedge cfg_local_gsrn_out)
    if(~cfg_local_gsrn_out)
      gsrn_out_reg <= 3'b0;
    else
      gsrn_out_reg <= {1'b1, gsrn_out_reg[2:1]};

  assign cfg_local_gsrn_out = (USR_GSRN == "ENABLE") ? gsrn : cfg_local_gsrn; 

  always @(*)
  begin
    #0.1 glbl.usr_gsrn_en <= (USR_GSRN == "ENABLE") ? 1'b1 : 1'b0;
    #0.1 glbl.gsrn = (GSRMODE == "SYNC") ? gsrn_out_reg[0] : cfg_local_gsrn_out;
    #0.1 glbl.gsr <= ~glbl.gsrn;
  end
endmodule

