///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011-2021 Anlogic, Inc.
// All Right Reserved.
///////////////////////////////////////////////////////////////////////////////
//
//
//   Vendor : Anlogic
//   Version : 0.1
//   Description : Anlogic Functional Simulation Library Component
//            PHY_CONFIG
//   Filename : elf_phy_config.v
//   Timestamp : Tue Jun 28 15:57:32 CST 2016
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    06/28/16 - Initial version.
//
///////////////////////////////////////////////////////////////////////////////
module ELF_PHY_CONFIG(
  wb_dat_o,wb_wt_en,wb_cyc_en,wb_stb_en,wb_rst, wb_dat_i,
  wb_adr_i, wb_clk_i,ufm_tbit_o, pib_ufm_cs,pib_ufm_ae,
  pib_ufm_oe,pib_ufm_prog,pib_ufm_nvstr,pib_ufm_sera,
  pib_done,pib_freeze_io,cclk,pib_programn,pib_ts_all,
  pib_user_clk,pib_user_gsr,gsrn_sync_clk,jce,jrstn,
  jrti,jshift,jtck,jtdi,jtms,jupdate,jtdo);

  output [7:0] wb_dat_o;
  input wb_wt_en;  
  input wb_cyc_en; 
  input wb_stb_en; 
  input wb_rst;    
  input [7:0] wb_dat_i;
  input [3:0] wb_adr_i;
  input wb_clk_i;  
  
  //ufm interface
  output ufm_tbit_o; 
  input pib_ufm_cs;
  input pib_ufm_ae;
  input pib_ufm_oe;
  input pib_ufm_prog;
  input pib_ufm_nvstr;
  input pib_ufm_sera;
  
  //to pib
  output pib_done;
  output pib_freeze_io;
  output cclk;
  
  //from pib
  input pib_programn;
  input pib_ts_all;
  input pib_user_clk;
  input pib_user_gsr;
  input gsrn_sync_clk;
  
  //isp interface
  output [1:0] jce;
  output jrstn;
  output [1:0] jrti;
  output jshift;
  output jtck;
  output jtdi;
  output jtms;
  output jupdate;
  input [1:0] jtdo;

  parameter DISOSC = "DISABLE";       //DISABLE, ENABLE
  parameter CFGSTDBY = "DISABLE";     //DISABLE, ENABLE
  parameter I2CWAKEUP = "DISABLE";    //DISABLE, ENABLE
  parameter TSALL = "DISABLE";        //DISABLE, ENABLE
  parameter TSINV = "DISABLE";        //DISABLE, ENABLE
  parameter ISP1 = "DISABLE";         //DISABLE, ENABLE
  parameter ISP2 = "DISABLE";         //DISABLE, ENABLE

  parameter DONE_PHASE = "0";         //0,1,2,3
  parameter GOE_PHASE = "0";          //0,1,2,3
  parameter GSR_PHASE = "0";          //0,1,2,3
  parameter GWE_PHASE = "0";          //0,1,2,3

  parameter SYNSRC = "DISABLE";       //DISABLE, ENABLE
  parameter SYNEXTDONE = "DISABLE";   //DISABLE, ENABLE
  parameter FORCEGSRN = "DISABLE";    //DISABLE, ENABLE
  parameter FORCEGWE = "DISABLE";     //DISABLE, ENABLE
  parameter USR_GSRN = "DISABLE";     //DISABLE, ENABLE
  parameter GSRMODE = "ASYNC";        //ASYNC, SYNC
  parameter I2C_PERSISTN = "DISABLE"; //DISABLE, ENABLE
  parameter MSPI_PERSIST = "DISABLE"; //DISABLE, ENABLE
  parameter PLLCHECK = "DISABLE";     //DISABLE, ENABLE
  parameter PIBMCLK = "DISABLE";      //DISABLE, ENABLE
  parameter PIBMCLKDIV = 1;         //1-64 (same as pll div for bitgen)
  parameter PWRDWN = "DISABLE";       //DISABLE, ENABL
  parameter mc1_i2c_mspi = "0";       // 0, 1
  parameter mc1_i2c_persistn = "0";       // 0, 1

wire done_gwe = glbl.done_gwe;
// TO BE FILLED

  // GSR implementation
  wire cfg_local_gsrn_out;

  reg cfg_local_gsrn;
  reg [2:0] gsrn_out_reg;

  initial
  begin
    cfg_local_gsrn = 1'b0;
    #10;
    cfg_local_gsrn = 1'b1;
  end

  always @(posedge gsrn_sync_clk or negedge cfg_local_gsrn_out)
    if(~cfg_local_gsrn_out)
      gsrn_out_reg <= 3'b0;
    else
      gsrn_out_reg <= {1'b1, gsrn_out_reg[2:1]};

  assign cfg_local_gsrn_out = (USR_GSRN == "ENABLE") ? pib_user_gsr : cfg_local_gsrn; 

  always @(*)
  begin
    #0.1 glbl.usr_gsrn_en <= (USR_GSRN == "ENABLE") ? 1'b1 : 1'b0;
    #0.1 glbl.gsrn = (GSRMODE == "SYNC") ? gsrn_out_reg[0] : cfg_local_gsrn_out;
    #0.1 glbl.gsr <= ~glbl.gsrn;
  end

 specify
        (wb_clk_i  *> wb_dat_o) = (0.5:0.5:0.5, 0.5:0.5:0.5);
        
        $setuphold (posedge wb_clk_i, posedge wb_wt_en 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);                        
	$setuphold (posedge wb_clk_i, posedge wb_cyc_en &&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (posedge wb_clk_i, posedge wb_stb_en &&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (posedge wb_clk_i, posedge wb_wt_en 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge wb_clk_i, posedge wb_rst 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (posedge wb_clk_i, posedge wb_dat_i 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (posedge wb_clk_i, posedge wb_adr_i 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);	

        $setuphold (posedge wb_clk_i, negedge wb_wt_en 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);                        
	$setuphold (posedge wb_clk_i, negedge wb_cyc_en &&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (posedge wb_clk_i, negedge wb_stb_en &&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (posedge wb_clk_i, negedge wb_wt_en 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge wb_clk_i, negedge wb_rst 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (posedge wb_clk_i, negedge wb_dat_i 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);	
	$setuphold (posedge wb_clk_i, negedge wb_adr_i 	&&& (done_gwe!=0) , 0:0:0, 0:0:0);	
 endspecify 
endmodule

