///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011-2021 Anlogic, Inc.
// All Right Reserved.
///////////////////////////////////////////////////////////////////////////////
//
//
//   Vendor : Anlogic
//   Version : 0.1
//   Description : Anlogic Functional Simulation Library Component
//            dram
//   Filename : elf_logic_dram.v
//   Timestamp : Mon Nov  4 14:20:41 CST 2015
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    11/04/15 - Initial version.
//
///////////////////////////////////////////////////////////////////////////////
// simulation model for ELF DRAM16X4 and DRAM16X8
`timescale 1 ns / 1 ps
module ELF_LOGIC_DRAM32X4 ( di, waddr, we, wclk, raddr, do);
parameter INIT_D0=16'h0000;
parameter INIT_D1=16'h0000;
parameter INIT_D2=16'h0000;
parameter INIT_D3=16'h0000;
parameter INIT_D4=16'h0000;
parameter INIT_D5=16'h0000;
parameter INIT_D6=16'h0000;
parameter INIT_D7=16'h0000;
  input [3:0]di;
  input [4:0]waddr;
  input wclk;
  input we;
  input [4:0]raddr;
  output [3:0]do;

wire dpram_mode,dpram_we;
wire [3:0]dpram_di;
wire [4:0]dpram_waddr;
wire dpram_wclk; 
wire [3:0]NOUSE;

// PLB MSLICE
	ELF_PHY_MSLICE 
	  #(
	  .MODE("DPRAM"),
	  .INIT_LUT0(INIT_D0),
	  .INIT_LUT1(INIT_D1),
	  .FMUX0("FUNC5")
	  )
	  	MSLICE_0  (
	  .glb(raddr[1]),
	  .a({raddr[0],raddr[0]}),
	  .b({waddr[3],di[0]}),
	  .c({raddr[2],raddr[2]}),
	  .d({raddr[3],raddr[3]}),
	  .clk(wclk),
	  .mi({1'b1,raddr[4]}),
	  .f({NOUSE[0],do[0]}),
	  .dpram_mode(dpram_mode),
	  .dpram_di(dpram_di[0]),
	  .dpram_we(dpram_we),
	  .dpram_waddr(dpram_waddr[4:0]),
	  .dpram_wclk(dpram_wclk),
	  .sload(we)
	  );
	
	ELF_PHY_MSLICE 
	  #(
	  .MODE("DPRAM"),
	  .INIT_LUT0(INIT_D2),
	  .INIT_LUT1(INIT_D3),
	  .FMUX0("FUNC5")
	   )
	  	MSLICE_1 (
	  .glb(raddr[1]),
	  .a({raddr[0],raddr[0]}),
	  .b({waddr[4],di[1]}),
	  .c({raddr[2],raddr[2]}),
	  .d({raddr[3],raddr[3]}),
	  .clk(),
	  .mi({1'b1,raddr[4]}),
	  .f({NOUSE[1],do[1]}),
	  .dpram_mode(dpram_mode),
	  .dpram_di(dpram_di[1]),
	  .dpram_we(dpram_we),
	  .dpram_waddr(dpram_waddr[4:0]),
	  .dpram_wclk(dpram_wclk),
	  .sload(we)
	  );

	ELF_PHY_MSLICE 
	  #(
	  .MODE("DPRAM"),
	  .INIT_LUT0(INIT_D4),
	  .INIT_LUT1(INIT_D5),
	  .FMUX0("FUNC5")
	  )
	  	MSLICE_2  (
	  .glb(raddr[1]),	
	  .a({raddr[0],raddr[0]}),
	  .b({waddr[0],di[2]}),
	  .c({raddr[2],raddr[2]}),
	  .d({raddr[3],raddr[3]}),
	  .clk(),
	  .mi({1'b1,raddr[4]}),
	  .f({NOUSE[2],do[2]}),
	  .dpram_mode(dpram_mode),
	  .dpram_di(dpram_di[2]),
	  .dpram_we(dpram_we),
	  .dpram_waddr(dpram_waddr[4:0]),
	  .dpram_wclk(dpram_wclk),
	  .sload(we)
	  );
	
	ELF_PHY_MSLICE 
	  #(
	  .MODE("DPRAM"),
	  .INIT_LUT0(INIT_D6),
	  .INIT_LUT1(INIT_D7),
	  .FMUX0("FUNC5")
	   )
	  	MSLICE_3 (
	  .glb(raddr[1]),	
	  .a({raddr[0],raddr[0]}),
	  .b({waddr[1],di[3]}),
	  .c({raddr[2],raddr[2]}),
	  .d({raddr[3],raddr[3]}),
	  .clk(),
	  .mi({waddr[2],raddr[4]}),
	  .f({NOUSE[3],do[3]}),
	  .dpram_mode(dpram_mode),
	  .dpram_di(dpram_di[3]),
	  .dpram_we(dpram_we),
	  .dpram_waddr(dpram_waddr[4:0]),
	  .dpram_wclk(dpram_wclk),
	  .sload(we)
	  );
	ELF_PHY_DRAMHARDCON
	   hardcon (  
	  .b({waddr[1],di[3],waddr[0],di[2],waddr[4],di[1],waddr[3],di[0]}),
	  .mi7(waddr[2]),
	  .clk0(wclk),
	  .sload(we),
	  .dpram_mode(dpram_mode),
	  .dpram_di(dpram_di),
	  .dpram_we(dpram_we),
	  .dpram_waddr(dpram_waddr),
	  .dpram_wclk(dpram_wclk)
	  ); 

and ( dpram_check,we,done_gwe);	  
 specify                                      
 
        (wclk  *> do) = (0:0:0, 0:0:0);
        (raddr *> do) = (0:0:0, 0:0:0);
		
	$setuphold (posedge wclk, posedge di 	&&& (dpram_check!=0) , 0:0:0, 0:0:0);	
	$setuphold (posedge wclk, posedge waddr &&& (dpram_check!=0) , 0:0:0, 0:0:0);	
	$setuphold (posedge wclk, posedge we 	&&& (done_gwe!=0)    , 0:0:0, 0:0:0);

	$setuphold (negedge wclk, posedge di 	&&& (dpram_check!=0) , 0:0:0, 0:0:0);	
	$setuphold (negedge wclk, posedge waddr &&& (dpram_check!=0) , 0:0:0, 0:0:0);	
	$setuphold (negedge wclk, posedge we 	&&& (done_gwe!=0)    , 0:0:0, 0:0:0);

	$setuphold (posedge wclk, negedge di 	&&& (dpram_check!=0) , 0:0:0, 0:0:0);	
	$setuphold (posedge wclk, negedge waddr &&& (dpram_check!=0) , 0:0:0, 0:0:0);	
	$setuphold (posedge wclk, negedge we 	&&& (done_gwe!=0)    , 0:0:0, 0:0:0);

	$setuphold (negedge wclk, negedge di 	&&& (dpram_check!=0) , 0:0:0, 0:0:0);	
	$setuphold (negedge wclk, negedge waddr &&& (dpram_check!=0) , 0:0:0, 0:0:0);	
	$setuphold (negedge wclk, negedge we 	&&& (done_gwe!=0)    , 0:0:0, 0:0:0);


 endspecify	  
	       
endmodule

module ELF_PHY_DRAMHARDCON ( b, sload, clk0, mi7,dpram_di,dpram_we,dpram_waddr,dpram_wclk,dpram_mode); 
    parameter MODE = "DPRAM" ;
    input [7:0] b;
    input sload;
    input clk0;
    input mi7;
    output [3:0] dpram_di;
    output dpram_we;
    output [4:0] dpram_waddr;
    output dpram_wclk;
    output dpram_mode;
    reg dpram_mode;
    
    wire done_gwe = glbl.done_gwe;
    initial begin
    	case (MODE)
   		"DPRAM"  : begin 
   				dpram_mode = 1'b1;
   			    end	
    		default  : begin 
   				dpram_mode = 1'b0;
   			    end  
   	endcase
    end		     

and (dpram_di[0] ,b[0], dpram_mode);
and (dpram_di[1] ,b[2], dpram_mode);
and (dpram_di[2] ,b[4], dpram_mode);
and (dpram_di[3] ,b[6], dpram_mode);
and (dpram_waddr[0] ,b[5], dpram_mode);
and (dpram_waddr[1] ,b[7], dpram_mode);
and (dpram_waddr[2] ,mi7, dpram_mode);
and (dpram_waddr[3] ,b[1], dpram_mode);
and (dpram_waddr[4] ,b[3], dpram_mode);
and (dpram_we,sload,dpram_mode,done_gwe);
and (dpram_wclk,clk0,dpram_mode);
    
endmodule

module ELF_LOGIC_DRAM32X4W4R8 ( di, waddr, we, wclk, raddr, do);
parameter INIT_D0=16'h0000;
parameter INIT_D1=16'h0000;
parameter INIT_D2=16'h0000;
parameter INIT_D3=16'h0000;
parameter INIT_D4=16'h0000;
parameter INIT_D5=16'h0000;
parameter INIT_D6=16'h0000;
parameter INIT_D7=16'h0000;
  input [3:0]di;
  input [4:0]waddr;
  input wclk;
  input we;
  input [3:0]raddr;
  output [7:0]do;
wire dpram_mode,dpram_we;
wire [3:0]dpram_di;
wire [4:0]dpram_waddr;
wire dpram_wclk; 

// PLB MSLICE0
	ELF_PHY_MSLICE 
	  #(
	  .MODE("DPRAM"),
	  .INIT_LUT0(INIT_D0),
	  .INIT_LUT1(INIT_D1)
	  )
	  	MSLICE_0  (
	  .glb(raddr[1]),
	  .a({raddr[0],raddr[0]}),
	  .b({waddr[4],di[0]}),
	  .c({raddr[2],raddr[2]}),
	  .d({raddr[3],raddr[3]}),
	  .ce(),
	  .clk(),
	  .mi(),
	  .ssr(),
	  .asr(),
	  .fci(),
	  .f({do[4],do[0]}),
	  .fco(),
	  .dpram_mode(dpram_mode),
	  .dpram_di(dpram_di[0]),
	  .dpram_we(dpram_we),
	  .dpram_waddr(dpram_waddr[4:0]),
	  .dpram_wclk(dpram_wclk)     
	  );
	
	ELF_PHY_MSLICE 
	  #(
	  .MODE("DPRAM"),
	  .INIT_LUT0(INIT_D2),
	  .INIT_LUT1(INIT_D3)
	   )
	  	MSLICE_1 (
	  .glb(raddr[1]),	
	  .a({raddr[0],raddr[0]}),
	  .b({waddr[0],di[1]}),
	  .c({raddr[2],raddr[2]}),
	  .d({raddr[3],raddr[3]}),
	  .ce(),
	  .clk(),
	  .mi(),
	  .ssr(),
	  .fci(),
	  .f({do[5],do[1]}),
	  .q(),
	  .fco(),
	  .dpram_mode(dpram_mode),
	  .dpram_di(dpram_di[1]),
	  .dpram_we(dpram_we),
	  .dpram_waddr(dpram_waddr[4:0]),
	  .dpram_wclk(dpram_wclk)       
	  );

	ELF_PHY_MSLICE 
	  #(
	  .MODE("DPRAM"),
	  .INIT_LUT0(INIT_D4),
	  .INIT_LUT1(INIT_D5)
	  )
	  	MSLICE_2  (
	  .glb(raddr[1]),	
	  .a({raddr[0],raddr[0]}),
	  .b({waddr[1],di[2]}),
	  .c({raddr[2],raddr[2]}),
	  .d({raddr[3],raddr[3]}),
	  .ce(),
	  .clk(),
	  .mi(),
	  .ssr(),
	  .fci(),
	  .f({do[6],do[2]}),
	  .q(),
	  .fco(),
	  .dpram_mode(dpram_mode),
	  .dpram_di(dpram_di[2]),
	  .dpram_we(dpram_we),
	  .dpram_waddr(dpram_waddr[4:0]),
	  .dpram_wclk(dpram_wclk)     
	  );
	
	ELF_PHY_MSLICE 
	  #(
	  .MODE("DPRAM"),
	  .INIT_LUT0(INIT_D6),
	  .INIT_LUT1(INIT_D7)
	   )
	  	MSLICE_3 (
	  .glb(raddr[1]),	
	  .a({raddr[0],raddr[0]}),
	  .b({waddr[2],di[3]}),
	  .c({raddr[2],raddr[2]}),
	  .d({raddr[3],raddr[3]}),
	  .ce(),
	  .clk(),
	  .mi(),
	  .ssr(),
	  .fci(),
	  .f({do[7],do[3]}),
	  .q(),
	  .fco(),
	  .dpram_mode(dpram_mode),
	  .dpram_di(dpram_di[3]),
	  .dpram_we(dpram_we),
	  .dpram_waddr(dpram_waddr[4:0]),
	  .dpram_wclk(dpram_wclk)       
	  );
	ELF_PHY_DRAMHARDCON
	   hardcon (  
	  .b({waddr[2],di[3],waddr[1],di[2],waddr[0],di[1],waddr[4],di[0]}),
	  .mi7(waddr[3]),
	  .clk0(wclk),
	  .sload(we),
	  .dpram_mode(dpram_mode),
	  .dpram_di(dpram_di),
	  .dpram_we(dpram_we),
	  .dpram_waddr(dpram_waddr),
	  .dpram_wclk(dpram_wclk) 
	  );      
endmodule
