///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011/2021 Anlogic, Inc.
// All Right Reserved.
///////////////////////////////////////////////////////////////////////////////
//
//
//   Vendor : Anlogic
//   Version : 0.1
//   Description : Anlogic Functional Simulation Library Component
//            elf device slice
//   Filename : elf_phy_slice.v
//   Timestamp : Mon Nov  4 14:20:41 CST 2015
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    11/04/15 - Initial version.
//    04/20/16 - update parameter
//    05/20/16 - add SSRMUX parameter
//    06/28/16 - correct latch d input
//    07/13/16 - correct glbl.gsrn reset dff function ; 308~309line
///////////////////////////////////////////////////////////////////////////////
module ELF_PHY_MSLICE
  (
  a,
  b,
  c,
  d,
  ce,
  clk,
  mi,
  ssr,
  asr,
  fci,
  fco,  
  sload,  
  f,
  q,
  shift_in,
  shift_out, 
  dpram_mode,
  dpram_di,
  dpram_we,
  dpram_waddr,
  dpram_wclk,
  glb
  );
  input [1:0] a,b,c,d,mi;
  input clk;
  input ce;
  input ssr;
  input asr;  
  input sload;
  input fci;
  input shift_in;
  output shift_out;
  output [1:0] f,q;
  output fco;    
// dpram special interface
  input dpram_mode;
  input dpram_di;
  input dpram_we;
  input dpram_wclk;
  input [4:0]dpram_waddr;
  input glb;
      
tri1 [1:0] a,b,c,d,mi;
tri1 clk;
tri1 ce;
tri0 ssr;
tri1 asr;
tri0 fci;
tri1 sload; 
tri0 dpram_mode;
tri1 dpram_di;
tri0 dpram_we;
tri0 dpram_wclk;
tri0 [4:0]dpram_waddr;
tri1 [1:0] ifx;
tri1 glb;
tri0 shift_in;

parameter INIT_LUT0 = 16'h0000 ;
parameter INIT_LUT1 = 16'h0000 ;
parameter MODE = "LOGIC" ; // LOGIC/RIPPLE/DPRAM
parameter FMUX0 = "LUT4"; //LUT4,FUNC5
parameter FMUX1 = "LUT4"; //LUT4,FUNC6,FUNC7
parameter GSR = "ENABLE" ;
parameter CEMUX = "CE" ;
parameter ALUTYPE = "";
parameter ASRMUX = "ASR" ;
parameter CLKMUX = "CLK" ;
parameter DFFMODE = "FF" ;
parameter SHIFT = "DISABLE"; // DISABLE/ENABLE
parameter REG0_SD = "F" ; // F/MI/DYN
parameter REG1_SD = "F" ; // 
parameter REG0_REGSET = "RESET" ; // RESET/SET
parameter REG1_REGSET = "RESET" ;
parameter SSRMUX = "0";
parameter SSRSTATUS = "DISABLE"; // DISABLE/ENABLE
parameter SSRMODE = "OFF"; // OFF/CE_OVER_SSR
parameter SELGLB = "DISABLE"; // DISABLE/ENABLE , this parameter is not be USED now.
// global control signs
wire gsrn = glbl.gsrn;
wire done_gwe = glbl.done_gwe;

reg mc1_rp, latchmode ;
reg [1:0] mc1_f5;
reg [1:0] mc1_asr;
reg mc1_disgsr ;
reg mc1_disram;
reg mc1_ceoverlsr,mc1_qshift;
reg mc1_selglB,mc1_selglB_logic,mc1_ssruse;
reg mc1_useload_bank_a,mc1_useload_bank_b;
reg [1:0] mc1_sloaduse;

wire [1:0] f,q,sum;
wire [1:0] qt;

or (selglB,mc1_selglB,mc1_selglB_logic);
reg asr_in;
  buf ( q[0] , qt[0] );
  buf ( q[1] , qt[1] );
  buf ( shift_out, q[1] );

 			  			  			
initial begin
        mc1_useload_bank_a=1'b0;
        mc1_useload_bank_b=1'b0;
        case (REG0_SD)
                "F"   : begin
                          mc1_sloaduse[0]=0;
                          mc1_useload_bank_a=1'b0;
                        end   
                "MI"  : begin
                          mc1_sloaduse[0]=1;
                          mc1_useload_bank_a=1'b0;
                        end   
                "DYN" : begin
                          mc1_sloaduse[0]=1;
                          mc1_useload_bank_a=1'b1;
                        end  
                default : begin
                          mc1_sloaduse[0]=0;
                          mc1_useload_bank_a=1'b0;
                        end 
        endcase
        case (REG1_SD)
                "F"   : begin
                          mc1_sloaduse[1]=0;
                          mc1_useload_bank_b=1'b0;
                        end   
                "MI"  : begin
                          mc1_sloaduse[1]=1;
                          mc1_useload_bank_b=1'b0;
                        end   
                "DYN" : begin
                          mc1_sloaduse[1]=1;
                          mc1_useload_bank_b=1'b1;
                        end  
                default : begin
                          mc1_sloaduse[1]=0;
                          mc1_useload_bank_b=1'b0;
                        end 
        endcase        

        case (SELGLB)
                "ENABLE"   : mc1_selglB_logic = 1'b1;                
                default : mc1_selglB_logic = 1'b0;
        endcase 

        case (FMUX0)
                "FUNC5"   : mc1_f5[0] = 1'b1;
                "LUT4"  : mc1_f5[0] = 1'b0;
                default : mc1_f5[0] = 1'b0;
        endcase  

        case (FMUX1)
                "FUNC7"   : mc1_f5[1] = 1'b1;
                "FUNC6"   : mc1_f5[1] = 1'b1;
                "LUT4"  : mc1_f5[1] = 1'b0;
                default : mc1_f5[1] = 1'b0;
        endcase

	case (MODE)
   		"LOGIC"  : begin 
   				mc1_rp = 0;
  				mc1_disram = 0;
  				mc1_selglB = 0; // special cas
   			    end	
   		"RIPPLE"  : begin // begin
   				mc1_rp = 1;   //
    				mc1_disram = 0;
    				mc1_selglB = 0; // special cas
   			    end	         // end
   		"DPRAM"   : begin
   				mc1_rp = 0;
   			        mc1_disram = 1 ;
   			        mc1_selglB = 1 ; // special case
   			    end	    
   		default :   begin 
   				mc1_rp = 0;
   				mc1_disram = 0;
   				mc1_selglB = 0; // special case
   			    end
   	endcase

// SLICE DFF CONFIG
	case (DFFMODE)
   		"FF"      : latchmode = 0;
   		"LATCH"   : latchmode = 1;
   		default : latchmode = 0;
   	endcase	 
	
	case (REG0_REGSET)
   		"RESET"   : mc1_asr[0] = 0;
   		"SET"     : mc1_asr[0] = 1;
   		default : mc1_asr[0] = 0;
   	endcase	

	case (REG1_REGSET)
   		"RESET"   : mc1_asr[1] = 0;
   		"SET"     : mc1_asr[1] = 1;
   		default : mc1_asr[1] = 0;
   	endcase	

	case (GSR)
   		"DISABLE"   : mc1_disgsr = 1;
   		"ENABLE"    : mc1_disgsr = 0;
   		default : mc1_disgsr = 0;
   	endcase
	case (SHIFT)
   		"DISABLE"   : mc1_qshift = 0;
   		"ENABLE"    : mc1_qshift = 1;
   		default : mc1_qshift = 0;
   	endcase
   	case (SSRMODE)
   		"SSR_OVER_CE"    : mc1_ceoverlsr = 0;
   		"CE_OVER_SSR"    : mc1_ceoverlsr = 1;   	
   		default : mc1_ceoverlsr = 0;
   	endcase
   	case (SSRSTATUS)
   		"DISABLE"    : mc1_ssruse = 0;
   		"ENABLE"     : mc1_ssruse = 1;   	
   		default : mc1_ssruse = 0;
   	endcase   	
   	   	   		
end // initial


always @( mc1_useload_bank_a or mc1_useload_bank_b)
   begin
      if( mc1_useload_bank_a ^ mc1_useload_bank_b )
         $display("Attribute Syntax Error : The Parameter REG0_SD/REG1_SD must both be DYN or both not DYN.");   
   end

   	
reg ce_in,clk_in,ssr_in ;   	

	// CEMUX/SRMUX/CLKMUX		
	always @(ce or CEMUX)
  		begin
    		case(CEMUX)
  			"CE"  : ce_in = ce ;
  			"INV" : ce_in = ~ce ;
  			"1"   : ce_in = 1'b1;
  			"0"   : ce_in = 1'b0;
  			//default: ce_in = 1'b1;    	
  			default: ce_in = ce;  //
		endcase
		end
    always @(ssr or SSRMUX)
        begin
            case(SSRMUX)
            "SSR" : ssr_in = ssr;
            "INV" : ssr_in = ~ssr;
            "1"   : ssr_in = 1'b1;
            "0"   : ssr_in = 1'b0;
            default: ssr_in = 1'b0;
            endcase
       end     
            
	always @(asr or ASRMUX)
  		begin
    		case(ASRMUX)
  			"ASR"  : asr_in = asr ;
  			"INV" : asr_in = ~asr ;
  			"1"   : asr_in = 1'b1;
  			"0"   : asr_in = 1'b0;
  			//default: sr_in = 1'b1;    	
  			default: asr_in = asr; //   	
		endcase
		end

	always @(clk or CLKMUX)
  		begin
    		case(CLKMUX)
  			"CLK"  	: clk_in = clk ;
  			"INV" 		: clk_in = ~clk ;  			
  			"1"   		: clk_in = 1'b1;
  			"0"   		: clk_in = 1'b0;
  			default		: clk_in = clk ;    	
		endcase
		end


wire sync_reset = (~mc1_ceoverlsr | ce_in) & ssr_in ;
// 2016/07/13 or asr_in with gsrn for power up initial
wire global_gsr=!gsrn;
or (async_sr,asr_in,global_gsr);
wire fxina_n = 1'b0;
wire fxinb_n = 1'b0;		

wire disram_mode = mc1_disram | dpram_mode ;
wire [1:0] lut4, gen_n ;
wire [1:0] btmp = selglB ? {glb,glb} : b[1:0];
wire [1:0] dpram_we_t ;
assign dpram_we_t[0] = ~dpram_waddr[4] & dpram_we ;
assign dpram_we_t[1] = dpram_waddr[4] & dpram_we ;

pfb_slice01_lut4 #(.INIT(INIT_LUT0)) lut4_0(.lut4_o(lut4[0]) , .lut00_n(gen_n[0]) , .mc1_ripmode(mc1_rp), .I0(a[0]), .I1(btmp[0]), .I2(c[0]), .I3(d[0]),
		.dpram_mode(disram_mode),.dpram_di(dpram_di),.dpram_we(dpram_we_t[0]),.dpram_wclk(dpram_wclk),.dpram_waddr(dpram_waddr[3:0])
		); 
pfb_slice01_lut4 #(.INIT(INIT_LUT1)) lut4_1(.lut4_o(lut4[1]) , .lut00_n(gen_n[1]) , .mc1_ripmode(mc1_rp), .I0(a[1]), .I1(btmp[1]), .I2(c[1]), .I3(d[1]),
		.dpram_mode(disram_mode),.dpram_di(dpram_di),.dpram_we(dpram_we_t[1]),.dpram_wclk(dpram_wclk),.dpram_waddr(dpram_waddr[3:0])
);

wire [1:0]fx,fxx;
wire mc1_fpathuse = mc1_f5[0] | mc1_f5[1] | mc1_rp ;
assign fx[1] = mc1_fpathuse & lut4[1] ;
assign fx[0] = mc1_fpathuse & lut4[0] ;
wire func5 = mi[0]? fx[1] : fx[0];
wire funcX = mi[1]? ifx[1] : ifx[0]; 


buf ( f5o , func5 );
buf ( fxo , funcX );

  assign fxx[1] = mc1_rp ? sum[1] : funcX;
  assign fxx[0] = mc1_rp ? sum[0] : func5;
  assign f1 = ( mc1_rp | mc1_f5[1] ) ?   fxx[1] : lut4[1];
  assign f0 = ( mc1_rp | mc1_f5[0] ) ?   fxx[0] : lut4[0];
  buf ( f[0] , f0 );
  buf ( f[1] , f1 );
   

elf_pfb_carrysum carrychain( .fco(fco), .sum(sum), .lut(lut4) , .gen_n(gen_n), .ci(fci), .mc1_rp(mc1_rp) );


wire [1:0] dff_di ;
assign dff_di[0] = mc1_qshift ? shift_in : mi[0]; 
assign dff_di[1] = mc1_qshift ? qt[0]    : mi[1];
or (sload0,sload,!mc1_useload_bank_a);
or (sload1,sload,!mc1_useload_bank_b);
elf_dff dff_slice_half0 ( .q(qt[0]) , .clk(clk_in), .ce(ce_in), .async_sr(async_sr), .sync_reset(sync_reset), .f(f0), .di(dff_di[0]), .sload(sload0), .mc1_ssruse(mc1_ssruse), .mc1_sloaduse(mc1_sloaduse[0]), .mc1_asr(mc1_asr[0]), .latchmode(1'b0) );  
elf_dff dff_slice_half1 ( .q(qt[1]) , .clk(clk_in), .ce(ce_in), .async_sr(async_sr), .sync_reset(sync_reset), .f(f1), .di(dff_di[1]), .sload(sload1), .mc1_ssruse(mc1_ssruse), .mc1_sloaduse(mc1_sloaduse[1]), .mc1_asr(mc1_asr[1]), .latchmode(latchmode) );

and ( ce_check_en,ce_in,done_gwe);
and ( dpram_check,dpram_we,done_gwe);
 specify                                      
 
        (clk  => shift_out) = (0:0:0, 0:0:0);
        (clk  => q[0]) = (0:0:0, 0:0:0);
        (clk  => q[1]) = (0:0:0, 0:0:0);                                                             
	(a[0] => f[0]) = (0:0:0, 0:0:0);              
	(b[0] => f[0]) = (0:0:0, 0:0:0);              
	(c[0] => f[0]) = (0:0:0, 0:0:0);              
	(d[0] => f[0]) = (0:0:0, 0:0:0);              
	(a[1] => f[1]) = (0:0:0, 0:0:0);              
	(b[1] => f[1]) = (0:0:0, 0:0:0);              
	(c[1] => f[1]) = (0:0:0, 0:0:0);              
	(d[1] => f[1]) = (0:0:0, 0:0:0);
	(a[0] => f[1]) = (0:0:0, 0:0:0);              
	(b[0] => f[1]) = (0:0:0, 0:0:0);              
	(c[0] => f[1]) = (0:0:0, 0:0:0);              
	(d[0] => f[1]) = (0:0:0, 0:0:0);              
	(a[1] => f[0]) = (0:0:0, 0:0:0);              
	(b[1] => f[0]) = (0:0:0, 0:0:0);              
	(c[1] => f[0]) = (0:0:0, 0:0:0);              
	(d[1] => f[0]) = (0:0:0, 0:0:0);	
	
	(mi[0] => f[0]) = (0:0:0, 0:0:0); 
	(mi[1] => f[1]) = (0:0:0, 0:0:0);
	
	(a[0] => fco) = (0:0:0, 0:0:0);              
	(b[0] => fco) = (0:0:0, 0:0:0);              
	(c[0] => fco) = (0:0:0, 0:0:0);              
	(d[0] => fco) = (0:0:0, 0:0:0);              
	(a[1] => fco) = (0:0:0, 0:0:0);              
	(b[1] => fco) = (0:0:0, 0:0:0);              
	(c[1] => fco) = (0:0:0, 0:0:0);              
	(d[1] => fco) = (0:0:0, 0:0:0);	
	(fci => fco)   = (0:0:0, 0:0:0);
	(fci => f[0])  = (0:0:0, 0:0:0);
	(fci => f[1])  = (0:0:0, 0:0:0);	
        $recovery (asr, posedge clk &&& (done_gwe!=0), 0:0:0 );
        $recovery (asr, negedge clk &&& (done_gwe!=0), 0:0:0 );
        $removal  (asr, posedge clk &&& (done_gwe!=0), 0:0:0 );
        $removal  (asr, negedge clk &&& (done_gwe!=0), 0:0:0 );
        	
	$setuphold (posedge clk, posedge ce  &&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge ce  &&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge clk, posedge ssr &&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge ssr &&& (done_gwe!=0) , 0:0:0, 0:0:0);

	$setuphold (posedge clk, negedge ce  &&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge ce  &&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge clk, negedge ssr &&& (done_gwe!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge ssr &&& (done_gwe!=0) , 0:0:0, 0:0:0);	
		
	$setuphold (posedge clk, posedge mi[0]    &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge clk, posedge mi[1]    &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge mi[0]    &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge mi[1]    &&& (ce_check_en!=0) , 0:0:0, 0:0:0);	
	$setuphold (posedge clk, posedge fci      &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge fci      &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge clk, posedge sload    &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge sload    &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge clk, posedge shift_in &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge shift_in &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	
	$setuphold (posedge clk, negedge mi[0]    &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge clk, negedge mi[1]    &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge mi[0]    &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge mi[1]    &&& (ce_check_en!=0) , 0:0:0, 0:0:0);	
	$setuphold (posedge clk, negedge fci      &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge fci      &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge clk, negedge sload    &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge sload    &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge clk, negedge shift_in &&& (ce_check_en!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge shift_in &&& (ce_check_en!=0) , 0:0:0, 0:0:0);	
		
	$setuphold (posedge clk, posedge a[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clk, posedge b[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clk, posedge c[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clk, posedge d[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clk, posedge a[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clk, posedge b[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clk, posedge c[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clk, posedge d[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);

	$setuphold (posedge clk, negedge a[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clk, negedge b[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clk, negedge c[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clk, negedge d[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clk, negedge a[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clk, negedge b[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (posedge clk, negedge c[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);
	$setuphold (posedge clk, negedge d[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);

	$setuphold (negedge clk, posedge a[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clk, posedge b[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clk, posedge c[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge d[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clk, posedge a[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clk, posedge b[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clk, posedge c[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clk, posedge d[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);

	$setuphold (negedge clk, negedge a[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clk, negedge b[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clk, negedge c[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge d[0] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clk, negedge a[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clk, negedge b[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);	
	$setuphold (negedge clk, negedge c[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);
	$setuphold (negedge clk, negedge d[1] &&& (ce_check_en!=0)	, 0:0:0, 0:0:0);

	$setuphold (posedge dpram_wclk, posedge dpram_we &&& (done_gwe!=0)	, 0:0:0, 0:0:0);
	$setuphold (posedge dpram_wclk, posedge dpram_di 	&&& (dpram_check!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge dpram_wclk, posedge dpram_waddr 	&&& (dpram_check!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge dpram_wclk, posedge dpram_we 	&&& (dpram_check!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge dpram_wclk, posedge dpram_di 	&&& (dpram_check!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge dpram_wclk, posedge dpram_waddr 	&&& (dpram_check!=0) , 0:0:0, 0:0:0);

	$setuphold (posedge dpram_wclk, negedge dpram_we &&& (done_gwe!=0)	, 0:0:0, 0:0:0);
	$setuphold (posedge dpram_wclk, negedge dpram_di 	&&& (dpram_check!=0) , 0:0:0, 0:0:0);
	$setuphold (posedge dpram_wclk, negedge dpram_waddr 	&&& (dpram_check!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge dpram_wclk, negedge dpram_we 	&&& (dpram_check!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge dpram_wclk, negedge dpram_di 	&&& (dpram_check!=0) , 0:0:0, 0:0:0);
	$setuphold (negedge dpram_wclk, negedge dpram_waddr 	&&& (dpram_check!=0) , 0:0:0, 0:0:0);	

 endspecify

endmodule


module pfb_slice01_lut4 (lut4_o , lut00_n , mc1_ripmode, I0, I1, I2, I3, dpram_mode,dpram_di,dpram_we,dpram_wclk,dpram_waddr); 
output lut4_o;
output lut00_n;
input mc1_ripmode;
input I0,I1,I2,I3;
parameter INIT=16'h0000 ;

//dpram interface
input dpram_mode;
input dpram_di,dpram_we,dpram_wclk;
input [3:0] dpram_waddr;

reg [15:0] value_lut4;
reg lut4_o;
reg lut00,lut01,lut10,lut11;
initial begin
	value_lut4 = INIT ;
end

wire dpram_weclk = dpram_mode & dpram_wclk ;
	always @(posedge dpram_weclk)
	begin
	   if(dpram_we) 
	   begin
	   value_lut4[dpram_waddr]= dpram_di ;
	   end  
	end

      always @(I1 or I0 or value_lut4[3:0])
      begin
      case({I1,I0})
      	2'b00 : lut00 = value_lut4[0];
      	2'b01 : lut00 = value_lut4[1];
      	2'b10 : lut00 = value_lut4[2];
      	2'b11 : lut00 = value_lut4[3];
      endcase
      end	
      always @(I1 or I0 or value_lut4[7:4])
      begin
      case({I1,I0})
      	2'b00 : lut01 = value_lut4[4];
      	2'b01 : lut01 = value_lut4[5];
      	2'b10 : lut01 = value_lut4[6];
      	2'b11 : lut01 = value_lut4[7];
      endcase
      end
      always @(I1 or I0 or value_lut4[11:8])
      begin
      case({I1,I0})
      	2'b00 : lut10 = value_lut4[8];
      	2'b01 : lut10 = value_lut4[9];
      	2'b10 : lut10 = value_lut4[10];
      	2'b11 : lut10 = value_lut4[11];
      endcase
	end
      always @(I1 or I0 or value_lut4[15:12])
      begin
      case({I1,I0})
      	2'b00 : lut11 = value_lut4[12];
      	2'b01 : lut11 = value_lut4[13];
      	2'b10 : lut11 = value_lut4[14];
      	2'b11 : lut11 = value_lut4[15];
      endcase      
	end	
      always @(lut00 or lut01 or lut10 or lut11 or I3 or I2)
      begin
      case({I3,I2})
      	2'b00 : lut4_o = lut00;
      	2'b01 : lut4_o = lut01;
      	2'b10 : lut4_o = lut10;
      	2'b11 : lut4_o = lut11;
      endcase     
      end
                         
      nand (lut00_n, mc1_ripmode, lut00);
      
endmodule


module elf_pfb_carrysum( fco, sum, lut, gen_n, ci, mc1_rp);
output fco;
output [1:0]sum;
input [1:0]lut, gen_n;
input ci,mc1_rp;
wire [1:0]sum;
wire co0b;
reg fco;

nand (ci0b,ci,mc1_rp);
assign co0b = lut[0] ? ci0b : gen_n[0];
assign sum[0] = ~lut[0] ^ ci0b ;
assign sum[1] = ~lut[1] ^ co0b ;

always @(ci or co0b or gen_n or lut )
   begin
      case(lut)
       2'b10 : fco = ~co0b;
       2'b11 : fco = ci;
       2'b00 : fco = ~gen_n[1];
       2'b01 : fco = ~gen_n[1];
       endcase
   end 
      
endmodule


module elf_dff ( q , clk, ce, async_sr, sync_reset, f, di, sload, mc1_ssruse, mc1_sloaduse, mc1_asr, latchmode); 
output q;
input ce,async_sr,sync_reset,clk,di,f;
input sload,mc1_ssruse,mc1_sloaduse,mc1_asr;
input latchmode;
buf (qfb,q);
wire s = sload & mc1_sloaduse ;
wire dit= ce ? (s ? di : f) : qfb ;
not (din ,dit);

wire di_dff = !( mc1_ssruse & sync_reset | din );
wire aset_n = !(mc1_asr & async_sr) ;
wire areset_n = !(!mc1_asr & async_sr) ;


reg q_latch,q_dff; 
 always @( clk or di_dff or areset_n or aset_n)
 begin
   if(!areset_n) q_latch = 1'b0;
   else if(!aset_n) q_latch = 1'b1;
   else if(!clk) q_latch = di_dff;
 end

 wire q = latchmode ? q_latch : q_dff ;
 
 always @(posedge clk or negedge aset_n or  negedge areset_n )
 begin
     if(!areset_n) q_dff <= 1'b0;
     else if(!aset_n) q_dff <= 1'b1;
     else
          q_dff <= di_dff ;          
 end
  
endmodule

