///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011-2021 Anlogic, Inc.
// All Right Reserved.
///////////////////////////////////////////////////////////////////////////////
//
//
//   Vendor : Anlogic
//   Version : 0.1
//   Description : Anlogic Functional Simulation Library Component
//            ufm
//   Filename : elf_syn_ufm.v
//   Timestamp : Fri Jul  22 15:55:41 CST 2016
//
///////////////////////////////////////////////////////////////////////////////
//
// Revision:
//    07/22/16 - Initial version.
//
///////////////////////////////////////////////////////////////////////////////
`timescale 1ns / 1ns

module ELF_SYN_UFM(CLK, RST,WRITE,READ,ERASE,DAT_SHIFT,ADR_SHIFT,DIN,DOUT,VALID,BUSY);
input CLK,RST;
input WRITE,READ,ERASE;
input DAT_SHIFT;
input ADR_SHIFT;
input [7:0] DIN;
output [7:0] DOUT;
output VALID;
output BUSY;

wire [3:0] wb_adr_i;
wire [7:0] wb_dat_i;

wire wb_clk_i;
wire wb_cyc_en;
wire wb_rst;
wire wb_stb_en;
wire wb_wt_en;
wire pib_ufm_ae;
wire pib_ufm_cs;
wire pib_ufm_nvstr;
wire pib_ufm_oe;
wire pib_ufm_prog;
wire pib_ufm_sera;
wire ufm_tbit_o;
wire [7:0] wb_dat_o;

ELF_SYN_UFM_INTERNAL u_ELF_SYN_UFM_INTERNAL(.CLK(CLK),
                    .RST(RST),
                    .READ(READ),
                    .ERASE(ERASE),
                    .DOUT(DOUT),
                    .VALID(VALID),
                    .WRITE(WRITE),
                    .ADR_SHIFT(ADR_SHIFT),
                    .DAT_SHIFT(DAT_SHIFT),
                    .BUSY(BUSY),
                    .DIN(DIN),
                    .wb_adr_i(wb_adr_i),      
                    .wb_clk_i(wb_clk_i),      
                    .wb_cyc_en(wb_cyc_en),     
                    .wb_dat_i(wb_dat_i),      
                    .wb_rst(wb_rst),        
                    .wb_stb_en(wb_stb_en),     
                    .wb_wt_en(wb_wt_en),      
                    .pib_ufm_ae(pib_ufm_ae),    
                    .pib_ufm_cs(pib_ufm_cs),    
                    .pib_ufm_nvstr(pib_ufm_nvstr), 
                    .pib_ufm_oe(pib_ufm_oe),    
                    .pib_ufm_prog(pib_ufm_prog),  
                    .pib_ufm_sera(pib_ufm_sera),
                    .ufm_tbit_o(ufm_tbit_o),
                    .wb_dat_o(wb_dat_o));


//PHY_CONFIG INST
`ifdef SIM
  ELF_CFGIP_SIM u_config(
`else 
  ELF_PHY_CONFIG u_config(
`endif
  .wb_dat_o(wb_dat_o),
  .wb_wt_en(wb_wt_en),
  .wb_cyc_en(wb_cyc_en),
  .wb_stb_en(wb_stb_en),
  .wb_rst(wb_rst),
  .wb_dat_i(wb_dat_i),
  .wb_adr_i(wb_adr_i),
  .wb_clk_i(wb_clk_i),
  
  //ufm interface
  .ufm_tbit_o(ufm_tbit_o),
  .pib_ufm_cs(pib_ufm_cs),
  .pib_ufm_ae(pib_ufm_ae),
  .pib_ufm_oe(pib_ufm_oe),
  .pib_ufm_prog(pib_ufm_prog),
  .pib_ufm_nvstr(pib_ufm_nvstr),
  .pib_ufm_sera(pib_ufm_sera),
  
  //to pib
  .pib_done(),
  //.pib_freeze_io(),
  //.pib_cclk(),
  
  //from pib
  .pib_programn(1'b1),
  .pib_ts_all(1'b0),
  //.pib_user_clk(),
  .pib_user_gsr()
  //.gsrn_sync_clk(),
  
  //isp interface
  //.jce(),
  //.jrstn(),
  //.jrti(),
  //.jshift(),
  //.jtck(),
  //.jtdi(),
  //.jtms(),
  //.jupdate(),
  //.jtdo()
  );

endmodule 
 

module ELF_SYN_UFM_INTERNAL(CLK, RST,WRITE,READ,ERASE,DAT_SHIFT,ADR_SHIFT,DIN,DOUT,VALID,BUSY,
wb_adr_i,
wb_clk_i,
wb_cyc_en,
wb_dat_i,
wb_rst,
wb_stb_en,
wb_wt_en,
pib_ufm_ae,
pib_ufm_cs,
pib_ufm_nvstr,
pib_ufm_oe,
pib_ufm_prog,
pib_ufm_sera,
ufm_tbit_o,
wb_dat_o
);

input  ufm_tbit_o;
input [7:0] wb_dat_o;
input CLK,RST;
input WRITE,READ,ERASE;
input DAT_SHIFT;
input ADR_SHIFT;
input [7:0] DIN;
output [7:0] DOUT;
output VALID;
output BUSY;

output [3:0] wb_adr_i;
output wb_clk_i;
output wb_cyc_en;
output [7:0] wb_dat_i;
output wb_rst;
output wb_stb_en;
output wb_wt_en;

output pib_ufm_ae;
output pib_ufm_cs;
output pib_ufm_nvstr;
output pib_ufm_oe;
output pib_ufm_prog;
output pib_ufm_sera;

reg [7:0] wb_dat_i;
reg wb_stb_en;
reg pib_ufm_ae;
reg pib_ufm_oe;
reg [15:0] pib_nj_addr;
reg [31:0] pib_nj_data;
reg ufm_prog,ufm_prog_d1;
reg ufm_sera,ufm_sera_d1;
reg pib_ufm_nvstr;
reg [3:0] byte_cnt;
reg update;
reg [3:0] wb_adr_i;
reg [1:0] c_state,n_state;
reg ufm_tbit_o_d2,ufm_tbit_o_d1;
reg start_d2,start_d1;

wire [7:0] wb_dat_o;
wire wb_rst;
wire wb_cyc_en;
wire wb_wt_en;
wire [6:0] byte_end;
wire pib_ufm_cs;
wire read_vld;
wire write_erase_vld;
wire ufm_tbit_o;

wire pib_ufm_prog;
wire pib_ufm_sera;
wire st_adr;
wire st_dat;

wire start,start_org;
wire update_en;
wire block_read;
wire block_write;


parameter IDLE  = 2'h0;
parameter EN  = 2'h1;
parameter ADR = 2'h2;
parameter DAT = 2'h3;


wire st_idle;
wire st_w_en;
//wire st_r_en;
//wire st_e_en;
wire st_r_adr;
wire st_e_adr;
wire st_w_dat;
wire st_r_dat;
wire st_en;


//  wb_adr_i = 4'hc;   //wr_ufmcr address : 8'hc
//  wb_dat_i = 8'h80;  //wb_dat_i -> ufm pib enable 
//  wb_adr_i = 4'hd;   //wr_ufmtxdr address : 8'hd
//  wb_dat_i = 8bits: read_ufm_addr;   //wr_ufmtxdr address : 8'hd


  //assign wb_clk_i = CLK;
  ELF_LOGIC_BUF u_buf (.o(wb_clk_i), .i(CLK));
  assign start_org = WRITE|READ|ERASE;
  assign wb_rst = RST | ~start_org; //RST or WRITE/READ/ERASE all are unasserted
  assign read_vld = wb_cyc_en & wb_stb_en & ~wb_wt_en & ~wb_rst;
  assign DOUT = wb_dat_o;
  assign wb_cyc_en = ~st_idle;
  assign pib_ufm_cs = ~wb_rst;
  assign VALID = read_vld | write_erase_vld;
  assign BUSY = ufm_tbit_o & (WRITE|ERASE);


  always @(negedge wb_clk_i or posedge wb_rst)
  if(wb_rst)
    {ufm_tbit_o_d2,ufm_tbit_o_d1} <= 2'b00;
  else 
    {ufm_tbit_o_d2,ufm_tbit_o_d1} <= {ufm_tbit_o_d1,ufm_tbit_o};
  
  assign write_erase_vld = ~ufm_tbit_o_d1 & ufm_tbit_o_d2;  // negedge pulse  of ufm_tbit_o 
  
//--------------sample address data-----------------
  always @(negedge wb_clk_i or posedge wb_rst) 
  if(wb_rst)
    pib_nj_addr <= 14'd0;
  else if(WRITE|READ|ERASE)
    if(ADR_SHIFT)
      pib_nj_addr <= {pib_nj_addr[7:0],DIN};
    else if(block_write || block_read)
      pib_nj_addr <= pib_nj_addr + 1'b1;
    else ;
  else;

  always @(negedge wb_clk_i or posedge wb_rst) 
  if(wb_rst)
    pib_nj_data <= 32'h0;
  else if(WRITE)
    if(DAT_SHIFT)
      pib_nj_data <= {pib_nj_data[23:0],DIN};
    else;
  else;
//--------------sample address data-----------------
  
  always @(negedge wb_clk_i or posedge wb_rst)
  if(wb_rst) 
    {start_d2,start_d1} <= 2'b00;
  else 
    {start_d2,start_d1} <= {start_d1,start_org};
  assign start = start_d1 && ~start_d2;
//-----------for FSM combination -----------------  
  reg start_level, block_write_level;
  always @(negedge wb_clk_i or posedge wb_rst) 
  if(wb_rst) begin
    start_level <= 1'b0;
  end
  else begin
    if(start)
      start_level <= 1'b1;
    else if(update)
      start_level <= 1'b0;
    else;
  end

  always @(negedge wb_clk_i or posedge wb_rst) 
  if(wb_rst) begin
    block_write_level <= 1'b0;
  end
  else begin
    if(block_write)
      block_write_level <= 1'b1;
    else if(update)
      block_write_level <= 1'b0;
    else;
  end
//-----------for FSM combination -----------------  
//----------FSM----------------
  always @(negedge wb_clk_i or posedge wb_rst) 
  if(wb_rst) begin
    c_state <= IDLE;
  end
  else begin
    if(update)
      c_state <= n_state;
    else;
  end
  always @(*) begin
    if(wb_rst)
      n_state <= IDLE;
    else begin
      case(c_state) 
        IDLE:  if(start_level) 
                n_state <= EN;
               else if(block_write_level)
                  n_state <= ADR;
               else
                  n_state <= IDLE;
        EN   :  n_state <= ADR;
        ADR  : if(WRITE|READ)
                n_state <= DAT; 
               else
                n_state <= IDLE; 
        DAT:   if(READ)
                 n_state <= ADR;
               else
                 n_state <= IDLE;
        default: n_state <= IDLE;
      endcase
    end
  end

  assign st_idle  = c_state == IDLE; 
  assign st_w_en  = (c_state == EN) && WRITE; 
  assign st_r_adr = (c_state == ADR) && READ;
  assign st_e_adr = (c_state == ADR) && ERASE;
  assign st_w_dat = (c_state == DAT) && WRITE;
  assign st_r_dat = (c_state == DAT) &&  READ;

  assign st_en     = c_state == EN; 
  assign st_adr    = c_state == ADR; 
  assign st_dat    = c_state == DAT; 
//----------FSM----------------
  assign byte_end = st_en?4'd2:(st_adr|st_dat)?4'd8:4'd15;

//-------------------block_read_write--------------
  assign block_read = READ && c_state == DAT && n_state == ADR && update;
  assign block_write = WRITE && VALID;
//-------------------block_read_write--------------

  always @(negedge wb_clk_i or posedge wb_rst) 
  if(wb_rst) begin
    byte_cnt <= 4'd0;
  end
  else begin
    if(READ|WRITE|ERASE|VALID) begin
      if(block_write ||(byte_cnt >= byte_end))
        byte_cnt <= 4'd0;
      else
        byte_cnt <= byte_cnt + 1'b1;
    end
    else
      byte_cnt <= 4'd0;
  end

  assign update_en = block_write || (~pib_ufm_nvstr && byte_cnt >= byte_end);
  always @(negedge wb_clk_i or posedge wb_rst) 
  if(wb_rst) begin
    update <= 1'b0;
  end
  else begin
    if(update_en)
      update <= 1'b1;
    else 
      update <= 1'b0;
  end


  always @(negedge wb_clk_i or posedge wb_rst) 
  if(wb_rst) begin
    wb_adr_i <= 4'h0;
  end
  else begin
    case ({st_en,st_adr,st_w_dat,st_r_dat})
      4'b1000: wb_adr_i <= 4'hc;
      4'b0100: wb_adr_i <= 4'hd;
      4'b0010: wb_adr_i <= 4'hd;
      4'b0001: wb_adr_i <= 4'he;
      default: wb_adr_i <= 4'h0;
    endcase
  end
 
  wire ufm_prog_en;
  assign ufm_prog_en = (st_w_dat&&byte_cnt==4'h0);
  always @(negedge wb_clk_i)
  if(ufm_prog_en)  begin
    ufm_prog <= 1'b1;
  end
  else begin
    ufm_prog <= 1'b0;
  end

  always @(negedge wb_clk_i)
    ufm_prog_d1 <= ufm_prog;

  assign pib_ufm_prog = ufm_prog || ufm_prog_d1;

  always @(negedge wb_clk_i)
  if(st_e_adr && update)  begin
    ufm_sera <= 1'b1;
  end
  else begin
    ufm_sera <= 1'b0;
  end

  always @(negedge wb_clk_i or posedge wb_rst) 
  if(wb_rst)
    ufm_sera_d1 <= 1'b0;
  else
    ufm_sera_d1 <= ufm_sera;

  assign pib_ufm_sera = ufm_sera || ufm_sera_d1;

  always @(negedge wb_clk_i or posedge wb_rst) 
  if(wb_rst) begin
    pib_ufm_ae <= 1'b0;
  end
  else begin
    if(st_r_adr &&  byte_cnt >= byte_end - 2'd2)
      pib_ufm_ae <= 1'b1;
    else 
      pib_ufm_ae <= ufm_prog|ufm_sera;
  end

  always @(negedge wb_clk_i or posedge wb_rst) 
  if(wb_rst)
    pib_ufm_nvstr <= 1'b0;
  else begin
    if(pib_ufm_ae && (WRITE||ERASE))
      pib_ufm_nvstr <= 1'b1;
    else if(VALID)
      pib_ufm_nvstr <= 1'b0;
  end


  always @(negedge wb_clk_i or posedge wb_rst) 
  if(wb_rst) begin
    pib_ufm_oe <= 1'b0;
  end
  else if(st_r_adr)begin
    pib_ufm_oe <= pib_ufm_ae;
  end
  else begin
    pib_ufm_oe <= 1'b0;
  end

  assign wb_wt_en = st_en|st_adr|st_w_dat;

  always @(negedge wb_clk_i or posedge wb_rst) 
  if(wb_rst) begin
    wb_dat_i <= 8'hff;
    wb_stb_en <= 1'b0;
  end
  else begin
    if(st_en) begin
      wb_stb_en <= ~wb_stb_en;
      wb_dat_i <= {1'b1,st_w_en,6'h0};  //8'hc0: 8'h80
    end
    else if (st_adr) begin
      case (byte_cnt)
        4'd2   : begin wb_dat_i <= {4'b0001,pib_nj_addr[11:8]}; wb_stb_en <= 1'b1; end
        4'd4   : begin wb_dat_i <= pib_nj_addr[7:0]; wb_stb_en <= 1'b1; end
        default: begin wb_dat_i <= 8'hff;      wb_stb_en <= 1'b0; end
      endcase 
    end
    else if (st_r_dat) begin
      wb_stb_en <= ~wb_stb_en;
    end
    else if (st_w_dat) begin
      case (byte_cnt)
        4'd1   : begin wb_dat_i <= pib_nj_data[31:24]; wb_stb_en <= 1'b1; end
        4'd3   : begin wb_dat_i <= pib_nj_data[23:16]; wb_stb_en <= 1'b1; end
        4'd5   : begin wb_dat_i <= pib_nj_data[15:8]; wb_stb_en <= 1'b1; end
        4'd7   : begin wb_dat_i <= pib_nj_data[7:0]; wb_stb_en <= 1'b1; end
        default: begin wb_dat_i <= 8'hff;      wb_stb_en <= 1'b0; end
      endcase 
    end
    else begin 
      wb_dat_i <= 8'hff;      
      wb_stb_en <= 1'b0; 
    end
  end

endmodule 

